@@ -15,6 +15,7 @@
 #include "base/files/memory_mapped_file.h"
 #include "base/files/scoped_temp_dir.h"
 #include "base/memory/ptr_util.h"
+#include "base/memory/read_only_shared_memory_region.h"
 #include "base/pending_task.h"
 #include "base/process/process.h"
 #include "base/stl_util.h"
@@ -37,7 +38,7 @@ class TestActivityTracker : public ThreadActivityTracker {
       : ThreadActivityTracker(memset(memory.get(), 0, mem_size), mem_size),
         mem_segment_(std::move(memory)) {}
 
-  ~TestActivityTracker() override {}
+  ~TestActivityTracker() override = default;
 
  private:
   std::unique_ptr<char[]> mem_segment_;
@@ -51,7 +52,7 @@ class ActivityAnalyzerTest : public testing::Test {
   const int kMemorySize = 1 << 20;  // 1MiB
   const int kStackSize  = 1 << 10;  // 1KiB
 
-  ActivityAnalyzerTest() {}
+  ActivityAnalyzerTest() = default;
 
   ~ActivityAnalyzerTest() override {
     GlobalActivityTracker* global_tracker = GlobalActivityTracker::Get();
@@ -63,7 +64,7 @@ class ActivityAnalyzerTest : public testing::Test {
 
   std::unique_ptr<ThreadActivityTracker> CreateActivityTracker() {
     std::unique_ptr<char[]> memory(new char[kStackSize]);
-    return MakeUnique<TestActivityTracker>(std::move(memory), kStackSize);
+    return std::make_unique<TestActivityTracker>(std::move(memory), kStackSize);
   }
 
   template <typename Function>
@@ -74,7 +75,7 @@ class ActivityAnalyzerTest : public testing::Test {
 
     PersistentMemoryAllocator* old_allocator = old_global->allocator();
     std::unique_ptr<PersistentMemoryAllocator> new_allocator(
-        MakeUnique<PersistentMemoryAllocator>(
+        std::make_unique<PersistentMemoryAllocator>(
             const_cast<void*>(old_allocator->data()), old_allocator->size(), 0,
             0, "", false));
     GlobalActivityTracker::CreateWithAllocator(std::move(new_allocator), 3,
@@ -119,7 +120,7 @@ class SimpleActivityThread : public SimpleThread {
         exit_(false),
         exit_condition_(&lock_) {}
 
-  ~SimpleActivityThread() override {}
+  ~SimpleActivityThread() override = default;
 
   void Run() override {
     ThreadActivityTracker::ActivityId id =
@@ -168,7 +169,7 @@ TEST_F(ActivityAnalyzerTest, GlobalAnalyzerConstruction) {
 
   PersistentMemoryAllocator* allocator =
       GlobalActivityTracker::Get()->allocator();
-  GlobalActivityAnalyzer analyzer(MakeUnique<PersistentMemoryAllocator>(
+  GlobalActivityAnalyzer analyzer(std::make_unique<PersistentMemoryAllocator>(
       const_cast<void*>(allocator->data()), allocator->size(), 0, 0, "", true));
 
   // The only thread at this point is the test thread of this process.
@@ -216,6 +217,29 @@ TEST_F(ActivityAnalyzerTest, GlobalAnalyzerConstruction) {
   EXPECT_EQ("bar", data_snapshot.at("foo").GetString());
 }
 
+TEST_F(ActivityAnalyzerTest, GlobalAnalyzerFromSharedMemory) {
+  base::MappedReadOnlyRegion shm =
+      base::ReadOnlySharedMemoryRegion::Create(kMemorySize);
+  ASSERT_TRUE(shm.IsValid());
+  base::WritableSharedMemoryMapping rw_mapping = std::move(shm.mapping);
+  base::ReadOnlySharedMemoryMapping ro_mapping = shm.region.Map();
+  ASSERT_TRUE(ro_mapping.IsValid());
+
+  GlobalActivityTracker::CreateWithSharedMemory(std::move(rw_mapping), 0, "",
+                                                3);
+  GlobalActivityTracker::Get()->process_data().SetString("foo", "bar");
+
+  std::unique_ptr<GlobalActivityAnalyzer> analyzer =
+      GlobalActivityAnalyzer::CreateWithSharedMemory(std::move(ro_mapping));
+
+  const int64_t pid = analyzer->GetFirstProcess();
+  ASSERT_NE(0, pid);
+  const ActivityUserData::Snapshot& data_snapshot =
+      analyzer->GetProcessDataSnapshot(pid);
+  ASSERT_LE(1U, data_snapshot.size());
+  EXPECT_EQ("bar", data_snapshot.at("foo").GetString());
+}
+
 TEST_F(ActivityAnalyzerTest, UserDataSnapshotTest) {
   GlobalActivityTracker::CreateWithLocalMemory(kMemorySize, 0, "", 3, 0);
   ThreadActivityAnalyzer::Snapshot tracker_snapshot;
@@ -227,8 +251,10 @@ TEST_F(ActivityAnalyzerTest, UserDataSnapshotTest) {
 
   PersistentMemoryAllocator* allocator =
       GlobalActivityTracker::Get()->allocator();
-  GlobalActivityAnalyzer global_analyzer(MakeUnique<PersistentMemoryAllocator>(
-      const_cast<void*>(allocator->data()), allocator->size(), 0, 0, "", true));
+  GlobalActivityAnalyzer global_analyzer(
+      std::make_unique<PersistentMemoryAllocator>(
+          const_cast<void*>(allocator->data()), allocator->size(), 0, 0, "",
+          true));
 
   ThreadActivityTracker* tracker =
       GlobalActivityTracker::Get()->GetOrCreateTrackerForCurrentThread();
@@ -268,22 +294,22 @@ TEST_F(ActivityAnalyzerTest, UserDataSnapshotTest) {
       const ActivityUserData::Snapshot& user_data =
           analyzer_snapshot.user_data_stack.at(1);
       EXPECT_EQ(8U, user_data.size());
-      ASSERT_TRUE(ContainsKey(user_data, "raw2"));
+      ASSERT_TRUE(Contains(user_data, "raw2"));
       EXPECT_EQ("foo2", user_data.at("raw2").Get().as_string());
-      ASSERT_TRUE(ContainsKey(user_data, "string2"));
+      ASSERT_TRUE(Contains(user_data, "string2"));
       EXPECT_EQ("bar2", user_data.at("string2").GetString().as_string());
-      ASSERT_TRUE(ContainsKey(user_data, "char2"));
+      ASSERT_TRUE(Contains(user_data, "char2"));
       EXPECT_EQ('2', user_data.at("char2").GetChar());
-      ASSERT_TRUE(ContainsKey(user_data, "int2"));
+      ASSERT_TRUE(Contains(user_data, "int2"));
       EXPECT_EQ(-2222, user_data.at("int2").GetInt());
-      ASSERT_TRUE(ContainsKey(user_data, "uint2"));
+      ASSERT_TRUE(Contains(user_data, "uint2"));
       EXPECT_EQ(2222U, user_data.at("uint2").GetUint());
-      ASSERT_TRUE(ContainsKey(user_data, "bool2"));
+      ASSERT_TRUE(Contains(user_data, "bool2"));
       EXPECT_FALSE(user_data.at("bool2").GetBool());
-      ASSERT_TRUE(ContainsKey(user_data, "ref2"));
+      ASSERT_TRUE(Contains(user_data, "ref2"));
       EXPECT_EQ(string2a, user_data.at("ref2").GetReference().data());
       EXPECT_EQ(sizeof(string2a), user_data.at("ref2").GetReference().size());
-      ASSERT_TRUE(ContainsKey(user_data, "sref2"));
+      ASSERT_TRUE(Contains(user_data, "sref2"));
       EXPECT_EQ(string2b, user_data.at("sref2").GetStringReference().data());
       EXPECT_EQ(strlen(string2b),
                 user_data.at("sref2").GetStringReference().size());
@@ -326,8 +352,10 @@ TEST_F(ActivityAnalyzerTest, GlobalUserDataTest) {
 
   PersistentMemoryAllocator* allocator =
       GlobalActivityTracker::Get()->allocator();
-  GlobalActivityAnalyzer global_analyzer(MakeUnique<PersistentMemoryAllocator>(
-      const_cast<void*>(allocator->data()), allocator->size(), 0, 0, "", true));
+  GlobalActivityAnalyzer global_analyzer(
+      std::make_unique<PersistentMemoryAllocator>(
+          const_cast<void*>(allocator->data()), allocator->size(), 0, 0, "",
+          true));
 
   ActivityUserData& process_data = GlobalActivityTracker::Get()->process_data();
   ASSERT_NE(0U, process_data.id());
@@ -344,33 +372,35 @@ TEST_F(ActivityAnalyzerTest, GlobalUserDataTest) {
   DCHECK_EQ(pid, first_pid);
   const ActivityUserData::Snapshot& snapshot =
       global_analyzer.GetProcessDataSnapshot(pid);
-  ASSERT_TRUE(ContainsKey(snapshot, "raw"));
+  ASSERT_TRUE(Contains(snapshot, "raw"));
   EXPECT_EQ("foo", snapshot.at("raw").Get().as_string());
-  ASSERT_TRUE(ContainsKey(snapshot, "string"));
+  ASSERT_TRUE(Contains(snapshot, "string"));
   EXPECT_EQ("bar", snapshot.at("string").GetString().as_string());
-  ASSERT_TRUE(ContainsKey(snapshot, "char"));
+  ASSERT_TRUE(Contains(snapshot, "char"));
   EXPECT_EQ('9', snapshot.at("char").GetChar());
-  ASSERT_TRUE(ContainsKey(snapshot, "int"));
+  ASSERT_TRUE(Contains(snapshot, "int"));
   EXPECT_EQ(-9999, snapshot.at("int").GetInt());
-  ASSERT_TRUE(ContainsKey(snapshot, "uint"));
+  ASSERT_TRUE(Contains(snapshot, "uint"));
   EXPECT_EQ(9999U, snapshot.at("uint").GetUint());
-  ASSERT_TRUE(ContainsKey(snapshot, "bool"));
+  ASSERT_TRUE(Contains(snapshot, "bool"));
   EXPECT_TRUE(snapshot.at("bool").GetBool());
-  ASSERT_TRUE(ContainsKey(snapshot, "ref"));
+  ASSERT_TRUE(Contains(snapshot, "ref"));
   EXPECT_EQ(string1, snapshot.at("ref").GetReference().data());
   EXPECT_EQ(sizeof(string1), snapshot.at("ref").GetReference().size());
-  ASSERT_TRUE(ContainsKey(snapshot, "sref"));
+  ASSERT_TRUE(Contains(snapshot, "sref"));
   EXPECT_EQ(string2, snapshot.at("sref").GetStringReference().data());
   EXPECT_EQ(strlen(string2), snapshot.at("sref").GetStringReference().size());
 }
 
 TEST_F(ActivityAnalyzerTest, GlobalModulesTest) {
   GlobalActivityTracker::CreateWithLocalMemory(kMemorySize, 0, "", 3, 0);
+  GlobalActivityTracker* global = GlobalActivityTracker::Get();
 
-  PersistentMemoryAllocator* allocator =
-      GlobalActivityTracker::Get()->allocator();
-  GlobalActivityAnalyzer global_analyzer(MakeUnique<PersistentMemoryAllocator>(
-      const_cast<void*>(allocator->data()), allocator->size(), 0, 0, "", true));
+  PersistentMemoryAllocator* allocator = global->allocator();
+  GlobalActivityAnalyzer global_analyzer(
+      std::make_unique<PersistentMemoryAllocator>(
+          const_cast<void*>(allocator->data()), allocator->size(), 0, 0, "",
+          true));
 
   GlobalActivityTracker::ModuleInfo info1;
   info1.is_loaded = true;
@@ -383,9 +413,9 @@ TEST_F(ActivityAnalyzerTest, GlobalModulesTest) {
   info1.file = "anything";
   info1.debug_file = "elsewhere";
 
-  GlobalActivityTracker::Get()->RecordModuleInfo(info1);
+  global->RecordModuleInfo(info1);
   std::vector<GlobalActivityTracker::ModuleInfo> modules1;
-  modules1 = global_analyzer.GetModules();
+  modules1 = global_analyzer.GetModules(global_analyzer.GetFirstProcess());
   ASSERT_EQ(1U, modules1.size());
   GlobalActivityTracker::ModuleInfo& stored1a = modules1[0];
   EXPECT_EQ(info1.is_loaded, stored1a.is_loaded);
@@ -399,8 +429,8 @@ TEST_F(ActivityAnalyzerTest, GlobalModulesTest) {
   EXPECT_EQ(info1.debug_file, stored1a.debug_file);
 
   info1.is_loaded = false;
-  GlobalActivityTracker::Get()->RecordModuleInfo(info1);
-  modules1 = global_analyzer.GetModules();
+  global->RecordModuleInfo(info1);
+  modules1 = global_analyzer.GetModules(global_analyzer.GetFirstProcess());
   ASSERT_EQ(1U, modules1.size());
   GlobalActivityTracker::ModuleInfo& stored1b = modules1[0];
   EXPECT_EQ(info1.is_loaded, stored1b.is_loaded);
@@ -424,9 +454,9 @@ TEST_F(ActivityAnalyzerTest, GlobalModulesTest) {
   info2.file = "nothing";
   info2.debug_file = "farewell";
 
-  GlobalActivityTracker::Get()->RecordModuleInfo(info2);
+  global->RecordModuleInfo(info2);
   std::vector<GlobalActivityTracker::ModuleInfo> modules2;
-  modules2 = global_analyzer.GetModules();
+  modules2 = global_analyzer.GetModules(global_analyzer.GetFirstProcess());
   ASSERT_EQ(2U, modules2.size());
   GlobalActivityTracker::ModuleInfo& stored2 = modules2[1];
   EXPECT_EQ(info2.is_loaded, stored2.is_loaded);
@@ -445,7 +475,7 @@ TEST_F(ActivityAnalyzerTest, GlobalLogMessages) {
 
   PersistentMemoryAllocator* allocator =
       GlobalActivityTracker::Get()->allocator();
-  GlobalActivityAnalyzer analyzer(MakeUnique<PersistentMemoryAllocator>(
+  GlobalActivityAnalyzer analyzer(std::make_unique<PersistentMemoryAllocator>(
       const_cast<void*>(allocator->data()), allocator->size(), 0, 0, "", true));
 
   GlobalActivityTracker::Get()->RecordLogMessage("hello world");
@@ -473,7 +503,7 @@ TEST_F(ActivityAnalyzerTest, GlobalMultiProcess) {
   GlobalActivityTracker::Get()->process_data().SetInt("pid",
                                                       global->process_id());
 
-  GlobalActivityAnalyzer analyzer(MakeUnique<PersistentMemoryAllocator>(
+  GlobalActivityAnalyzer analyzer(std::make_unique<PersistentMemoryAllocator>(
       const_cast<void*>(allocator->data()), allocator->size(), 0, 0, "", true));
 
   AsOtherProcess(2002, [&global]() {
