@@ -6,6 +6,7 @@
 
 #include <stddef.h>
 
+#include <algorithm>
 #include <utility>
 
 #include "base/json/json_reader.h"
@@ -31,6 +32,8 @@ const char kEnableArgumentFilter[] = "enable-argument-filter";
 
 // String parameters that can be used to parse the trace config string.
 const char kRecordModeParam[] = "record_mode";
+const char kTraceBufferSizeInEvents[] = "trace_buffer_size_in_events";
+const char kTraceBufferSizeInKb[] = "trace_buffer_size_in_kb";
 const char kEnableSystraceParam[] = "enable_systrace";
 const char kEnableArgumentFilterParam[] = "enable_argument_filter";
 
@@ -51,13 +54,18 @@ const char kEventFiltersParam[] = "event_filters";
 const char kFilterPredicateParam[] = "filter_predicate";
 const char kFilterArgsParam[] = "filter_args";
 
+// String parameter used to parse process filter.
+const char kIncludedProcessesParam[] = "included_process_ids";
+
+const char kHistogramNamesParam[] = "histogram_names";
+
 class ConvertableTraceConfigToTraceFormat
     : public base::trace_event::ConvertableToTraceFormat {
  public:
   explicit ConvertableTraceConfigToTraceFormat(const TraceConfig& trace_config)
       : trace_config_(trace_config) {}
 
-  ~ConvertableTraceConfigToTraceFormat() override {}
+  ~ConvertableTraceConfigToTraceFormat() override = default;
 
   void AppendAsTraceFormat(std::string* out) const override {
     out->append(trace_config_.ToString());
@@ -91,12 +99,12 @@ void TraceConfig::ResetMemoryDumpConfig(
   memory_dump_config_ = memory_dump_config;
 }
 
-TraceConfig::MemoryDumpConfig::MemoryDumpConfig() {}
+TraceConfig::MemoryDumpConfig::MemoryDumpConfig() = default;
 
 TraceConfig::MemoryDumpConfig::MemoryDumpConfig(
     const MemoryDumpConfig& other) = default;
 
-TraceConfig::MemoryDumpConfig::~MemoryDumpConfig() {}
+TraceConfig::MemoryDumpConfig::~MemoryDumpConfig() = default;
 
 void TraceConfig::MemoryDumpConfig::Clear() {
   allowed_dump_modes.clear();
@@ -115,11 +123,60 @@ void TraceConfig::MemoryDumpConfig::Merge(
                config.heap_profiler_options.breakdown_threshold_bytes);
 }
 
+TraceConfig::ProcessFilterConfig::ProcessFilterConfig() = default;
+
+TraceConfig::ProcessFilterConfig::ProcessFilterConfig(
+    const ProcessFilterConfig& other) = default;
+
+TraceConfig::ProcessFilterConfig::ProcessFilterConfig(
+    const std::unordered_set<base::ProcessId>& included_process_ids)
+    : included_process_ids_(included_process_ids) {}
+
+TraceConfig::ProcessFilterConfig::~ProcessFilterConfig() = default;
+
+void TraceConfig::ProcessFilterConfig::Clear() {
+  included_process_ids_.clear();
+}
+
+void TraceConfig::ProcessFilterConfig::Merge(
+    const ProcessFilterConfig& config) {
+  included_process_ids_.insert(config.included_process_ids_.begin(),
+                               config.included_process_ids_.end());
+}
+
+void TraceConfig::ProcessFilterConfig::InitializeFromConfigDict(
+    const Value& dict) {
+  included_process_ids_.clear();
+  const Value* value = dict.FindListKey(kIncludedProcessesParam);
+  if (!value)
+    return;
+  for (auto& pid_value : value->GetList()) {
+    if (pid_value.is_int())
+      included_process_ids_.insert(pid_value.GetInt());
+  }
+}
+
+void TraceConfig::ProcessFilterConfig::ToDict(Value* dict) const {
+  if (included_process_ids_.empty())
+    return;
+  Value* list = dict->SetKey(kIncludedProcessesParam, Value(Value::Type::LIST));
+  std::set<base::ProcessId> ordered_set(included_process_ids_.begin(),
+                                        included_process_ids_.end());
+  for (auto process_id : ordered_set)
+    list->GetList().emplace_back(static_cast<int>(process_id));
+}
+
+bool TraceConfig::ProcessFilterConfig::IsEnabled(
+    base::ProcessId process_id) const {
+  return included_process_ids_.empty() ||
+         included_process_ids_.count(process_id);
+}
+
 TraceConfig::EventFilterConfig::EventFilterConfig(
     const std::string& predicate_name)
     : predicate_name_(predicate_name) {}
 
-TraceConfig::EventFilterConfig::~EventFilterConfig() {}
+TraceConfig::EventFilterConfig::~EventFilterConfig() = default;
 
 TraceConfig::EventFilterConfig::EventFilterConfig(const EventFilterConfig& tc) {
   *this = tc;
@@ -133,19 +190,19 @@ TraceConfig::EventFilterConfig& TraceConfig::EventFilterConfig::operator=(
   predicate_name_ = rhs.predicate_name_;
   category_filter_ = rhs.category_filter_;
 
-  if (rhs.args_)
-    args_ = rhs.args_->CreateDeepCopy();
+  if (!rhs.args_.is_none())
+    args_ = rhs.args_.Clone();
 
   return *this;
 }
 
 void TraceConfig::EventFilterConfig::InitializeFromConfigDict(
-    const base::DictionaryValue* event_filter) {
-  category_filter_.InitializeFromConfigDict(*event_filter);
+    const Value& event_filter) {
+  category_filter_.InitializeFromConfigDict(event_filter);
 
-  const base::DictionaryValue* args_dict = nullptr;
-  if (event_filter->GetDictionary(kFilterArgsParam, &args_dict))
-    args_ = args_dict->CreateDeepCopy();
+  const Value* args_dict = event_filter.FindDictKey(kFilterArgsParam);
+  if (args_dict)
+    args_ = args_dict->Clone();
 }
 
 void TraceConfig::EventFilterConfig::SetCategoryFilter(
@@ -153,26 +210,24 @@ void TraceConfig::EventFilterConfig::SetCategoryFilter(
   category_filter_ = category_filter;
 }
 
-void TraceConfig::EventFilterConfig::ToDict(
-    DictionaryValue* filter_dict) const {
-  filter_dict->SetString(kFilterPredicateParam, predicate_name());
+void TraceConfig::EventFilterConfig::ToDict(Value* filter_dict) const {
+  filter_dict->SetStringKey(kFilterPredicateParam, predicate_name());
 
   category_filter_.ToDict(filter_dict);
 
-  if (args_)
-    filter_dict->Set(kFilterArgsParam, args_->CreateDeepCopy());
+  if (!args_.is_none())
+    filter_dict->SetKey(kFilterArgsParam, args_.Clone());
 }
 
 bool TraceConfig::EventFilterConfig::GetArgAsSet(
     const char* key,
     std::unordered_set<std::string>* out_set) const {
-  const ListValue* list = nullptr;
-  if (!args_->GetList(key, &list))
+  const Value* list = args_.FindListPath(key);
+  if (!list)
     return false;
-  for (size_t i = 0; i < list->GetSize(); ++i) {
-    std::string value;
-    if (list->GetString(i, &value))
-      out_set->insert(value);
+  for (const Value& item : list->GetList()) {
+    if (item.is_string())
+      out_set->insert(item.GetString());
   }
   return true;
 }
@@ -182,6 +237,23 @@ bool TraceConfig::EventFilterConfig::IsCategoryGroupEnabled(
   return category_filter_.IsCategoryGroupEnabled(category_group_name);
 }
 
+// static
+std::string TraceConfig::TraceRecordModeToStr(TraceRecordMode record_mode) {
+  switch (record_mode) {
+    case RECORD_UNTIL_FULL:
+      return kRecordUntilFull;
+    case RECORD_CONTINUOUSLY:
+      return kRecordContinuously;
+    case RECORD_AS_MUCH_AS_POSSIBLE:
+      return kRecordAsMuchAsPossible;
+    case ECHO_TO_CONSOLE:
+      return kTraceToConsole;
+    default:
+      NOTREACHED();
+  }
+  return kRecordUntilFull;
+}
+
 TraceConfig::TraceConfig() {
   InitializeDefault();
 }
@@ -193,27 +265,11 @@ TraceConfig::TraceConfig(StringPiece category_filter_string,
 
 TraceConfig::TraceConfig(StringPiece category_filter_string,
                          TraceRecordMode record_mode) {
-  std::string trace_options_string;
-  switch (record_mode) {
-    case RECORD_UNTIL_FULL:
-      trace_options_string = kRecordUntilFull;
-      break;
-    case RECORD_CONTINUOUSLY:
-      trace_options_string = kRecordContinuously;
-      break;
-    case RECORD_AS_MUCH_AS_POSSIBLE:
-      trace_options_string = kRecordAsMuchAsPossible;
-      break;
-    case ECHO_TO_CONSOLE:
-      trace_options_string = kTraceToConsole;
-      break;
-    default:
-      NOTREACHED();
-  }
-  InitializeFromStrings(category_filter_string, trace_options_string);
+  InitializeFromStrings(category_filter_string,
+                        TraceConfig::TraceRecordModeToStr(record_mode));
 }
 
-TraceConfig::TraceConfig(const DictionaryValue& config) {
+TraceConfig::TraceConfig(const Value& config) {
   InitializeFromConfigDict(config);
 }
 
@@ -224,40 +280,37 @@ TraceConfig::TraceConfig(StringPiece config_string) {
     InitializeDefault();
 }
 
-TraceConfig::TraceConfig(const TraceConfig& tc)
-    : record_mode_(tc.record_mode_),
-      enable_systrace_(tc.enable_systrace_),
-      enable_argument_filter_(tc.enable_argument_filter_),
-      category_filter_(tc.category_filter_),
-      memory_dump_config_(tc.memory_dump_config_),
-      event_filters_(tc.event_filters_) {}
+TraceConfig::TraceConfig(const TraceConfig& tc) = default;
 
-TraceConfig::~TraceConfig() {
-}
+TraceConfig::~TraceConfig() = default;
 
 TraceConfig& TraceConfig::operator=(const TraceConfig& rhs) {
   if (this == &rhs)
     return *this;
 
   record_mode_ = rhs.record_mode_;
+  trace_buffer_size_in_events_ = rhs.trace_buffer_size_in_events_;
+  trace_buffer_size_in_kb_ = rhs.trace_buffer_size_in_kb_;
   enable_systrace_ = rhs.enable_systrace_;
   enable_argument_filter_ = rhs.enable_argument_filter_;
   category_filter_ = rhs.category_filter_;
+  process_filter_config_ = rhs.process_filter_config_;
   memory_dump_config_ = rhs.memory_dump_config_;
   event_filters_ = rhs.event_filters_;
+  histogram_names_ = rhs.histogram_names_;
   return *this;
 }
 
 std::string TraceConfig::ToString() const {
-  std::unique_ptr<DictionaryValue> dict = ToDict();
+  Value dict = ToValue();
   std::string json;
-  JSONWriter::Write(*dict, &json);
+  JSONWriter::Write(dict, &json);
   return json;
 }
 
 std::unique_ptr<ConvertableToTraceFormat>
 TraceConfig::AsConvertableToTraceFormat() const {
-  return MakeUnique<ConvertableTraceConfigToTraceFormat>(*this);
+  return std::make_unique<ConvertableTraceConfigToTraceFormat>(*this);
 }
 
 std::string TraceConfig::ToCategoryFilterString() const {
@@ -278,61 +331,77 @@ void TraceConfig::Merge(const TraceConfig& config) {
     DLOG(ERROR) << "Attempting to merge trace config with a different "
                 << "set of options.";
   }
+  DCHECK_EQ(trace_buffer_size_in_events_, config.trace_buffer_size_in_events_)
+      << "Cannot change trace buffer size";
 
   category_filter_.Merge(config.category_filter_);
-
   memory_dump_config_.Merge(config.memory_dump_config_);
+  process_filter_config_.Merge(config.process_filter_config_);
 
   event_filters_.insert(event_filters_.end(), config.event_filters().begin(),
                         config.event_filters().end());
+  histogram_names_.insert(config.histogram_names().begin(),
+                          config.histogram_names().end());
 }
 
 void TraceConfig::Clear() {
   record_mode_ = RECORD_UNTIL_FULL;
+  trace_buffer_size_in_events_ = 0;
+  trace_buffer_size_in_kb_ = 0;
   enable_systrace_ = false;
   enable_argument_filter_ = false;
   category_filter_.Clear();
   memory_dump_config_.Clear();
+  process_filter_config_.Clear();
   event_filters_.clear();
+  histogram_names_.clear();
 }
 
 void TraceConfig::InitializeDefault() {
   record_mode_ = RECORD_UNTIL_FULL;
+  trace_buffer_size_in_events_ = 0;
+  trace_buffer_size_in_kb_ = 0;
   enable_systrace_ = false;
   enable_argument_filter_ = false;
 }
 
-void TraceConfig::InitializeFromConfigDict(const DictionaryValue& dict) {
+void TraceConfig::InitializeFromConfigDict(const Value& dict) {
   record_mode_ = RECORD_UNTIL_FULL;
-  std::string record_mode;
-  if (dict.GetString(kRecordModeParam, &record_mode)) {
-    if (record_mode == kRecordUntilFull) {
+  const std::string* record_mode = dict.FindStringKey(kRecordModeParam);
+  if (record_mode) {
+    if (*record_mode == kRecordUntilFull) {
       record_mode_ = RECORD_UNTIL_FULL;
-    } else if (record_mode == kRecordContinuously) {
+    } else if (*record_mode == kRecordContinuously) {
       record_mode_ = RECORD_CONTINUOUSLY;
-    } else if (record_mode == kTraceToConsole) {
+    } else if (*record_mode == kTraceToConsole) {
       record_mode_ = ECHO_TO_CONSOLE;
-    } else if (record_mode == kRecordAsMuchAsPossible) {
+    } else if (*record_mode == kRecordAsMuchAsPossible) {
       record_mode_ = RECORD_AS_MUCH_AS_POSSIBLE;
     }
   }
+  trace_buffer_size_in_events_ =
+      dict.FindIntKey(kTraceBufferSizeInEvents).value_or(0);
+  trace_buffer_size_in_kb_ = dict.FindIntKey(kTraceBufferSizeInKb).value_or(0);
 
-  bool val;
-  enable_systrace_ = dict.GetBoolean(kEnableSystraceParam, &val) ? val : false;
+  enable_systrace_ = dict.FindBoolKey(kEnableSystraceParam).value_or(false);
   enable_argument_filter_ =
-      dict.GetBoolean(kEnableArgumentFilterParam, &val) ? val : false;
+      dict.FindBoolKey(kEnableArgumentFilterParam).value_or(false);
 
   category_filter_.InitializeFromConfigDict(dict);
+  process_filter_config_.InitializeFromConfigDict(dict);
 
-  const base::ListValue* category_event_filters = nullptr;
-  if (dict.GetList(kEventFiltersParam, &category_event_filters))
+  const Value* category_event_filters = dict.FindListKey(kEventFiltersParam);
+  if (category_event_filters)
     SetEventFiltersFromConfigList(*category_event_filters);
+  const Value* histogram_names = dict.FindListKey(kHistogramNamesParam);
+  if (histogram_names)
+    SetHistogramNamesFromConfigList(*histogram_names);
 
   if (category_filter_.IsCategoryEnabled(MemoryDumpManager::kTraceCategory)) {
     // If dump triggers not set, the client is using the legacy with just
     // category enabled. So, use the default periodic dump config.
-    const DictionaryValue* memory_dump_config = nullptr;
-    if (dict.GetDictionary(kMemoryDumpConfigParam, &memory_dump_config))
+    const Value* memory_dump_config = dict.FindDictKey(kMemoryDumpConfigParam);
+    if (memory_dump_config)
       SetMemoryDumpConfigFromConfigDict(*memory_dump_config);
     else
       SetDefaultMemoryDumpConfig();
@@ -340,8 +409,8 @@ void TraceConfig::InitializeFromConfigDict(const DictionaryValue& dict) {
 }
 
 void TraceConfig::InitializeFromConfigString(StringPiece config_string) {
-  auto dict = DictionaryValue::From(JSONReader::Read(config_string));
-  if (dict)
+  base::Optional<Value> dict = JSONReader::Read(config_string);
+  if (dict && dict->is_dict())
     InitializeFromConfigDict(*dict);
   else
     InitializeDefault();
@@ -353,6 +422,8 @@ void TraceConfig::InitializeFromStrings(StringPiece category_filter_string,
     category_filter_.InitializeFromString(category_filter_string);
 
   record_mode_ = RECORD_UNTIL_FULL;
+  trace_buffer_size_in_events_ = 0;
+  trace_buffer_size_in_kb_ = 0;
   enable_systrace_ = false;
   enable_argument_filter_ = false;
   if (!trace_options_string.empty()) {
@@ -381,16 +452,16 @@ void TraceConfig::InitializeFromStrings(StringPiece category_filter_string,
 }
 
 void TraceConfig::SetMemoryDumpConfigFromConfigDict(
-    const DictionaryValue& memory_dump_config) {
+    const Value& memory_dump_config) {
   // Set allowed dump modes.
   memory_dump_config_.allowed_dump_modes.clear();
-  const ListValue* allowed_modes_list;
-  if (memory_dump_config.GetList(kAllowedDumpModesParam, &allowed_modes_list)) {
-    for (size_t i = 0; i < allowed_modes_list->GetSize(); ++i) {
-      std::string level_of_detail_str;
-      allowed_modes_list->GetString(i, &level_of_detail_str);
+  const Value* allowed_modes_list =
+      memory_dump_config.FindListKey(kAllowedDumpModesParam);
+  if (allowed_modes_list) {
+    for (const Value& item : allowed_modes_list->GetList()) {
+      DCHECK(item.is_string());
       memory_dump_config_.allowed_dump_modes.insert(
-          StringToMemoryDumpLevelOfDetail(level_of_detail_str));
+          StringToMemoryDumpLevelOfDetail(item.GetString()));
     }
   } else {
     // If allowed modes param is not given then allow all modes by default.
@@ -399,48 +470,48 @@ void TraceConfig::SetMemoryDumpConfigFromConfigDict(
 
   // Set triggers
   memory_dump_config_.triggers.clear();
-  const ListValue* trigger_list = nullptr;
-  if (memory_dump_config.GetList(kTriggersParam, &trigger_list) &&
-      trigger_list->GetSize() > 0) {
-    for (size_t i = 0; i < trigger_list->GetSize(); ++i) {
-      const DictionaryValue* trigger = nullptr;
-      if (!trigger_list->GetDictionary(i, &trigger))
+  const Value* trigger_list = memory_dump_config.FindListKey(kTriggersParam);
+  if (trigger_list) {
+    for (const Value& trigger : trigger_list->GetList()) {
+      if (!trigger.is_dict())
         continue;
 
       MemoryDumpConfig::Trigger dump_config;
-      int interval = 0;
-      if (!trigger->GetInteger(kMinTimeBetweenDumps, &interval)) {
+      base::Optional<int> interval = trigger.FindIntKey(kMinTimeBetweenDumps);
+      if (!interval) {
         // If "min_time_between_dumps_ms" param was not given, then the trace
         // config uses old format where only periodic dumps are supported.
-        trigger->GetInteger(kPeriodicIntervalLegacyParam, &interval);
+        interval = trigger.FindIntKey(kPeriodicIntervalLegacyParam);
         dump_config.trigger_type = MemoryDumpType::PERIODIC_INTERVAL;
       } else {
-        std::string trigger_type_str;
-        trigger->GetString(kTriggerTypeParam, &trigger_type_str);
-        dump_config.trigger_type = StringToMemoryDumpType(trigger_type_str);
+        const std::string* trigger_type_str =
+            trigger.FindStringKey(kTriggerTypeParam);
+        DCHECK(trigger_type_str);
+        dump_config.trigger_type = StringToMemoryDumpType(*trigger_type_str);
       }
-      DCHECK_GT(interval, 0);
-      dump_config.min_time_between_dumps_ms = static_cast<uint32_t>(interval);
+      DCHECK(interval.has_value());
+      DCHECK_GT(*interval, 0);
+      dump_config.min_time_between_dumps_ms = static_cast<uint32_t>(*interval);
 
-      std::string level_of_detail_str;
-      trigger->GetString(kTriggerModeParam, &level_of_detail_str);
+      const std::string* level_of_detail_str =
+          trigger.FindStringKey(kTriggerModeParam);
+      DCHECK(level_of_detail_str);
       dump_config.level_of_detail =
-          StringToMemoryDumpLevelOfDetail(level_of_detail_str);
+          StringToMemoryDumpLevelOfDetail(*level_of_detail_str);
 
       memory_dump_config_.triggers.push_back(dump_config);
     }
   }
 
   // Set heap profiler options
-  const DictionaryValue* heap_profiler_options = nullptr;
-  if (memory_dump_config.GetDictionary(kHeapProfilerOptions,
-                                       &heap_profiler_options)) {
-    int min_size_bytes = 0;
-    if (heap_profiler_options->GetInteger(kBreakdownThresholdBytes,
-                                         &min_size_bytes)
-        && min_size_bytes >= 0) {
+  const Value* heap_profiler_options =
+      memory_dump_config.FindDictKey(kHeapProfilerOptions);
+  if (heap_profiler_options) {
+    base::Optional<int> min_size_bytes =
+        heap_profiler_options->FindIntKey(kBreakdownThresholdBytes);
+    if (min_size_bytes && *min_size_bytes >= 0) {
       memory_dump_config_.heap_profiler_options.breakdown_threshold_bytes =
-          static_cast<size_t>(min_size_bytes);
+          *min_size_bytes;
     } else {
       memory_dump_config_.heap_profiler_options.breakdown_threshold_bytes =
           MemoryDumpConfig::HeapProfiler::kDefaultBreakdownThresholdBytes;
@@ -453,102 +524,111 @@ void TraceConfig::SetDefaultMemoryDumpConfig() {
   memory_dump_config_.allowed_dump_modes = GetDefaultAllowedMemoryDumpModes();
 }
 
+void TraceConfig::SetProcessFilterConfig(const ProcessFilterConfig& config) {
+  process_filter_config_ = config;
+}
+
+void TraceConfig::SetHistogramNamesFromConfigList(
+    const Value& histogram_names) {
+  histogram_names_.clear();
+  for (const Value& value : histogram_names.GetList())
+    histogram_names_.insert(value.GetString());
+}
+
 void TraceConfig::SetEventFiltersFromConfigList(
-    const base::ListValue& category_event_filters) {
+    const Value& category_event_filters) {
   event_filters_.clear();
 
-  for (size_t event_filter_index = 0;
-       event_filter_index < category_event_filters.GetSize();
-       ++event_filter_index) {
-    const base::DictionaryValue* event_filter = nullptr;
-    if (!category_event_filters.GetDictionary(event_filter_index,
-                                              &event_filter))
+  for (const Value& event_filter : category_event_filters.GetList()) {
+    if (!event_filter.is_dict())
       continue;
 
-    std::string predicate_name;
-    CHECK(event_filter->GetString(kFilterPredicateParam, &predicate_name))
-        << "Invalid predicate name in category event filter.";
+    const std::string* predicate_name =
+        event_filter.FindStringKey(kFilterPredicateParam);
+    CHECK(predicate_name) << "Invalid predicate name in category event filter.";
 
-    EventFilterConfig new_config(predicate_name);
+    EventFilterConfig new_config(*predicate_name);
     new_config.InitializeFromConfigDict(event_filter);
     event_filters_.push_back(new_config);
   }
 }
 
-std::unique_ptr<DictionaryValue> TraceConfig::ToDict() const {
-  auto dict = MakeUnique<DictionaryValue>();
-  switch (record_mode_) {
-    case RECORD_UNTIL_FULL:
-      dict->SetString(kRecordModeParam, kRecordUntilFull);
-      break;
-    case RECORD_CONTINUOUSLY:
-      dict->SetString(kRecordModeParam, kRecordContinuously);
-      break;
-    case RECORD_AS_MUCH_AS_POSSIBLE:
-      dict->SetString(kRecordModeParam, kRecordAsMuchAsPossible);
-      break;
-    case ECHO_TO_CONSOLE:
-      dict->SetString(kRecordModeParam, kTraceToConsole);
-      break;
-    default:
-      NOTREACHED();
-  }
+Value TraceConfig::ToValue() const {
+  Value dict(Value::Type::DICTIONARY);
+  dict.SetStringKey(kRecordModeParam,
+                    TraceConfig::TraceRecordModeToStr(record_mode_));
+  dict.SetBoolKey(kEnableSystraceParam, enable_systrace_);
+  dict.SetBoolKey(kEnableArgumentFilterParam, enable_argument_filter_);
+  if (trace_buffer_size_in_events_ > 0)
+    dict.SetIntKey(kTraceBufferSizeInEvents, trace_buffer_size_in_events_);
+  if (trace_buffer_size_in_kb_ > 0)
+    dict.SetIntKey(kTraceBufferSizeInKb, trace_buffer_size_in_kb_);
 
-  dict->SetBoolean(kEnableSystraceParam, enable_systrace_);
-  dict->SetBoolean(kEnableArgumentFilterParam, enable_argument_filter_);
-
-  category_filter_.ToDict(dict.get());
+  category_filter_.ToDict(&dict);
+  process_filter_config_.ToDict(&dict);
 
   if (!event_filters_.empty()) {
-    std::unique_ptr<base::ListValue> filter_list(new base::ListValue());
+    std::vector<Value> filter_list;
     for (const EventFilterConfig& filter : event_filters_) {
-      std::unique_ptr<base::DictionaryValue> filter_dict(
-          new base::DictionaryValue());
-      filter.ToDict(filter_dict.get());
-      filter_list->Append(std::move(filter_dict));
+      filter_list.emplace_back(Value::Type::DICTIONARY);
+      filter.ToDict(&filter_list.back());
     }
-    dict->Set(kEventFiltersParam, std::move(filter_list));
+    dict.SetKey(kEventFiltersParam, Value(std::move(filter_list)));
   }
 
   if (category_filter_.IsCategoryEnabled(MemoryDumpManager::kTraceCategory)) {
-    auto allowed_modes = MakeUnique<ListValue>();
+    std::vector<Value> allowed_modes;
     for (auto dump_mode : memory_dump_config_.allowed_dump_modes)
-      allowed_modes->AppendString(MemoryDumpLevelOfDetailToString(dump_mode));
+      allowed_modes.emplace_back(MemoryDumpLevelOfDetailToString(dump_mode));
 
-    auto memory_dump_config = MakeUnique<DictionaryValue>();
-    memory_dump_config->Set(kAllowedDumpModesParam, std::move(allowed_modes));
+    Value memory_dump_config(Value::Type::DICTIONARY);
+    memory_dump_config.SetKey(kAllowedDumpModesParam,
+                              Value(std::move(allowed_modes)));
 
-    auto triggers_list = MakeUnique<ListValue>();
+    std::vector<Value> triggers_list;
     for (const auto& config : memory_dump_config_.triggers) {
-      auto trigger_dict = MakeUnique<DictionaryValue>();
-      trigger_dict->SetString(kTriggerTypeParam,
-                              MemoryDumpTypeToString(config.trigger_type));
-      trigger_dict->SetInteger(
+      triggers_list.emplace_back(Value::Type::DICTIONARY);
+      Value& trigger_dict = triggers_list.back();
+
+      trigger_dict.SetStringKey(kTriggerTypeParam,
+                                MemoryDumpTypeToString(config.trigger_type));
+      trigger_dict.SetIntKey(
           kMinTimeBetweenDumps,
           static_cast<int>(config.min_time_between_dumps_ms));
-      trigger_dict->SetString(
+      trigger_dict.SetStringKey(
           kTriggerModeParam,
           MemoryDumpLevelOfDetailToString(config.level_of_detail));
-      triggers_list->Append(std::move(trigger_dict));
     }
 
     // Empty triggers will still be specified explicitly since it means that
     // the periodic dumps are not enabled.
-    memory_dump_config->Set(kTriggersParam, std::move(triggers_list));
+    memory_dump_config.SetKey(kTriggersParam, Value(std::move(triggers_list)));
 
     if (memory_dump_config_.heap_profiler_options.breakdown_threshold_bytes !=
         MemoryDumpConfig::HeapProfiler::kDefaultBreakdownThresholdBytes) {
-      auto options = MakeUnique<DictionaryValue>();
-      options->SetInteger(
+      Value options(Value::Type::DICTIONARY);
+      options.SetIntKey(
           kBreakdownThresholdBytes,
           memory_dump_config_.heap_profiler_options.breakdown_threshold_bytes);
-      memory_dump_config->Set(kHeapProfilerOptions, std::move(options));
+      memory_dump_config.SetKey(kHeapProfilerOptions, std::move(options));
     }
-    dict->Set(kMemoryDumpConfigParam, std::move(memory_dump_config));
+    dict.SetKey(kMemoryDumpConfigParam, std::move(memory_dump_config));
   }
+
+  if (!histogram_names_.empty()) {
+    std::vector<Value> histogram_names;
+    for (const std::string& histogram_name : histogram_names_)
+      histogram_names.emplace_back(histogram_name);
+    dict.SetKey(kHistogramNamesParam, Value(std::move(histogram_names)));
+  }
+
   return dict;
 }
 
+void TraceConfig::EnableHistogram(const std::string& histogram_name) {
+  histogram_names_.insert(histogram_name);
+}
+
 std::string TraceConfig::ToTraceOptionsString() const {
   std::string ret;
   switch (record_mode_) {
