@@ -7,12 +7,16 @@
 #include <utility>
 
 #include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/optional.h"
 #include "base/run_loop.h"
 #include "base/test/test_simple_task_runner.h"
 #include "base/threading/sequenced_task_runner_handle.h"
 #include "base/threading/thread_checker_impl.h"
+#include "cc/paint/paint_image_builder.h"
 #include "cc/test/skia_common.h"
+#include "cc/test/stub_decode_cache.h"
+#include "cc/test/test_paint_worklet_input.h"
 #include "cc/tiles/image_decode_cache.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -67,13 +71,13 @@ class WorkerTaskRunner : public base::SequencedTaskRunner {
  public:
   WorkerTaskRunner() { thread_.Start(); }
 
-  bool PostNonNestableDelayedTask(const tracked_objects::Location& from_here,
+  bool PostNonNestableDelayedTask(const base::Location& from_here,
                                   base::OnceClosure task,
                                   base::TimeDelta delay) override {
     return PostDelayedTask(from_here, std::move(task), delay);
   }
 
-  bool PostDelayedTask(const tracked_objects::Location& from_here,
+  bool PostDelayedTask(const base::Location& from_here,
                        base::OnceClosure task,
                        base::TimeDelta delay) override {
     thread_.PostTask(std::move(task));
@@ -92,44 +96,32 @@ class WorkerTaskRunner : public base::SequencedTaskRunner {
 };
 
 // Image decode cache with introspection!
-class TestableCache : public ImageDecodeCache {
+class TestableCache : public StubDecodeCache {
  public:
   ~TestableCache() override { EXPECT_EQ(number_of_refs_, 0); }
 
-  bool GetTaskForImageAndRef(const DrawImage& image,
-                             const TracingInfo& tracing_info,
-                             scoped_refptr<TileTask>* task) override {
+  TaskResult GetTaskForImageAndRef(const DrawImage& image,
+                                   const TracingInfo& tracing_info) override {
     // Return false for large images to mimic "won't fit in memory"
     // behavior.
-    if (image.image() &&
-        image.image()->width() * image.image()->height() >= 1000 * 1000) {
-      return false;
+    if (image.paint_image() &&
+        image.paint_image().width() * image.paint_image().height() >=
+            1000 * 1000) {
+      return TaskResult(false);
     }
 
-    *task = task_to_use_;
     ++number_of_refs_;
-    return true;
+    return TaskResult(task_to_use_);
   }
-  bool GetOutOfRasterDecodeTaskForImageAndRef(
-      const DrawImage& image,
-      scoped_refptr<TileTask>* task) override {
-    return GetTaskForImageAndRef(image, TracingInfo(), task);
+  TaskResult GetOutOfRasterDecodeTaskForImageAndRef(
+      const DrawImage& image) override {
+    return GetTaskForImageAndRef(image, TracingInfo());
   }
 
   void UnrefImage(const DrawImage& image) override {
     ASSERT_GT(number_of_refs_, 0);
     --number_of_refs_;
   }
-  DecodedDrawImage GetDecodedImageForDraw(const DrawImage& image) override {
-    return DecodedDrawImage(nullptr, kNone_SkFilterQuality);
-  }
-  void DrawWithImageFinished(const DrawImage& image,
-                             const DecodedDrawImage& decoded_image) override {}
-  void ReduceCacheUsage() override {}
-  void SetShouldAggressivelyFreeResources(
-      bool aggressively_free_resources) override {}
-  void ClearCache() override {}
-  void NotifyImageUnused(uint32_t skimage_id) override {}
   size_t GetMaximumMemoryLimitBytes() const override {
     return 256 * 1024 * 1024;
   }
@@ -145,7 +137,7 @@ class TestableCache : public ImageDecodeCache {
 // A simple class that can receive decode callbacks.
 class DecodeClient {
  public:
-  DecodeClient() {}
+  DecodeClient() = default;
   void Callback(base::OnceClosure quit_closure,
                 ImageController::ImageDecodeRequestId id,
                 ImageController::ImageDecodeResult result) {
@@ -169,6 +161,9 @@ class SimpleTask : public TileTask {
   SimpleTask() : TileTask(true /* supports_concurrent_execution */) {
     EXPECT_TRUE(thread_checker_.CalledOnValidThread());
   }
+  SimpleTask(const SimpleTask&) = delete;
+
+  SimpleTask& operator=(const SimpleTask&) = delete;
 
   void RunOnWorkerThread() override {
     EXPECT_FALSE(HasCompleted());
@@ -185,8 +180,6 @@ class SimpleTask : public TileTask {
 
   base::ThreadChecker thread_checker_;
   bool has_run_ = false;
-
-  DISALLOW_COPY_AND_ASSIGN(SimpleTask);
 };
 
 // A task that blocks until instructed otherwise.
@@ -196,6 +189,9 @@ class BlockingTask : public TileTask {
       : TileTask(true /* supports_concurrent_execution */), run_cv_(&lock_) {
     EXPECT_TRUE(thread_checker_.CalledOnValidThread());
   }
+  BlockingTask(const BlockingTask&) = delete;
+
+  BlockingTask& operator=(const BlockingTask&) = delete;
 
   void RunOnWorkerThread() override {
     EXPECT_FALSE(HasCompleted());
@@ -227,8 +223,6 @@ class BlockingTask : public TileTask {
   base::Lock lock_;
   base::ConditionVariable run_cv_;
   bool can_run_ = false;
-
-  DISALLOW_COPY_AND_ASSIGN(BlockingTask);
 };
 
 // For tests that exercise image controller's thread, this is the timeout value
@@ -236,10 +230,16 @@ class BlockingTask : public TileTask {
 int kDefaultTimeoutSeconds = 10;
 
 DrawImage CreateDiscardableDrawImage(gfx::Size size) {
+  return DrawImage(CreateDiscardablePaintImage(size),
+                   SkIRect::MakeWH(size.width(), size.height()),
+                   kNone_SkFilterQuality, SkMatrix::I(),
+                   PaintImage::kDefaultFrameIndex, gfx::ColorSpace());
+}
+
+DrawImage CreateBitmapDrawImage(gfx::Size size) {
   return DrawImage(
-      PaintImage(PaintImage::kUnknownStableId, CreateDiscardableImage(size)),
-      SkIRect::MakeWH(size.width(), size.height()), kNone_SkFilterQuality,
-      SkMatrix::I(), gfx::ColorSpace());
+      CreateBitmapImage(size), SkIRect::MakeWH(size.width(), size.height()),
+      kNone_SkFilterQuality, SkMatrix::I(), PaintImage::kDefaultFrameIndex);
 }
 
 class ImageControllerTest : public testing::Test {
@@ -250,7 +250,7 @@ class ImageControllerTest : public testing::Test {
   ~ImageControllerTest() override = default;
 
   void SetUp() override {
-    worker_task_runner_ = make_scoped_refptr(new WorkerTaskRunner);
+    worker_task_runner_ = base::MakeRefCounted<WorkerTaskRunner>();
     controller_.reset(
         new ImageController(task_runner_.get(), worker_task_runner_));
     cache_ = TestableCache();
@@ -260,6 +260,7 @@ class ImageControllerTest : public testing::Test {
   void TearDown() override {
     controller_.reset();
     worker_task_runner_ = nullptr;
+    weak_ptr_factory_.InvalidateWeakPtrs();
   }
 
   base::SequencedTaskRunner* task_runner() { return task_runner_.get(); }
@@ -268,7 +269,7 @@ class ImageControllerTest : public testing::Test {
   const DrawImage& image() const { return image_; }
 
   // Timeout callback, which errors and exits the runloop.
-  static void Timeout(base::RunLoop* run_loop) {
+  void Timeout(base::RunLoop* run_loop) {
     ADD_FAILURE() << "Timeout.";
     run_loop->Quit();
   }
@@ -278,6 +279,7 @@ class ImageControllerTest : public testing::Test {
     task_runner_->PostDelayedTask(
         FROM_HERE,
         base::BindOnce(&ImageControllerTest::Timeout,
+                       weak_ptr_factory_.GetWeakPtr(),
                        base::Unretained(run_loop)),
         base::TimeDelta::FromSeconds(kDefaultTimeoutSeconds));
     run_loop->Run();
@@ -285,12 +287,32 @@ class ImageControllerTest : public testing::Test {
 
   void ResetController() { controller_.reset(); }
 
+  SkMatrix CreateMatrix(const SkSize& scale, bool is_decomposable) {
+    SkMatrix matrix;
+    matrix.setScale(scale.width(), scale.height());
+
+    if (!is_decomposable) {
+      // Perspective is not decomposable, add it.
+      matrix[SkMatrix::kMPersp0] = 0.1f;
+    }
+
+    return matrix;
+  }
+
+  PaintImage CreatePaintImage(int width, int height) {
+    scoped_refptr<TestPaintWorkletInput> input =
+        base::MakeRefCounted<TestPaintWorkletInput>(gfx::SizeF(width, height));
+    return CreatePaintWorkletPaintImage(input);
+  }
+
  private:
   scoped_refptr<base::SequencedTaskRunner> task_runner_;
   scoped_refptr<WorkerTaskRunner> worker_task_runner_;
   TestableCache cache_;
   std::unique_ptr<ImageController> controller_;
   DrawImage image_;
+
+  base::WeakPtrFactory<ImageControllerTest> weak_ptr_factory_{this};
 };
 
 TEST_F(ImageControllerTest, NullControllerUnrefsImages) {
@@ -310,12 +332,12 @@ TEST_F(ImageControllerTest, NullControllerUnrefsImages) {
 TEST_F(ImageControllerTest, QueueImageDecode) {
   base::RunLoop run_loop;
   DecodeClient decode_client;
-  EXPECT_EQ(image().image()->bounds().width(), 1);
+  EXPECT_EQ(image().paint_image().width(), 1);
   ImageController::ImageDecodeRequestId expected_id =
       controller()->QueueImageDecode(
-          image(),
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client),
-                     run_loop.QuitClosure()));
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client),
+                                  run_loop.QuitClosure()));
   RunOrTimeout(&run_loop);
   EXPECT_EQ(expected_id, decode_client.id());
   EXPECT_EQ(ImageController::ImageDecodeResult::SUCCESS,
@@ -326,18 +348,13 @@ TEST_F(ImageControllerTest, QueueImageDecodeNonLazy) {
   base::RunLoop run_loop;
   DecodeClient decode_client;
 
-  SkBitmap bitmap;
-  bitmap.allocN32Pixels(1, 1);
-  DrawImage image = DrawImage(
-      PaintImage(PaintImage::kUnknownStableId, SkImage::MakeFromBitmap(bitmap)),
-      SkIRect::MakeWH(1, 1), kNone_SkFilterQuality, SkMatrix::I(),
-      gfx::ColorSpace());
+  DrawImage image = CreateBitmapDrawImage(gfx::Size(1, 1));
 
   ImageController::ImageDecodeRequestId expected_id =
       controller()->QueueImageDecode(
-          image,
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client),
-                     run_loop.QuitClosure()));
+          image, base::BindOnce(&DecodeClient::Callback,
+                                base::Unretained(&decode_client),
+                                run_loop.QuitClosure()));
   RunOrTimeout(&run_loop);
   EXPECT_EQ(expected_id, decode_client.id());
   EXPECT_EQ(ImageController::ImageDecodeResult::DECODE_NOT_REQUIRED,
@@ -351,9 +368,9 @@ TEST_F(ImageControllerTest, QueueImageDecodeTooLarge) {
   DrawImage image = CreateDiscardableDrawImage(gfx::Size(2000, 2000));
   ImageController::ImageDecodeRequestId expected_id =
       controller()->QueueImageDecode(
-          image,
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client),
-                     run_loop.QuitClosure()));
+          image, base::BindOnce(&DecodeClient::Callback,
+                                base::Unretained(&decode_client),
+                                run_loop.QuitClosure()));
   RunOrTimeout(&run_loop);
   EXPECT_EQ(expected_id, decode_client.id());
   EXPECT_EQ(ImageController::ImageDecodeResult::FAILURE,
@@ -365,21 +382,21 @@ TEST_F(ImageControllerTest, QueueImageDecodeMultipleImages) {
   DecodeClient decode_client1;
   ImageController::ImageDecodeRequestId expected_id1 =
       controller()->QueueImageDecode(
-          image(),
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client1),
-                     base::Bind([] {})));
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client1),
+                                  base::DoNothing::Once()));
   DecodeClient decode_client2;
   ImageController::ImageDecodeRequestId expected_id2 =
       controller()->QueueImageDecode(
-          image(),
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client2),
-                     base::Bind([] {})));
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client2),
+                                  base::DoNothing::Once()));
   DecodeClient decode_client3;
   ImageController::ImageDecodeRequestId expected_id3 =
       controller()->QueueImageDecode(
-          image(),
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client3),
-                     run_loop.QuitClosure()));
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client3),
+                                  run_loop.QuitClosure()));
   RunOrTimeout(&run_loop);
   EXPECT_EQ(expected_id1, decode_client1.id());
   EXPECT_EQ(ImageController::ImageDecodeResult::SUCCESS,
@@ -400,9 +417,9 @@ TEST_F(ImageControllerTest, QueueImageDecodeWithTask) {
   DecodeClient decode_client;
   ImageController::ImageDecodeRequestId expected_id =
       controller()->QueueImageDecode(
-          image(),
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client),
-                     run_loop.QuitClosure()));
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client),
+                                  run_loop.QuitClosure()));
   RunOrTimeout(&run_loop);
   EXPECT_EQ(expected_id, decode_client.id());
   EXPECT_TRUE(task->has_run());
@@ -417,21 +434,21 @@ TEST_F(ImageControllerTest, QueueImageDecodeMultipleImagesSameTask) {
   DecodeClient decode_client1;
   ImageController::ImageDecodeRequestId expected_id1 =
       controller()->QueueImageDecode(
-          image(),
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client1),
-                     base::Bind([] {})));
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client1),
+                                  base::DoNothing::Once()));
   DecodeClient decode_client2;
   ImageController::ImageDecodeRequestId expected_id2 =
       controller()->QueueImageDecode(
-          image(),
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client2),
-                     base::Bind([] {})));
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client2),
+                                  base::DoNothing::Once()));
   DecodeClient decode_client3;
   ImageController::ImageDecodeRequestId expected_id3 =
       controller()->QueueImageDecode(
-          image(),
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client3),
-                     run_loop.QuitClosure()));
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client3),
+                                  run_loop.QuitClosure()));
   RunOrTimeout(&run_loop);
   EXPECT_EQ(expected_id1, decode_client1.id());
   EXPECT_EQ(ImageController::ImageDecodeResult::SUCCESS,
@@ -453,9 +470,9 @@ TEST_F(ImageControllerTest, QueueImageDecodeChangeControllerWithTaskQueued) {
   DecodeClient decode_client1;
   ImageController::ImageDecodeRequestId expected_id1 =
       controller()->QueueImageDecode(
-          image(),
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client1),
-                     base::Bind([] {})));
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client1),
+                                  base::DoNothing::Once()));
 
   scoped_refptr<BlockingTask> task_two(new BlockingTask);
   cache()->SetTaskToUse(task_two);
@@ -464,9 +481,9 @@ TEST_F(ImageControllerTest, QueueImageDecodeChangeControllerWithTaskQueued) {
   DecodeClient decode_client2;
   ImageController::ImageDecodeRequestId expected_id2 =
       controller()->QueueImageDecode(
-          image(),
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client2),
-                     run_loop.QuitClosure()));
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client2),
+                                  run_loop.QuitClosure()));
 
   task_one->AllowToRun();
   task_two->AllowToRun();
@@ -489,9 +506,9 @@ TEST_F(ImageControllerTest, QueueImageDecodeImageAlreadyLocked) {
   DecodeClient decode_client1;
   ImageController::ImageDecodeRequestId expected_id1 =
       controller()->QueueImageDecode(
-          image(),
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client1),
-                     run_loop1.QuitClosure()));
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client1),
+                                  run_loop1.QuitClosure()));
   RunOrTimeout(&run_loop1);
   EXPECT_EQ(expected_id1, decode_client1.id());
   EXPECT_TRUE(task->has_run());
@@ -501,9 +518,9 @@ TEST_F(ImageControllerTest, QueueImageDecodeImageAlreadyLocked) {
   DecodeClient decode_client2;
   ImageController::ImageDecodeRequestId expected_id2 =
       controller()->QueueImageDecode(
-          image(),
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client2),
-                     run_loop2.QuitClosure()));
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client2),
+                                  run_loop2.QuitClosure()));
   RunOrTimeout(&run_loop2);
   EXPECT_EQ(expected_id2, decode_client2.id());
   EXPECT_EQ(ImageController::ImageDecodeResult::SUCCESS,
@@ -518,9 +535,9 @@ TEST_F(ImageControllerTest, QueueImageDecodeLockedImageControllerChange) {
   DecodeClient decode_client1;
   ImageController::ImageDecodeRequestId expected_id1 =
       controller()->QueueImageDecode(
-          image(),
-          base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client1),
-                     run_loop1.QuitClosure()));
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client1),
+                                  run_loop1.QuitClosure()));
   RunOrTimeout(&run_loop1);
   EXPECT_EQ(expected_id1, decode_client1.id());
   EXPECT_TRUE(task->has_run());
@@ -541,12 +558,12 @@ TEST_F(ImageControllerTest, DispatchesDecodeCallbacksAfterCacheReset) {
 
   controller()->QueueImageDecode(
       image(),
-      base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client1),
-                 run_loop1.QuitClosure()));
+      base::BindOnce(&DecodeClient::Callback, base::Unretained(&decode_client1),
+                     run_loop1.QuitClosure()));
   controller()->QueueImageDecode(
       image(),
-      base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client2),
-                 run_loop2.QuitClosure()));
+      base::BindOnce(&DecodeClient::Callback, base::Unretained(&decode_client2),
+                     run_loop2.QuitClosure()));
 
   // Now reset the image cache before decode completed callbacks are posted to
   // the compositor thread. Ensure that the completion callbacks for the decode
@@ -574,12 +591,12 @@ TEST_F(ImageControllerTest, DispatchesDecodeCallbacksAfterCacheChanged) {
 
   controller()->QueueImageDecode(
       image(),
-      base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client1),
-                 run_loop1.QuitClosure()));
+      base::BindOnce(&DecodeClient::Callback, base::Unretained(&decode_client1),
+                     run_loop1.QuitClosure()));
   controller()->QueueImageDecode(
       image(),
-      base::Bind(&DecodeClient::Callback, base::Unretained(&decode_client2),
-                 run_loop2.QuitClosure()));
+      base::BindOnce(&DecodeClient::Callback, base::Unretained(&decode_client2),
+                     run_loop2.QuitClosure()));
 
   // Now reset the image cache before decode completed callbacks are posted to
   // the compositor thread. This should orphan the requests.
@@ -607,5 +624,92 @@ TEST_F(ImageControllerTest, DispatchesDecodeCallbacksAfterCacheChanged) {
   ResetController();
 }
 
+TEST_F(ImageControllerTest, QueueImageDecodeLazyCancelImmediately) {
+  DecodeClient decode_client1;
+  DecodeClient decode_client2;
+
+  // Create two images so that there is always one that is queued up and
+  // not run yet.  This prevents raciness in this test.
+  DrawImage image1 = CreateDiscardableDrawImage(gfx::Size(1, 1));
+  DrawImage image2 = CreateDiscardableDrawImage(gfx::Size(1, 1));
+
+  ImageController::ImageDecodeRequestId expected_id1 =
+      controller()->QueueImageDecode(
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client1),
+                                  base::DoNothing::Once()));
+
+  ImageController::ImageDecodeRequestId expected_id2 =
+      controller()->QueueImageDecode(
+          image(), base::BindOnce(&DecodeClient::Callback,
+                                  base::Unretained(&decode_client2),
+                                  base::DoNothing::Once()));
+
+  // This needs a ref because it is lazy.
+  EXPECT_EQ(2, cache()->number_of_refs());
+
+  // Instead of running, immediately cancel everything.
+  controller()->SetImageDecodeCache(nullptr);
+
+  // This should not crash, and nothing should have run.
+  EXPECT_NE(expected_id1, decode_client1.id());
+  EXPECT_NE(expected_id2, decode_client2.id());
+  EXPECT_EQ(0u, decode_client1.id());
+  EXPECT_EQ(0u, decode_client2.id());
+  EXPECT_EQ(ImageController::ImageDecodeResult::FAILURE,
+            decode_client1.result());
+  EXPECT_EQ(ImageController::ImageDecodeResult::FAILURE,
+            decode_client2.result());
+
+  // Refs should still be cleaned up.
+  EXPECT_EQ(0, cache()->number_of_refs());
+
+  // Explicitly reset the controller so that orphaned task callbacks run
+  // while the decode clients still exist.
+  ResetController();
+}
+
+TEST_F(ImageControllerTest, QueueImageDecodeNonLazyCancelImmediately) {
+  DecodeClient decode_client1;
+  DecodeClient decode_client2;
+
+  // Create two images so that there is always one that is queued up and
+  // not run yet.  This prevents raciness in this test.
+  DrawImage image1 = CreateBitmapDrawImage(gfx::Size(1, 1));
+  DrawImage image2 = CreateBitmapDrawImage(gfx::Size(1, 1));
+
+  ImageController::ImageDecodeRequestId expected_id1 =
+      controller()->QueueImageDecode(
+          image1, base::BindOnce(&DecodeClient::Callback,
+                                 base::Unretained(&decode_client1),
+                                 base::DoNothing::Once()));
+  ImageController::ImageDecodeRequestId expected_id2 =
+      controller()->QueueImageDecode(
+          image2, base::BindOnce(&DecodeClient::Callback,
+                                 base::Unretained(&decode_client2),
+                                 base::DoNothing::Once()));
+
+  // No ref needed here, because it is non-lazy.
+  EXPECT_EQ(0, cache()->number_of_refs());
+
+  // Instead of running, immediately cancel everything.
+  controller()->SetImageDecodeCache(nullptr);
+
+  // This should not crash, and nothing should have run.
+  EXPECT_NE(expected_id1, decode_client1.id());
+  EXPECT_NE(expected_id2, decode_client2.id());
+  EXPECT_EQ(0u, decode_client1.id());
+  EXPECT_EQ(0u, decode_client2.id());
+  EXPECT_EQ(ImageController::ImageDecodeResult::FAILURE,
+            decode_client1.result());
+  EXPECT_EQ(ImageController::ImageDecodeResult::FAILURE,
+            decode_client2.result());
+  EXPECT_EQ(0, cache()->number_of_refs());
+
+  // Explicitly reset the controller so that orphaned task callbacks run
+  // while the decode clients still exist.
+  ResetController();
+}
+
 }  // namespace
 }  // namespace cc
