@@ -6,10 +6,10 @@
 
 #include <stddef.h>
 
-#include "cc/base/filter_operation.h"
-#include "cc/base/filter_operations.h"
 #include "cc/base/math_util.h"
 #include "cc/layers/layer_impl.h"
+#include "cc/paint/filter_operation.h"
+#include "cc/paint/filter_operations.h"
 #include "cc/test/fake_impl_task_runner_provider.h"
 #include "cc/test/fake_layer_tree_host_impl.h"
 #include "cc/test/geometry_test_utils.h"
@@ -18,14 +18,44 @@
 #include "cc/trees/layer_tree_host_common.h"
 #include "cc/trees/layer_tree_impl.h"
 #include "cc/trees/single_thread_proxy.h"
+#include "cc/trees/transform_node.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/skia/include/effects/SkBlurImageFilter.h"
 #include "ui/gfx/geometry/quad_f.h"
 #include "ui/gfx/geometry/rect_conversions.h"
 
 namespace cc {
 namespace {
 
+class TestLayerImpl : public LayerImpl {
+ public:
+  TestLayerImpl(LayerTreeImpl* tree_impl, int id);
+
+  void AddDamageRect(const gfx::Rect& damage_rect);
+
+  // LayerImpl overrides.
+  gfx::Rect GetDamageRect() const override;
+  void ResetChangeTracking() override;
+
+ private:
+  gfx::Rect damage_rect_;
+};
+
+TestLayerImpl::TestLayerImpl(LayerTreeImpl* tree_impl, int id)
+    : LayerImpl(tree_impl, id) {}
+
+void TestLayerImpl::AddDamageRect(const gfx::Rect& damage_rect) {
+  damage_rect_.Union(damage_rect);
+}
+
+gfx::Rect TestLayerImpl::GetDamageRect() const {
+  return damage_rect_;
+}
+
+void TestLayerImpl::ResetChangeTracking() {
+  LayerImpl::ResetChangeTracking();
+  damage_rect_.SetRect(0, 0, 0, 0);
+}
+
 void ExecuteCalculateDrawProperties(LayerImpl* root,
                                     float device_scale_factor,
                                     RenderSurfaceList* render_surface_list) {
@@ -71,18 +101,16 @@ class DamageTrackerTest : public testing::Test {
 
   LayerImpl* CreateTestTreeWithOneSurface(int number_of_children) {
     host_impl_.active_tree()->DetachLayers();
-    std::unique_ptr<LayerImpl> root =
-        LayerImpl::Create(host_impl_.active_tree(), 1);
+    auto root = std::make_unique<TestLayerImpl>(host_impl_.active_tree(), 1);
 
-    root->SetPosition(gfx::PointF());
     root->SetBounds(gfx::Size(500, 500));
     root->SetDrawsContent(true);
     root->test_properties()->force_render_surface = true;
 
     for (int i = 0; i < number_of_children; ++i) {
-      std::unique_ptr<LayerImpl> child =
-          LayerImpl::Create(host_impl_.active_tree(), 2 + i);
-      child->SetPosition(gfx::PointF(100.f, 100.f));
+      auto child =
+          std::make_unique<TestLayerImpl>(host_impl_.active_tree(), 2 + i);
+      child->test_properties()->position = gfx::PointF(100.f, 100.f);
       child->SetBounds(gfx::Size(30, 30));
       child->SetDrawsContent(true);
       root->test_properties()->AddChild(std::move(child));
@@ -99,23 +127,19 @@ class DamageTrackerTest : public testing::Test {
     // two children of its own.
 
     host_impl_.active_tree()->DetachLayers();
-    std::unique_ptr<LayerImpl> root =
-        LayerImpl::Create(host_impl_.active_tree(), 1);
-    std::unique_ptr<LayerImpl> child1 =
-        LayerImpl::Create(host_impl_.active_tree(), 2);
-    std::unique_ptr<LayerImpl> child2 =
-        LayerImpl::Create(host_impl_.active_tree(), 3);
-    std::unique_ptr<LayerImpl> grand_child1 =
-        LayerImpl::Create(host_impl_.active_tree(), 4);
-    std::unique_ptr<LayerImpl> grand_child2 =
-        LayerImpl::Create(host_impl_.active_tree(), 5);
+    auto root = std::make_unique<TestLayerImpl>(host_impl_.active_tree(), 1);
+    auto child1 = std::make_unique<TestLayerImpl>(host_impl_.active_tree(), 2);
+    auto child2 = std::make_unique<TestLayerImpl>(host_impl_.active_tree(), 3);
+    auto grand_child1 =
+        std::make_unique<TestLayerImpl>(host_impl_.active_tree(), 4);
+    auto grand_child2 =
+        std::make_unique<TestLayerImpl>(host_impl_.active_tree(), 5);
 
-    root->SetPosition(gfx::PointF());
     root->SetBounds(gfx::Size(500, 500));
     root->SetDrawsContent(true);
     root->test_properties()->force_render_surface = true;
 
-    child1->SetPosition(gfx::PointF(100.f, 100.f));
+    child1->test_properties()->position = gfx::PointF(100.f, 100.f);
     child1->SetBounds(gfx::Size(30, 30));
     // With a child that draws_content, opacity will cause the layer to create
     // its own RenderSurface. This layer does not draw, but is intended to
@@ -123,15 +147,15 @@ class DamageTrackerTest : public testing::Test {
     child1->SetDrawsContent(false);
     child1->test_properties()->force_render_surface = true;
 
-    child2->SetPosition(gfx::PointF(11.f, 11.f));
+    child2->test_properties()->position = gfx::PointF(11.f, 11.f);
     child2->SetBounds(gfx::Size(18, 18));
     child2->SetDrawsContent(true);
 
-    grand_child1->SetPosition(gfx::PointF(200.f, 200.f));
+    grand_child1->test_properties()->position = gfx::PointF(200.f, 200.f);
     grand_child1->SetBounds(gfx::Size(6, 8));
     grand_child1->SetDrawsContent(true);
 
-    grand_child2->SetPosition(gfx::PointF(190.f, 190.f));
+    grand_child2->test_properties()->position = gfx::PointF(190.f, 190.f);
     grand_child2->SetBounds(gfx::Size(6, 8));
     grand_child2->SetDrawsContent(true);
 
@@ -278,7 +302,8 @@ TEST_F(DamageTrackerTest, VerifyDamageForUpdateRects) {
 
 TEST_F(DamageTrackerTest, VerifyDamageForLayerDamageRects) {
   LayerImpl* root = CreateAndSetUpTestTreeWithOneSurface();
-  LayerImpl* child = root->test_properties()->children[0];
+  auto* child =
+      static_cast<TestLayerImpl*>(root->test_properties()->children[0]);
 
   // CASE 1: Adding the layer damage rect should cause the corresponding damage
   // to the surface.
@@ -338,7 +363,8 @@ TEST_F(DamageTrackerTest, VerifyDamageForLayerDamageRects) {
 
 TEST_F(DamageTrackerTest, VerifyDamageForLayerUpdateAndDamageRects) {
   LayerImpl* root = CreateAndSetUpTestTreeWithOneSurface();
-  LayerImpl* child = root->test_properties()->children[0];
+  auto* child =
+      static_cast<TestLayerImpl*>(root->test_properties()->children[0]);
 
   // CASE 1: Adding the layer damage rect and update rect should cause the
   // corresponding damage to the surface.
@@ -498,12 +524,63 @@ TEST_F(DamageTrackerTest,
                   ->has_damage_from_contributing_content());
 }
 
+// Regression test for http://crbug.com/923794
+TEST_F(DamageTrackerTest, EffectPropertyChangeNoSurface) {
+  LayerImpl* root = CreateAndSetUpTestTreeWithOneSurface();
+  LayerImpl* child = root->test_properties()->children[0];
+
+  // Create a separate effect node for the child, but no render surface.
+  child->test_properties()->opacity = 0.5;
+  root->layer_tree_impl()->property_trees()->needs_rebuild = true;
+  EmulateDrawingOneFrame(root);
+
+  EXPECT_EQ(root->transform_tree_index(), child->transform_tree_index());
+  EXPECT_NE(root->effect_tree_index(), child->effect_tree_index());
+
+  // Change the child's opacity, which should damage its target.
+  ClearDamageForAllSurfaces(root);
+  root->layer_tree_impl()->SetOpacityMutated(child->element_id(), 0.4f);
+  EmulateDrawingOneFrame(root);
+  EXPECT_TRUE(GetRenderSurface(root)
+                  ->damage_tracker()
+                  ->has_damage_from_contributing_content());
+}
+
+// Regression test for http://crbug.com/923794
+TEST_F(DamageTrackerTest, TransformPropertyChangeNoSurface) {
+  LayerImpl* root = CreateAndSetUpTestTreeWithOneSurface();
+  LayerImpl* child = root->test_properties()->children[0];
+
+  // Create a separate transform node for the child, but no render surface.
+  gfx::Transform trans1;
+  trans1.Scale(2, 1);
+  child->test_properties()->transform = trans1;
+  root->layer_tree_impl()->property_trees()->needs_rebuild = true;
+  EmulateDrawingOneFrame(root);
+
+  EXPECT_NE(root->transform_tree_index(), child->transform_tree_index());
+  EXPECT_EQ(root->effect_tree_index(), child->effect_tree_index());
+
+  // Change the child's transform , which should damage its target.
+  ClearDamageForAllSurfaces(root);
+  gfx::Transform trans2;
+  trans2.Scale(1, 2);
+  root->layer_tree_impl()->SetTransformMutated(child->element_id(), trans2);
+  EmulateDrawingOneFrame(root);
+  EXPECT_TRUE(GetRenderSurface(root)
+                  ->damage_tracker()
+                  ->has_damage_from_contributing_content());
+}
+
 TEST_F(DamageTrackerTest,
        VerifyDamageForUpdateAndDamageRectsFromContributingContents) {
   LayerImpl* root = CreateAndSetUpTestTreeWithTwoSurfaces();
-  LayerImpl* child1 = root->test_properties()->children[0];
-  LayerImpl* child2 = root->test_properties()->children[1];
-  LayerImpl* grandchild1 = child1->test_properties()->children[0];
+  auto* child1 =
+      static_cast<TestLayerImpl*>(root->test_properties()->children[0]);
+  auto* child2 =
+      static_cast<TestLayerImpl*>(root->test_properties()->children[1]);
+  auto* grandchild1 =
+      static_cast<TestLayerImpl*>(child1->test_properties()->children[0]);
 
   // CASE 1: Adding the layer1's damage rect and update rect should cause the
   // corresponding damage to the surface.
@@ -583,7 +660,7 @@ TEST_F(DamageTrackerTest, VerifyDamageForTransformedLayer) {
   ClearDamageForAllSurfaces(root);
   child->test_properties()->transform_origin = gfx::Point3F(
       child->bounds().width() * 0.5f, child->bounds().height() * 0.5f, 0.f);
-  child->SetPosition(gfx::PointF(85.f, 85.f));
+  child->test_properties()->position = gfx::PointF(85.f, 85.f);
   child->NoteLayerPropertyChanged();
   root->layer_tree_impl()->property_trees()->needs_rebuild = true;
   EmulateDrawingOneFrame(root);
@@ -597,8 +674,8 @@ TEST_F(DamageTrackerTest, VerifyDamageForTransformedLayer) {
   EXPECT_TRUE(GetRenderSurface(root)
                   ->damage_tracker()
                   ->has_damage_from_contributing_content());
-  // Layer's layer_property_changed_ should be considered as damage to render
-  // surface.
+  // Layer's layer_property_changed_not_from_property_trees_ should be
+  // considered as damage to render surface.
   EXPECT_TRUE(GetRenderSurface(child)
                   ->damage_tracker()
                   ->has_damage_from_contributing_content());
@@ -654,7 +731,7 @@ TEST_F(DamageTrackerTest, VerifyDamageForPerspectiveClippedLayer) {
   transform.Translate3d(-50.0, -50.0, 0.0);
 
   // Set up the child
-  child->SetPosition(gfx::PointF(0.f, 0.f));
+  child->test_properties()->position = gfx::PointF(0.f, 0.f);
   child->SetBounds(gfx::Size(100, 100));
   child->test_properties()->transform = transform;
   root->layer_tree_impl()->property_trees()->needs_rebuild = true;
@@ -662,7 +739,8 @@ TEST_F(DamageTrackerTest, VerifyDamageForPerspectiveClippedLayer) {
 
   // Sanity check that the child layer's bounds would actually get clipped by
   // w < 0, otherwise this test is not actually testing the intended scenario.
-  gfx::RectF test_rect(child->position(), gfx::SizeF(child->bounds()));
+  gfx::RectF test_rect(child->test_properties()->position,
+                       gfx::SizeF(child->bounds()));
   bool clipped = false;
   MathUtil::MapQuad(transform, gfx::QuadF(test_rect), &clipped);
   EXPECT_TRUE(clipped);
@@ -742,8 +820,9 @@ TEST_F(DamageTrackerTest, VerifyDamageForImageFilter) {
   child->SetDrawsContent(true);
 
   FilterOperations filters;
-  filters.Append(FilterOperation::CreateReferenceFilter(
-      SkBlurImageFilter::Make(2, 2, nullptr)));
+  filters.Append(
+      FilterOperation::CreateReferenceFilter(sk_make_sp<BlurPaintFilter>(
+          2, 2, BlurPaintFilter::TileMode::kClampToBlack_TileMode, nullptr)));
 
   // Setting the filter will damage the whole surface.
   child->test_properties()->force_render_surface = true;
@@ -823,8 +902,9 @@ TEST_F(DamageTrackerTest, VerifyDamageForTransformedImageFilter) {
   child->SetDrawsContent(true);
 
   FilterOperations filters;
-  filters.Append(FilterOperation::CreateReferenceFilter(
-      SkBlurImageFilter::Make(2, 2, nullptr)));
+  filters.Append(
+      FilterOperation::CreateReferenceFilter(sk_make_sp<BlurPaintFilter>(
+          2, 2, BlurPaintFilter::TileMode::kClampToBlack_TileMode, nullptr)));
 
   // Setting the filter will damage the whole surface.
   gfx::Transform transform;
@@ -923,7 +1003,7 @@ TEST_F(DamageTrackerTest, VerifyDamageForHighDPIImageFilter) {
   EXPECT_EQ(expected_child_damage_rect, child_damage_rect);
 }
 
-TEST_F(DamageTrackerTest, VerifyDamageForBackgroundBlurredChild) {
+TEST_F(DamageTrackerTest, VerifyDamageForBackdropBlurredChild) {
   LayerImpl* root = CreateAndSetUpTestTreeWithTwoSurfaces();
   LayerImpl* child1 = root->test_properties()->children[0];
   LayerImpl* child2 = root->test_properties()->children[1];
@@ -936,13 +1016,13 @@ TEST_F(DamageTrackerTest, VerifyDamageForBackgroundBlurredChild) {
 
   // Setting the filter will damage the whole surface.
   ClearDamageForAllSurfaces(root);
-  child1->test_properties()->background_filters = filters;
+  child1->test_properties()->backdrop_filters = filters;
   child1->NoteLayerPropertyChanged();
   root->layer_tree_impl()->property_trees()->needs_rebuild = true;
   EmulateDrawingOneFrame(root);
 
   // CASE 1: Setting the update rect should cause the corresponding damage to
-  //         the surface, blurred based on the size of the child's background
+  //         the surface, blurred based on the size of the child's backdrop
   //         blur filter. Note that child1's render surface has a size of
   //         206x208 due to contributions from grand_child1 and grand_child2.
   ClearDamageForAllSurfaces(root);
@@ -963,7 +1043,7 @@ TEST_F(DamageTrackerTest, VerifyDamageForBackgroundBlurredChild) {
   EXPECT_EQ(expected_damage_rect.ToString(), root_damage_rect.ToString());
 
   // CASE 2: Setting the update rect should cause the corresponding damage to
-  //         the surface, blurred based on the size of the child's background
+  //         the surface, blurred based on the size of the child's backdrop
   //         blur filter. Since the damage extends to the right/bottom outside
   //         of the blurred layer, only the left/top should end up expanded.
   ClearDamageForAllSurfaces(root);
@@ -1052,11 +1132,11 @@ TEST_F(DamageTrackerTest, VerifyDamageForBackgroundBlurredChild) {
 
   // CASE 7: No changes, so should not damage the surface.
   ClearDamageForAllSurfaces(root);
-  // We want to make sure that the background filter doesn't cause empty damage
+  // We want to make sure that the backdrop filter doesn't cause empty damage
   // to get expanded. We position child1 so that an expansion of the empty rect
   // would have non-empty intersection with child1 in its target space (root
   // space).
-  child1->SetPosition(gfx::PointF());
+  child1->test_properties()->position = gfx::PointF();
   root->layer_tree_impl()->property_trees()->needs_rebuild = true;
   EmulateDrawingOneFrame(root);
 
@@ -1081,7 +1161,7 @@ TEST_F(DamageTrackerTest, VerifyDamageForAddingAndRemovingLayer) {
   {
     std::unique_ptr<LayerImpl> child2 =
         LayerImpl::Create(host_impl_.active_tree(), 3);
-    child2->SetPosition(gfx::PointF(400.f, 380.f));
+    child2->test_properties()->position = gfx::PointF(400.f, 380.f);
     child2->SetBounds(gfx::Size(6, 8));
     child2->SetDrawsContent(true);
     root->test_properties()->AddChild(std::move(child2));
@@ -1116,7 +1196,7 @@ TEST_F(DamageTrackerTest, VerifyDamageForAddingAndRemovingLayer) {
 
   // Then, test removing child1.
   root->test_properties()->RemoveChild(child1);
-  child1 = NULL;
+  child1 = nullptr;
   root->layer_tree_impl()->property_trees()->needs_rebuild = true;
   EmulateDrawingOneFrame(root);
 
@@ -1139,7 +1219,7 @@ TEST_F(DamageTrackerTest, VerifyDamageForNewUnchangedLayer) {
   {
     std::unique_ptr<LayerImpl> child2 =
         LayerImpl::Create(host_impl_.active_tree(), 3);
-    child2->SetPosition(gfx::PointF(400.f, 380.f));
+    child2->test_properties()->position = gfx::PointF(400.f, 380.f);
     child2->SetBounds(gfx::Size(6, 8));
     child2->SetDrawsContent(true);
     root->test_properties()->AddChild(std::move(child2));
@@ -1178,7 +1258,7 @@ TEST_F(DamageTrackerTest, VerifyDamageForMultipleLayers) {
   {
     std::unique_ptr<LayerImpl> child2 =
         LayerImpl::Create(host_impl_.active_tree(), 3);
-    child2->SetPosition(gfx::PointF(400.f, 380.f));
+    child2->test_properties()->position = gfx::PointF(400.f, 380.f);
     child2->SetBounds(gfx::Size(6, 8));
     child2->SetDrawsContent(true);
     root->test_properties()->AddChild(std::move(child2));
@@ -1289,7 +1369,7 @@ TEST_F(DamageTrackerTest, VerifyDamageForSurfaceChangeFromDescendantLayer) {
   gfx::Rect root_damage_rect;
 
   ClearDamageForAllSurfaces(root);
-  grand_child1->SetPosition(gfx::PointF(195.f, 205.f));
+  grand_child1->test_properties()->position = gfx::PointF(195.f, 205.f);
   root->layer_tree_impl()->property_trees()->needs_rebuild = true;
   EmulateDrawingOneFrame(root);
   EXPECT_TRUE(GetRenderSurface(child1)->damage_tracker()->GetDamageRectIfValid(
@@ -1316,6 +1396,46 @@ TEST_F(DamageTrackerTest, VerifyDamageForSurfaceChangeFromDescendantLayer) {
                   ->has_damage_from_contributing_content());
 }
 
+TEST_F(DamageTrackerTest, VerifyDamageForSurfaceChangeFromDescendantSurface) {
+  // If descendant surface changes position, the ancestor surface should be
+  // damaged with the old and new descendant surface regions.
+
+  LayerImpl* root = CreateAndSetUpTestTreeWithTwoSurfaces();
+  LayerImpl* child1 = root->test_properties()->children[0];
+  child1->SetDrawsContent(true);
+  EmulateDrawingOneFrame(root);
+
+  gfx::Rect child_damage_rect;
+  gfx::Rect root_damage_rect;
+
+  ClearDamageForAllSurfaces(root);
+  child1->test_properties()->position = gfx::PointF(105.f, 107.f);
+  child1->NoteLayerPropertyChanged();
+  TransformNode* child1_transform =
+      root->layer_tree_impl()->property_trees()->transform_tree.Node(
+          child1->transform_tree_index());
+  child1_transform->transform_changed = true;
+  EmulateDrawingOneFrame(root);
+  EXPECT_TRUE(GetRenderSurface(child1)->damage_tracker()->GetDamageRectIfValid(
+      &child_damage_rect));
+  EXPECT_TRUE(GetRenderSurface(root)->damage_tracker()->GetDamageRectIfValid(
+      &root_damage_rect));
+
+  // Damage to the root surface should be the union of child1's *entire* render
+  // surface (in target space), and its old exposed area (also in target
+  // space).
+  EXPECT_EQ(gfx::Rect(100, 100, 206, 208).ToString(),
+            root_damage_rect.ToString());
+  // The child surface should also be damaged.
+  EXPECT_EQ(gfx::Rect(0, 0, 206, 208).ToString(), child_damage_rect.ToString());
+  EXPECT_TRUE(GetRenderSurface(root)
+                  ->damage_tracker()
+                  ->has_damage_from_contributing_content());
+  EXPECT_TRUE(GetRenderSurface(child1)
+                  ->damage_tracker()
+                  ->has_damage_from_contributing_content());
+}
+
 TEST_F(DamageTrackerTest, VerifyDamageForSurfaceChangeFromAncestorLayer) {
   // An ancestor/owning layer changes that affects the position/transform of
   // the render surface. Note that in this case, the layer_property_changed flag
@@ -1500,7 +1620,8 @@ TEST_F(DamageTrackerTest, VerifyDamageForMask) {
   {
     std::unique_ptr<LayerImpl> mask_layer =
         LayerImpl::Create(host_impl_.active_tree(), 3);
-    mask_layer->SetPosition(child->position());
+    mask_layer->test_properties()->position =
+        child->test_properties()->position;
     mask_layer->SetBounds(child->bounds());
     child->test_properties()->SetMaskLayer(std::move(mask_layer));
     child->test_properties()->force_render_surface = true;
@@ -1512,7 +1633,7 @@ TEST_F(DamageTrackerTest, VerifyDamageForMask) {
   {
     std::unique_ptr<LayerImpl> grand_child =
         LayerImpl::Create(host_impl_.active_tree(), 4);
-    grand_child->SetPosition(gfx::PointF(2.f, 2.f));
+    grand_child->test_properties()->position = gfx::PointF(2.f, 2.f);
     grand_child->SetBounds(gfx::Size(2, 2));
     grand_child->SetDrawsContent(true);
     child->test_properties()->AddChild(std::move(grand_child));
@@ -1714,7 +1835,7 @@ TEST_F(DamageTrackerTest, VerifyDamageAccumulatesUntilReset) {
 }
 
 TEST_F(DamageTrackerTest, HugeDamageRect) {
-  // This number is so large that we start losting floating point accuracy.
+  // This number is so large that we start losing floating point accuracy.
   const int kBigNumber = 900000000;
   // Walk over a range to find floating point inaccuracy boundaries that move
   // toward the wrong direction.
@@ -1729,7 +1850,6 @@ TEST_F(DamageTrackerTest, HugeDamageRect) {
 
     // The child layer covers (0, 0, i, i) of the viewport,
     // but has a huge negative position.
-    child->SetPosition(gfx::PointF());
     child->SetBounds(gfx::Size(kBigNumber + i, kBigNumber + i));
     child->test_properties()->transform = transform;
     root->layer_tree_impl()->property_trees()->needs_rebuild = true;
@@ -1759,11 +1879,13 @@ TEST_F(DamageTrackerTest, DamageRectTooBig) {
   LayerImpl* child2 = root->test_properties()->children[1];
 
   // Really far left.
-  child1->SetPosition(gfx::PointF(std::numeric_limits<int>::min() + 100, 0));
+  child1->test_properties()->position =
+      gfx::PointF(std::numeric_limits<int>::min() + 100, 0);
   child1->SetBounds(gfx::Size(1, 1));
 
   // Really far right.
-  child2->SetPosition(gfx::PointF(std::numeric_limits<int>::max() - 100, 0));
+  child2->test_properties()->position =
+      gfx::PointF(std::numeric_limits<int>::max() - 100, 0);
   child2->SetBounds(gfx::Size(1, 1));
 
   root->layer_tree_impl()->property_trees()->needs_rebuild = true;
@@ -1790,14 +1912,16 @@ TEST_F(DamageTrackerTest, DamageRectTooBigWithFilter) {
   FilterOperations filters;
   filters.Append(FilterOperation::CreateBlurFilter(5.f));
   root->SetDrawsContent(true);
-  root->test_properties()->background_filters = filters;
+  root->test_properties()->backdrop_filters = filters;
 
   // Really far left.
-  child1->SetPosition(gfx::PointF(std::numeric_limits<int>::min() + 100, 0));
+  child1->test_properties()->position =
+      gfx::PointF(std::numeric_limits<int>::min() + 100, 0);
   child1->SetBounds(gfx::Size(1, 1));
 
   // Really far right.
-  child2->SetPosition(gfx::PointF(std::numeric_limits<int>::max() - 100, 0));
+  child2->test_properties()->position =
+      gfx::PointF(std::numeric_limits<int>::max() - 100, 0);
   child2->SetBounds(gfx::Size(1, 1));
 
   root->layer_tree_impl()->property_trees()->needs_rebuild = true;
@@ -1819,18 +1943,20 @@ TEST_F(DamageTrackerTest, DamageRectTooBigWithFilter) {
 TEST_F(DamageTrackerTest, DamageRectTooBigInRenderSurface) {
   LayerImpl* root = CreateAndSetUpTestTreeWithTwoSurfaces();
   LayerImpl* child1 = root->test_properties()->children[0];
-  LayerImpl* grandchild1 = child1->test_properties()->children[0];
-  LayerImpl* grandchild2 = child1->test_properties()->children[1];
+  auto* grandchild1 =
+      static_cast<TestLayerImpl*>(child1->test_properties()->children[0]);
+  auto* grandchild2 =
+      static_cast<TestLayerImpl*>(child1->test_properties()->children[1]);
 
   // Really far left.
-  grandchild1->SetPosition(
-      gfx::PointF(std::numeric_limits<int>::min() + 500, 0));
+  grandchild1->test_properties()->position =
+      gfx::PointF(std::numeric_limits<int>::min() + 500, 0);
   grandchild1->SetBounds(gfx::Size(1, 1));
   grandchild1->SetDrawsContent(true);
 
   // Really far right.
-  grandchild2->SetPosition(
-      gfx::PointF(std::numeric_limits<int>::max() - 500, 0));
+  grandchild2->test_properties()->position =
+      gfx::PointF(std::numeric_limits<int>::max() - 500, 0);
   grandchild2->SetBounds(gfx::Size(1, 1));
   grandchild2->SetDrawsContent(true);
 
@@ -1909,24 +2035,26 @@ TEST_F(DamageTrackerTest, DamageRectTooBigInRenderSurface) {
 TEST_F(DamageTrackerTest, DamageRectTooBigInRenderSurfaceWithFilter) {
   LayerImpl* root = CreateAndSetUpTestTreeWithTwoSurfaces();
   LayerImpl* child1 = root->test_properties()->children[0];
-  LayerImpl* grandchild1 = child1->test_properties()->children[0];
-  LayerImpl* grandchild2 = child1->test_properties()->children[1];
+  auto* grandchild1 =
+      static_cast<TestLayerImpl*>(child1->test_properties()->children[0]);
+  auto* grandchild2 =
+      static_cast<TestLayerImpl*>(child1->test_properties()->children[1]);
 
   // Set up a moving pixels filter on the child.
   FilterOperations filters;
   filters.Append(FilterOperation::CreateBlurFilter(5.f));
   child1->SetDrawsContent(true);
-  child1->test_properties()->background_filters = filters;
+  child1->test_properties()->backdrop_filters = filters;
 
   // Really far left.
-  grandchild1->SetPosition(
-      gfx::PointF(std::numeric_limits<int>::min() + 500, 0));
+  grandchild1->test_properties()->position =
+      gfx::PointF(std::numeric_limits<int>::min() + 500, 0);
   grandchild1->SetBounds(gfx::Size(1, 1));
   grandchild1->SetDrawsContent(true);
 
   // Really far right.
-  grandchild2->SetPosition(
-      gfx::PointF(std::numeric_limits<int>::max() - 500, 0));
+  grandchild2->test_properties()->position =
+      gfx::PointF(std::numeric_limits<int>::max() - 500, 0);
   grandchild2->SetBounds(gfx::Size(1, 1));
   grandchild2->SetDrawsContent(true);
 
