@@ -5,8 +5,9 @@
 #include <stddef.h>
 #include <stdint.h>
 
-#include "base/memory/ptr_util.h"
-#include "cc/base/filter_operations.h"
+#include "base/bind.h"
+#include "base/stl_util.h"
+#include "build/build_config.h"
 #include "cc/layers/heads_up_display_layer.h"
 #include "cc/layers/layer_impl.h"
 #include "cc/layers/painted_scrollbar_layer.h"
@@ -15,29 +16,32 @@
 #include "cc/layers/texture_layer_impl.h"
 #include "cc/layers/video_layer.h"
 #include "cc/layers/video_layer_impl.h"
+#include "cc/paint/filter_operations.h"
 #include "cc/paint/paint_flags.h"
-#include "cc/resources/single_release_callback.h"
 #include "cc/resources/ui_resource_manager.h"
 #include "cc/test/fake_content_layer_client.h"
 #include "cc/test/fake_layer_tree_host_client.h"
 #include "cc/test/fake_painted_scrollbar_layer.h"
 #include "cc/test/fake_picture_layer.h"
 #include "cc/test/fake_picture_layer_impl.h"
-#include "cc/test/fake_resource_provider.h"
 #include "cc/test/fake_scoped_ui_resource.h"
 #include "cc/test/fake_scrollbar.h"
 #include "cc/test/fake_video_frame_provider.h"
 #include "cc/test/layer_tree_test.h"
 #include "cc/test/render_pass_test_utils.h"
-#include "cc/test/test_context_provider.h"
-#include "cc/test/test_shared_bitmap_manager.h"
-#include "cc/test/test_web_graphics_context_3d.h"
+#include "cc/test/test_layer_tree_frame_sink.h"
 #include "cc/trees/layer_tree_host.h"
 #include "cc/trees/layer_tree_host_impl.h"
 #include "cc/trees/layer_tree_impl.h"
 #include "cc/trees/single_thread_proxy.h"
-#include "components/viz/test/test_layer_tree_frame_sink.h"
+#include "components/viz/client/client_resource_provider.h"
+#include "components/viz/common/resources/single_release_callback.h"
+#include "components/viz/test/fake_output_surface.h"
+#include "components/viz/test/test_context_provider.h"
+#include "components/viz/test/test_gles2_interface.h"
+#include "components/viz/test/test_shared_bitmap_manager.h"
 #include "gpu/GLES2/gl2extchromium.h"
+#include "gpu/command_buffer/client/raster_interface.h"
 #include "media/base/media.h"
 
 using media::VideoFrame;
@@ -45,12 +49,16 @@ using media::VideoFrame;
 namespace cc {
 namespace {
 
+// Returns a fake TimeTicks based on the given microsecond offset.
+base::TimeTicks TicksFromMicroseconds(int64_t micros) {
+  return base::TimeTicks() + base::TimeDelta::FromMicroseconds(micros);
+}
+
 // These tests deal with losing the 3d graphics context.
 class LayerTreeHostContextTest : public LayerTreeTest {
  public:
   LayerTreeHostContextTest()
       : LayerTreeTest(),
-        context3d_(NULL),
         times_to_fail_create_(0),
         times_to_lose_during_commit_(0),
         times_to_lose_during_draw_(0),
@@ -66,42 +74,45 @@ class LayerTreeHostContextTest : public LayerTreeTest {
 
   void LoseContext() {
     // CreateDisplayLayerTreeFrameSink happens on a different thread, so lock
-    // context3d_ to make sure we don't set it to null after recreating it
+    // gl_ to make sure we don't set it to null after recreating it
     // there.
-    base::AutoLock lock(context3d_lock_);
+    base::AutoLock lock(gl_lock_);
     // For sanity-checking tests, they should only call this when the
     // context is not lost.
-    CHECK(context3d_);
-    context3d_->loseContextCHROMIUM(GL_GUILTY_CONTEXT_RESET_ARB,
-                                    GL_INNOCENT_CONTEXT_RESET_ARB);
-    context3d_ = nullptr;
+    CHECK(gl_);
+    gl_->LoseContextCHROMIUM(GL_GUILTY_CONTEXT_RESET_ARB,
+                             GL_INNOCENT_CONTEXT_RESET_ARB);
+    gl_ = nullptr;
   }
 
-  std::unique_ptr<viz::TestLayerTreeFrameSink> CreateLayerTreeFrameSink(
+  std::unique_ptr<TestLayerTreeFrameSink> CreateLayerTreeFrameSink(
       const viz::RendererSettings& renderer_settings,
       double refresh_rate,
       scoped_refptr<viz::ContextProvider> compositor_context_provider,
-      scoped_refptr<viz::ContextProvider> worker_context_provider) override {
-    base::AutoLock lock(context3d_lock_);
+      scoped_refptr<viz::RasterContextProvider> worker_context_provider)
+      override {
+    base::AutoLock lock(gl_lock_);
 
-    std::unique_ptr<TestWebGraphicsContext3D> compositor_context3d =
-        TestWebGraphicsContext3D::Create();
+    auto gl_owned = std::make_unique<viz::TestGLES2Interface>();
     if (context_should_support_io_surface_) {
-      compositor_context3d->set_have_extension_io_surface(true);
-      compositor_context3d->set_have_extension_egl_image(true);
+      gl_owned->set_have_extension_io_surface(true);
+      gl_owned->set_have_extension_egl_image(true);
     }
-    context3d_ = compositor_context3d.get();
 
+    gl_ = gl_owned.get();
+
+    auto provider = viz::TestContextProvider::Create(std::move(gl_owned));
     if (times_to_fail_create_) {
       --times_to_fail_create_;
       ExpectCreateToFail();
-      context3d_->loseContextCHROMIUM(GL_GUILTY_CONTEXT_RESET_ARB,
-                                      GL_INNOCENT_CONTEXT_RESET_ARB);
+      gl_->LoseContextCHROMIUM(GL_GUILTY_CONTEXT_RESET_ARB,
+                               GL_INNOCENT_CONTEXT_RESET_ARB);
     }
 
+    sii_ = provider->SharedImageInterface();
+
     return LayerTreeTest::CreateLayerTreeFrameSink(
-        renderer_settings, refresh_rate,
-        TestContextProvider::Create(std::move(compositor_context3d)),
+        renderer_settings, refresh_rate, std::move(provider),
         std::move(worker_context_provider));
   }
 
@@ -151,10 +162,11 @@ class LayerTreeHostContextTest : public LayerTreeTest {
   void ExpectCreateToFail() { ++times_to_expect_create_failed_; }
 
  protected:
-  // Protects use of context3d_ so LoseContext and
+  // Protects use of gl_ so LoseContext and
   // CreateDisplayLayerTreeFrameSink can both use it on different threads.
-  base::Lock context3d_lock_;
-  TestWebGraphicsContext3D* context3d_;
+  base::Lock gl_lock_;
+  viz::TestGLES2Interface* gl_ = nullptr;
+  viz::TestSharedImageInterface* sii_ = nullptr;
 
   int times_to_fail_create_;
   int times_to_lose_during_commit_;
@@ -318,7 +330,7 @@ class LayerTreeHostContextTestLostContextSucceeds
         },
     };
 
-    if (test_case_ >= arraysize(kTests))
+    if (test_case_ >= base::size(kTests))
       return false;
     // Make sure that we lost our context at least once in the last test run so
     // the test did something.
@@ -460,8 +472,8 @@ class MultipleCompositeDoesNotCreateLayerTreeFrameSink
   }
 
   void BeginTest() override {
-    layer_tree_host()->Composite(base::TimeTicks::FromInternalValue(1));
-    layer_tree_host()->Composite(base::TimeTicks::FromInternalValue(2));
+    layer_tree_host()->Composite(TicksFromMicroseconds(1), false);
+    layer_tree_host()->Composite(TicksFromMicroseconds(2), false);
   }
 
   void DidInitializeLayerTreeFrameSink() override { EXPECT_TRUE(false); }
@@ -504,12 +516,12 @@ class FailedCreateDoesNotCreateExtraLayerTreeFrameSink
 
   void BeginTest() override {
     // First composite tries to create a surface.
-    layer_tree_host()->Composite(base::TimeTicks::FromInternalValue(1));
+    layer_tree_host()->Composite(TicksFromMicroseconds(1), false);
     EXPECT_EQ(num_requests_, 2);
     EXPECT_TRUE(has_failed_);
 
     // Second composite should not request or fail.
-    layer_tree_host()->Composite(base::TimeTicks::FromInternalValue(2));
+    layer_tree_host()->Composite(TicksFromMicroseconds(2), false);
     EXPECT_EQ(num_requests_, 2);
     EndTest();
   }
@@ -557,7 +569,7 @@ class LayerTreeHostContextTestCommitAfterDelayedLayerTreeFrameSink
   }
 
   void BeginTest() override {
-    layer_tree_host()->Composite(base::TimeTicks::FromInternalValue(1));
+    layer_tree_host()->Composite(TicksFromMicroseconds(1), false);
   }
 
   void ScheduleComposite() override {
@@ -592,7 +604,7 @@ class LayerTreeHostContextTestAvoidUnnecessaryComposite
 
   void BeginTest() override {
     in_composite_ = true;
-    layer_tree_host()->Composite(base::TimeTicks::FromInternalValue(1));
+    layer_tree_host()->Composite(TicksFromMicroseconds(1), false);
     in_composite_ = false;
   }
 
@@ -689,7 +701,7 @@ class LayerTreeHostContextTestLostContextAndEvictTextures
  public:
   LayerTreeHostContextTestLostContextAndEvictTextures()
       : LayerTreeHostContextTest(),
-        impl_host_(0),
+        impl_host_(nullptr),
         num_commits_(0),
         lost_context_(false) {}
 
@@ -811,30 +823,46 @@ class LayerTreeHostContextTestLayersNotified : public LayerTreeHostContextTest {
     root_->AddChild(child_);
     child_->AddChild(grandchild_);
 
-    layer_tree_host()->SetRootLayer(root_);
     LayerTreeHostContextTest::SetupTree();
     client_.set_bounds(root_->bounds());
   }
 
   void BeginTest() override { PostSetNeedsCommitToMainThread(); }
 
+  void AttachTree() { layer_tree_host()->SetRootLayer(root_); }
+
   void DidActivateTreeOnThread(LayerTreeHostImpl* host_impl) override {
     LayerTreeHostContextTest::DidActivateTreeOnThread(host_impl);
 
-    FakePictureLayerImpl* root_picture = NULL;
-    FakePictureLayerImpl* child_picture = NULL;
-    FakePictureLayerImpl* grandchild_picture = NULL;
-
-    root_picture = static_cast<FakePictureLayerImpl*>(
-        host_impl->active_tree()->root_layer_for_testing());
-    child_picture = static_cast<FakePictureLayerImpl*>(
-        host_impl->active_tree()->LayerById(child_->id()));
-    grandchild_picture = static_cast<FakePictureLayerImpl*>(
-        host_impl->active_tree()->LayerById(grandchild_->id()));
-
     ++num_commits_;
+
+    FakePictureLayerImpl* root_picture = nullptr;
+    FakePictureLayerImpl* child_picture = nullptr;
+    FakePictureLayerImpl* grandchild_picture = nullptr;
+    // Root layer isn't attached on first activation so the static_cast will
+    // fail before second activation.
+    if (num_commits_ >= 2) {
+      root_picture = static_cast<FakePictureLayerImpl*>(
+          host_impl->active_tree()->root_layer_for_testing());
+      child_picture = static_cast<FakePictureLayerImpl*>(
+          host_impl->active_tree()->LayerById(child_->id()));
+      grandchild_picture = static_cast<FakePictureLayerImpl*>(
+          host_impl->active_tree()->LayerById(grandchild_->id()));
+    }
     switch (num_commits_) {
       case 1:
+        // Because setting the colorspace on the first activation releases
+        // resources, don't attach the layers until the first activation.
+        // Because of single thread vs multi thread differences (i.e.
+        // commit to active tree), if this delay is not done, then the
+        // active tree layers will have a different number of resource
+        // releasing.
+        MainThreadTaskRunner()->PostTask(
+            FROM_HERE,
+            base::BindOnce(&LayerTreeHostContextTestLayersNotified::AttachTree,
+                           base::Unretained(this)));
+        break;
+      case 2:
         EXPECT_EQ(0u, root_picture->release_resources_count());
         EXPECT_EQ(0u, child_picture->release_resources_count());
         EXPECT_EQ(0u, grandchild_picture->release_resources_count());
@@ -843,7 +871,7 @@ class LayerTreeHostContextTestLayersNotified : public LayerTreeHostContextTest {
         LoseContext();
         times_to_fail_create_ = 1;
         break;
-      case 2:
+      case 3:
         EXPECT_TRUE(root_picture->release_resources_count());
         EXPECT_TRUE(child_picture->release_resources_count());
         EXPECT_TRUE(grandchild_picture->release_resources_count());
@@ -874,11 +902,12 @@ class LayerTreeHostContextTestDontUseLostResources
   LayerTreeHostContextTestDontUseLostResources() : lost_context_(false) {
     context_should_support_io_surface_ = true;
 
-    child_context_provider_ = TestContextProvider::Create();
-    CHECK(child_context_provider_->BindToCurrentThread());
-    shared_bitmap_manager_.reset(new TestSharedBitmapManager);
-    child_resource_provider_ = FakeResourceProvider::Create(
-        child_context_provider_.get(), shared_bitmap_manager_.get());
+    child_context_provider_ = viz::TestContextProvider::Create();
+    auto result = child_context_provider_->BindToCurrentThread();
+    CHECK_EQ(result, gpu::ContextResult::kSuccess);
+    shared_bitmap_manager_ = std::make_unique<viz::TestSharedBitmapManager>();
+    child_resource_provider_ =
+        std::make_unique<viz::ClientResourceProvider>(true);
   }
 
   static void EmptyReleaseCallback(const gpu::SyncToken& sync_token,
@@ -887,20 +916,10 @@ class LayerTreeHostContextTestDontUseLostResources
   void SetupTree() override {
     gpu::gles2::GLES2Interface* gl = child_context_provider_->ContextGL();
 
-    ResourceId resource = child_resource_provider_->CreateResource(
-        gfx::Size(4, 4), ResourceProvider::TEXTURE_HINT_IMMUTABLE,
-        viz::RGBA_8888, gfx::ColorSpace());
-    ResourceProvider::ScopedWriteLockGL lock(child_resource_provider_.get(),
-                                             resource, false);
-
-    gpu::Mailbox mailbox;
-    gl->GenMailboxCHROMIUM(mailbox.name);
-
-    const GLuint64 fence_sync = gl->InsertFenceSyncCHROMIUM();
-    gl->ShallowFlushCHROMIUM();
+    gpu::Mailbox mailbox = gpu::Mailbox::Generate();
 
     gpu::SyncToken sync_token;
-    gl->GenSyncTokenCHROMIUM(fence_sync, sync_token.GetData());
+    gl->GenSyncTokenCHROMIUM(sync_token.GetData());
 
     scoped_refptr<Layer> root = Layer::Create();
     root->SetBounds(gfx::Size(10, 10));
@@ -911,14 +930,18 @@ class LayerTreeHostContextTestDontUseLostResources
     layer->SetIsDrawable(true);
     root->AddChild(layer);
 
-    scoped_refptr<TextureLayer> texture = TextureLayer::CreateForMailbox(NULL);
+    scoped_refptr<TextureLayer> texture =
+        TextureLayer::CreateForMailbox(nullptr);
     texture->SetBounds(gfx::Size(10, 10));
     texture->SetIsDrawable(true);
-    texture->SetTextureMailbox(
-        viz::TextureMailbox(mailbox, sync_token, GL_TEXTURE_2D),
-        SingleReleaseCallback::Create(
-            base::Bind(&LayerTreeHostContextTestDontUseLostResources::
-                           EmptyReleaseCallback)));
+    constexpr gfx::Size size(64, 64);
+    auto resource = viz::TransferableResource::MakeGL(
+        mailbox, GL_LINEAR, GL_TEXTURE_2D, sync_token, size,
+        false /* is_overlay_candidate */);
+    texture->SetTransferableResource(
+        resource, viz::SingleReleaseCallback::Create(base::BindOnce(
+                      &LayerTreeHostContextTestDontUseLostResources::
+                          EmptyReleaseCallback)));
     root->AddChild(texture);
 
     scoped_refptr<PictureLayer> mask = PictureLayer::Create(&client_);
@@ -995,14 +1018,19 @@ class LayerTreeHostContextTestDontUseLostResources
       // On the third commit we're recovering from context loss. Hardware
       // video frames should not be reused by the VideoFrameProvider, but
       // software frames can be.
-      hw_frame_provider_.set_frame(NULL);
-      scaled_hw_frame_provider_.set_frame(NULL);
+      hw_frame_provider_.set_frame(nullptr);
+      scaled_hw_frame_provider_.set_frame(nullptr);
     }
   }
 
   DrawResult PrepareToDrawOnThread(LayerTreeHostImpl* host_impl,
                                    LayerTreeHostImpl::FrameData* frame,
                                    DrawResult draw_result) override {
+    if (host_impl->active_tree()->source_frame_number() == 2) {
+      // Lose the context after draw on the second commit. This will cause
+      // a third commit to recover.
+      gl_->set_times_bind_texture_succeeds(0);
+    }
     return draw_result;
   }
 
@@ -1018,13 +1046,6 @@ class LayerTreeHostContextTestDontUseLostResources
 
   void DidCommitAndDrawFrame() override {
     ASSERT_TRUE(layer_tree_host()->hud_layer());
-
-    if (layer_tree_host()->SourceFrameNumber() == 2) {
-      // Lose the context after draw on the second commit. This will cause
-      // a third commit to recover.
-      context3d_->set_times_bind_texture_succeeds(0);
-    }
-
     // End the test once we know the 3nd frame drew.
     if (layer_tree_host()->SourceFrameNumber() < 5) {
       layer_tree_host()->root_layer()->SetNeedsDisplay();
@@ -1040,9 +1061,9 @@ class LayerTreeHostContextTestDontUseLostResources
   FakeContentLayerClient client_;
   bool lost_context_;
 
-  scoped_refptr<TestContextProvider> child_context_provider_;
+  scoped_refptr<viz::TestContextProvider> child_context_provider_;
   std::unique_ptr<viz::SharedBitmapManager> shared_bitmap_manager_;
-  std::unique_ptr<ResourceProvider> child_resource_provider_;
+  std::unique_ptr<viz::ClientResourceProvider> child_resource_provider_;
 
   scoped_refptr<VideoFrame> color_video_frame_;
   scoped_refptr<VideoFrame> hw_video_frame_;
@@ -1136,9 +1157,6 @@ SINGLE_AND_MULTI_THREAD_TEST_F(ScrollbarLayerLostContext);
 class UIResourceLostTest : public LayerTreeHostContextTest {
  public:
   UIResourceLostTest() : time_step_(0) {}
-  void InitializeSettings(LayerTreeSettings* settings) override {
-    settings->resource_settings.texture_id_allocation_chunk_size = 1;
-  }
   void BeginTest() override { PostSetNeedsCommitToMainThread(); }
   void AfterTest() override {}
 
@@ -1348,7 +1366,12 @@ class UIResourceLostBeforeCommit : public UIResourceLostTestSimple {
   UIResourceId test_id1_;
 };
 
-SINGLE_AND_MULTI_THREAD_TEST_F(UIResourceLostBeforeCommit);
+// http://crbug.com/803532 : Flaky on Win 7 (dbg) and linux tsan
+#if (defined(NDEBUG) || !defined(OS_WIN)) && \
+    (!defined(THREAD_SANITIZER) || !defined(OS_LINUX))
+SINGLE_THREAD_TEST_F(UIResourceLostBeforeCommit);
+#endif
+MULTI_THREAD_TEST_F(UIResourceLostBeforeCommit);
 
 // Losing UI resource before the pending trees is activated but after the
 // commit.  Impl-side-painting only.
@@ -1478,7 +1501,7 @@ class UIResourceLostEviction : public UIResourceLostTestSimple {
   void DidSetVisibleOnImplTree(LayerTreeHostImpl* impl, bool visible) override {
     if (!visible) {
       // All resources should have been evicted.
-      ASSERT_EQ(0u, context3d_->NumTextures());
+      ASSERT_EQ(0u, sii_->shared_image_count());
       EXPECT_EQ(0u, impl->ResourceIdForUIResource(ui_resource_->id()));
       EXPECT_EQ(0u, impl->ResourceIdForUIResource(ui_resource2_->id()));
       EXPECT_EQ(0u, impl->ResourceIdForUIResource(ui_resource3_->id()));
@@ -1498,7 +1521,7 @@ class UIResourceLostEviction : public UIResourceLostTestSimple {
       case 1:
         // The first two resources should have been created on LTHI after the
         // commit.
-        ASSERT_EQ(2u, context3d_->NumTextures());
+        ASSERT_EQ(2u, sii_->shared_image_count());
         EXPECT_NE(0u, impl->ResourceIdForUIResource(ui_resource_->id()));
         EXPECT_NE(0u, impl->ResourceIdForUIResource(ui_resource2_->id()));
         EXPECT_EQ(1, ui_resource_->resource_create_count);
@@ -1506,7 +1529,7 @@ class UIResourceLostEviction : public UIResourceLostTestSimple {
         EXPECT_TRUE(impl->CanDraw());
         // Evict all UI resources. This will trigger a commit.
         impl->EvictAllUIResources();
-        ASSERT_EQ(0u, context3d_->NumTextures());
+        ASSERT_EQ(0u, sii_->shared_image_count());
         EXPECT_EQ(0u, impl->ResourceIdForUIResource(ui_resource_->id()));
         EXPECT_EQ(0u, impl->ResourceIdForUIResource(ui_resource2_->id()));
         EXPECT_EQ(1, ui_resource_->resource_create_count);
@@ -1515,7 +1538,7 @@ class UIResourceLostEviction : public UIResourceLostTestSimple {
         break;
       case 2:
         // The first two resources should have been recreated.
-        ASSERT_EQ(2u, context3d_->NumTextures());
+        ASSERT_EQ(2u, sii_->shared_image_count());
         EXPECT_NE(0u, impl->ResourceIdForUIResource(ui_resource_->id()));
         EXPECT_EQ(2, ui_resource_->resource_create_count);
         EXPECT_EQ(1, ui_resource_->lost_resource_count);
@@ -1527,7 +1550,7 @@ class UIResourceLostEviction : public UIResourceLostTestSimple {
       case 3:
         // The first resource should have been recreated after visibility was
         // restored.
-        ASSERT_EQ(2u, context3d_->NumTextures());
+        ASSERT_EQ(2u, sii_->shared_image_count());
         EXPECT_NE(0u, impl->ResourceIdForUIResource(ui_resource_->id()));
         EXPECT_EQ(3, ui_resource_->resource_create_count);
         EXPECT_EQ(2, ui_resource_->lost_resource_count);
@@ -1553,6 +1576,181 @@ class UIResourceLostEviction : public UIResourceLostTestSimple {
 
 SINGLE_AND_MULTI_THREAD_TEST_F(UIResourceLostEviction);
 
+class UIResourceFreedIfLostWhileExported : public LayerTreeHostContextTest {
+ protected:
+  void BeginTest() override {
+    // Make 1 UIResource, post it to the compositor thread, where it will be
+    // uploaded.
+    ui_resource_ =
+        FakeScopedUIResource::Create(layer_tree_host()->GetUIResourceManager());
+    EXPECT_NE(0, ui_resource_->id());
+    PostSetNeedsCommitToMainThread();
+  }
+
+  void DidActivateTreeOnThread(LayerTreeHostImpl* impl) override {
+    switch (impl->active_tree()->source_frame_number()) {
+      case 0:
+        // The UIResource has been created and a gpu resource made for it.
+        EXPECT_NE(0u, impl->ResourceIdForUIResource(ui_resource_->id()));
+        EXPECT_EQ(1u, sii_->shared_image_count());
+        // Lose the LayerTreeFrameSink connection. The UI resource should
+        // be replaced and the old texture should be destroyed.
+        impl->DidLoseLayerTreeFrameSink();
+        break;
+      case 1:
+        // The UIResource has been recreated, the old texture is not kept
+        // around.
+        EXPECT_NE(0u, impl->ResourceIdForUIResource(ui_resource_->id()));
+        EXPECT_EQ(1u, sii_->shared_image_count());
+        MainThreadTaskRunner()->PostTask(
+            FROM_HERE,
+            base::BindOnce(
+                &UIResourceFreedIfLostWhileExported::DeleteAndEndTest,
+                base::Unretained(this)));
+    }
+  }
+
+  void DeleteAndEndTest() {
+    ui_resource_->DeleteResource();
+    EndTest();
+  }
+
+  void AfterTest() override {}
+
+  std::unique_ptr<FakeScopedUIResource> ui_resource_;
+};
+
+SINGLE_AND_MULTI_THREAD_TEST_F(UIResourceFreedIfLostWhileExported);
+
+class TileResourceFreedIfLostWhileExported : public LayerTreeHostContextTest {
+ protected:
+  void SetupTree() override {
+    PaintFlags flags;
+    client_.set_fill_with_nonsolid_color(true);
+
+    scoped_refptr<FakePictureLayer> picture_layer =
+        FakePictureLayer::Create(&client_);
+    picture_layer->SetBounds(gfx::Size(10, 20));
+    client_.set_bounds(picture_layer->bounds());
+    layer_tree_host()->SetRootLayer(std::move(picture_layer));
+
+    LayerTreeTest::SetupTree();
+  }
+
+  void BeginTest() override { PostSetNeedsCommitToMainThread(); }
+
+  void DrawLayersOnThread(LayerTreeHostImpl* impl) override {
+    auto* context_provider = static_cast<viz::TestContextProvider*>(
+        impl->layer_tree_frame_sink()->worker_context_provider());
+    viz::TestSharedImageInterface* sii =
+        context_provider->SharedImageInterface();
+    switch (impl->active_tree()->source_frame_number()) {
+      case 0:
+        // The PicturLayer has a texture for a tile, that has been exported to
+        // the display compositor now.
+        EXPECT_EQ(1u, impl->resource_provider()->num_resources_for_testing());
+        EXPECT_EQ(1u, impl->resource_pool()->resource_count());
+        // Shows that the tile texture is allocated with the current worker
+        // context.
+        num_textures_ = sii->shared_image_count();
+        EXPECT_GT(num_textures_, 0u);
+
+        // Lose the LayerTreeFrameSink connection. The tile resource should
+        // be replaced and the old texture should be destroyed.
+        LoseContext();
+        break;
+      case 1:
+        // The tile has been recreated, the old texture is not kept around in
+        // the pool indefinitely. It can be dropped as soon as the context is
+        // known to be lost.
+        EXPECT_EQ(1u, impl->resource_provider()->num_resources_for_testing());
+        EXPECT_EQ(1u, impl->resource_pool()->resource_count());
+        // Shows that the replacement tile texture is re-allocated with the
+        // current worker context, not just the previous one.
+        EXPECT_EQ(num_textures_, sii->shared_image_count());
+        EndTest();
+    }
+  }
+
+  void AfterTest() override {}
+
+  FakeContentLayerClient client_;
+  size_t num_textures_ = 0;
+};
+
+SINGLE_AND_MULTI_THREAD_TEST_F(TileResourceFreedIfLostWhileExported);
+
+class SoftwareTileResourceFreedIfLostWhileExported : public LayerTreeTest {
+ protected:
+  std::unique_ptr<TestLayerTreeFrameSink> CreateLayerTreeFrameSink(
+      const viz::RendererSettings& renderer_settings,
+      double refresh_rate,
+      scoped_refptr<viz::ContextProvider> compositor_context_provider,
+      scoped_refptr<viz::RasterContextProvider> worker_context_provider)
+      override {
+    // Induce software compositing in cc.
+    return LayerTreeTest::CreateLayerTreeFrameSink(
+        renderer_settings, refresh_rate, nullptr, nullptr);
+  }
+
+  std::unique_ptr<viz::OutputSurface> CreateDisplayOutputSurfaceOnThread(
+      scoped_refptr<viz::ContextProvider> compositor_context_provider)
+      override {
+    // Induce software compositing in the display compositor.
+    return viz::FakeOutputSurface::CreateSoftware(
+        std::make_unique<viz::SoftwareOutputDevice>());
+  }
+
+  void SetupTree() override {
+    PaintFlags flags;
+    client_.set_fill_with_nonsolid_color(true);
+
+    scoped_refptr<FakePictureLayer> picture_layer =
+        FakePictureLayer::Create(&client_);
+    picture_layer->SetBounds(gfx::Size(10, 20));
+    client_.set_bounds(picture_layer->bounds());
+    layer_tree_host()->SetRootLayer(std::move(picture_layer));
+
+    LayerTreeTest::SetupTree();
+  }
+
+  void BeginTest() override { PostSetNeedsCommitToMainThread(); }
+
+  void DrawLayersOnThread(LayerTreeHostImpl* impl) override {
+    switch (impl->active_tree()->source_frame_number()) {
+      case 0: {
+        // The PicturLayer has a bitmap for a tile, that has been exported to
+        // the display compositor now.
+        EXPECT_EQ(1u, impl->resource_provider()->num_resources_for_testing());
+        EXPECT_EQ(1u, impl->resource_pool()->resource_count());
+
+        impl->DidLoseLayerTreeFrameSink();
+        break;
+      }
+      case 1: {
+        // The tile did not need to be recreated, the same bitmap/resource
+        // should be used for it.
+        EXPECT_EQ(1u, impl->resource_provider()->num_resources_for_testing());
+        EXPECT_EQ(1u, impl->resource_pool()->resource_count());
+
+        // TODO(danakj): It'd be possible to not destroy and recreate the
+        // software bitmap, however for simplicity we do the same for software
+        // and for gpu paths. If we didn't destroy it we could see the same
+        // bitmap on PictureLayerImpl's tile.
+
+        EndTest();
+      }
+    }
+  }
+
+  void AfterTest() override {}
+
+  FakeContentLayerClient client_;
+  viz::ResourceId exported_resource_id_ = 0;
+};
+
+SINGLE_AND_MULTI_THREAD_TEST_F(SoftwareTileResourceFreedIfLostWhileExported);
+
 class LayerTreeHostContextTestLoseAfterSendingBeginMainFrame
     : public LayerTreeHostContextTest {
  protected:
@@ -1566,9 +1764,12 @@ class LayerTreeHostContextTestLoseAfterSendingBeginMainFrame
       return;
     deferred_ = true;
 
-    // Defer commits before the BeginFrame completes, causing it to be delayed.
-    layer_tree_host()->SetDeferCommits(true);
-    // Meanwhile, lose the context while we are in defer commits.
+    // TODO(schenney): This should switch back to defer_commits_ because there
+    // is no way in the real code to start deferring main frame updates when
+    // inside WillBeginMainFrame. Defer commits before the BeginFrame completes,
+    // causing it to be delayed.
+    scoped_defer_main_frame_update_ = layer_tree_host()->DeferMainFrameUpdate();
+    // Meanwhile, lose the context while we are in defer BeginMainFrame.
     ImplThreadTaskRunner()->PostTask(
         FROM_HERE,
         base::BindOnce(&LayerTreeHostContextTestLoseAfterSendingBeginMainFrame::
@@ -1576,8 +1777,8 @@ class LayerTreeHostContextTestLoseAfterSendingBeginMainFrame
                        base::Unretained(this)));
 
     // After the first frame, we will lose the context and then not start
-    // allowing commits until that happens. The 2nd frame should not happen
-    // before DidInitializeLayerTreeFrameSink occurs.
+    // lifecycle updates and commits until that happens. The 2nd frame should
+    // not happen before DidInitializeLayerTreeFrameSink occurs.
     lost_ = true;
   }
 
@@ -1589,14 +1790,18 @@ class LayerTreeHostContextTestLoseAfterSendingBeginMainFrame
   void LoseContextOnImplThread() {
     LoseContext();
 
+    // TODO(schenney): This should switch back to defer_commits_ to match the
+    // change above.
     // After losing the context, stop deferring commits.
-    PostSetDeferCommitsToMainThread(false);
+    PostReturnDeferMainFrameUpdateToMainThread(
+        std::move(scoped_defer_main_frame_update_));
   }
 
   void DidCommitAndDrawFrame() override { EndTest(); }
 
   void AfterTest() override {}
 
+  std::unique_ptr<ScopedDeferMainFrameUpdate> scoped_defer_main_frame_update_;
   bool deferred_ = false;
   bool lost_ = true;
 };
@@ -1604,11 +1809,7 @@ class LayerTreeHostContextTestLoseAfterSendingBeginMainFrame
 SINGLE_AND_MULTI_THREAD_TEST_F(
     LayerTreeHostContextTestLoseAfterSendingBeginMainFrame);
 
-// This test causes context loss on the worker context but not the compositor
-// context and checks that draw still occurs. The resources might be in a bad
-// state e.g. null sync tokens but that shouldn't cause the draw to fail.
-class LayerTreeHostContextTestLoseWorkerContextDuringPrepareTiles
-    : public LayerTreeTest {
+class LayerTreeHostContextTestWorkerContextLostRecovery : public LayerTreeTest {
  protected:
   void SetupTree() override {
     PaintFlags flags;
@@ -1627,27 +1828,31 @@ class LayerTreeHostContextTestLoseWorkerContextDuringPrepareTiles
   void BeginTest() override { PostSetNeedsCommitToMainThread(); }
 
   void WillPrepareTilesOnThread(LayerTreeHostImpl* host_impl) override {
-    viz::ContextProvider::ScopedContextLock scoped_context(
+    if (did_lose_context)
+      return;
+    did_lose_context = true;
+    viz::RasterContextProvider::ScopedRasterContextLock scoped_context(
         host_impl->layer_tree_frame_sink()->worker_context_provider());
-    gpu::gles2::GLES2Interface* gl = scoped_context.ContextGL();
-    gl->LoseContextCHROMIUM(GL_GUILTY_CONTEXT_RESET_ARB,
+    gpu::raster::RasterInterface* ri = scoped_context.RasterInterface();
+    ri->LoseContextCHROMIUM(GL_GUILTY_CONTEXT_RESET_ARB,
                             GL_INNOCENT_CONTEXT_RESET_ARB);
   }
 
-  void DrawLayersOnThread(LayerTreeHostImpl* host_impl) override {
-    ++num_draws_;
-  }
+  void DidInitializeLayerTreeFrameSink() override { num_frame_sinks_++; }
 
   void DidCommitAndDrawFrame() override { EndTest(); }
 
-  void AfterTest() override { EXPECT_EQ(1, num_draws_); }
+  void AfterTest() override {
+    EXPECT_TRUE(did_lose_context);
+    EXPECT_EQ(num_frame_sinks_, 2);
+  }
 
   FakeContentLayerClient client_;
-  int num_draws_ = 0;
+  bool did_lose_context = false;
+  int num_frame_sinks_ = 0;
 };
 
-MULTI_THREAD_TEST_F(
-    LayerTreeHostContextTestLoseWorkerContextDuringPrepareTiles);
+MULTI_THREAD_TEST_F(LayerTreeHostContextTestWorkerContextLostRecovery);
 
 }  // namespace
 }  // namespace cc
