@@ -10,10 +10,10 @@
 #include <memory>
 #include <utility>
 
+#include "base/bind.h"
 #include "base/lazy_instance.h"
-#include "base/macros.h"
-#include "base/memory/ptr_util.h"
 #include "base/memory/singleton.h"
+#include "base/stl_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/values.h"
 #include "build/build_config.h"
@@ -26,7 +26,7 @@
 #include "chrome/browser/net/prediction_options.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/common/pref_names.h"
-#include "components/autofill/core/common/autofill_pref_names.h"
+#include "components/autofill/core/common/autofill_prefs.h"
 #include "components/content_settings/core/common/pref_names.h"
 #include "components/data_reduction_proxy/core/common/data_reduction_proxy_pref_names.h"
 #include "components/password_manager/core/common/password_manager_pref_names.h"
@@ -48,12 +48,11 @@
 #include "extensions/common/error_utils.h"
 #include "extensions/common/permissions/api_permission.h"
 #include "extensions/common/permissions/permissions_data.h"
-#include "media/media_features.h"
+#include "media/media_buildflags.h"
 
-namespace keys = extensions::preference_api_constants;
-namespace helpers = extensions::preference_helpers;
-
-using base::DictionaryValue;
+#if defined(OS_CHROMEOS)
+#include "ash/public/cpp/ash_pref_names.h"  // nogncheck
+#endif
 
 namespace extensions {
 
@@ -82,8 +81,8 @@ const char kConversionErrorMessage[] =
     "Internal error: Stored value for preference '*' cannot be converted "
     "properly.";
 
-PrefMappingEntry kPrefMapping[] = {
-    {"spdy_proxy.enabled", prefs::kDataSaverEnabled,
+const PrefMappingEntry kPrefMapping[] = {
+    {"spdy_proxy.enabled", data_reduction_proxy::prefs::kDataSaverEnabled,
      APIPermission::kDataReductionProxy, APIPermission::kDataReductionProxy},
     {"data_reduction.daily_original_length",
      data_reduction_proxy::prefs::kDailyHttpOriginalContentLength,
@@ -96,12 +95,14 @@ PrefMappingEntry kPrefMapping[] = {
      APIPermission::kDataReductionProxy, APIPermission::kDataReductionProxy},
     {"alternateErrorPagesEnabled", prefs::kAlternateErrorPagesEnabled,
      APIPermission::kPrivacy, APIPermission::kPrivacy},
-    {"autofillEnabled", autofill::prefs::kAutofillEnabled,
+    {"autofillEnabled", autofill::prefs::kAutofillEnabledDeprecated,
+     APIPermission::kPrivacy, APIPermission::kPrivacy},
+    {"autofillAddressEnabled", autofill::prefs::kAutofillProfileEnabled,
+     APIPermission::kPrivacy, APIPermission::kPrivacy},
+    {"autofillCreditCardEnabled", autofill::prefs::kAutofillCreditCardEnabled,
      APIPermission::kPrivacy, APIPermission::kPrivacy},
     {"hyperlinkAuditingEnabled", prefs::kEnableHyperlinkAuditing,
      APIPermission::kPrivacy, APIPermission::kPrivacy},
-    {"hotwordSearchEnabled", prefs::kHotwordSearchEnabled,
-     APIPermission::kPrivacy, APIPermission::kPrivacy},
     {"networkPredictionEnabled", prefs::kNetworkPredictionOptions,
      APIPermission::kPrivacy, APIPermission::kPrivacy},
     {"passwordSavingEnabled",
@@ -113,10 +114,12 @@ PrefMappingEntry kPrefMapping[] = {
      APIPermission::kProxy},
     {"referrersEnabled", prefs::kEnableReferrers, APIPermission::kPrivacy,
      APIPermission::kPrivacy},
+    {"doNotTrackEnabled", prefs::kEnableDoNotTrack, APIPermission::kPrivacy,
+     APIPermission::kPrivacy},
     {"safeBrowsingEnabled", prefs::kSafeBrowsingEnabled,
      APIPermission::kPrivacy, APIPermission::kPrivacy},
     {"safeBrowsingExtendedReportingEnabled",
-     prefs::kSafeBrowsingExtendedReportingEnabled, APIPermission::kPrivacy,
+     prefs::kSafeBrowsingScoutReportingEnabled, APIPermission::kPrivacy,
      APIPermission::kPrivacy},
     {"searchSuggestEnabled", prefs::kSearchSuggestEnabled,
      APIPermission::kPrivacy, APIPermission::kPrivacy},
@@ -124,7 +127,7 @@ PrefMappingEntry kPrefMapping[] = {
      APIPermission::kPrivacy, APIPermission::kPrivacy},
     {"thirdPartyCookiesAllowed", prefs::kBlockThirdPartyCookies,
      APIPermission::kPrivacy, APIPermission::kPrivacy},
-    {"translationServiceEnabled", prefs::kEnableTranslate,
+    {"translationServiceEnabled", prefs::kOfferTranslateEnabled,
      APIPermission::kPrivacy, APIPermission::kPrivacy},
 #if BUILDFLAG(ENABLE_WEBRTC)
     // webRTCMultipleRoutesEnabled and webRTCNonProxiedUdpEnabled have been
@@ -146,25 +149,28 @@ PrefMappingEntry kPrefMapping[] = {
      APIPermission::kAccessibilityFeaturesRead,
      APIPermission::kAccessibilityFeaturesModify},
 #if defined(OS_CHROMEOS)
-    {"autoclick", prefs::kAccessibilityAutoclickEnabled,
+    {"autoclick", ash::prefs::kAccessibilityAutoclickEnabled,
      APIPermission::kAccessibilityFeaturesRead,
      APIPermission::kAccessibilityFeaturesModify},
-    {"highContrast", prefs::kAccessibilityHighContrastEnabled,
+    {"highContrast", ash::prefs::kAccessibilityHighContrastEnabled,
      APIPermission::kAccessibilityFeaturesRead,
      APIPermission::kAccessibilityFeaturesModify},
-    {"largeCursor", prefs::kAccessibilityLargeCursorEnabled,
+    {"largeCursor", ash::prefs::kAccessibilityLargeCursorEnabled,
      APIPermission::kAccessibilityFeaturesRead,
      APIPermission::kAccessibilityFeaturesModify},
-    {"screenMagnifier", prefs::kAccessibilityScreenMagnifierEnabled,
+    {"screenMagnifier", ash::prefs::kAccessibilityScreenMagnifierEnabled,
      APIPermission::kAccessibilityFeaturesRead,
      APIPermission::kAccessibilityFeaturesModify},
-    {"spokenFeedback", prefs::kAccessibilitySpokenFeedbackEnabled,
+    {"selectToSpeak", ash::prefs::kAccessibilitySelectToSpeakEnabled,
      APIPermission::kAccessibilityFeaturesRead,
      APIPermission::kAccessibilityFeaturesModify},
-    {"stickyKeys", prefs::kAccessibilityStickyKeysEnabled,
+    {"spokenFeedback", ash::prefs::kAccessibilitySpokenFeedbackEnabled,
      APIPermission::kAccessibilityFeaturesRead,
      APIPermission::kAccessibilityFeaturesModify},
-    {"virtualKeyboard", prefs::kAccessibilityVirtualKeyboardEnabled,
+    {"stickyKeys", ash::prefs::kAccessibilityStickyKeysEnabled,
+     APIPermission::kAccessibilityFeaturesRead,
+     APIPermission::kAccessibilityFeaturesModify},
+    {"virtualKeyboard", ash::prefs::kAccessibilityVirtualKeyboardEnabled,
      APIPermission::kAccessibilityFeaturesRead,
      APIPermission::kAccessibilityFeaturesModify},
 #endif
@@ -205,7 +211,7 @@ class InvertBooleanTransformer : public PrefTransformerInterface {
     bool bool_value = false;
     bool result = value->GetAsBoolean(&bool_value);
     DCHECK(result);
-    return base::MakeUnique<base::Value>(!bool_value);
+    return std::make_unique<base::Value>(!bool_value);
   }
 };
 
@@ -219,12 +225,11 @@ class NetworkPredictionTransformer : public PrefTransformerInterface {
     const bool pref_found = extension_pref->GetAsBoolean(&bool_value);
     DCHECK(pref_found) << "Preference not found.";
     if (bool_value) {
-      return base::MakeUnique<base::Value>(
+      return std::make_unique<base::Value>(
           chrome_browser_net::NETWORK_PREDICTION_DEFAULT);
-    } else {
-      return base::MakeUnique<base::Value>(
-          chrome_browser_net::NETWORK_PREDICTION_NEVER);
     }
+    return std::make_unique<base::Value>(
+        chrome_browser_net::NETWORK_PREDICTION_NEVER);
   }
 
   std::unique_ptr<base::Value> BrowserToExtensionPref(
@@ -232,7 +237,7 @@ class NetworkPredictionTransformer : public PrefTransformerInterface {
     int int_value = chrome_browser_net::NETWORK_PREDICTION_DEFAULT;
     const bool pref_found = browser_pref->GetAsInteger(&int_value);
     DCHECK(pref_found) << "Preference not found.";
-    return base::MakeUnique<base::Value>(
+    return std::make_unique<base::Value>(
         int_value != chrome_browser_net::NETWORK_PREDICTION_NEVER);
   }
 };
@@ -247,7 +252,7 @@ class PrefMapping {
                                        std::string* browser_pref,
                                        APIPermission::ID* read_permission,
                                        APIPermission::ID* write_permission) {
-    PrefMap::iterator it = mapping_.find(extension_pref);
+    auto it = mapping_.find(extension_pref);
     if (it != mapping_.end()) {
       *browser_pref = it->second.pref_name;
       *read_permission = it->second.read_permission;
@@ -260,7 +265,7 @@ class PrefMapping {
   bool FindEventForBrowserPref(const std::string& browser_pref,
                                std::string* event_name,
                                APIPermission::ID* permission) {
-    PrefMap::iterator it = event_mapping_.find(browser_pref);
+    auto it = event_mapping_.find(browser_pref);
     if (it != event_mapping_.end()) {
       *event_name = it->second.pref_name;
       *permission = it->second.read_permission;
@@ -274,36 +279,30 @@ class PrefMapping {
     auto it = transformers_.find(browser_pref);
     if (it != transformers_.end())
       return it->second.get();
-    else
-      return identity_transformer_.get();
+    return identity_transformer_.get();
   }
 
  private:
   friend struct base::DefaultSingletonTraits<PrefMapping>;
 
   PrefMapping() {
-    identity_transformer_.reset(new IdentityPrefTransformer());
-    for (size_t i = 0; i < arraysize(kPrefMapping); ++i) {
-      mapping_[kPrefMapping[i].extension_pref] =
-          PrefMapData(kPrefMapping[i].browser_pref,
-                      kPrefMapping[i].read_permission,
-                      kPrefMapping[i].write_permission);
+    identity_transformer_ = std::make_unique<IdentityPrefTransformer>();
+    for (const auto& pref : kPrefMapping) {
+      mapping_[pref.extension_pref] = PrefMapData(
+          pref.browser_pref, pref.read_permission, pref.write_permission);
       std::string event_name =
-          base::StringPrintf(kOnPrefChangeFormat,
-                             kPrefMapping[i].extension_pref);
-      event_mapping_[kPrefMapping[i].browser_pref] =
-          PrefMapData(event_name,
-                      kPrefMapping[i].read_permission,
-                      kPrefMapping[i].write_permission);
+          base::StringPrintf(kOnPrefChangeFormat, pref.extension_pref);
+      event_mapping_[pref.browser_pref] =
+          PrefMapData(event_name, pref.read_permission, pref.write_permission);
     }
-    DCHECK_EQ(arraysize(kPrefMapping), mapping_.size());
-    DCHECK_EQ(arraysize(kPrefMapping), event_mapping_.size());
+    DCHECK_EQ(base::size(kPrefMapping), mapping_.size());
+    DCHECK_EQ(base::size(kPrefMapping), event_mapping_.size());
     RegisterPrefTransformer(proxy_config::prefs::kProxy,
-                            base::MakeUnique<ProxyPrefTransformer>());
+                            std::make_unique<ProxyPrefTransformer>());
     RegisterPrefTransformer(prefs::kBlockThirdPartyCookies,
-                            base::MakeUnique<InvertBooleanTransformer>());
+                            std::make_unique<InvertBooleanTransformer>());
     RegisterPrefTransformer(prefs::kNetworkPredictionOptions,
-                            base::MakeUnique<NetworkPredictionTransformer>());
+                            std::make_unique<NetworkPredictionTransformer>());
   }
 
   ~PrefMapping() {
@@ -312,8 +311,9 @@ class PrefMapping {
   void RegisterPrefTransformer(
       const std::string& browser_pref,
       std::unique_ptr<PrefTransformerInterface> transformer) {
-    DCHECK_EQ(0u, transformers_.count(browser_pref)) <<
-        "Trying to register pref transformer for " << browser_pref << " twice";
+    DCHECK(!base::Contains(transformers_, browser_pref))
+        << "Trying to register pref transformer for " << browser_pref
+        << " twice";
     transformers_[browser_pref] = std::move(transformer);
   }
 
@@ -339,7 +339,7 @@ class PrefMapping {
     APIPermission::ID write_permission;
   };
 
-  typedef std::map<std::string, PrefMapData> PrefMap;
+  using PrefMap = std::map<std::string, PrefMapData>;
 
   // Mapping from extension pref keys to browser pref keys and permissions.
   PrefMap mapping_;
@@ -361,11 +361,10 @@ class PrefMapping {
 PreferenceEventRouter::PreferenceEventRouter(Profile* profile)
     : profile_(profile) {
   registrar_.Init(profile_->GetPrefs());
-  for (size_t i = 0; i < arraysize(kPrefMapping); ++i) {
-    registrar_.Add(kPrefMapping[i].browser_pref,
+  for (const auto& pref : kPrefMapping) {
+    registrar_.Add(pref.browser_pref,
                    base::Bind(&PreferenceEventRouter::OnPrefChanged,
-                              base::Unretained(this),
-                              registrar_.prefs()));
+                              base::Unretained(this), registrar_.prefs()));
   }
   notification_registrar_.Add(this, chrome::NOTIFICATION_PROFILE_CREATED,
                               content::NotificationService::AllSources());
@@ -400,11 +399,12 @@ void PreferenceEventRouter::OnPrefChanged(PrefService* pref_service,
     return;
   }
 
-  std::unique_ptr<base::DictionaryValue> dict(new base::DictionaryValue());
-  dict->Set(keys::kValue, std::move(transformed_value));
+  auto dict = std::make_unique<base::DictionaryValue>();
+  dict->Set(extensions::preference_api_constants::kValue,
+            std::move(transformed_value));
   if (incognito) {
     ExtensionPrefs* ep = ExtensionPrefs::Get(profile_);
-    dict->SetBoolean(keys::kIncognitoSpecific,
+    dict->SetBoolean(extensions::preference_api_constants::kIncognitoSpecific,
                      ep->HasIncognitoPrefValue(browser_pref));
   }
   args.Append(std::move(dict));
@@ -420,9 +420,9 @@ void PreferenceEventRouter::OnPrefChanged(PrefService* pref_service,
   // to change.
   events::HistogramValue histogram_value =
       events::TYPES_CHROME_SETTING_ON_CHANGE;
-  helpers::DispatchEventToExtensions(profile_, histogram_value, event_name,
-                                     &args, permission, incognito,
-                                     browser_pref);
+  extensions::preference_helpers::DispatchEventToExtensions(
+      profile_, histogram_value, event_name, &args, permission, incognito,
+      browser_pref);
 }
 
 void PreferenceEventRouter::Observe(
@@ -452,11 +452,11 @@ void PreferenceEventRouter::Observe(
 }
 
 void PreferenceEventRouter::OnIncognitoProfileCreated(PrefService* prefs) {
-  incognito_registrar_.reset(new PrefChangeRegistrar());
+  incognito_registrar_ = std::make_unique<PrefChangeRegistrar>();
   incognito_registrar_->Init(prefs);
-  for (size_t i = 0; i < arraysize(kPrefMapping); ++i) {
+  for (const auto& pref : kPrefMapping) {
     incognito_registrar_->Add(
-        kPrefMapping[i].browser_pref,
+        pref.browser_pref,
         base::Bind(&PreferenceEventRouter::OnPrefChanged,
                    base::Unretained(this), incognito_registrar_->prefs()));
   }
@@ -466,13 +466,13 @@ void PreferenceAPIBase::SetExtensionControlledPref(
     const std::string& extension_id,
     const std::string& pref_key,
     ExtensionPrefsScope scope,
-    base::Value* value) {
+    base::Value value) {
 #ifndef NDEBUG
   const PrefService::Preference* pref =
       extension_prefs()->pref_service()->FindPreference(pref_key);
   DCHECK(pref) << "Extension controlled preference key " << pref_key
                << " not registered.";
-  DCHECK_EQ(pref->GetType(), value->GetType())
+  DCHECK_EQ(pref->GetType(), value.type())
       << "Extension controlled preference " << pref_key << " has wrong type.";
 #endif
 
@@ -485,10 +485,10 @@ void PreferenceAPIBase::SetExtensionControlledPref(
                                                   extension_id,
                                                   scope_string);
     auto preference = update.Create();
-    preference->SetWithoutPathExpansion(pref_key, value->CreateDeepCopy());
+    preference->SetWithoutPathExpansion(pref_key, value.CreateDeepCopy());
   }
-  extension_pref_value_map()->SetExtensionPref(
-      extension_id, pref_key, scope, value);
+  extension_pref_value_map()->SetExtensionPref(extension_id, pref_key, scope,
+                                               std::move(value));
 }
 
 void PreferenceAPIBase::RemoveExtensionControlledPref(
@@ -506,7 +506,7 @@ void PreferenceAPIBase::RemoveExtensionControlledPref(
                                                   scope_string);
     auto preference = update.Get();
     if (preference)
-      preference->RemoveWithoutPathExpansion(pref_key, NULL);
+      preference->RemoveWithoutPathExpansion(pref_key, nullptr);
   }
   extension_pref_value_map()->RemoveExtensionPref(
       extension_id, pref_key, scope);
@@ -538,11 +538,11 @@ bool PreferenceAPIBase::DoesExtensionControlPref(
 
 PreferenceAPI::PreferenceAPI(content::BrowserContext* context)
     : profile_(Profile::FromBrowserContext(context)) {
-  for (size_t i = 0; i < arraysize(kPrefMapping); ++i) {
+  for (const auto& pref : kPrefMapping) {
     std::string event_name;
     APIPermission::ID permission = APIPermission::kInvalid;
     bool rv = PrefMapping::GetInstance()->FindEventForBrowserPref(
-        kPrefMapping[i].browser_pref, &event_name, &permission);
+        pref.browser_pref, &event_name, &permission);
     DCHECK(rv);
     EventRouter::Get(profile_)->RegisterObserver(this, event_name);
   }
@@ -559,14 +559,13 @@ void PreferenceAPI::Shutdown() {
   content_settings_store()->RemoveObserver(this);
 }
 
-static base::LazyInstance<
-    BrowserContextKeyedAPIFactory<PreferenceAPI>>::DestructorAtExit g_factory =
-    LAZY_INSTANCE_INITIALIZER;
+static base::LazyInstance<BrowserContextKeyedAPIFactory<PreferenceAPI>>::
+    DestructorAtExit g_preference_api_factory = LAZY_INSTANCE_INITIALIZER;
 
 // static
 BrowserContextKeyedAPIFactory<PreferenceAPI>*
 PreferenceAPI::GetFactoryInstance() {
-  return g_factory.Pointer();
+  return g_preference_api_factory.Pointer();
 }
 
 // static
@@ -575,7 +574,7 @@ PreferenceAPI* PreferenceAPI::Get(content::BrowserContext* context) {
 }
 
 void PreferenceAPI::OnListenerAdded(const EventListenerInfo& details) {
-  preference_event_router_.reset(new PreferenceEventRouter(profile_));
+  preference_event_router_ = std::make_unique<PreferenceEventRouter>(profile_);
   EventRouter::Get(profile_)->UnregisterObserver(this);
 }
 
@@ -599,10 +598,9 @@ void PreferenceAPI::OnContentSettingChanged(const std::string& extension_id,
 void PreferenceAPI::ClearIncognitoSessionOnlyContentSettings() {
   ExtensionIdList extension_ids;
   extension_prefs()->GetExtensions(&extension_ids);
-  for (ExtensionIdList::iterator extension_id = extension_ids.begin();
-       extension_id != extension_ids.end(); ++extension_id) {
+  for (const auto& id : extension_ids) {
     content_settings_store()->ClearContentSettingsForExtension(
-        *extension_id, kExtensionPrefsScopeIncognitoSessionOnly);
+        id, kExtensionPrefsScopeIncognitoSessionOnly);
   }
 }
 
@@ -634,17 +632,27 @@ GetPreferenceFunction::~GetPreferenceFunction() { }
 ExtensionFunction::ResponseAction GetPreferenceFunction::Run() {
   std::string pref_key;
   EXTENSION_FUNCTION_VALIDATE(args_->GetString(0, &pref_key));
-  base::DictionaryValue* details = NULL;
+  base::DictionaryValue* details = nullptr;
   EXTENSION_FUNCTION_VALIDATE(args_->GetDictionary(1, &details));
 
   bool incognito = false;
-  if (details->HasKey(keys::kIncognitoKey))
-    EXTENSION_FUNCTION_VALIDATE(details->GetBoolean(keys::kIncognitoKey,
-                                                    &incognito));
+  if (details->HasKey(extensions::preference_api_constants::kIncognitoKey))
+    EXTENSION_FUNCTION_VALIDATE(details->GetBoolean(
+        extensions::preference_api_constants::kIncognitoKey, &incognito));
 
   // Check incognito access.
-  if (incognito && !include_incognito())
-    return RespondNow(Error(keys::kIncognitoErrorMessage));
+  if (incognito) {
+    // Extensions are only allowed to modify incognito preferences if they are
+    // enabled in incognito. If the calling browser context is off the record,
+    // then the extension must be allowed to run incognito. Otherwise, this
+    // could be a spanning mode extension, and we need to check its incognito
+    // access.
+    if (!browser_context()->IsOffTheRecord() &&
+        !include_incognito_information()) {
+      return RespondNow(
+          Error(extensions::preference_api_constants::kIncognitoErrorMessage));
+    }
+  }
 
   // Obtain pref.
   std::string browser_pref;
@@ -654,7 +662,9 @@ ExtensionFunction::ResponseAction GetPreferenceFunction::Run() {
       PrefMapping::GetInstance()->FindBrowserPrefForExtensionPref(
       pref_key, &browser_pref, &read_permission, &write_permission));
   if (!extension()->permissions_data()->HasAPIPermission(read_permission))
-    return RespondNow(Error(keys::kPermissionErrorMessage, pref_key));
+    return RespondNow(
+        Error(extensions::preference_api_constants::kPermissionErrorMessage,
+              pref_key));
 
   Profile* profile = Profile::FromBrowserContext(browser_context());
   const PrefService* prefs =
@@ -662,12 +672,14 @@ ExtensionFunction::ResponseAction GetPreferenceFunction::Run() {
   const PrefService::Preference* pref = prefs->FindPreference(browser_pref);
   CHECK(pref);
 
-  std::unique_ptr<base::DictionaryValue> result(new base::DictionaryValue);
+  auto result = std::make_unique<base::DictionaryValue>();
 
   // Retrieve level of control.
-  std::string level_of_control = helpers::GetLevelOfControl(
-      profile, extension_id(), browser_pref, incognito);
-  result->SetString(keys::kLevelOfControl, level_of_control);
+  std::string level_of_control =
+      extensions::preference_helpers::GetLevelOfControl(
+          profile, extension_id(), browser_pref, incognito);
+  result->SetString(extensions::preference_api_constants::kLevelOfControl,
+                    level_of_control);
 
   // Retrieve pref value.
   PrefTransformerInterface* transformer =
@@ -677,41 +689,43 @@ ExtensionFunction::ResponseAction GetPreferenceFunction::Run() {
   if (!transformed_value) {
     // TODO(devlin): Can this happen?  When?  Should it be an error, or a bad
     // message?
-    LOG(ERROR) <<
-        ErrorUtils::FormatErrorMessage(kConversionErrorMessage,
-                                                pref->name());
+    LOG(ERROR) << ErrorUtils::FormatErrorMessage(kConversionErrorMessage,
+                                                 pref->name());
     return RespondNow(Error(kUnknownErrorDoNotUse));
   }
-  result->Set(keys::kValue, std::move(transformed_value));
+  result->Set(extensions::preference_api_constants::kValue,
+              std::move(transformed_value));
 
   // Retrieve incognito status.
   if (incognito) {
     ExtensionPrefs* ep = ExtensionPrefs::Get(browser_context());
-    result->SetBoolean(keys::kIncognitoSpecific,
+    result->SetBoolean(extensions::preference_api_constants::kIncognitoSpecific,
                        ep->HasIncognitoPrefValue(browser_pref));
   }
 
   return RespondNow(OneArgument(std::move(result)));
 }
 
-SetPreferenceFunction::~SetPreferenceFunction() { }
+SetPreferenceFunction::~SetPreferenceFunction() {}
 
 ExtensionFunction::ResponseAction SetPreferenceFunction::Run() {
   std::string pref_key;
   EXTENSION_FUNCTION_VALIDATE(args_->GetString(0, &pref_key));
-  base::DictionaryValue* details = NULL;
+  base::DictionaryValue* details = nullptr;
   EXTENSION_FUNCTION_VALIDATE(args_->GetDictionary(1, &details));
 
-  base::Value* value = NULL;
-  EXTENSION_FUNCTION_VALIDATE(details->Get(keys::kValue, &value));
+  base::Value* value = nullptr;
+  EXTENSION_FUNCTION_VALIDATE(
+      details->Get(extensions::preference_api_constants::kValue, &value));
 
   ExtensionPrefsScope scope = kExtensionPrefsScopeRegular;
-  if (details->HasKey(keys::kScopeKey)) {
+  if (details->HasKey(extensions::preference_api_constants::kScopeKey)) {
     std::string scope_str;
-    EXTENSION_FUNCTION_VALIDATE(
-        details->GetString(keys::kScopeKey, &scope_str));
+    EXTENSION_FUNCTION_VALIDATE(details->GetString(
+        extensions::preference_api_constants::kScopeKey, &scope_str));
 
-    EXTENSION_FUNCTION_VALIDATE(helpers::StringToScope(scope_str, &scope));
+    EXTENSION_FUNCTION_VALIDATE(
+        extensions::preference_helpers::StringToScope(scope_str, &scope));
   }
 
   // Check incognito scope.
@@ -719,12 +733,18 @@ ExtensionFunction::ResponseAction SetPreferenceFunction::Run() {
       (scope == kExtensionPrefsScopeIncognitoPersistent ||
        scope == kExtensionPrefsScopeIncognitoSessionOnly);
   if (incognito) {
-    // Regular profiles can't access incognito unless include_incognito is true.
-    if (!browser_context()->IsOffTheRecord() && !include_incognito())
-      return RespondNow(Error(keys::kIncognitoErrorMessage));
+    // Regular profiles can't access incognito unless
+    // include_incognito_information is true.
+    if (!browser_context()->IsOffTheRecord() &&
+        !include_incognito_information())
+      return RespondNow(
+          Error(extensions::preference_api_constants::kIncognitoErrorMessage));
   } else if (browser_context()->IsOffTheRecord()) {
-    // Incognito profiles can't access regular mode ever, they only exist in
-    // split mode.
+    // If the browser_context associated with this ExtensionFunction is off the
+    // record, it must have come from the incognito process for a split-mode
+    // extension (spanning mode extensions only run in the on-the-record
+    // process). The incognito profile of a split-mode extension should never be
+    // able to modify the on-the-record profile, so error out.
     return RespondNow(
         Error("Can't modify regular settings from an incognito context."));
   }
@@ -732,7 +752,8 @@ ExtensionFunction::ResponseAction SetPreferenceFunction::Run() {
   Profile* profile = Profile::FromBrowserContext(browser_context());
   if (scope == kExtensionPrefsScopeIncognitoSessionOnly &&
       !profile->HasOffTheRecordProfile()) {
-    return RespondNow(Error(keys::kIncognitoSessionOnlyErrorMessage));
+    return RespondNow(Error(extensions::preference_api_constants::
+                                kIncognitoSessionOnlyErrorMessage));
   }
 
   // Obtain pref.
@@ -743,7 +764,9 @@ ExtensionFunction::ResponseAction SetPreferenceFunction::Run() {
       PrefMapping::GetInstance()->FindBrowserPrefForExtensionPref(
       pref_key, &browser_pref, &read_permission, &write_permission));
   if (!extension()->permissions_data()->HasAPIPermission(write_permission))
-    return RespondNow(Error(keys::kPermissionErrorMessage, pref_key));
+    return RespondNow(
+        Error(extensions::preference_api_constants::kPermissionErrorMessage,
+              pref_key));
 
   ExtensionPrefs* prefs = ExtensionPrefs::Get(browser_context());
   const PrefService::Preference* pref =
@@ -761,7 +784,7 @@ ExtensionFunction::ResponseAction SetPreferenceFunction::Run() {
     EXTENSION_FUNCTION_VALIDATE(!bad_message);
     return RespondNow(Error(error));
   }
-  EXTENSION_FUNCTION_VALIDATE(browser_pref_value->GetType() == pref->GetType());
+  EXTENSION_FUNCTION_VALIDATE(browser_pref_value->type() == pref->GetType());
 
   // Validate also that the stored value can be converted back by the
   // transformer.
@@ -769,9 +792,25 @@ ExtensionFunction::ResponseAction SetPreferenceFunction::Run() {
       transformer->BrowserToExtensionPref(browser_pref_value.get()));
   EXTENSION_FUNCTION_VALIDATE(extension_pref_value);
 
-  PreferenceAPI::Get(browser_context())
-      ->SetExtensionControlledPref(extension_id(), browser_pref, scope,
-                                   browser_pref_value.release());
+  PreferenceAPI* preference_api = PreferenceAPI::Get(browser_context());
+
+  // Set the new Autofill prefs if the extension sets the deprecated pref in
+  // order to maintain backward compatibility in the extensions preference API.
+  // TODO(crbug.com/870328): Remove this once the deprecated pref is retired.
+  if (autofill::prefs::kAutofillEnabledDeprecated == browser_pref) {
+    // |SetExtensionControlledPref| takes ownership of the base::Value pointer.
+    preference_api->SetExtensionControlledPref(
+        extension_id(), autofill::prefs::kAutofillCreditCardEnabled, scope,
+        base::Value(browser_pref_value->GetBool()));
+    preference_api->SetExtensionControlledPref(
+        extension_id(), autofill::prefs::kAutofillProfileEnabled, scope,
+        base::Value(browser_pref_value->GetBool()));
+  }
+
+  preference_api->SetExtensionControlledPref(
+      extension_id(), browser_pref, scope,
+      base::Value::FromUniquePtrValue(std::move(browser_pref_value)));
+
   return RespondNow(NoArguments());
 }
 
@@ -780,16 +819,17 @@ ClearPreferenceFunction::~ClearPreferenceFunction() { }
 ExtensionFunction::ResponseAction ClearPreferenceFunction::Run() {
   std::string pref_key;
   EXTENSION_FUNCTION_VALIDATE(args_->GetString(0, &pref_key));
-  base::DictionaryValue* details = NULL;
+  base::DictionaryValue* details = nullptr;
   EXTENSION_FUNCTION_VALIDATE(args_->GetDictionary(1, &details));
 
   ExtensionPrefsScope scope = kExtensionPrefsScopeRegular;
-  if (details->HasKey(keys::kScopeKey)) {
+  if (details->HasKey(extensions::preference_api_constants::kScopeKey)) {
     std::string scope_str;
-    EXTENSION_FUNCTION_VALIDATE(
-        details->GetString(keys::kScopeKey, &scope_str));
+    EXTENSION_FUNCTION_VALIDATE(details->GetString(
+        extensions::preference_api_constants::kScopeKey, &scope_str));
 
-    EXTENSION_FUNCTION_VALIDATE(helpers::StringToScope(scope_str, &scope));
+    EXTENSION_FUNCTION_VALIDATE(
+        extensions::preference_helpers::StringToScope(scope_str, &scope));
   }
 
   // Check incognito scope.
@@ -813,7 +853,9 @@ ExtensionFunction::ResponseAction ClearPreferenceFunction::Run() {
       PrefMapping::GetInstance()->FindBrowserPrefForExtensionPref(
       pref_key, &browser_pref, &read_permission, &write_permission));
   if (!extension()->permissions_data()->HasAPIPermission(write_permission))
-    return RespondNow(Error(keys::kPermissionErrorMessage, pref_key));
+    return RespondNow(
+        Error(extensions::preference_api_constants::kPermissionErrorMessage,
+              pref_key));
 
   PreferenceAPI::Get(browser_context())
       ->RemoveExtensionControlledPref(extension_id(), browser_pref, scope);
