@@ -10,15 +10,18 @@
 #include "base/bind.h"
 #include "base/files/file_path.h"
 #include "base/path_service.h"
-#include "base/task_scheduler/post_task.h"
+#include "base/task/post_task.h"
 #include "base/threading/thread.h"
+#include "base/win/windows_version.h"
+#include "chrome/browser/platform_util.h"
+#include "url/gurl.h"
 
 namespace {
 
 // A helper callback that opens the printer management dialog.
 void OpenPrintersDialogCallback() {
   base::FilePath sys_dir;
-  PathService::Get(base::DIR_SYSTEM, &sys_dir);
+  base::PathService::Get(base::DIR_SYSTEM, &sys_dir);
   base::FilePath rundll32 = sys_dir.Append(L"rundll32.exe");
   base::FilePath shell32dll = sys_dir.Append(L"shell32.dll");
 
@@ -32,10 +35,14 @@ void OpenPrintersDialogCallback() {
 
 namespace printing {
 
-void PrinterManagerDialog::ShowPrinterManagerDialog() {
-  base::PostTaskWithTraits(
-      FROM_HERE, {base::MayBlock(), base::TaskPriority::USER_BLOCKING},
-      base::Bind(OpenPrintersDialogCallback));
+void PrinterManagerDialog::ShowPrinterManagerDialog(Profile* profile) {
+  if (base::win::GetVersion() >= base::win::Version::WIN10_RS1) {
+    platform_util::OpenExternal(profile, GURL("ms-settings:printers"));
+  } else {
+    base::PostTaskWithTraits(
+        FROM_HERE, {base::MayBlock(), base::TaskPriority::USER_BLOCKING},
+        base::BindOnce(OpenPrintersDialogCallback));
+  }
 }
 
 }  // namespace printing
