@@ -12,12 +12,6 @@ Polymer({
       notify: true,
     },
 
-    /** @type {!Array<!CupsPrinterInfo>} */
-    cupsPrinters: {
-      type: Array,
-      notify: true,
-    },
-
     searchTerm: {
       type: String,
     },
@@ -26,22 +20,30 @@ Polymer({
     focusConfig_: {
       type: Object,
       value: function() {
-        var map = new Map();
+        const map = new Map();
         if (settings.routes.CLOUD_PRINTERS) {
-          map.set(
-              settings.routes.CLOUD_PRINTERS.path,
-              '#cloudPrinters .subpage-arrow');
+          map.set(settings.routes.CLOUD_PRINTERS.path, '#cloudPrinters');
         }
         // <if expr="chromeos">
         if (settings.routes.CUPS_PRINTERS) {
-          map.set(
-              settings.routes.CUPS_PRINTERS.path,
-              '#cupsPrinters .subpage-arrow');
+          map.set(settings.routes.CUPS_PRINTERS.path, '#cupsPrinters');
         }
         // </if>
         return map;
       },
     },
+
+    // <if expr="chromeos">
+    /**
+     * TODO(crbug.com/950007): Remove when SplitSettings is the default because
+     * CUPS printers will exist only in the OS settings page.
+     * @private
+     */
+    showCupsPrinters_: {
+      type: Boolean,
+      value: () => loadTimeData.getBoolean('showOSSettings'),
+    }
+    // </if>
   },
 
   // <if expr="chromeos">
@@ -51,6 +53,12 @@ Polymer({
   },
   // </if>
 
+  // <if expr="not chromeos">
+  onTapLocalPrinters_: function() {
+    settings.PrintingBrowserProxyImpl.getInstance().openSystemPrintDialog();
+  },
+  // </if>
+
   /** @private */
   onTapCloudPrinters_: function() {
     settings.navigateTo(settings.routes.CLOUD_PRINTERS);
