@@ -15,18 +15,20 @@
 #include "base/memory/ptr_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_piece.h"
+#include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/browser/certificate_viewer.h"
 #include "chrome/browser/platform_util.h"
-#include "chrome/browser/ui/browser_dialogs.h"
 #include "chrome/browser/ui/certificate_dialogs.h"
 #include "chrome/browser/ui/webui/certificate_viewer_ui.h"
 #include "chrome/browser/ui/webui/constrained_web_dialog_ui.h"
-#include "chrome/common/net/x509_certificate_model.h"
+#include "chrome/common/net/x509_certificate_model_nss.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/grit/generated_resources.h"
+#include "content/public/browser/host_zoom_map.h"
 #include "content/public/browser/web_contents.h"
+#include "net/cert/x509_util_nss.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/gfx/geometry/size.h"
 
@@ -103,10 +105,10 @@ CertNodeBuilder& CertNodeBuilder::ChildIfNotNull(
 std::unique_ptr<base::DictionaryValue> CertNodeBuilder::Build() {
   DCHECK(!built_);
   if (!children_.empty()) {
-    node_.Set("children", base::MakeUnique<base::Value>(std::move(children_)));
+    node_.SetKey("children", std::move(children_));
   }
   built_ = true;
-  return base::MakeUnique<base::DictionaryValue>(std::move(node_));
+  return std::make_unique<base::DictionaryValue>(std::move(node_));
 }
 
 }  // namespace
@@ -115,102 +117,104 @@ std::unique_ptr<base::DictionaryValue> CertNodeBuilder::Build() {
 void ShowCertificateViewer(WebContents* web_contents,
                            gfx::NativeWindow parent,
                            net::X509Certificate* cert) {
-  CertificateViewerDialog* dialog = new CertificateViewerDialog(cert);
-  dialog->Show(web_contents, parent);
+  net::ScopedCERTCertificateList nss_certs =
+      net::x509_util::CreateCERTCertificateListFromX509Certificate(cert);
+  if (nss_certs.empty())
+    return;
+
+  CertificateViewerDialog::ShowConstrained(std::move(nss_certs), web_contents,
+                                           parent);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // CertificateViewerDialog
 
-CertificateViewerModalDialog::CertificateViewerModalDialog(
-    net::X509Certificate* cert)
-    : cert_(cert), webui_(NULL), window_(NULL) {
+// static
+CertificateViewerDialog* CertificateViewerDialog::ShowConstrained(
+    net::ScopedCERTCertificateList certs,
+    WebContents* web_contents,
+    gfx::NativeWindow parent) {
+  CertificateViewerDialog* dialog_ptr =
+      new CertificateViewerDialog(std::move(certs));
+  auto dialog = base::WrapUnique(dialog_ptr);
+
+  // TODO(bshe): UI tweaks needed for Aura HTML Dialog, such as adding padding
+  // on the title for Aura ConstrainedWebDialogUI.
+  dialog_ptr->delegate_ = ShowConstrainedWebDialog(
+      web_contents->GetBrowserContext(), std::move(dialog), web_contents);
+
+  // Clear the zoom level for the dialog so that it is not affected by the page
+  // zoom setting.
+  content::WebContents* dialog_web_contents =
+      dialog_ptr->delegate_->GetWebContents();
+  const GURL dialog_url = dialog_ptr->GetDialogContentURL();
+  content::HostZoomMap::Get(dialog_web_contents->GetSiteInstance())
+      ->SetZoomLevelForHostAndScheme(dialog_url.scheme(), dialog_url.host(), 0);
+  return dialog_ptr;  // For tests.
+}
+
+gfx::NativeWindow CertificateViewerDialog::GetNativeWebContentsModalDialog() {
+  return delegate_->GetNativeDialog();
+}
+
+CertificateViewerDialog::CertificateViewerDialog(
+    net::ScopedCERTCertificateList certs)
+    : nss_certs_(std::move(certs)) {
   // Construct the dialog title from the certificate.
   title_ = l10n_util::GetStringFUTF16(
       IDS_CERT_INFO_DIALOG_TITLE,
       base::UTF8ToUTF16(
-          x509_certificate_model::GetTitle(cert_->os_cert_handle())));
+          x509_certificate_model::GetTitle(nss_certs_.front().get())));
 }
 
-CertificateViewerModalDialog::~CertificateViewerModalDialog() {
+CertificateViewerDialog::~CertificateViewerDialog() = default;
+
+ui::ModalType CertificateViewerDialog::GetDialogModalType() const {
+  return ui::MODAL_TYPE_NONE;
 }
 
-void CertificateViewerModalDialog::Show(content::WebContents* web_contents,
-                                        gfx::NativeWindow parent) {
-  window_ = chrome::ShowWebDialog(parent,
-                                  web_contents->GetBrowserContext(),
-                                  this);
-}
-
-gfx::NativeWindow
-CertificateViewerModalDialog::GetNativeWebContentsModalDialog() {
-#if defined(USE_AURA)
-  return window_;
-#else
-  NOTREACHED();
-  return NULL;
-#endif
-}
-
-ui::ModalType CertificateViewerModalDialog::GetDialogModalType() const {
-  return ui::MODAL_TYPE_SYSTEM;
-}
-
-base::string16 CertificateViewerModalDialog::GetDialogTitle() const {
+base::string16 CertificateViewerDialog::GetDialogTitle() const {
   return title_;
 }
 
-GURL CertificateViewerModalDialog::GetDialogContentURL() const {
-  return GURL(chrome::kChromeUICertificateViewerDialogURL);
+GURL CertificateViewerDialog::GetDialogContentURL() const {
+  return GURL(chrome::kChromeUICertificateViewerURL);
 }
 
-void CertificateViewerModalDialog::GetWebUIMessageHandlers(
+void CertificateViewerDialog::GetWebUIMessageHandlers(
     std::vector<WebUIMessageHandler*>* handlers) const {
   handlers->push_back(new CertificateViewerDialogHandler(
-      const_cast<CertificateViewerModalDialog*>(this), cert_.get()));
+      const_cast<CertificateViewerDialog*>(this),
+      net::x509_util::DupCERTCertificateList(nss_certs_)));
 }
 
-void CertificateViewerModalDialog::GetDialogSize(gfx::Size* size) const {
+void CertificateViewerDialog::GetDialogSize(gfx::Size* size) const {
   const int kDefaultWidth = 544;
   const int kDefaultHeight = 628;
   size->SetSize(kDefaultWidth, kDefaultHeight);
 }
 
-std::string CertificateViewerModalDialog::GetDialogArgs() const {
+std::string CertificateViewerDialog::GetDialogArgs() const {
   std::string data;
 
   // Certificate information. The keys in this dictionary's general key
   // correspond to the IDs in the Html page.
   base::DictionaryValue cert_info;
-  net::X509Certificate::OSCertHandle cert_hnd = cert_->os_cert_handle();
-
-  // Get the certificate chain.
-  net::X509Certificate::OSCertHandles cert_chain;
-  cert_chain.push_back(cert_->os_cert_handle());
-  const net::X509Certificate::OSCertHandles& certs =
-      cert_->GetIntermediateCertificates();
-  cert_chain.insert(cert_chain.end(), certs.begin(), certs.end());
+  CERTCertificate* cert_hnd = nss_certs_.front().get();
 
   // Certificate usage.
   std::vector<std::string> usages;
   x509_certificate_model::GetUsageStrings(cert_hnd, &usages);
-  std::string usagestr;
-  for (std::vector<std::string>::iterator it = usages.begin();
-      it != usages.end(); ++it) {
-    if (usagestr.length() > 0) {
-      usagestr += "\n";
-    }
-    usagestr += *it;
-  }
-  cert_info.SetString("general.usages", usagestr);
+  cert_info.SetString("general.usages", base::JoinString(usages, "\n"));
 
   // Standard certificate details.
   const std::string alternative_text =
       l10n_util::GetStringUTF8(IDS_CERT_INFO_FIELD_NOT_PRESENT);
-  cert_info.SetString("general.title", l10n_util::GetStringFUTF8(
-      IDS_CERT_INFO_DIALOG_TITLE,
-      base::UTF8ToUTF16(x509_certificate_model::GetTitle(
-          cert_chain.front()))));
+  cert_info.SetString(
+      "general.title",
+      l10n_util::GetStringFUTF8(
+          IDS_CERT_INFO_DIALOG_TITLE,
+          base::UTF8ToUTF16(x509_certificate_model::GetTitle(cert_hnd))));
 
   // Issued to information.
   cert_info.SetString("general.issued-cn",
@@ -250,104 +254,70 @@ std::string CertificateViewerModalDialog::GetDialogArgs() const {
       x509_certificate_model::HashCertSHA1(cert_hnd));
 
   // Certificate hierarchy is constructed from bottom up.
-  std::unique_ptr<base::ListValue> children;
+  base::Value children;
   int index = 0;
-  for (net::X509Certificate::OSCertHandles::const_iterator i =
-      cert_chain.begin(); i != cert_chain.end(); ++i, ++index) {
-    std::unique_ptr<base::DictionaryValue> cert_node(
-        new base::DictionaryValue());
-    base::ListValue cert_details;
-    cert_node->SetString("label", x509_certificate_model::GetTitle(*i).c_str());
-    cert_node->SetDouble("payload.index", index);
+  for (const auto& cert : nss_certs_) {
+    base::Value cert_node(base::Value::Type::DICTIONARY);
+    cert_node.SetKey("label",
+                     base::Value(x509_certificate_model::GetTitle(cert.get())));
+    cert_node.SetPath({"payload", "index"}, base::Value(index));
     // Add the child from the previous iteration.
-    if (children)
-      cert_node->Set("children", std::move(children));
+    if (!children.is_none())
+      cert_node.SetKey("children", std::move(children));
 
     // Add this node to the children list for the next iteration.
-    children = base::MakeUnique<base::ListValue>();
-    children->Append(std::move(cert_node));
+    children = base::Value(base::Value::Type::LIST);
+    children.GetList().push_back(std::move(cert_node));
+    ++index;
   }
   // Set the last node as the top of the certificate hierarchy.
-  cert_info.Set("hierarchy", std::move(children));
+  cert_info.SetKey("hierarchy", std::move(children));
 
   base::JSONWriter::Write(cert_info, &data);
 
   return data;
 }
 
-void CertificateViewerModalDialog::OnDialogShown(
+void CertificateViewerDialog::OnDialogShown(
     content::WebUI* webui,
     content::RenderViewHost* render_view_host) {
   webui_ = webui;
 }
 
-void CertificateViewerModalDialog::OnDialogClosed(
-    const std::string& json_retval) {
+void CertificateViewerDialog::OnDialogClosed(const std::string& json_retval) {
+  // Don't |delete this|: owned by the constrained dialog manager.
 }
 
-void CertificateViewerModalDialog::OnCloseContents(WebContents* source,
+void CertificateViewerDialog::OnCloseContents(WebContents* source,
                                               bool* out_close_dialog) {
   *out_close_dialog = true;
 }
 
-bool CertificateViewerModalDialog::ShouldShowDialogTitle() const {
+bool CertificateViewerDialog::ShouldShowDialogTitle() const {
   return true;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// CertificateViewerDialog
-
-CertificateViewerDialog::CertificateViewerDialog(net::X509Certificate* cert)
-    : CertificateViewerModalDialog(cert),
-      dialog_(NULL) {
-}
-
-CertificateViewerDialog::~CertificateViewerDialog() {
-}
-
-void CertificateViewerDialog::Show(WebContents* web_contents,
-                                   gfx::NativeWindow parent) {
-  // TODO(bshe): UI tweaks needed for Aura HTML Dialog, such as adding padding
-  // on the title for Aura ConstrainedWebDialogUI.
-  dialog_ = ShowConstrainedWebDialog(web_contents->GetBrowserContext(), this,
-                                     web_contents);
-}
-
-gfx::NativeWindow CertificateViewerDialog::GetNativeWebContentsModalDialog() {
-  return dialog_->GetNativeDialog();
-}
-
-GURL CertificateViewerDialog::GetDialogContentURL() const {
-  return GURL(chrome::kChromeUICertificateViewerURL);
-}
-
-ui::ModalType CertificateViewerDialog::GetDialogModalType() const {
-  return ui::MODAL_TYPE_NONE;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // CertificateViewerDialogHandler
 
 CertificateViewerDialogHandler::CertificateViewerDialogHandler(
-    CertificateViewerModalDialog* dialog,
-    net::X509Certificate* cert)
-    : cert_(cert), dialog_(dialog) {
-  cert_chain_.push_back(cert_->os_cert_handle());
-  const net::X509Certificate::OSCertHandles& certs =
-      cert_->GetIntermediateCertificates();
-  cert_chain_.insert(cert_chain_.end(), certs.begin(), certs.end());
-}
+    CertificateViewerDialog* dialog,
+    net::ScopedCERTCertificateList cert_chain)
+    : dialog_(dialog), cert_chain_(std::move(cert_chain)) {}
 
 CertificateViewerDialogHandler::~CertificateViewerDialogHandler() {
 }
 
 void CertificateViewerDialogHandler::RegisterMessages() {
-  web_ui()->RegisterMessageCallback("exportCertificate",
-      base::Bind(&CertificateViewerDialogHandler::ExportCertificate,
-                 base::Unretained(this)));
-  web_ui()->RegisterMessageCallback("requestCertificateFields",
-      base::Bind(&CertificateViewerDialogHandler::RequestCertificateFields,
-                 base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      "exportCertificate",
+      base::BindRepeating(&CertificateViewerDialogHandler::ExportCertificate,
+                          base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      "requestCertificateFields",
+      base::BindRepeating(
+          &CertificateViewerDialogHandler::RequestCertificateFields,
+          base::Unretained(this)));
 }
 
 void CertificateViewerDialogHandler::ExportCertificate(
@@ -370,7 +340,7 @@ void CertificateViewerDialogHandler::RequestCertificateFields(
   if (cert_index < 0)
     return;
 
-  net::X509Certificate::OSCertHandle cert = cert_chain_[cert_index];
+  CERTCertificate* cert = cert_chain_[cert_index].get();
 
   CertNodeBuilder version_node(IDS_CERT_DETAILS_VERSION);
   std::string version = x509_certificate_model::GetVersion(cert);
