@@ -6,54 +6,63 @@
 
 #include <type_traits>
 
-#include "ash/accessibility_types.h"
+#include "ash/public/cpp/ash_features.h"
+#include "ash/public/cpp/event_rewriter_controller.h"
+#include "ash/public/interfaces/constants.mojom.h"
 #include "ash/shell.h"
 #include "base/bind.h"
+#include "base/command_line.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/chromeos/accessibility/accessibility_manager.h"
 #include "chrome/browser/chromeos/accessibility/magnification_manager.h"
-#include "chrome/browser/chromeos/events/keyboard_driven_event_rewriter.h"
-#include "chrome/browser/chromeos/login/enrollment/auto_enrollment_controller.h"
+#include "chrome/browser/chromeos/login/configuration_keys.h"
+#include "chrome/browser/chromeos/login/demo_mode/demo_session.h"
+#include "chrome/browser/chromeos/login/demo_mode/demo_setup_controller.h"
 #include "chrome/browser/chromeos/login/helper.h"
 #include "chrome/browser/chromeos/login/lock/screen_locker.h"
-#include "chrome/browser/chromeos/login/lock/webui_screen_locker.h"
+#include "chrome/browser/chromeos/login/screens/reset_screen.h"
 #include "chrome/browser/chromeos/login/ui/login_display_host.h"
 #include "chrome/browser/chromeos/login/wizard_controller.h"
 #include "chrome/browser/chromeos/policy/browser_policy_connector_chromeos.h"
 #include "chrome/browser/chromeos/policy/device_cloud_policy_manager_chromeos.h"
 #include "chrome/browser/chromeos/system/input_device_settings.h"
-#include "chrome/browser/chromeos/tpm_firmware_update.h"
+#include "chrome/browser/chromeos/system/timezone_resolver_manager.h"
 #include "chrome/browser/lifetime/application_lifetime.h"
 #include "chrome/browser/ui/ash/ash_util.h"
+#include "chrome/browser/ui/ash/keyboard/chrome_keyboard_controller_client.h"
+#include "chrome/browser/ui/ash/tablet_mode_client.h"
+#include "chrome/browser/ui/webui/chromeos/login/demo_setup_screen_handler.h"
+#include "chrome/browser/ui/webui/chromeos/login/eula_screen_handler.h"
 #include "chrome/browser/ui/webui/chromeos/login/oobe_ui.h"
+#include "chrome/browser/ui/webui/chromeos/login/reset_screen_handler.h"
 #include "chrome/browser/ui/webui/chromeos/login/signin_screen_handler.h"
 #include "chrome/common/channel_info.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/grit/chromium_strings.h"
 #include "chrome/grit/generated_resources.h"
-#include "chromeos/chromeos_constants.h"
+#include "chromeos/constants/chromeos_constants.h"
+#include "chromeos/constants/chromeos_switches.h"
 #include "components/login/base_screen_handler_utils.h"
 #include "components/login/localized_values_builder.h"
 #include "components/prefs/pref_service.h"
 #include "components/strings/grit/components_strings.h"
 #include "components/version_info/version_info.h"
+#include "content/public/browser/system_connector.h"
 #include "google_apis/google_api_keys.h"
+#include "services/service_manager/public/cpp/connector.h"
+#include "ui/accessibility/accessibility_switches.h"
 #include "ui/aura/window_tree_host.h"
-#include "ui/display/display.h"
 #include "ui/display/screen.h"
 #include "ui/events/event_sink.h"
 #include "ui/gfx/geometry/size.h"
-#include "ui/keyboard/keyboard_controller.h"
 
 namespace chromeos {
 
 namespace {
 
-const char kJsScreenPath[] = "cr.ui.Oobe";
-
 bool IsRemoraRequisition() {
   policy::DeviceCloudPolicyManagerChromeOS* policy_manager =
       g_browser_process->platform_part()
@@ -67,11 +76,10 @@ void LaunchResetScreen() {
   WizardController* const wizard_controller =
       WizardController::default_controller();
   if (wizard_controller && !wizard_controller->login_screen_started()) {
-    wizard_controller->AdvanceToScreen(OobeScreen::SCREEN_OOBE_RESET);
+    wizard_controller->AdvanceToScreen(ResetView::kScreenId);
   } else {
     DCHECK(LoginDisplayHost::default_host());
-    LoginDisplayHost::default_host()->StartWizard(
-        OobeScreen::SCREEN_OOBE_RESET);
+    LoginDisplayHost::default_host()->StartWizard(ResetView::kScreenId);
   }
 }
 
@@ -79,25 +87,32 @@ void LaunchResetScreen() {
 
 // Note that show_oobe_ui_ defaults to false because WizardController assumes
 // OOBE UI is not visible by default.
-CoreOobeHandler::CoreOobeHandler(OobeUI* oobe_ui,
-                                 JSCallsContainer* js_calls_container)
+CoreOobeHandler::CoreOobeHandler(JSCallsContainer* js_calls_container)
     : BaseWebUIHandler(js_calls_container),
-      oobe_ui_(oobe_ui),
-      version_info_updater_(this) {
+      version_info_updater_(this),
+      weak_ptr_factory_(this) {
   DCHECK(js_calls_container);
-  set_call_js_prefix(kJsScreenPath);
-  if (!ash_util::IsRunningInMash()) {
-    AccessibilityManager* accessibility_manager = AccessibilityManager::Get();
-    CHECK(accessibility_manager);
-    accessibility_subscription_ = accessibility_manager->RegisterCallback(
-        base::Bind(&CoreOobeHandler::OnAccessibilityStatusChanged,
-                   base::Unretained(this)));
-  } else {
-    NOTIMPLEMENTED();
+  AccessibilityManager* accessibility_manager = AccessibilityManager::Get();
+  CHECK(accessibility_manager);
+  accessibility_subscription_ = accessibility_manager->RegisterCallback(
+      base::Bind(&CoreOobeHandler::OnAccessibilityStatusChanged,
+                 base::Unretained(this)));
+
+  TabletModeClient* tablet_mode_client = TabletModeClient::Get();
+  tablet_mode_client->AddObserver(this);
+
+  // |connector| may be null in tests.
+  auto* connector = content::GetSystemConnector();
+  if (connector) {
+    connector->BindInterface(ash::mojom::kServiceName,
+                             &cros_display_config_ptr_);
   }
+  OobeConfiguration::Get()->AddAndFireObserver(this);
 }
 
 CoreOobeHandler::~CoreOobeHandler() {
+  OobeConfiguration::Get()->RemoveObserver(this);
+  TabletModeClient::Get()->RemoveObserver(this);
 }
 
 void CoreOobeHandler::DeclareLocalizedValues(
@@ -109,9 +124,11 @@ void CoreOobeHandler::DeclareLocalizedValues(
   // OOBE accessibility options menu strings shown on each screen.
   builder->Add("accessibilityLink", IDS_OOBE_ACCESSIBILITY_LINK);
   builder->Add("spokenFeedbackOption", IDS_OOBE_SPOKEN_FEEDBACK_OPTION);
+  builder->Add("selectToSpeakOption", IDS_OOBE_SELECT_TO_SPEAK_OPTION);
   builder->Add("largeCursorOption", IDS_OOBE_LARGE_CURSOR_OPTION);
   builder->Add("highContrastOption", IDS_OOBE_HIGH_CONTRAST_MODE_OPTION);
   builder->Add("screenMagnifierOption", IDS_OOBE_SCREEN_MAGNIFIER_OPTION);
+  builder->Add("dockedMagnifierOption", IDS_OOBE_DOCKED_MAGNIFIER_OPTION);
   builder->Add("virtualKeyboardOption", IDS_OOBE_VIRTUAL_KEYBOARD_OPTION);
   builder->Add("closeAccessibilityMenu", IDS_OOBE_CLOSE_ACCESSIBILITY_MENU);
 
@@ -131,6 +148,14 @@ void CoreOobeHandler::DeclareLocalizedValues(
   builder->Add("deviceRequisitionSharkPromptText",
                IDS_ENTERPRISE_DEVICE_REQUISITION_SHARK_PROMPT_TEXT);
 
+  // Strings for enable demo mode dialog.
+  builder->Add("enableDemoModeDialogTitle", IDS_ENABLE_DEMO_MODE_DIALOG_TITLE);
+  builder->Add("enableDemoModeDialogText", IDS_ENABLE_DEMO_MODE_DIALOG_TEXT);
+  builder->Add("enableDemoModeDialogConfirm",
+               IDS_ENABLE_DEMO_MODE_DIALOG_CONFIRM);
+  builder->Add("enableDemoModeDialogCancel",
+               IDS_ENABLE_DEMO_MODE_DIALOG_CANCEL);
+
   // Strings for Asset Identifier shown in version string.
   builder->Add("assetIdLabel", IDS_OOBE_ASSET_ID_LABEL);
 
@@ -151,39 +176,52 @@ void CoreOobeHandler::Initialize() {
   UpdateClientAreaSize();
 }
 
+void CoreOobeHandler::GetAdditionalParameters(base::DictionaryValue* dict) {
+  dict->SetKey("isInTabletMode",
+               base::Value(TabletModeClient::Get()->tablet_mode_enabled()));
+  dict->SetKey("isDemoModeEnabled",
+               base::Value(DemoSetupController::IsDemoModeAllowed()));
+  dict->SetKey("showTechnologyBadge",
+               base::Value(!ash::features::IsSeparateNetworkIconsEnabled()));
+}
+
 void CoreOobeHandler::RegisterMessages() {
-  AddCallback("screenStateInitialize",
-              &CoreOobeHandler::HandleInitialized);
+  AddCallback("screenStateInitialize", &CoreOobeHandler::HandleInitialized);
   AddCallback("skipUpdateEnrollAfterEula",
               &CoreOobeHandler::HandleSkipUpdateEnrollAfterEula);
   AddCallback("updateCurrentScreen",
               &CoreOobeHandler::HandleUpdateCurrentScreen);
-  AddCallback("enableHighContrast",
-              &CoreOobeHandler::HandleEnableHighContrast);
-  AddCallback("enableLargeCursor",
-              &CoreOobeHandler::HandleEnableLargeCursor);
+  AddCallback("enableHighContrast", &CoreOobeHandler::HandleEnableHighContrast);
+  AddCallback("enableLargeCursor", &CoreOobeHandler::HandleEnableLargeCursor);
   AddCallback("enableVirtualKeyboard",
               &CoreOobeHandler::HandleEnableVirtualKeyboard);
   AddCallback("enableScreenMagnifier",
               &CoreOobeHandler::HandleEnableScreenMagnifier);
   AddCallback("enableSpokenFeedback",
               &CoreOobeHandler::HandleEnableSpokenFeedback);
+  AddCallback("enableSelectToSpeak",
+              &CoreOobeHandler::HandleEnableSelectToSpeak);
+  AddCallback("enableDockedMagnifier",
+              &CoreOobeHandler::HandleEnableDockedMagnifier);
   AddCallback("setDeviceRequisition",
               &CoreOobeHandler::HandleSetDeviceRequisition);
-  AddCallback("screenAssetsLoaded",
-              &CoreOobeHandler::HandleScreenAssetsLoaded);
   AddRawCallback("skipToLoginForTesting",
                  &CoreOobeHandler::HandleSkipToLoginForTesting);
-  AddCallback("launchHelpApp",
-              &CoreOobeHandler::HandleLaunchHelpApp);
+  AddCallback("skipToUpdateForTesting",
+              &CoreOobeHandler::HandleSkipToUpdateForTesting);
+  AddCallback("launchHelpApp", &CoreOobeHandler::HandleLaunchHelpApp);
   AddCallback("toggleResetScreen", &CoreOobeHandler::HandleToggleResetScreen);
   AddCallback("toggleEnableDebuggingScreen",
               &CoreOobeHandler::HandleEnableDebuggingScreen);
-  AddCallback("headerBarVisible",
-              &CoreOobeHandler::HandleHeaderBarVisible);
   AddCallback("raiseTabKeyEvent", &CoreOobeHandler::HandleRaiseTabKeyEvent);
-  AddCallback("setOobeBootstrappingSlave",
-              &CoreOobeHandler::HandleSetOobeBootstrappingSlave);
+  // Note: Used by enterprise_RemoraRequisitionDisplayUsage.py:
+  // TODO(felixe): Use chrome.system.display or cros_display_config.mojom,
+  // https://crbug.com/858958.
+  AddRawCallback("getPrimaryDisplayNameForTesting",
+                 &CoreOobeHandler::HandleGetPrimaryDisplayNameForTesting);
+  AddCallback("setupDemoMode", &CoreOobeHandler::HandleSetupDemoMode);
+  AddCallback("startDemoModeSetupForTesting",
+              &CoreOobeHandler::HandleStartDemoModeSetupForTesting);
 }
 
 void CoreOobeHandler::ShowSignInError(
@@ -192,121 +230,84 @@ void CoreOobeHandler::ShowSignInError(
     const std::string& help_link_text,
     HelpAppLauncher::HelpTopic help_topic_id) {
   LOG(ERROR) << "CoreOobeHandler::ShowSignInError: error_text=" << error_text;
-  CallJSOrDefer("showSignInError", login_attempts, error_text,
+  CallJS("cr.ui.Oobe.showSignInError", login_attempts, error_text,
          help_link_text, static_cast<int>(help_topic_id));
 }
 
 void CoreOobeHandler::ShowTpmError() {
-  CallJSOrDefer("showTpmError");
+  CallJS("cr.ui.Oobe.showTpmError");
 }
 
 void CoreOobeHandler::ShowDeviceResetScreen() {
-  // Powerwash is generally not available on enterprise devices. First, check
-  // the common case of a correctly enrolled device.
-  if (g_browser_process->platform_part()
-          ->browser_policy_connector_chromeos()
-          ->IsEnterpriseManaged()) {
-    // Powerwash not allowed, except if allowed by the admin specifically for
-    // the purpose of installing a TPM firmware update.
-    tpm_firmware_update::ShouldOfferUpdateViaPowerwash(
-        base::Bind([](bool offer_update) {
-          if (offer_update) {
-            // Force the TPM firmware update option to be enabled.
-            g_browser_process->local_state()->SetBoolean(
-                prefs::kFactoryResetTPMFirmwareUpdateRequested, true);
-            LaunchResetScreen();
-          }
-        }));
-    return;
-  }
-
-  // Devices that are still in OOBE may be subject to forced re-enrollment (FRE)
-  // and thus pending for enterprise management. These should not be allowed to
-  // powerwash either. Note that taking consumer device ownership has the side
-  // effect of dropping the FRE requirement if it was previously in effect.
-  const AutoEnrollmentController::FRERequirement requirement =
-      AutoEnrollmentController::GetFRERequirement();
-  if (requirement != AutoEnrollmentController::EXPLICITLY_REQUIRED) {
-    LaunchResetScreen();
-  }
+  LaunchResetScreen();
 }
 
 void CoreOobeHandler::ShowEnableDebuggingScreen() {
   // Don't recreate WizardController if it already exists.
-  WizardController* wizard_controller =
-      WizardController::default_controller();
+  WizardController* wizard_controller = WizardController::default_controller();
   if (wizard_controller && !wizard_controller->login_screen_started()) {
-    wizard_controller->AdvanceToScreen(
-        OobeScreen::SCREEN_OOBE_ENABLE_DEBUGGING);
+    wizard_controller->AdvanceToScreen(EnableDebuggingScreenView::kScreenId);
   }
 }
 
 void CoreOobeHandler::ShowActiveDirectoryPasswordChangeScreen(
     const std::string& username) {
-  CallJSOrDefer("showActiveDirectoryPasswordChangeScreen", username);
+  CallJS("cr.ui.Oobe.showActiveDirectoryPasswordChangeScreen", username);
 }
 
 void CoreOobeHandler::ShowSignInUI(const std::string& email) {
-  CallJSOrDefer("showSigninUI", email);
+  CallJS("cr.ui.Oobe.showSigninUI", email);
 }
 
 void CoreOobeHandler::ResetSignInUI(bool force_online) {
-  CallJSOrDefer("resetSigninUI", force_online);
+  CallJS("cr.ui.Oobe.resetSigninUI", force_online);
 }
 
 void CoreOobeHandler::ClearUserPodPassword() {
-  CallJSOrDefer("clearUserPodPassword");
+  CallJS("cr.ui.Oobe.clearUserPodPassword");
 }
 
 void CoreOobeHandler::RefocusCurrentPod() {
-  CallJSOrDefer("refocusCurrentPod");
+  CallJS("cr.ui.Oobe.refocusCurrentPod");
 }
 
 void CoreOobeHandler::ShowPasswordChangedScreen(bool show_password_error,
                                                 const std::string& email) {
-  CallJSOrDefer("showPasswordChangedScreen", show_password_error, email);
+  CallJS("cr.ui.Oobe.showPasswordChangedScreen", show_password_error, email);
 }
 
 void CoreOobeHandler::SetUsageStats(bool checked) {
-  CallJSOrDefer("setUsageStats", checked);
-}
-
-void CoreOobeHandler::SetOemEulaUrl(const std::string& oem_eula_url) {
-  CallJSOrDefer("setOemEulaUrl", oem_eula_url);
+  CallJS("cr.ui.Oobe.setUsageStats", checked);
 }
 
 void CoreOobeHandler::SetTpmPassword(const std::string& tpm_password) {
-  CallJSOrDefer("setTpmPassword", tpm_password);
+  CallJS("cr.ui.Oobe.setTpmPassword", tpm_password);
 }
 
 void CoreOobeHandler::ClearErrors() {
-  CallJSOrDefer("clearErrors");
+  CallJS("cr.ui.Oobe.clearErrors");
 }
 
 void CoreOobeHandler::ReloadContent(const base::DictionaryValue& dictionary) {
-  CallJSOrDefer("reloadContent", dictionary);
+  CallJS("cr.ui.Oobe.reloadContent", dictionary);
 }
 
 void CoreOobeHandler::ReloadEulaContent(
     const base::DictionaryValue& dictionary) {
-  CallJSOrDefer("reloadEulaContent", dictionary);
-}
-
-void CoreOobeHandler::ShowControlBar(bool show) {
-  CallJSOrDefer("showControlBar", show);
+  CallJS("cr.ui.Oobe.reloadEulaContent", dictionary);
 }
 
 void CoreOobeHandler::SetVirtualKeyboardShown(bool shown) {
-  CallJSOrDefer("setVirtualKeyboardShown", shown);
+  CallJS("cr.ui.Oobe.setVirtualKeyboardShown", shown);
 }
 
 void CoreOobeHandler::SetClientAreaSize(int width, int height) {
-  CallJSOrDefer("setClientAreaSize", width, height);
+  CallJS("cr.ui.Oobe.setClientAreaSize", width, height);
 }
 
 void CoreOobeHandler::HandleInitialized() {
-  ExecuteDeferredJSCalls();
-  oobe_ui_->InitializeHandlers();
+  GetOobeUI()->InitializeHandlers();
+  AllowJavascript();
 }
 
 void CoreOobeHandler::HandleSkipUpdateEnrollAfterEula() {
@@ -318,13 +319,10 @@ void CoreOobeHandler::HandleSkipUpdateEnrollAfterEula() {
 
 void CoreOobeHandler::HandleUpdateCurrentScreen(
     const std::string& screen_name) {
-  const OobeScreen screen = GetOobeScreenFromName(screen_name);
-  oobe_ui_->CurrentScreenChanged(screen);
-  // TODO(mash): Support EventRewriterController; see crbug.com/647781
-  if (!ash_util::IsRunningInMash()) {
-    KeyboardDrivenEventRewriter::GetInstance()->SetArrowToTabRewritingEnabled(
-        screen == OobeScreen::SCREEN_OOBE_EULA);
-  }
+  const OobeScreenId screen(screen_name);
+  GetOobeUI()->CurrentScreenChanged(screen);
+  ash::EventRewriterController::Get()->SetArrowToTabRewritingEnabled(
+      screen == EulaView::kScreenId);
 }
 
 void CoreOobeHandler::HandleEnableHighContrast(bool enabled) {
@@ -340,7 +338,6 @@ void CoreOobeHandler::HandleEnableVirtualKeyboard(bool enabled) {
 }
 
 void CoreOobeHandler::HandleEnableScreenMagnifier(bool enabled) {
-  // TODO(nkostylev): Add support for partial screen magnifier.
   DCHECK(MagnificationManager::Get());
   MagnificationManager::Get()->SetMagnifierEnabled(enabled);
 }
@@ -348,8 +345,22 @@ void CoreOobeHandler::HandleEnableScreenMagnifier(bool enabled) {
 void CoreOobeHandler::HandleEnableSpokenFeedback(bool /* enabled */) {
   // Checkbox is initialized on page init and updates when spoken feedback
   // setting is changed so just toggle spoken feedback here.
-  AccessibilityManager::Get()->ToggleSpokenFeedback(
-      ash::A11Y_NOTIFICATION_NONE);
+  AccessibilityManager::Get()->EnableSpokenFeedback(
+      !AccessibilityManager::Get()->IsSpokenFeedbackEnabled());
+}
+
+void CoreOobeHandler::HandleEnableSelectToSpeak(bool /* enabled */) {
+  // Checkbox is initialized on page init and updates when Select to Speak
+  // setting is changed so just toggle Select to Speak here.
+  AccessibilityManager::Get()->SetSelectToSpeakEnabled(
+      !AccessibilityManager::Get()->IsSelectToSpeakEnabled());
+}
+
+void CoreOobeHandler::HandleEnableDockedMagnifier(bool enabled) {
+  // Checkbox is initialized on page init and updates when the docked magnifier
+  // setting is changed so just toggle Select to Speak here.
+  DCHECK(MagnificationManager::Get());
+  MagnificationManager::Get()->SetDockedMagnifierEnabled(enabled);
 }
 
 void CoreOobeHandler::HandleSetDeviceRequisition(
@@ -362,31 +373,58 @@ void CoreOobeHandler::HandleSetDeviceRequisition(
 
   if (IsRemoraRequisition()) {
     // CfM devices default to static timezone.
-    g_browser_process->local_state()->SetBoolean(
-        prefs::kResolveDeviceTimezoneByGeolocation, false);
+    g_browser_process->local_state()->SetInteger(
+        prefs::kResolveDeviceTimezoneByGeolocationMethod,
+        static_cast<int>(chromeos::system::TimeZoneResolverManager::
+                             TimeZoneResolveMethod::DISABLED));
   }
 
   // Exit Chrome to force the restart as soon as a new requisition is set.
   if (initial_requisition !=
-          connector->GetDeviceCloudPolicyManager()->GetDeviceRequisition()) {
+      connector->GetDeviceCloudPolicyManager()->GetDeviceRequisition()) {
     chrome::AttemptRestart();
   }
 }
 
-void CoreOobeHandler::HandleScreenAssetsLoaded(
-    const std::string& screen_async_load_id) {
-  oobe_ui_->OnScreenAssetsLoaded(screen_async_load_id);
+void CoreOobeHandler::HandleSkipToLoginForTesting(const base::ListValue* args) {
+  LoginScreenContext context;
+
+  // Parse |args|.
+  if (args->GetSize() > 0) {
+    std::string email;
+    if (args->GetString(0, &email))
+      context.set_email(email);
+  }
+
+  if (WizardController::default_controller())
+    WizardController::default_controller()->SkipToLoginForTesting(context);
 }
 
-void CoreOobeHandler::HandleSkipToLoginForTesting(
-    const base::ListValue* args) {
-  LoginScreenContext context(args);
+void CoreOobeHandler::HandleSkipToUpdateForTesting() {
   if (WizardController::default_controller())
-      WizardController::default_controller()->SkipToLoginForTesting(context);
+    WizardController::default_controller()->SkipToUpdateForTesting();
 }
 
 void CoreOobeHandler::HandleToggleResetScreen() {
-  ShowDeviceResetScreen();
+  base::OnceCallback<void(bool, base::Optional<tpm_firmware_update::Mode>)>
+      callback =
+          base::BindOnce(&CoreOobeHandler::HandleToggleResetScreenCallback,
+                         weak_ptr_factory_.GetWeakPtr());
+  ResetScreen::CheckIfPowerwashAllowed(std::move(callback));
+}
+
+void CoreOobeHandler::HandleToggleResetScreenCallback(
+    bool is_reset_allowed,
+    base::Optional<tpm_firmware_update::Mode> tpm_firmware_update_mode) {
+  if (!is_reset_allowed)
+    return;
+  if (tpm_firmware_update_mode.has_value()) {
+    // Force the TPM firmware update option to be enabled.
+    g_browser_process->local_state()->SetInteger(
+        prefs::kFactoryResetTPMFirmwareUpdateMode,
+        static_cast<int>(tpm_firmware_update_mode.value()));
+  }
+  LaunchResetScreen();
 }
 
 void CoreOobeHandler::HandleEnableDebuggingScreen() {
@@ -403,18 +441,15 @@ void CoreOobeHandler::ShowOobeUI(bool show) {
     UpdateOobeUIVisibility();
 }
 
-void CoreOobeHandler::UpdateShutdownAndRebootVisibility(
-    bool reboot_on_shutdown) {
-  CallJSOrDefer("showShutdown", !reboot_on_shutdown);
+void CoreOobeHandler::SetLoginUserCount(int user_count) {
+  CallJS("cr.ui.Oobe.setLoginUserCount", user_count);
+}
+
+void CoreOobeHandler::ForwardAccelerator(std::string accelerator_name) {
+  CallJS("cr.ui.Oobe.handleAccelerator", accelerator_name);
 }
 
 void CoreOobeHandler::UpdateA11yState() {
-  if (ash_util::IsRunningInMash()) {
-    NOTIMPLEMENTED();
-    return;
-  }
-  // TODO(dpolukhin): crbug.com/412891
-  DCHECK(MagnificationManager::Get());
   base::DictionaryValue a11y_info;
   a11y_info.SetBoolean("highContrastEnabled",
                        AccessibilityManager::Get()->IsHighContrastEnabled());
@@ -422,18 +457,29 @@ void CoreOobeHandler::UpdateA11yState() {
                        AccessibilityManager::Get()->IsLargeCursorEnabled());
   a11y_info.SetBoolean("spokenFeedbackEnabled",
                        AccessibilityManager::Get()->IsSpokenFeedbackEnabled());
+  a11y_info.SetBoolean("selectToSpeakEnabled",
+                       AccessibilityManager::Get()->IsSelectToSpeakEnabled());
+  a11y_info.SetBoolean(
+      "enableExperimentalA11yFeatures",
+      base::CommandLine::ForCurrentProcess()->HasSwitch(
+          ::switches::kEnableExperimentalAccessibilityFeatures));
+  DCHECK(MagnificationManager::Get());
   a11y_info.SetBoolean("screenMagnifierEnabled",
                        MagnificationManager::Get()->IsMagnifierEnabled());
+  a11y_info.SetBoolean("dockedMagnifierEnabled",
+                       MagnificationManager::Get()->IsDockedMagnifierEnabled());
   a11y_info.SetBoolean("virtualKeyboardEnabled",
                        AccessibilityManager::Get()->IsVirtualKeyboardEnabled());
-  CallJSOrDefer("refreshA11yInfo", a11y_info);
+  CallJS("cr.ui.Oobe.refreshA11yInfo", a11y_info);
 }
 
 void CoreOobeHandler::UpdateOobeUIVisibility() {
-  const std::string& display = oobe_ui_->display_type();
-  CallJSOrDefer("showAPIKeysNotice", !google_apis::HasKeysConfigured() &&
-                                  (display == OobeUI::kOobeDisplay ||
-                                   display == OobeUI::kLoginDisplay));
+  const std::string& display = GetOobeUI()->display_type();
+  bool has_api_keys_configured = google_apis::HasAPIKeyConfigured() &&
+                                 google_apis::HasOAuthClientConfigured();
+  CallJS("cr.ui.Oobe.showAPIKeysNotice",
+         !has_api_keys_configured && (display == OobeUI::kOobeDisplay ||
+                                      display == OobeUI::kLoginDisplay));
 
   // Don't show version label on the stable channel by default.
   bool should_show_version = true;
@@ -442,10 +488,10 @@ void CoreOobeHandler::UpdateOobeUIVisibility() {
       channel == version_info::Channel::BETA) {
     should_show_version = false;
   }
-  CallJSOrDefer("showVersion", should_show_version);
-  CallJSOrDefer("showOobeUI", show_oobe_ui_);
+  CallJS("cr.ui.Oobe.showVersion", should_show_version);
+  CallJS("cr.ui.Oobe.showOobeUI", show_oobe_ui_);
   if (system::InputDeviceSettings::Get()->ForceKeyboardDrivenUINavigation())
-    CallJSOrDefer("enableKeyboardFlow", true);
+    CallJS("cr.ui.Oobe.enableKeyboardFlow", true);
 }
 
 void CoreOobeHandler::OnOSVersionLabelTextUpdated(
@@ -453,13 +499,13 @@ void CoreOobeHandler::OnOSVersionLabelTextUpdated(
   UpdateLabel("version", os_version_label_text);
 }
 
-void CoreOobeHandler::OnEnterpriseInfoUpdated(
-    const std::string& message_text, const std::string& asset_id) {
-  CallJSOrDefer("setEnterpriseInfo", message_text, asset_id);
+void CoreOobeHandler::OnEnterpriseInfoUpdated(const std::string& message_text,
+                                              const std::string& asset_id) {
+  CallJS("cr.ui.Oobe.setEnterpriseInfo", message_text, asset_id);
 }
 
 void CoreOobeHandler::OnDeviceInfoUpdated(const std::string& bluetooth_name) {
-  CallJSOrDefer("setBluetoothDeviceInfo", bluetooth_name);
+  CallJS("cr.ui.Oobe.setBluetoothDeviceInfo", bluetooth_name);
 }
 
 ui::EventSink* CoreOobeHandler::GetEventSink() {
@@ -468,7 +514,7 @@ ui::EventSink* CoreOobeHandler::GetEventSink() {
 
 void CoreOobeHandler::UpdateLabel(const std::string& id,
                                   const std::string& text) {
-  CallJSOrDefer("setLabelText", id, text);
+  CallJS("cr.ui.Oobe.setLabelText", id, text);
 }
 
 void CoreOobeHandler::UpdateDeviceRequisition() {
@@ -477,19 +523,19 @@ void CoreOobeHandler::UpdateDeviceRequisition() {
           ->browser_policy_connector_chromeos()
           ->GetDeviceCloudPolicyManager();
   if (policy_manager) {
-    CallJSOrDefer("updateDeviceRequisition",
-                  policy_manager->GetDeviceRequisition());
+    CallJS("cr.ui.Oobe.updateDeviceRequisition",
+           policy_manager->GetDeviceRequisition());
   }
 }
 
 void CoreOobeHandler::UpdateKeyboardState() {
-  keyboard::KeyboardController* keyboard_controller =
-      keyboard::KeyboardController::GetInstance();
-  if (keyboard_controller) {
-    gfx::Rect bounds = keyboard_controller->current_keyboard_bounds();
-    ShowControlBar(bounds.IsEmpty());
-    SetVirtualKeyboardShown(!bounds.IsEmpty());
-  }
+  const bool is_keyboard_shown =
+      ChromeKeyboardControllerClient::Get()->is_keyboard_visible();
+  SetVirtualKeyboardShown(is_keyboard_shown);
+}
+
+void CoreOobeHandler::OnTabletModeToggled(bool enabled) {
+  CallJS("cr.ui.Oobe.setTabletModeState", enabled);
 }
 
 void CoreOobeHandler::UpdateClientAreaSize() {
@@ -498,6 +544,15 @@ void CoreOobeHandler::UpdateClientAreaSize() {
   SetClientAreaSize(size.width(), size.height());
 }
 
+void CoreOobeHandler::OnOobeConfigurationChanged() {
+  base::Value configuration(base::Value::Type::DICTIONARY);
+  chromeos::configuration::FilterConfiguration(
+      OobeConfiguration::Get()->GetConfiguration(),
+      chromeos::configuration::ConfigurationHandlerSide::HANDLER_JS,
+      configuration);
+  CallJS("cr.ui.Oobe.updateOobeConfiguration", configuration);
+}
+
 void CoreOobeHandler::OnAccessibilityStatusChanged(
     const AccessibilityStatusEventDetails& details) {
   if (details.notification_type == ACCESSIBILITY_MANAGER_SHUTDOWN)
@@ -508,19 +563,12 @@ void CoreOobeHandler::OnAccessibilityStatusChanged(
 
 void CoreOobeHandler::HandleLaunchHelpApp(double help_topic_id) {
   if (!help_app_.get())
-    help_app_ = new HelpAppLauncher(GetNativeWindow());
+    help_app_ = new HelpAppLauncher(
+        LoginDisplayHost::default_host()->GetNativeWindow());
   help_app_->ShowHelpTopic(
       static_cast<HelpAppLauncher::HelpTopic>(help_topic_id));
 }
 
-void CoreOobeHandler::HandleHeaderBarVisible() {
-  LoginDisplayHost* login_display_host = LoginDisplayHost::default_host();
-  if (login_display_host)
-    login_display_host->SetStatusAreaVisible(true);
-  if (ScreenLocker::default_screen_locker())
-    ScreenLocker::default_screen_locker()->delegate()->OnHeaderBarVisible();
-}
-
 void CoreOobeHandler::HandleRaiseTabKeyEvent(bool reverse) {
   ui::KeyEvent event(ui::ET_KEY_PRESSED, ui::VKEY_TAB, ui::EF_NONE);
   if (reverse)
@@ -528,14 +576,56 @@ void CoreOobeHandler::HandleRaiseTabKeyEvent(bool reverse) {
   SendEventToSink(&event);
 }
 
-void CoreOobeHandler::HandleSetOobeBootstrappingSlave() {
-  const bool is_slave = g_browser_process->local_state()->GetBoolean(
-      prefs::kIsBootstrappingSlave);
-  if (is_slave)
-    return;
-  g_browser_process->local_state()->SetBoolean(prefs::kIsBootstrappingSlave,
-                                               true);
-  chrome::AttemptRestart();
+void CoreOobeHandler::HandleGetPrimaryDisplayNameForTesting(
+    const base::ListValue* args) {
+  CHECK_EQ(1U, args->GetSize());
+  const base::Value* callback_id;
+  CHECK(args->Get(0, &callback_id));
+
+  cros_display_config_ptr_->GetDisplayUnitInfoList(
+      false /* single_unified */,
+      base::BindOnce(&CoreOobeHandler::GetPrimaryDisplayNameCallback,
+                     weak_ptr_factory_.GetWeakPtr(), callback_id->Clone()));
+}
+
+void CoreOobeHandler::GetPrimaryDisplayNameCallback(
+    const base::Value& callback_id,
+    std::vector<ash::mojom::DisplayUnitInfoPtr> info_list) {
+  AllowJavascript();
+  std::string display_name;
+  for (const ash::mojom::DisplayUnitInfoPtr& info : info_list) {
+    if (info->is_primary) {
+      display_name = info->name;
+      break;
+    }
+  }
+  DCHECK(!display_name.empty());
+  ResolveJavascriptCallback(callback_id, base::Value(display_name));
+}
+
+void CoreOobeHandler::HandleSetupDemoMode() {
+  WizardController* wizard_controller = WizardController::default_controller();
+  if (wizard_controller && !wizard_controller->login_screen_started()) {
+    wizard_controller->StartDemoModeSetup();
+  }
+}
+
+void CoreOobeHandler::HandleStartDemoModeSetupForTesting(
+    const std::string& demo_config) {
+  DemoSession::DemoModeConfig config;
+  if (demo_config == "online") {
+    config = DemoSession::DemoModeConfig::kOnline;
+  } else if (demo_config == "offline") {
+    config = DemoSession::DemoModeConfig::kOffline;
+  } else {
+    NOTREACHED() << "Unknown demo config passed for tests";
+  }
+
+  WizardController* wizard_controller = WizardController::default_controller();
+  if (wizard_controller && !wizard_controller->login_screen_started()) {
+    wizard_controller->SimulateDemoModeSetupForTesting(config);
+    wizard_controller->AdvanceToScreen(DemoSetupScreenView::kScreenId);
+  }
 }
 
 void CoreOobeHandler::InitDemoModeDetection() {
