@@ -8,11 +8,11 @@
 #include <vector>
 
 #include "base/optional.h"
-#include "components/autofill/core/browser/autofill_profile.h"
-#include "components/autofill/core/browser/credit_card.h"
+#include "components/autofill/core/browser/data_model/autofill_profile.h"
+#include "components/autofill/core/browser/data_model/credit_card.h"
 #include "components/autofill/core/browser/payments/payments_customer_data.h"
 #include "components/autofill/core/browser/personal_data_manager.h"
-#include "components/signin/core/browser/account_info.h"
+#include "components/signin/public/identity_manager/account_info.h"
 
 namespace autofill {
 
@@ -28,6 +28,7 @@ class TestPersonalDataManager : public PersonalDataManager {
   // for various tests, whether to skip calls to uncreated databases/services,
   // or to make things easier in general to toggle.
   void OnSyncServiceInitialized(syncer::SyncService* sync_service) override;
+  AutofillSyncSigninState GetSyncSigninState() const override;
   void RecordUseOf(const AutofillDataModel& data_model) override;
   std::string SaveImportedProfile(
       const AutofillProfile& imported_profile) override;
@@ -55,7 +56,7 @@ class TestPersonalDataManager : public PersonalDataManager {
   CreditCard* GetCreditCardByNumber(const std::string& number) override;
   bool IsDataLoaded() const override;
   bool IsSyncFeatureEnabled() const override;
-  AccountInfo GetAccountInfoForPaymentsServer() const override;
+  CoreAccountInfo GetAccountInfoForPaymentsServer() const override;
 
   // Unique to TestPersonalDataManager:
 
@@ -115,7 +116,11 @@ class TestPersonalDataManager : public PersonalDataManager {
 
   void SetSyncFeatureEnabled(bool enabled) { sync_feature_enabled_ = enabled; }
 
-  void SetAccountInfoForPayments(const AccountInfo& account_info) {
+  void SetSyncAndSignInState(AutofillSyncSigninState sync_and_signin_state) {
+    sync_and_signin_state_ = sync_and_signin_state;
+  }
+
+  void SetAccountInfoForPayments(const CoreAccountInfo& account_info) {
     account_info_ = account_info;
   }
 
@@ -129,8 +134,10 @@ class TestPersonalDataManager : public PersonalDataManager {
   base::Optional<bool> autofill_credit_card_enabled_;
   base::Optional<bool> autofill_wallet_import_enabled_;
   bool sync_feature_enabled_ = false;
+  AutofillSyncSigninState sync_and_signin_state_ =
+      AutofillSyncSigninState::kSignedInAndSyncFeatureEnabled;
   bool sync_service_initialized_ = false;
-  AccountInfo account_info_;
+  CoreAccountInfo account_info_;
 
   DISALLOW_COPY_AND_ASSIGN(TestPersonalDataManager);
 };
