@@ -72,7 +72,7 @@ std::unique_ptr<EntityData> CreateEntityData(const AutofillEntry& entry) {
   autofill->add_usage_timestamp(entry.date_created().ToInternalValue());
   if (entry.date_created() != entry.date_last_used())
     autofill->add_usage_timestamp(entry.date_last_used().ToInternalValue());
-  entity_data->non_unique_name = EscapeIdentifiers(*autofill);
+  entity_data->name = EscapeIdentifiers(*autofill);
   return entity_data;
 }
 
@@ -330,22 +330,16 @@ Optional<syncer::ModelError> AutocompleteSyncBridge::MergeSyncData(
 
   SyncDifferenceTracker tracker(GetAutofillTable());
   for (const auto& change : entity_data) {
-    DCHECK(change.data().specifics.has_autofill());
+    DCHECK(change->data().specifics.has_autofill());
     RETURN_IF_ERROR(tracker.IncorporateRemoteSpecifics(
-        change.storage_key(), change.data().specifics.autofill()));
+        change->storage_key(), change->data().specifics.autofill()));
   }
 
   RETURN_IF_ERROR(tracker.FlushToLocal(web_data_backend_));
   RETURN_IF_ERROR(tracker.FlushToSync(true, std::move(metadata_change_list),
                                 change_processor()));
 
-  // TODO(crbug.com/920214) Deprecated, clean-up as part of the
-  // Autocomplete Retention Policy flag cleanup.
-  if (!base::FeatureList::IsEnabled(
-          autofill::features::kAutocompleteRetentionPolicyEnabled)) {
-    web_data_backend_->RemoveExpiredFormElements();
-  }
-
+  web_data_backend_->CommitChanges();
   web_data_backend_->NotifyThatSyncHasStarted(syncer::AUTOFILL);
   return {};
 }
@@ -356,13 +350,13 @@ Optional<ModelError> AutocompleteSyncBridge::ApplySyncChanges(
   DCHECK(thread_checker_.CalledOnValidThread());
 
   SyncDifferenceTracker tracker(GetAutofillTable());
-  for (const EntityChange& change : entity_changes) {
-    if (change.type() == EntityChange::ACTION_DELETE) {
-      RETURN_IF_ERROR(tracker.IncorporateRemoteDelete(change.storage_key()));
+  for (const std::unique_ptr<EntityChange>& change : entity_changes) {
+    if (change->type() == EntityChange::ACTION_DELETE) {
+      RETURN_IF_ERROR(tracker.IncorporateRemoteDelete(change->storage_key()));
     } else {
-      DCHECK(change.data().specifics.has_autofill());
+      DCHECK(change->data().specifics.has_autofill());
       RETURN_IF_ERROR(tracker.IncorporateRemoteSpecifics(
-          change.storage_key(), change.data().specifics.autofill()));
+          change->storage_key(), change->data().specifics.autofill()));
     }
   }
 
@@ -370,12 +364,7 @@ Optional<ModelError> AutocompleteSyncBridge::ApplySyncChanges(
   RETURN_IF_ERROR(tracker.FlushToSync(false, std::move(metadata_change_list),
                                       change_processor()));
 
-  // TODO(crbug.com/920214) Deprecated, clean-up as part of the
-  // Autocomplete Retention Policy flag cleanup.
-  if (!base::FeatureList::IsEnabled(
-          autofill::features::kAutocompleteRetentionPolicyEnabled)) {
-    web_data_backend_->RemoveExpiredFormElements();
-  }
+  web_data_backend_->CommitChanges();
   return {};
 }
 
@@ -470,6 +459,11 @@ void AutocompleteSyncBridge::ActOnLocalChanges(
     }
   }
 
+  // We do not need to commit any local changes (written by the processor via
+  // the metadata change list) because the open WebDatabase transaction is
+  // committed by the AutofillWebDataService when the original local write
+  // operation (that triggered this notification to the bridge) finishes.
+
   if (Optional<ModelError> error = metadata_change_list->TakeError())
     change_processor()->ReportError(*error);
 }
