@@ -9,18 +9,18 @@
 #include "base/base64.h"
 #include "base/feature_list.h"
 #include "base/logging.h"
-#include "base/metrics/histogram_macros.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
-#include "components/autofill/core/browser/autofill_profile.h"
 #include "components/autofill/core/browser/autofill_profile_sync_util.h"
-#include "components/autofill/core/browser/credit_card.h"
+#include "components/autofill/core/browser/data_model/autofill_profile.h"
+#include "components/autofill/core/browser/data_model/credit_card.h"
 #include "components/autofill/core/browser/payments/payments_customer_data.h"
 #include "components/autofill/core/browser/webdata/autofill_sync_bridge_util.h"
 #include "components/autofill/core/browser/webdata/autofill_table.h"
 #include "components/autofill/core/browser/webdata/autofill_webdata_backend.h"
 #include "components/autofill/core/browser/webdata/autofill_webdata_service.h"
 #include "components/autofill/core/common/autofill_util.h"
+#include "components/sync/base/data_type_histogram.h"
 #include "components/sync/driver/sync_driver_switches.h"
 #include "components/sync/model/entity_data.h"
 #include "components/sync/model/mutable_data_batch.h"
@@ -91,7 +91,7 @@ std::unique_ptr<EntityData> CreateEntityDataFromAutofillServerProfile(
     const AutofillProfile& address,
     bool enforce_utf8) {
   auto entity_data = std::make_unique<EntityData>();
-  entity_data->non_unique_name =
+  entity_data->name =
       "Server profile " +
       GetBase64EncodedId(GetSpecificsIdFromAutofillProfile(address));
 
@@ -107,7 +107,7 @@ std::unique_ptr<EntityData> CreateEntityDataFromAutofillServerProfile(
 std::unique_ptr<EntityData> CreateEntityDataFromCard(const CreditCard& card,
                                                      bool enforce_utf8) {
   auto entity_data = std::make_unique<EntityData>();
-  entity_data->non_unique_name =
+  entity_data->name =
       "Server card " + GetBase64EncodedId(GetSpecificsIdFromCreditCard(card));
 
   AutofillWalletSpecifics* wallet_specifics =
@@ -122,7 +122,7 @@ std::unique_ptr<EntityData> CreateEntityDataFromCard(const CreditCard& card,
 std::unique_ptr<EntityData> CreateEntityDataFromPaymentsCustomerData(
     const PaymentsCustomerData& customer_data) {
   auto entity_data = std::make_unique<EntityData>();
-  entity_data->non_unique_name =
+  entity_data->name =
       "Payments customer data " +
       GetBase64EncodedId(GetSpecificsIdFromPaymentsCustomerData(customer_data));
 
@@ -197,7 +197,9 @@ base::Optional<syncer::ModelError> AutofillWalletSyncBridge::MergeSyncData(
     return error;
   }
 
-  SetSyncData(entity_data);
+  // We want to notify the metadata bridge about all changes so that the
+  // metadata bridge can track changes in the data bridge and react accordingly.
+  SetSyncData(entity_data, /*notify_metadata_bridge=*/true);
 
   // After the first sync, we are sure that initial sync is done.
   if (!initial_sync_done_) {
@@ -253,8 +255,7 @@ bool AutofillWalletSyncBridge::SupportsIncrementalUpdates() const {
   return false;
 }
 
-AutofillWalletSyncBridge::StopSyncResponse
-AutofillWalletSyncBridge::ApplyStopSyncChanges(
+void AutofillWalletSyncBridge::ApplyStopSyncChanges(
     std::unique_ptr<syncer::MetadataChangeList> delete_metadata_change_list) {
   // If a metadata change list gets passed in, that means sync is actually
   // disabled, so we want to delete the payments data.
@@ -262,11 +263,14 @@ AutofillWalletSyncBridge::ApplyStopSyncChanges(
     if (initial_sync_done_) {
       active_callback_.Run(false);
     }
-    SetSyncData(syncer::EntityChangeList());
+
+    // Do not notify the metadata bridge because we do not want to upstream the
+    // deletions. The metadata bridge deletes its data independently when sync
+    // gets stopped.
+    SetSyncData(syncer::EntityChangeList(), /*notify_metadata_bridge=*/false);
 
     initial_sync_done_ = false;
   }
-  return StopSyncResponse::kModelStillReadyToSync;
 }
 
 void AutofillWalletSyncBridge::GetAllDataForTesting(DataCallback callback) {
@@ -309,7 +313,8 @@ void AutofillWalletSyncBridge::GetAllDataImpl(DataCallback callback,
 }
 
 void AutofillWalletSyncBridge::SetSyncData(
-    const syncer::EntityChangeList& entity_data) {
+    const syncer::EntityChangeList& entity_data,
+    bool notify_metadata_bridge) {
   bool wallet_data_changed = false;
 
   // Extract the Autofill types from the sync |entity_data|.
@@ -319,13 +324,18 @@ void AutofillWalletSyncBridge::SetSyncData(
   PopulateWalletTypesFromSyncData(entity_data, &wallet_cards, &wallet_addresses,
                                   &customer_data);
 
-  bool should_log_diff;
+  wallet_data_changed |= SetPaymentsCustomerData(std::move(customer_data));
   wallet_data_changed |=
-      SetPaymentsCustomerData(std::move(customer_data), &should_log_diff);
+      SetWalletCards(std::move(wallet_cards), notify_metadata_bridge);
   wallet_data_changed |=
-      SetWalletCards(std::move(wallet_cards), should_log_diff);
-  wallet_data_changed |=
-      SetWalletAddresses(std::move(wallet_addresses), should_log_diff);
+      SetWalletAddresses(std::move(wallet_addresses), notify_metadata_bridge);
+
+  // Commit the transaction to make sure the data and the metadata with the
+  // new progress marker is written down (especially on Android where we
+  // cannot rely on commiting transactions on shutdown). We need to commit
+  // even if the wallet data has not changed because the model type state incl.
+  // the progress marker always changes.
+  web_data_backend_->CommitChanges();
 
   if (web_data_backend_ && wallet_data_changed)
     web_data_backend_->NotifyOfMultipleAutofillChanges();
@@ -333,7 +343,7 @@ void AutofillWalletSyncBridge::SetSyncData(
 
 bool AutofillWalletSyncBridge::SetWalletCards(
     std::vector<CreditCard> wallet_cards,
-    bool log_diff) {
+    bool notify_metadata_bridge) {
   // Users can set billing address of the server credit card locally, but that
   // information does not propagate to either Chrome Sync or Google Payments
   // server. To preserve user's preferred billing address and most recent use
@@ -350,14 +360,6 @@ bool AutofillWalletSyncBridge::SetWalletCards(
   AutofillWalletDiff<CreditCard> diff =
       ComputeAutofillWalletDiff(existing_cards, wallet_cards);
 
-  if (log_diff) {
-    UMA_HISTOGRAM_COUNTS_100("Autofill.WalletCards2.Added", diff.items_added);
-    UMA_HISTOGRAM_COUNTS_100("Autofill.WalletCards2.Removed",
-                             diff.items_removed);
-    UMA_HISTOGRAM_COUNTS_100("Autofill.WalletCards2.AddedOrRemoved",
-                             diff.items_added + diff.items_removed);
-  }
-
   if (!diff.IsEmpty()) {
     if (base::FeatureList::IsEnabled(
             ::switches::kSyncUSSAutofillWalletMetadata)) {
@@ -365,8 +367,11 @@ bool AutofillWalletSyncBridge::SetWalletCards(
     } else {
       table->SetServerCreditCards(wallet_cards);
     }
-    for (const CreditCardChange& change : diff.changes)
-      web_data_backend_->NotifyOfCreditCardChanged(change);
+    if (notify_metadata_bridge) {
+      for (const CreditCardChange& change : diff.changes) {
+        web_data_backend_->NotifyOfCreditCardChanged(change);
+      }
+    }
     return true;
   }
   return false;
@@ -374,7 +379,15 @@ bool AutofillWalletSyncBridge::SetWalletCards(
 
 bool AutofillWalletSyncBridge::SetWalletAddresses(
     std::vector<AutofillProfile> wallet_addresses,
-    bool log_diff) {
+    bool notify_metadata_bridge) {
+  // We do not have to CopyRelevantWalletMetadataFromDisk() because we will
+  // never overwrite the same entity with different data (server_id is generated
+  // based on content so addresses have the same server_id iff they have the
+  // same content). For that reason it is impossible to issue a DELETE and ADD
+  // for the same entity just because some of its fields got changed. As a
+  // result, we do not need to care to have up-to-date use stats for cards
+  // because we never notify on an existing one.
+
   // In the common case, the database won't have changed. Committing an update
   // to the database will require at least one DB page write and will schedule
   // a fsync. To avoid this I/O, it should be more efficient to do a read and
@@ -385,15 +398,6 @@ bool AutofillWalletSyncBridge::SetWalletAddresses(
   AutofillWalletDiff<AutofillProfile> diff =
       ComputeAutofillWalletDiff(existing_addresses, wallet_addresses);
 
-  if (log_diff) {
-    UMA_HISTOGRAM_COUNTS_100("Autofill.WalletAddresses2.Added",
-                             diff.items_added);
-    UMA_HISTOGRAM_COUNTS_100("Autofill.WalletAddresses2.Removed",
-                             diff.items_removed);
-    UMA_HISTOGRAM_COUNTS_100("Autofill.WalletAddresses2.AddedOrRemoved",
-                             diff.items_added + diff.items_removed);
-  }
-
   if (!diff.IsEmpty()) {
     if (base::FeatureList::IsEnabled(
             ::switches::kSyncUSSAutofillWalletMetadata)) {
@@ -401,16 +405,18 @@ bool AutofillWalletSyncBridge::SetWalletAddresses(
     } else {
       table->SetServerProfiles(wallet_addresses);
     }
-    for (const AutofillProfileChange& change : diff.changes)
-      web_data_backend_->NotifyOfAutofillProfileChanged(change);
+    if (notify_metadata_bridge) {
+      for (const AutofillProfileChange& change : diff.changes) {
+        web_data_backend_->NotifyOfAutofillProfileChanged(change);
+      }
+    }
     return true;
   }
   return false;
 }
 
 bool AutofillWalletSyncBridge::SetPaymentsCustomerData(
-    std::vector<PaymentsCustomerData> customer_data,
-    bool* should_log_diff) {
+    std::vector<PaymentsCustomerData> customer_data) {
   AutofillTable* table = GetAutofillTable();
   std::unique_ptr<PaymentsCustomerData> existing_entry;
   table->GetPaymentsCustomerData(&existing_entry);
@@ -427,15 +433,6 @@ bool AutofillWalletSyncBridge::SetPaymentsCustomerData(
   }
 #endif  // DCHECK_IS_ON()
 
-  // We report the diff to metrics only if this is an incremental change where
-  // the user had sync set-up (having PaymentsCustomerData is a pre-requisite
-  // for having any other data) and continues to have sync set-up (continuing
-  // having a PaymentsCustomerData entity). As a side effect, this excludes
-  // reporting diffs for users that newly got a GPay account and sync
-  // PaymentsCustomerData for the first time but this is the best we can do to
-  // have the metrics consistent with Directory implementation.
-  *should_log_diff = existing_entry && new_entry;
-
   if (!new_entry && existing_entry) {
     // Clear the existing entry in the DB.
     GetAutofillTable()->SetPaymentsCustomerData(nullptr);
@@ -470,8 +467,11 @@ AutofillWalletSyncBridge::ComputeAutofillWalletDiff(
   std::sort(old_ptrs.begin(), old_ptrs.end(), compare);
   std::sort(new_ptrs.begin(), new_ptrs.end(), compare);
 
-  // Walk over both of them and count added/removed elements.
   AutofillWalletDiff<Item> result;
+  // We collect ADD changes separately to ensure proper order.
+  std::vector<AutofillDataModelChange<Item>> add_changes;
+
+  // Walk over both of them and count added/removed elements.
   auto old_it = old_ptrs.begin();
   auto new_it = new_ptrs.begin();
   while (old_it != old_ptrs.end() || new_it != new_ptrs.end()) {
@@ -487,19 +487,25 @@ AutofillWalletSyncBridge::ComputeAutofillWalletDiff(
     if (cmp < 0) {
       ++result.items_removed;
       result.changes.emplace_back(AutofillDataModelChange<Item>::REMOVE,
-                                  (*old_it)->server_id(), nullptr);
+                                  (*old_it)->server_id(), *old_it);
       ++old_it;
     } else if (cmp == 0) {
       ++old_it;
       ++new_it;
     } else {
       ++result.items_added;
-      result.changes.emplace_back(AutofillDataModelChange<Item>::ADD,
-                                  (*new_it)->server_id(), *new_it);
+      add_changes.emplace_back(AutofillDataModelChange<Item>::ADD,
+                               (*new_it)->server_id(), *new_it);
       ++new_it;
     }
   }
 
+  // Append ADD changes to make sure they all come after all REMOVE changes.
+  // Since we CopyRelevantWalletMetadataFromDisk(), the ADD contains all current
+  // metadata if we happen to REMOVE and ADD the same entity.
+  result.changes.insert(result.changes.end(), add_changes.begin(),
+                        add_changes.end());
+
   DCHECK_EQ(old_data.size() + result.items_added - result.items_removed,
             new_data.size());
 
