@@ -138,10 +138,8 @@ void PopulateNodeImpl(const std::vector<std::string>& description,
       // value. The folders need not have a name, but one is assigned to help
       // in debugging.
       static int next_folder_id = 1;
-      TestNode* new_node = parent->Add(
-          std::make_unique<TestNode>(base::IntToString16(next_folder_id++),
-                                     BookmarkNode::FOLDER),
-          parent->child_count());
+      TestNode* new_node = parent->Add(std::make_unique<TestNode>(
+          base::NumberToString16(next_folder_id++), BookmarkNode::FOLDER));
       PopulateNodeImpl(description, index, new_node);
     } else if (element == "]") {
       // End the current folder.
@@ -154,8 +152,7 @@ void PopulateNodeImpl(const std::vector<std::string>& description,
       DCHECK(element.find('[') == std::string::npos);
       DCHECK(element.find(']') == std::string::npos);
       parent->Add(std::make_unique<TestNode>(base::UTF8ToUTF16(element),
-                                             BookmarkNode::URL),
-                  parent->child_count());
+                                             BookmarkNode::URL));
     }
   }
 }
@@ -189,8 +186,8 @@ void PopulateNodeFromString(const std::string& description, TestNode* parent) {
 void PopulateBookmarkNode(TestNode* parent,
                           BookmarkModel* model,
                           const BookmarkNode* bb_node) {
-  for (int i = 0; i < parent->child_count(); ++i) {
-    TestNode* child = parent->GetChild(i);
+  for (size_t i = 0; i < parent->children().size(); ++i) {
+    TestNode* child = parent->children()[i].get();
     if (child->value == BookmarkNode::FOLDER) {
       const BookmarkNode* new_bb_node =
           model->AddFolder(bb_node, i, child->GetTitle());
@@ -205,10 +202,10 @@ void PopulateBookmarkNode(TestNode* parent,
 // Verifies the contents of the bookmark bar node match the contents of the
 // TestNode.
 void VerifyModelMatchesNode(TestNode* expected, const BookmarkNode* actual) {
-  ASSERT_EQ(expected->child_count(), actual->child_count());
-  for (int i = 0; i < expected->child_count(); ++i) {
-    TestNode* expected_child = expected->GetChild(i);
-    const BookmarkNode* actual_child = actual->GetChild(i);
+  ASSERT_EQ(expected->children().size(), actual->children().size());
+  for (size_t i = 0; i < expected->children().size(); ++i) {
+    TestNode* expected_child = expected->children()[i].get();
+    const BookmarkNode* actual_child = actual->children()[i].get();
     ASSERT_EQ(expected_child->GetTitle(), actual_child->GetTitle());
     if (expected_child->value == BookmarkNode::FOLDER) {
       ASSERT_TRUE(actual_child->type() == BookmarkNode::FOLDER);
@@ -233,12 +230,12 @@ class BookmarkModelTest : public testing::Test,
                           public BookmarkUndoDelegate {
  public:
   struct ObserverDetails {
-    ObserverDetails() { Set(nullptr, nullptr, -1, -1); }
+    ObserverDetails() { Set(nullptr, nullptr, size_t{-1}, size_t{-1}); }
 
     void Set(const BookmarkNode* node1,
              const BookmarkNode* node2,
-             int index1,
-             int index2) {
+             size_t index1,
+             size_t index2) {
       node1_ = node1;
       node2_ = node2;
       index1_ = index1;
@@ -247,8 +244,8 @@ class BookmarkModelTest : public testing::Test,
 
     void ExpectEquals(const BookmarkNode* node1,
                       const BookmarkNode* node2,
-                      int index1,
-                      int index2) {
+                      size_t index1,
+                      size_t index2) {
       EXPECT_EQ(node1_, node1);
       EXPECT_EQ(node2_, node2);
       EXPECT_EQ(index1_, index1);
@@ -258,13 +255,13 @@ class BookmarkModelTest : public testing::Test,
    private:
     const BookmarkNode* node1_;
     const BookmarkNode* node2_;
-    int index1_;
-    int index2_;
+    size_t index1_;
+    size_t index2_;
   };
 
   struct NodeRemovalDetail {
     NodeRemovalDetail(const BookmarkNode* parent,
-                      int index,
+                      size_t index,
                       const BookmarkNode* node)
         : parent_node_id(parent->id()), index(index), node_id(node->id()) {}
 
@@ -275,7 +272,7 @@ class BookmarkModelTest : public testing::Test,
     }
 
     int64_t parent_node_id;
-    int index;
+    size_t index;
     int64_t node_id;
   };
 
@@ -291,23 +288,23 @@ class BookmarkModelTest : public testing::Test,
 
   void BookmarkNodeMoved(BookmarkModel* model,
                          const BookmarkNode* old_parent,
-                         int old_index,
+                         size_t old_index,
                          const BookmarkNode* new_parent,
-                         int new_index) override {
+                         size_t new_index) override {
     ++moved_count_;
     observer_details_.Set(old_parent, new_parent, old_index, new_index);
   }
 
   void BookmarkNodeAdded(BookmarkModel* model,
                          const BookmarkNode* parent,
-                         int index) override {
+                         size_t index) override {
     ++added_count_;
-    observer_details_.Set(parent, nullptr, index, -1);
+    observer_details_.Set(parent, nullptr, index, size_t{-1});
   }
 
   void OnWillRemoveBookmarks(BookmarkModel* model,
                              const BookmarkNode* parent,
-                             int old_index,
+                             size_t old_index,
                              const BookmarkNode* node) override {
     ++before_remove_count_;
   }
@@ -316,17 +313,17 @@ class BookmarkModelTest : public testing::Test,
 
   void BookmarkNodeRemoved(BookmarkModel* model,
                            const BookmarkNode* parent,
-                           int old_index,
+                           size_t old_index,
                            const BookmarkNode* node,
                            const std::set<GURL>& removed_urls) override {
     ++removed_count_;
-    observer_details_.Set(parent, nullptr, old_index, -1);
+    observer_details_.Set(parent, nullptr, old_index, size_t{-1});
   }
 
   void BookmarkNodeChanged(BookmarkModel* model,
                            const BookmarkNode* node) override {
     ++changed_count_;
-    observer_details_.Set(node, nullptr, -1, -1);
+    observer_details_.Set(node, nullptr, size_t{-1}, size_t{-1});
   }
 
   void OnWillChangeBookmarkNode(BookmarkModel* model,
@@ -378,7 +375,7 @@ class BookmarkModelTest : public testing::Test,
 
   void OnBookmarkNodeRemoved(BookmarkModel* model,
                              const BookmarkNode* parent,
-                             int index,
+                             size_t index,
                              std::unique_ptr<BookmarkNode> node) override {
     node_removal_details_.push_back(
         NodeRemovalDetail(parent, index, node.get()));
@@ -431,24 +428,24 @@ class BookmarkModelTest : public testing::Test,
 
   int AllNodesRemovedObserverCount() const { return all_bookmarks_removed_; }
 
-  BookmarkPermanentNode* ReloadModelWithExtraNode() {
+  BookmarkPermanentNode* ReloadModelWithManagedNode() {
     model_->RemoveObserver(this);
 
-    BookmarkPermanentNodeList extra_nodes;
-    extra_nodes.push_back(std::make_unique<BookmarkPermanentNode>(100));
-    BookmarkPermanentNode* extra_node = extra_nodes.back().get();
+    auto owned_managed_node =
+        std::make_unique<BookmarkPermanentNode>(100, BookmarkNode::FOLDER);
+    BookmarkPermanentNode* managed_node = owned_managed_node.get();
 
     std::unique_ptr<TestBookmarkClient> client(new TestBookmarkClient);
-    client->SetExtraNodesToLoad(std::move(extra_nodes));
+    client->SetManagedNodeToLoad(std::move(owned_managed_node));
 
     model_ = TestBookmarkClient::CreateModelWithClient(std::move(client));
     model_->AddObserver(this);
     ClearCounts();
 
-    if (model_->root_node()->GetIndexOf(extra_node) == -1)
+    if (model_->root_node()->GetIndexOf(managed_node) == -1)
       ADD_FAILURE();
 
-    return extra_node;
+    return managed_node;
   }
 
  protected:
@@ -478,17 +475,17 @@ class BookmarkModelTest : public testing::Test,
 TEST_F(BookmarkModelTest, InitialState) {
   const BookmarkNode* bb_node = model_->bookmark_bar_node();
   ASSERT_TRUE(bb_node != nullptr);
-  EXPECT_EQ(0, bb_node->child_count());
+  EXPECT_EQ(0u, bb_node->children().size());
   EXPECT_EQ(BookmarkNode::BOOKMARK_BAR, bb_node->type());
 
   const BookmarkNode* other_node = model_->other_node();
   ASSERT_TRUE(other_node != nullptr);
-  EXPECT_EQ(0, other_node->child_count());
+  EXPECT_EQ(0u, other_node->children().size());
   EXPECT_EQ(BookmarkNode::OTHER_NODE, other_node->type());
 
   const BookmarkNode* mobile_node = model_->mobile_node();
   ASSERT_TRUE(mobile_node != nullptr);
-  EXPECT_EQ(0, mobile_node->child_count());
+  EXPECT_EQ(0u, mobile_node->children().size());
   EXPECT_EQ(BookmarkNode::MOBILE, mobile_node->type());
 
   EXPECT_TRUE(bb_node->id() != other_node->id());
@@ -503,9 +500,9 @@ TEST_F(BookmarkModelTest, AddURL) {
 
   const BookmarkNode* new_node = model_->AddURL(root, 0, title, url);
   AssertObserverCount(1, 0, 0, 0, 0, 0, 0, 0, 0);
-  observer_details_.ExpectEquals(root, nullptr, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, size_t{-1});
 
-  ASSERT_EQ(1, root->child_count());
+  ASSERT_EQ(1u, root->children().size());
   ASSERT_EQ(title, new_node->GetTitle());
   ASSERT_TRUE(url == new_node->url());
   ASSERT_EQ(BookmarkNode::URL, new_node->type());
@@ -524,9 +521,9 @@ TEST_F(BookmarkModelTest, AddURLWithUnicodeTitle) {
 
   const BookmarkNode* new_node = model_->AddURL(root, 0, title, url);
   AssertObserverCount(1, 0, 0, 0, 0, 0, 0, 0, 0);
-  observer_details_.ExpectEquals(root, nullptr, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, size_t{-1});
 
-  ASSERT_EQ(1, root->child_count());
+  ASSERT_EQ(1u, root->children().size());
   ASSERT_EQ(title, new_node->GetTitle());
   ASSERT_TRUE(url == new_node->url());
   ASSERT_EQ(BookmarkNode::URL, new_node->type());
@@ -546,8 +543,7 @@ TEST_F(BookmarkModelTest, AddURLWithWhitespaceTitle) {
 
     const BookmarkNode* new_node = model_->AddURL(root, i, title, url);
 
-    int size = i + 1;
-    EXPECT_EQ(size, root->child_count());
+    EXPECT_EQ(i + 1, root->children().size());
     EXPECT_EQ(ASCIIToUTF16(url_whitespace_test_cases[i].expected_title),
               new_node->GetTitle());
     EXPECT_EQ(BookmarkNode::URL, new_node->type());
@@ -565,9 +561,9 @@ TEST_F(BookmarkModelTest, AddURLWithCreationTimeAndMetaInfo) {
   const BookmarkNode* new_node = model_->AddURLWithCreationTimeAndMetaInfo(
       root, 0, title, url, time, &meta_info);
   AssertObserverCount(1, 0, 0, 0, 0, 0, 0, 0, 0);
-  observer_details_.ExpectEquals(root, nullptr, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, size_t{-1});
 
-  ASSERT_EQ(1, root->child_count());
+  ASSERT_EQ(1u, root->children().size());
   ASSERT_EQ(title, new_node->GetTitle());
   ASSERT_TRUE(url == new_node->url());
   ASSERT_EQ(BookmarkNode::URL, new_node->type());
@@ -588,9 +584,9 @@ TEST_F(BookmarkModelTest, AddURLToMobileBookmarks) {
 
   const BookmarkNode* new_node = model_->AddURL(root, 0, title, url);
   AssertObserverCount(1, 0, 0, 0, 0, 0, 0, 0, 0);
-  observer_details_.ExpectEquals(root, nullptr, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, size_t{-1});
 
-  ASSERT_EQ(1, root->child_count());
+  ASSERT_EQ(1u, root->children().size());
   ASSERT_EQ(title, new_node->GetTitle());
   ASSERT_TRUE(url == new_node->url());
   ASSERT_EQ(BookmarkNode::URL, new_node->type());
@@ -607,9 +603,9 @@ TEST_F(BookmarkModelTest, AddFolder) {
 
   const BookmarkNode* new_node = model_->AddFolder(root, 0, title);
   AssertObserverCount(1, 0, 0, 0, 0, 0, 0, 0, 0);
-  observer_details_.ExpectEquals(root, nullptr, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, size_t{-1});
 
-  ASSERT_EQ(1, root->child_count());
+  ASSERT_EQ(1u, root->children().size());
   ASSERT_EQ(title, new_node->GetTitle());
   ASSERT_EQ(BookmarkNode::FOLDER, new_node->type());
 
@@ -621,7 +617,7 @@ TEST_F(BookmarkModelTest, AddFolder) {
   ClearCounts();
   model_->AddFolder(root, 0, title);
   AssertObserverCount(1, 0, 0, 0, 0, 0, 0, 0, 0);
-  observer_details_.ExpectEquals(root, nullptr, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, size_t{-1});
 }
 
 TEST_F(BookmarkModelTest, AddFolderWithWhitespaceTitle) {
@@ -632,8 +628,7 @@ TEST_F(BookmarkModelTest, AddFolderWithWhitespaceTitle) {
 
     const BookmarkNode* new_node = model_->AddFolder(root, i, title);
 
-    int size = i + 1;
-    EXPECT_EQ(size, root->child_count());
+    EXPECT_EQ(i + 1, root->children().size());
     EXPECT_EQ(ASCIIToUTF16(title_whitespace_test_cases[i].expected_title),
               new_node->GetTitle());
     EXPECT_EQ(BookmarkNode::FOLDER, new_node->type());
@@ -647,10 +642,10 @@ TEST_F(BookmarkModelTest, RemoveURL) {
   model_->AddURL(root, 0, title, url);
   ClearCounts();
 
-  model_->Remove(root->GetChild(0));
-  ASSERT_EQ(0, root->child_count());
+  model_->Remove(root->children().front().get());
+  ASSERT_EQ(0u, root->children().size());
   AssertObserverCount(0, 0, 1, 0, 0, 1, 0, 0, 0);
-  observer_details_.ExpectEquals(root, nullptr, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, size_t{-1});
 
   // Make sure there is no mapping for the URL.
   ASSERT_TRUE(model_->GetMostRecentlyAddedUserNodeForURL(url) == nullptr);
@@ -670,10 +665,10 @@ TEST_F(BookmarkModelTest, RemoveFolder) {
   ClearCounts();
 
   // Now remove the folder.
-  model_->Remove(root->GetChild(0));
-  ASSERT_EQ(0, root->child_count());
+  model_->Remove(root->children().front().get());
+  ASSERT_EQ(0u, root->children().size());
   AssertObserverCount(0, 0, 1, 0, 0, 1, 0, 0, 0);
-  observer_details_.ExpectEquals(root, nullptr, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, size_t{-1});
 
   // Make sure there is no mapping for the URL.
   ASSERT_TRUE(model_->GetMostRecentlyAddedUserNodeForURL(url) == nullptr);
@@ -697,7 +692,7 @@ TEST_F(BookmarkModelTest, RemoveAllUserBookmarks) {
   AssertObserverCount(3, 0, 0, 0, 0, 0, 0, 0, 0);
   ClearCounts();
 
-  int permanent_node_count = model_->root_node()->child_count();
+  size_t permanent_node_count = model_->root_node()->children().size();
 
   NodeRemovalDetail expected_node_removal_details[] = {
     NodeRemovalDetail(bookmark_bar_node, 1, url_node),
@@ -707,9 +702,9 @@ TEST_F(BookmarkModelTest, RemoveAllUserBookmarks) {
   model_->SetUndoDelegate(this);
   model_->RemoveAllUserBookmarks();
 
-  EXPECT_EQ(0, bookmark_bar_node->child_count());
+  EXPECT_EQ(0u, bookmark_bar_node->children().size());
   // No permanent node should be removed.
-  EXPECT_EQ(permanent_node_count, model_->root_node()->child_count());
+  EXPECT_EQ(permanent_node_count, model_->root_node()->children().size());
   // No individual BookmarkNodeRemoved events are fired, so removed count
   // should be 0.
   AssertObserverCount(0, 0, 0, 0, 0, 0, 0, 0, 1);
@@ -733,7 +728,7 @@ TEST_F(BookmarkModelTest, SetTitle) {
   title = ASCIIToUTF16("foo2");
   model_->SetTitle(node, title);
   AssertObserverCount(0, 0, 0, 1, 0, 0, 1, 0, 0);
-  observer_details_.ExpectEquals(node, nullptr, -1, -1);
+  observer_details_.ExpectEquals(node, nullptr, size_t{-1}, size_t{-1});
   EXPECT_EQ(title, node->GetTitle());
 }
 
@@ -762,7 +757,7 @@ TEST_F(BookmarkModelTest, SetURL) {
   url = GURL("http://foo2.com");
   model_->SetURL(node, url);
   AssertObserverCount(0, 0, 0, 1, 0, 0, 1, 0, 0);
-  observer_details_.ExpectEquals(node, nullptr, -1, -1);
+  observer_details_.ExpectEquals(node, nullptr, size_t{-1}, size_t{-1});
   EXPECT_EQ(url, node->url());
 }
 
@@ -794,18 +789,18 @@ TEST_F(BookmarkModelTest, Move) {
   AssertObserverCount(0, 1, 0, 0, 0, 0, 0, 0, 0);
   observer_details_.ExpectEquals(root, folder1, 1, 0);
   EXPECT_TRUE(folder1 == node->parent());
-  EXPECT_EQ(1, root->child_count());
-  EXPECT_EQ(folder1, root->GetChild(0));
-  EXPECT_EQ(1, folder1->child_count());
-  EXPECT_EQ(node, folder1->GetChild(0));
+  EXPECT_EQ(1u, root->children().size());
+  EXPECT_EQ(folder1, root->children().front().get());
+  EXPECT_EQ(1u, folder1->children().size());
+  EXPECT_EQ(node, folder1->children().front().get());
 
   // And remove the folder.
   ClearCounts();
-  model_->Remove(root->GetChild(0));
+  model_->Remove(root->children().front().get());
   AssertObserverCount(0, 0, 1, 0, 0, 1, 0, 0, 0);
-  observer_details_.ExpectEquals(root, nullptr, 0, -1);
+  observer_details_.ExpectEquals(root, nullptr, 0, size_t{-1});
   EXPECT_TRUE(model_->GetMostRecentlyAddedUserNodeForURL(url) == nullptr);
-  EXPECT_EQ(0, root->child_count());
+  EXPECT_EQ(0u, root->children().size());
 }
 
 TEST_F(BookmarkModelTest, NonMovingMoveCall) {
@@ -834,44 +829,44 @@ TEST_F(BookmarkModelTest, Copy) {
   EXPECT_EQ(model_string, actual_model_string);
 
   // Copy 'd' to be after '1:b': URL item from bar to folder.
-  const BookmarkNode* node_to_copy = root->GetChild(2);
-  const BookmarkNode* destination = root->GetChild(1);
+  const BookmarkNode* node_to_copy = root->children()[2].get();
+  const BookmarkNode* destination = root->children()[1].get();
   model_->Copy(node_to_copy, destination, 1);
   actual_model_string = test::ModelStringFromNode(root);
   EXPECT_EQ("a 1:[ b d c ] d 2:[ e f g ] h ", actual_model_string);
 
   // Copy '1:d' to be after 'a': URL item from folder to bar.
-  const BookmarkNode* folder = root->GetChild(1);
-  node_to_copy = folder->GetChild(1);
+  const BookmarkNode* folder = root->children()[1].get();
+  node_to_copy = folder->children()[1].get();
   model_->Copy(node_to_copy, root, 1);
   actual_model_string = test::ModelStringFromNode(root);
   EXPECT_EQ("a d 1:[ b d c ] d 2:[ e f g ] h ", actual_model_string);
 
   // Copy '1' to be after '2:e': Folder from bar to folder.
-  node_to_copy = root->GetChild(2);
-  destination = root->GetChild(4);
+  node_to_copy = root->children()[2].get();
+  destination = root->children()[4].get();
   model_->Copy(node_to_copy, destination, 1);
   actual_model_string = test::ModelStringFromNode(root);
   EXPECT_EQ("a d 1:[ b d c ] d 2:[ e 1:[ b d c ] f g ] h ",
             actual_model_string);
 
   // Copy '2:1' to be after '2:f': Folder within same folder.
-  folder = root->GetChild(4);
-  node_to_copy = folder->GetChild(1);
+  folder = root->children()[4].get();
+  node_to_copy = folder->children()[1].get();
   model_->Copy(node_to_copy, folder, 3);
   actual_model_string = test::ModelStringFromNode(root);
   EXPECT_EQ("a d 1:[ b d c ] d 2:[ e 1:[ b d c ] f 1:[ b d c ] g ] h ",
             actual_model_string);
 
   // Copy first 'd' to be after 'h': URL item within the bar.
-  node_to_copy = root->GetChild(1);
+  node_to_copy = root->children()[1].get();
   model_->Copy(node_to_copy, root, 6);
   actual_model_string = test::ModelStringFromNode(root);
   EXPECT_EQ("a d 1:[ b d c ] d 2:[ e 1:[ b d c ] f 1:[ b d c ] g ] h d ",
             actual_model_string);
 
   // Copy '2' to be after 'a': Folder within the bar.
-  node_to_copy = root->GetChild(4);
+  node_to_copy = root->children()[4].get();
   model_->Copy(node_to_copy, root, 1);
   actual_model_string = test::ModelStringFromNode(root);
   EXPECT_EQ("a 2:[ e 1:[ b d c ] f 1:[ b d c ] g ] d 1:[ b d c ] "
@@ -934,7 +929,7 @@ TEST_F(BookmarkModelTest, MostRecentlyModifiedFolders) {
 
   // Nuke the folder and do another fetch, making sure folder isn't in the
   // returned list.
-  model_->Remove(folder->parent()->GetChild(0));
+  model_->Remove(folder->parent()->children().front().get());
   most_recent_folders = GetMostRecentlyModifiedUserFolders(model_.get(), 1);
   ASSERT_EQ(1U, most_recent_folders.size());
   ASSERT_TRUE(most_recent_folders[0] != folder);
@@ -1041,12 +1036,12 @@ TEST_F(BookmarkModelTest, DISABLED_Sort) {
   const BookmarkNode* parent = model_->bookmark_bar_node();
   PopulateBookmarkNode(&bbn, model_.get(), parent);
 
-  BookmarkNode* child1 = AsMutable(parent->GetChild(1));
+  BookmarkNode* child1 = parent->children()[1].get();
   child1->SetTitle(ASCIIToUTF16("a"));
-  child1->Remove(child1->GetChild(0));
-  BookmarkNode* child3 = AsMutable(parent->GetChild(3));
+  child1->Remove(0);
+  BookmarkNode* child3 = parent->children()[3].get();
   child3->SetTitle(ASCIIToUTF16("C"));
-  child3->Remove(child3->GetChild(0));
+  child3->Remove(0);
 
   ClearCounts();
 
@@ -1058,10 +1053,10 @@ TEST_F(BookmarkModelTest, DISABLED_Sort) {
 
   // Make sure the order matches (remember, 'a' and 'C' are folders and
   // come first).
-  EXPECT_EQ(parent->GetChild(0)->GetTitle(), ASCIIToUTF16("a"));
-  EXPECT_EQ(parent->GetChild(1)->GetTitle(), ASCIIToUTF16("C"));
-  EXPECT_EQ(parent->GetChild(2)->GetTitle(), ASCIIToUTF16("B"));
-  EXPECT_EQ(parent->GetChild(3)->GetTitle(), ASCIIToUTF16("d"));
+  EXPECT_EQ(parent->children()[0]->GetTitle(), ASCIIToUTF16("a"));
+  EXPECT_EQ(parent->children()[1]->GetTitle(), ASCIIToUTF16("C"));
+  EXPECT_EQ(parent->children()[2]->GetTitle(), ASCIIToUTF16("B"));
+  EXPECT_EQ(parent->children()[3]->GetTitle(), ASCIIToUTF16("d"));
 }
 
 TEST_F(BookmarkModelTest, Reorder) {
@@ -1074,22 +1069,23 @@ TEST_F(BookmarkModelTest, Reorder) {
   ClearCounts();
 
   // Reorder bar node's bookmarks in reverse order.
-  std::vector<const BookmarkNode*> new_order;
-  new_order.push_back(parent->GetChild(3));
-  new_order.push_back(parent->GetChild(2));
-  new_order.push_back(parent->GetChild(1));
-  new_order.push_back(parent->GetChild(0));
+  std::vector<const BookmarkNode*> new_order = {
+      parent->children()[3].get(),
+      parent->children()[2].get(),
+      parent->children()[1].get(),
+      parent->children()[0].get(),
+  };
   model_->ReorderChildren(parent, new_order);
 
   // Make sure we were notified.
   AssertObserverCount(0, 0, 0, 0, 1, 0, 0, 1, 0);
 
   // Make sure the order matches is correct (it should be reversed).
-  ASSERT_EQ(4, parent->child_count());
-  EXPECT_EQ("D", base::UTF16ToASCII(parent->GetChild(0)->GetTitle()));
-  EXPECT_EQ("C", base::UTF16ToASCII(parent->GetChild(1)->GetTitle()));
-  EXPECT_EQ("B", base::UTF16ToASCII(parent->GetChild(2)->GetTitle()));
-  EXPECT_EQ("A", base::UTF16ToASCII(parent->GetChild(3)->GetTitle()));
+  ASSERT_EQ(4u, parent->children().size());
+  EXPECT_EQ("D", base::UTF16ToASCII(parent->children()[0]->GetTitle()));
+  EXPECT_EQ("C", base::UTF16ToASCII(parent->children()[1]->GetTitle()));
+  EXPECT_EQ("B", base::UTF16ToASCII(parent->children()[2]->GetTitle()));
+  EXPECT_EQ("A", base::UTF16ToASCII(parent->children()[3]->GetTitle()));
 }
 
 TEST_F(BookmarkModelTest, NodeVisibility) {
@@ -1114,7 +1110,7 @@ TEST_F(BookmarkModelTest, NodeVisibility) {
   PopulateNodeFromString("B", &bbn);
   const BookmarkNode* parent = model_->mobile_node();
   PopulateBookmarkNode(&bbn, model_.get(), parent);
-  EXPECT_TRUE(parent->GetChild(0)->IsVisible());
+  EXPECT_TRUE(parent->children().front()->IsVisible());
 
   // Mobile folder should be visible now that it has a child.
   EXPECT_TRUE(model_->mobile_node()->IsVisible());
@@ -1161,14 +1157,14 @@ TEST_F(BookmarkModelTest, MultipleExtensiveChangesObserver) {
 // and that IsBookmarkedByUser is true only if at least one of the matching
 // bookmarks can be edited by the user.
 TEST_F(BookmarkModelTest, IsBookmarked) {
-  // Reload the model with an extra node that is not editable by the user.
-  BookmarkPermanentNode* extra_node = ReloadModelWithExtraNode();
+  // Reload the model with a managed node that is not editable by the user.
+  BookmarkPermanentNode* managed_node = ReloadModelWithManagedNode();
 
   // "google.com" is a "user" bookmark.
   model_->AddURL(model_->other_node(), 0, base::ASCIIToUTF16("User"),
                  GURL("http://google.com"));
   // "youtube.com" is not.
-  model_->AddURL(extra_node, 0, base::ASCIIToUTF16("Extra"),
+  model_->AddURL(managed_node, 0, base::ASCIIToUTF16("Managed"),
                  GURL("http://youtube.com"));
 
   EXPECT_TRUE(model_->IsBookmarked(GURL("http://google.com")));
@@ -1183,12 +1179,12 @@ TEST_F(BookmarkModelTest, IsBookmarked) {
 // Verifies that GetMostRecentlyAddedUserNodeForURL skips bookmarks that
 // are not owned by the user.
 TEST_F(BookmarkModelTest, GetMostRecentlyAddedUserNodeForURLSkipsManagedNodes) {
-  // Reload the model with an extra node that is not editable by the user.
-  BookmarkPermanentNode* extra_node = ReloadModelWithExtraNode();
+  // Reload the model with a managed node that is not editable by the user.
+  BookmarkPermanentNode* managed_node = ReloadModelWithManagedNode();
 
   const base::string16 title = base::ASCIIToUTF16("Title");
   const BookmarkNode* user_parent = model_->other_node();
-  const BookmarkNode* managed_parent = extra_node;
+  const BookmarkNode* managed_parent = managed_node;
   const GURL url("http://google.com");
 
   // |url| is not bookmarked yet.
@@ -1359,7 +1355,7 @@ class BookmarkModelFaviconTest : public testing::Test,
   }
 
   bool WasNodeUpdated(const BookmarkNode* node) {
-    return base::ContainsValue(updated_nodes_, node);
+    return base::Contains(updated_nodes_, node);
   }
 
   void ClearUpdatedNodes() {
@@ -1372,17 +1368,17 @@ class BookmarkModelFaviconTest : public testing::Test,
 
   void BookmarkNodeMoved(BookmarkModel* model,
                          const BookmarkNode* old_parent,
-                         int old_index,
+                         size_t old_index,
                          const BookmarkNode* new_parent,
-                         int new_index) override {}
+                         size_t new_index) override {}
 
   void BookmarkNodeAdded(BookmarkModel* model,
                          const BookmarkNode* parent,
-                         int index) override {}
+                         size_t index) override {}
 
   void BookmarkNodeRemoved(BookmarkModel* model,
                            const BookmarkNode* parent,
-                           int old_index,
+                           size_t old_index,
                            const BookmarkNode* node,
                            const std::set<GURL>& removed_urls) override {}
 
