@@ -9,22 +9,20 @@
 
 #include <string>
 #include <utility>
-#include <vector>
 
+#include "base/metrics/field_trial_params.h"
 #include "base/rand_util.h"
+#include "base/stl_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/time/time.h"
-#include "components/data_reduction_proxy/core/common/data_reduction_proxy_event_creator.h"
+#include "components/data_reduction_proxy/core/common/data_reduction_proxy_features.h"
 #include "components/data_reduction_proxy/core/common/data_reduction_proxy_params.h"
 #include "net/http/http_response_headers.h"
 #include "net/http/http_status_code.h"
 #include "net/http/http_util.h"
-#include "net/url_request/url_request.h"
-
-using base::StringPiece;
-using base::TimeDelta;
+#include "services/network/public/cpp/resource_response.h"
 
 namespace {
 
@@ -41,40 +39,29 @@ const char kChromeProxyContentTransformHeader[] =
 const char kActionValueDelimiter = '=';
 
 // Previews directives.
-const char kEmptyImageDirective[] = "empty-image";
 const char kLitePageDirective[] = "lite-page";
 const char kCompressedVideoDirective[] = "compressed-video";
 const char kIdentityDirective[] = "identity";
 const char kChromeProxyPagePoliciesDirective[] = "page-policies";
 
-// The legacy Chrome-Proxy response header directive for LoFi images.
-const char kLegacyChromeProxyLoFiResponseDirective[] = "q=low";
-
-const char kChromeProxyExperimentForceLitePage[] = "force_lite_page";
-const char kChromeProxyExperimentForceEmptyImage[] =
-    "force_page_policies_empty_image";
-
-const char kIfHeavyQualifier[] = "if-heavy";
-
 const char kChromeProxyActionBlockOnce[] = "block-once";
 const char kChromeProxyActionBlock[] = "block";
 const char kChromeProxyActionBypass[] = "bypass";
 
-// Actions for tamper detection fingerprints.
-const char kChromeProxyActionFingerprintChromeProxy[]   = "fcp";
-const char kChromeProxyActionFingerprintVia[]           = "fvia";
-const char kChromeProxyActionFingerprintOtherHeaders[]  = "foh";
-const char kChromeProxyActionFingerprintContentLength[] = "fcl";
-
 const int kShortBypassMaxSeconds = 59;
 const int kMediumBypassMaxSeconds = 300;
 
+base::TimeDelta GetRandomBypassTime(base::TimeDelta min_time,
+                                    base::TimeDelta max_time) {
+  const int64_t delta_ms =
+      base::RandInt(min_time.InMilliseconds(), max_time.InMilliseconds());
+  return base::TimeDelta::FromMilliseconds(delta_ms);
+}
+
 // Returns a random bypass duration between 1 and 5 minutes.
 base::TimeDelta GetDefaultBypassDuration() {
-  const int64_t delta_ms =
-      base::RandInt(base::TimeDelta::FromMinutes(1).InMilliseconds(),
-                    base::TimeDelta::FromMinutes(5).InMilliseconds());
-  return TimeDelta::FromMilliseconds(delta_ms);
+  return GetRandomBypassTime(base::TimeDelta::FromMinutes(1),
+                             base::TimeDelta::FromMinutes(5));
 }
 
 bool StartsWithActionPrefix(base::StringPiece header_value,
@@ -113,41 +100,6 @@ bool IsPreviewType(const net::HttpResponseHeaders& headers,
          IsPreviewTypeInHeaderValue(value, transform_type);
 }
 
-// Returns true if there is a cycle in |url_chain|.
-bool HasURLRedirectCycle(const std::vector<GURL>& url_chain) {
-  if (url_chain.size() <= 1)
-    return false;
-
-  // If the last entry occurs earlier in the |url_chain|, then very likely there
-  // is a redirect cycle.
-  return std::find(url_chain.rbegin() + 1, url_chain.rend(),
-                   url_chain.back()) != url_chain.rend();
-}
-
-data_reduction_proxy::TransformDirective ParsePagePolicyDirective(
-    const std::string chrome_proxy_header_value) {
-  for (const auto& directive : base::SplitStringPiece(
-           chrome_proxy_header_value, ",", base::TRIM_WHITESPACE,
-           base::SPLIT_WANT_NONEMPTY)) {
-    if (!base::StartsWith(directive, kChromeProxyPagePoliciesDirective,
-                          base::CompareCase::INSENSITIVE_ASCII)) {
-      continue;
-    }
-
-    // Check policy directive for empty-image entry.
-    base::StringPiece page_policies_value = base::StringPiece(directive).substr(
-        arraysize(kChromeProxyPagePoliciesDirective));
-    for (const auto& policy :
-         base::SplitStringPiece(page_policies_value, "|", base::TRIM_WHITESPACE,
-                                base::SPLIT_WANT_NONEMPTY)) {
-      if (base::LowerCaseEqualsASCII(policy, kEmptyImageDirective)) {
-        return data_reduction_proxy::TRANSFORM_PAGE_POLICIES_EMPTY_IMAGE;
-      }
-    }
-  }
-  return data_reduction_proxy::TRANSFORM_NONE;
-}
-
 }  // namespace
 
 namespace data_reduction_proxy {
@@ -172,10 +124,6 @@ const char* chrome_proxy_content_transform_header() {
   return kChromeProxyContentTransformHeader;
 }
 
-const char* empty_image_directive() {
-  return kEmptyImageDirective;
-}
-
 const char* lite_page_directive() {
   return kLitePageDirective;
 }
@@ -184,16 +132,18 @@ const char* compressed_video_directive() {
   return kCompressedVideoDirective;
 }
 
-const char* chrome_proxy_experiment_force_lite_page() {
-  return kChromeProxyExperimentForceLitePage;
+const char* page_policies_directive() {
+  return kChromeProxyPagePoliciesDirective;
 }
 
-const char* chrome_proxy_experiment_force_empty_image() {
-  return kChromeProxyExperimentForceEmptyImage;
-}
+bool HasURLRedirectCycle(const std::vector<GURL>& url_chain) {
+  if (url_chain.size() <= 1)
+    return false;
 
-const char* if_heavy_qualifier() {
-  return kIfHeavyQualifier;
+  // If the last entry occurs earlier in the |url_chain|, then very likely there
+  // is a redirect cycle.
+  return std::find(url_chain.rbegin() + 1, url_chain.rend(),
+                   url_chain.back()) != url_chain.rend();
 }
 
 TransformDirective ParseRequestTransform(
@@ -207,18 +157,16 @@ TransformDirective ParseRequestTransform(
   if (base::LowerCaseEqualsASCII(accept_transform_value,
                                  lite_page_directive())) {
     return TRANSFORM_LITE_PAGE;
-  } else if (base::LowerCaseEqualsASCII(accept_transform_value,
-                                        empty_image_directive())) {
-    return TRANSFORM_EMPTY_IMAGE;
-  } else if (base::LowerCaseEqualsASCII(accept_transform_value,
-                                        compressed_video_directive())) {
-    return TRANSFORM_COMPRESSED_VIDEO;
-  } else if (base::LowerCaseEqualsASCII(accept_transform_value,
-                                        kIdentityDirective)) {
-    return TRANSFORM_IDENTITY;
-  } else {
-    return TRANSFORM_NONE;
   }
+  if (base::LowerCaseEqualsASCII(accept_transform_value,
+                                 compressed_video_directive())) {
+    return TRANSFORM_COMPRESSED_VIDEO;
+  }
+  if (base::LowerCaseEqualsASCII(accept_transform_value, kIdentityDirective)) {
+    return TRANSFORM_IDENTITY;
+  }
+
+  return TRANSFORM_NONE;
 }
 
 TransformDirective ParseResponseTransform(
@@ -226,50 +174,20 @@ TransformDirective ParseResponseTransform(
   std::string content_transform_value;
   if (!headers.GetNormalizedHeader(chrome_proxy_content_transform_header(),
                                    &content_transform_value)) {
-    // No content-transform so check for page-policies in chrome-proxy header.
-    std::string chrome_proxy_header_value;
-    if (headers.GetNormalizedHeader(chrome_proxy_header(),
-                                    &chrome_proxy_header_value)) {
-      return ParsePagePolicyDirective(chrome_proxy_header_value);
-    }
-  } else if (base::LowerCaseEqualsASCII(content_transform_value,
-                                        lite_page_directive())) {
+    return TRANSFORM_NONE;
+  }
+  if (base::LowerCaseEqualsASCII(content_transform_value,
+                                 lite_page_directive())) {
     return TRANSFORM_LITE_PAGE;
-  } else if (base::LowerCaseEqualsASCII(content_transform_value,
-                                        empty_image_directive())) {
-    return TRANSFORM_EMPTY_IMAGE;
-  } else if (base::LowerCaseEqualsASCII(content_transform_value,
-                                        kIdentityDirective)) {
+  }
+  if (base::LowerCaseEqualsASCII(content_transform_value, kIdentityDirective)) {
     return TRANSFORM_IDENTITY;
-  } else {
-    NOTREACHED() << "Unexpected content transform header: "
-                 << content_transform_value;
   }
-  return TRANSFORM_NONE;
-}
-
-bool IsEmptyImagePreview(const net::HttpResponseHeaders& headers) {
-  return IsPreviewType(headers, kEmptyImageDirective) ||
-         headers.HasHeaderValue(kChromeProxyHeader,
-                                kLegacyChromeProxyLoFiResponseDirective);
-}
-
-bool IsEmptyImagePreview(const std::string& content_transform_value,
-                         const std::string& chrome_proxy_value) {
-  if (IsPreviewTypeInHeaderValue(content_transform_value, kEmptyImageDirective))
-    return true;
-
-  // Look for "q=low" in the "Chrome-Proxy" response header.
-  net::HttpUtil::ValuesIterator values(chrome_proxy_value.begin(),
-                                       chrome_proxy_value.end(), ',');
-  while (values.GetNext()) {
-    base::StringPiece value(values.value_begin(), values.value_end());
-    if (base::LowerCaseEqualsASCII(value,
-                                   kLegacyChromeProxyLoFiResponseDirective)) {
-      return true;
-    }
+  if (base::LowerCaseEqualsASCII(content_transform_value,
+                                 compressed_video_directive())) {
+    return TRANSFORM_COMPRESSED_VIDEO;
   }
-  return false;
+  return TRANSFORM_UNKNOWN;
 }
 
 bool IsLitePagePreview(const net::HttpResponseHeaders& headers) {
@@ -303,12 +221,13 @@ bool ParseHeadersAndSetBypassDuration(const net::HttpResponseHeaders& headers,
     if (StartsWithActionPrefix(value, action_prefix)) {
       int64_t seconds;
       if (!base::StringToInt64(
-              StringPiece(value).substr(action_prefix.size() + 1), &seconds) ||
+              base::StringPiece(value).substr(action_prefix.size() + 1),
+              &seconds) ||
           seconds < 0) {
         continue;  // In case there is a well formed instruction.
       }
       if (seconds != 0) {
-        *bypass_duration = TimeDelta::FromSeconds(seconds);
+        *bypass_duration = base::TimeDelta::FromSeconds(seconds);
       } else {
         // Server deferred to us to choose a duration. Default to a random
         // duration between one and five minutes.
@@ -339,7 +258,6 @@ bool ParseHeadersForBypassInfo(const net::HttpResponseHeaders& headers,
           headers, kChromeProxyActionBlock, &proxy_info->bypass_duration)) {
     proxy_info->bypass_all = true;
     proxy_info->mark_proxies_as_bad = true;
-    proxy_info->bypass_action = BYPASS_ACTION_TYPE_BLOCK;
     return true;
   }
 
@@ -348,7 +266,6 @@ bool ParseHeadersForBypassInfo(const net::HttpResponseHeaders& headers,
           headers, kChromeProxyActionBypass, &proxy_info->bypass_duration)) {
     proxy_info->bypass_all = false;
     proxy_info->mark_proxies_as_bad = true;
-    proxy_info->bypass_action = BYPASS_ACTION_TYPE_BYPASS;
     return true;
   }
 
@@ -360,8 +277,7 @@ bool ParseHeadersForBypassInfo(const net::HttpResponseHeaders& headers,
   if (headers.HasHeaderValue(kChromeProxyHeader, kChromeProxyActionBlockOnce)) {
     proxy_info->bypass_all = true;
     proxy_info->mark_proxies_as_bad = false;
-    proxy_info->bypass_duration = TimeDelta();
-    proxy_info->bypass_action = BYPASS_ACTION_TYPE_BLOCK_ONCE;
+    proxy_info->bypass_duration = base::TimeDelta();
     return true;
   }
 
@@ -380,7 +296,7 @@ bool HasDataReductionProxyViaHeader(const net::HttpResponseHeaders& headers,
   // 'Via: 1.1 Chrome-Compression-Proxy'
   while (headers.EnumerateHeader(&iter, "via", &value)) {
     if (base::StringPiece(value).substr(
-            kVersionSize, arraysize(kDataReductionProxyViaValue) - 1) ==
+            kVersionSize, base::size(kDataReductionProxyViaValue) - 1) ==
         kDataReductionProxyViaValue) {
       if (has_intermediary)
         // We assume intermediary exists if there is another Via header after
@@ -399,13 +315,22 @@ DataReductionProxyBypassType GetDataReductionProxyBypassType(
     DataReductionProxyInfo* data_reduction_proxy_info) {
   DCHECK(data_reduction_proxy_info);
 
+  // Responses from the warmup URL probe should not be checked for bypass types.
+  // Doing so may unnecessarily cause all data saver proxies to be marked as
+  // bad (e.g., when via header is missing on the response to the probe from the
+  // HTTP proxy).
+  DCHECK(url_chain.empty() || (params::GetWarmupURL() != url_chain.back()));
+
   bool has_via_header = HasDataReductionProxyViaHeader(headers, nullptr);
 
-  if (has_via_header && HasURLRedirectCycle(url_chain)) {
+  // The following logic to detect redirect cycles works only when network
+  // servicification is disabled.
+  if (!base::FeatureList::IsEnabled(
+          features::kDataReductionProxyEnabledWithNetworkService) &&
+      has_via_header && HasURLRedirectCycle(url_chain)) {
     data_reduction_proxy_info->bypass_all = true;
     data_reduction_proxy_info->mark_proxies_as_bad = false;
     data_reduction_proxy_info->bypass_duration = base::TimeDelta();
-    data_reduction_proxy_info->bypass_action = BYPASS_ACTION_TYPE_BLOCK_ONCE;
     return BYPASS_EVENT_TYPE_URL_REDIRECT_CYCLE;
   }
 
@@ -415,10 +340,11 @@ DataReductionProxyBypassType GetDataReductionProxyBypassType(
     if (!data_reduction_proxy_info->mark_proxies_as_bad)
       return BYPASS_EVENT_TYPE_CURRENT;
 
-    const TimeDelta& duration = data_reduction_proxy_info->bypass_duration;
-    if (duration <= TimeDelta::FromSeconds(kShortBypassMaxSeconds))
+    const base::TimeDelta& duration =
+        data_reduction_proxy_info->bypass_duration;
+    if (duration <= base::TimeDelta::FromSeconds(kShortBypassMaxSeconds))
       return BYPASS_EVENT_TYPE_SHORT;
-    if (duration <= TimeDelta::FromSeconds(kMediumBypassMaxSeconds))
+    if (duration <= base::TimeDelta::FromSeconds(kMediumBypassMaxSeconds))
       return BYPASS_EVENT_TYPE_MEDIUM;
     return BYPASS_EVENT_TYPE_LONG;
   }
@@ -432,17 +358,41 @@ DataReductionProxyBypassType GetDataReductionProxyBypassType(
   // Fall back if a 500, 502 or 503 is returned.
   if (headers.response_code() == net::HTTP_INTERNAL_SERVER_ERROR)
     return BYPASS_EVENT_TYPE_STATUS_500_HTTP_INTERNAL_SERVER_ERROR;
-  if (headers.response_code() == net::HTTP_BAD_GATEWAY)
+  if (headers.response_code() == net::HTTP_BAD_GATEWAY) {
+    if (base::FeatureList::IsEnabled(
+            features::kDataReductionProxyBlockOnBadGatewayResponse)) {
+      // When 502 response is received with no valid directive in Chrome-Proxy
+      // header, it is likely the renderer may have been blocked in receiving
+      // the headers due to CORB, etc. In this case, block-once or a block all
+      // proxies for a small number of seconds can be an alternative.
+      data_reduction_proxy_info->bypass_all = true;
+      data_reduction_proxy_info->bypass_duration =
+          base::TimeDelta::FromSeconds(base::GetFieldTrialParamByFeatureAsInt(
+              features::kDataReductionProxyBlockOnBadGatewayResponse,
+              "block_duration_seconds", 1));
+      return BYPASS_EVENT_TYPE_SHORT;
+    }
     return BYPASS_EVENT_TYPE_STATUS_502_HTTP_BAD_GATEWAY;
+  }
   if (headers.response_code() == net::HTTP_SERVICE_UNAVAILABLE)
     return BYPASS_EVENT_TYPE_STATUS_503_HTTP_SERVICE_UNAVAILABLE;
   // TODO(kundaji): Bypass if Proxy-Authenticate header value cannot be
   // interpreted by data reduction proxy.
   if (headers.response_code() == net::HTTP_PROXY_AUTHENTICATION_REQUIRED &&
       !headers.HasHeader("Proxy-Authenticate")) {
+    // Bypass all proxies for a few RTTs until the config fetch could update the
+    // session key. The value 500 ms is the RTT observed for config fetches.
+    data_reduction_proxy_info->bypass_all = true;
+    data_reduction_proxy_info->bypass_duration =
+        base::TimeDelta::FromMilliseconds(3 * 500);
     return BYPASS_EVENT_TYPE_MALFORMED_407;
   }
-  if (!has_via_header && (headers.response_code() != net::HTTP_NOT_MODIFIED)) {
+
+  bool disable_bypass_on_missing_via_header =
+      params::IsWarmupURLFetchCallbackEnabled();
+
+  if (!has_via_header && !disable_bypass_on_missing_via_header &&
+      (headers.response_code() != net::HTTP_NOT_MODIFIED)) {
     // A Via header might not be present in a 304. Since the goal of a 304
     // response is to minimize information transfer, a sender in general
     // should not generate representation metadata other than Cache-Control,
@@ -457,67 +407,45 @@ DataReductionProxyBypassType GetDataReductionProxyBypassType(
       // bypass the data reduction proxy for the current request.
       data_reduction_proxy_info->bypass_all = true;
       data_reduction_proxy_info->mark_proxies_as_bad = false;
-      data_reduction_proxy_info->bypass_duration = TimeDelta();
+      data_reduction_proxy_info->bypass_duration = base::TimeDelta();
       return BYPASS_EVENT_TYPE_MISSING_VIA_HEADER_4XX;
     }
 
-    // Missing the via header should not trigger bypass if the client is
-    // included in the tamper detection experiment.
-    if (!params::IsIncludedInTamperDetectionExperiment())
-      return BYPASS_EVENT_TYPE_MISSING_VIA_HEADER_OTHER;
+    data_reduction_proxy_info->mark_proxies_as_bad = true;
+    data_reduction_proxy_info->bypass_duration = GetRandomBypassTime(
+        base::TimeDelta::FromSeconds(60), base::TimeDelta::FromSeconds(300));
+
+    return BYPASS_EVENT_TYPE_MISSING_VIA_HEADER_OTHER;
   }
   // There is no bypass event.
   return BYPASS_EVENT_TYPE_MAX;
 }
 
-bool GetDataReductionProxyActionFingerprintChromeProxy(
-    const net::HttpResponseHeaders* headers,
-    std::string* chrome_proxy_fingerprint) {
-  return GetDataReductionProxyActionValue(
-      headers,
-      kChromeProxyActionFingerprintChromeProxy,
-      chrome_proxy_fingerprint);
-}
-
-bool GetDataReductionProxyActionFingerprintVia(
-    const net::HttpResponseHeaders* headers,
-    std::string* via_fingerprint) {
-  return GetDataReductionProxyActionValue(
-      headers,
-      kChromeProxyActionFingerprintVia,
-      via_fingerprint);
-}
-
-bool GetDataReductionProxyActionFingerprintOtherHeaders(
-    const net::HttpResponseHeaders* headers,
-    std::string* other_headers_fingerprint) {
-  return GetDataReductionProxyActionValue(
-      headers,
-      kChromeProxyActionFingerprintOtherHeaders,
-      other_headers_fingerprint);
-}
-
-bool GetDataReductionProxyActionFingerprintContentLength(
-    const net::HttpResponseHeaders* headers,
-    std::string* content_length_fingerprint) {
-  return GetDataReductionProxyActionValue(
-      headers,
-      kChromeProxyActionFingerprintContentLength,
-      content_length_fingerprint);
-}
-
-void GetDataReductionProxyHeaderWithFingerprintRemoved(
-    const net::HttpResponseHeaders* headers,
-    std::vector<std::string>* values) {
-  DCHECK(values);
-
-  std::string value;
-  size_t iter = 0;
-  while (headers->EnumerateHeader(&iter, kChromeProxyHeader, &value)) {
-    if (StartsWithActionPrefix(value, kChromeProxyActionFingerprintChromeProxy))
-      continue;
-    values->push_back(std::move(value));
+int64_t GetDataReductionProxyOFCL(const net::HttpResponseHeaders* headers) {
+  std::string ofcl_str;
+  int64_t ofcl;
+  if (GetDataReductionProxyActionValue(headers, "ofcl", &ofcl_str) &&
+      base::StringToInt64(ofcl_str, &ofcl) && ofcl >= 0) {
+    return ofcl;
   }
+  return -1;
+}
+
+double EstimateCompressionRatioFromHeaders(
+    const network::ResourceResponseHead* response_head) {
+  if (!response_head->network_accessed || !response_head->headers ||
+      response_head->headers->GetContentLength() <= 0 ||
+      response_head->proxy_server.is_direct()) {
+    return 1.0;  // No compression
+  }
+
+  int64_t original_content_length =
+      GetDataReductionProxyOFCL(response_head->headers.get());
+  if (original_content_length > 0) {
+    return static_cast<double>(original_content_length) /
+           static_cast<double>(response_head->headers->GetContentLength());
+  }
+  return 1.0;  // No compression
 }
 
 }  // namespace data_reduction_proxy
