@@ -34,6 +34,7 @@ const char* kLogTypeUsbDesc = "USB";
 const char* kLogTypeHidDesc = "HID";
 const char* kLogTypeMemoryDesc = "Memory";
 const char* kLogTypePrinterDesc = "Printer";
+const char* kLogTypeFidoDesc = "FIDO";
 
 std::string GetLogTypeString(LogType type) {
   switch (type) {
@@ -53,6 +54,8 @@ std::string GetLogTypeString(LogType type) {
       return kLogTypeMemoryDesc;
     case LOG_TYPE_PRINTER:
       return kLogTypePrinterDesc;
+    case LOG_TYPE_FIDO:
+      return kLogTypeFidoDesc;
     case LOG_TYPE_UNKNOWN:
       break;
   }
@@ -248,9 +251,7 @@ void DeviceEventLogImpl::SendToVLogOrErrorLog(const char* file,
 DeviceEventLogImpl::DeviceEventLogImpl(
     scoped_refptr<base::SingleThreadTaskRunner> task_runner,
     size_t max_entries)
-    : task_runner_(task_runner),
-      max_entries_(max_entries),
-      weak_ptr_factory_(this) {
+    : task_runner_(task_runner), max_entries_(max_entries) {
   DCHECK(task_runner_);
 }
 
@@ -401,6 +402,10 @@ std::string DeviceEventLogImpl::GetAsString(StringOrder order,
   return result;
 }
 
+void DeviceEventLogImpl::ClearAll() {
+  entries_.clear();
+}
+
 void DeviceEventLogImpl::Clear(const base::Time& begin, const base::Time& end) {
   auto begin_it = std::find_if(
       entries_.begin(), entries_.end(),
