@@ -6,6 +6,7 @@
 
 #include <utility>
 
+#include "base/bind.h"
 #include "base/location.h"
 #include "base/memory/weak_ptr.h"
 #include "base/path_service.h"
@@ -112,8 +113,8 @@ class DistillerPageWebContentsTest : public ContentBrowserTest {
     quit_closure.Run();
   }
 
-  void OnJsExecutionDone(base::Closure callback, const base::Value* value) {
-    js_result_.reset(value->DeepCopy());
+  void OnJsExecutionDone(base::Closure callback, base::Value value) {
+    js_result_ = std::move(value);
     callback.Run();
   }
 
@@ -151,7 +152,7 @@ class DistillerPageWebContentsTest : public ContentBrowserTest {
 
   DistillerPageWebContents* distiller_page_;
   std::unique_ptr<proto::DomDistillerResult> distiller_result_;
-  std::unique_ptr<base::Value> js_result_;
+  base::Value js_result_;
 };
 
 // Use this class to be able to leak the WebContents, which is needed for when
@@ -330,8 +331,7 @@ IN_PROC_BROWSER_TEST_F(DistillerPageWebContentsTest,
   bool expect_new_web_contents = true;
   bool setup_main_frame_observer = true;
   bool wait_for_document_loaded = true;
-  RunUseCurrentWebContentsTest(url,
-                               expect_new_web_contents,
+  RunUseCurrentWebContentsTest(url, expect_new_web_contents,
                                setup_main_frame_observer,
                                wait_for_document_loaded);
 }
@@ -349,8 +349,7 @@ IN_PROC_BROWSER_TEST_F(DistillerPageWebContentsTest,
   bool expect_new_web_contents = true;
   bool setup_main_frame_observer = false;
   bool wait_for_document_loaded = true;
-  RunUseCurrentWebContentsTest(url,
-                               expect_new_web_contents,
+  RunUseCurrentWebContentsTest(url, expect_new_web_contents,
                                setup_main_frame_observer,
                                wait_for_document_loaded);
 }
@@ -368,8 +367,7 @@ IN_PROC_BROWSER_TEST_F(DistillerPageWebContentsTest,
   bool expect_new_web_contents = false;
   bool setup_main_frame_observer = true;
   bool wait_for_document_loaded = false;
-  RunUseCurrentWebContentsTest(url,
-                               expect_new_web_contents,
+  RunUseCurrentWebContentsTest(url, expect_new_web_contents,
                                setup_main_frame_observer,
                                wait_for_document_loaded);
 }
@@ -387,8 +385,7 @@ IN_PROC_BROWSER_TEST_F(DistillerPageWebContentsTest,
   bool expect_new_web_contents = false;
   bool setup_main_frame_observer = true;
   bool wait_for_document_loaded = true;
-  RunUseCurrentWebContentsTest(url,
-                               expect_new_web_contents,
+  RunUseCurrentWebContentsTest(url, expect_new_web_contents,
                                setup_main_frame_observer,
                                wait_for_document_loaded);
 }
@@ -408,10 +405,8 @@ void DistillerPageWebContentsTest::RunUseCurrentWebContentsTest(
                                                url_loaded_runner.QuitClosure(),
                                                wait_for_document_loaded);
   current_web_contents->GetController().LoadURL(
-      embedded_test_server()->GetURL(url),
-      content::Referrer(),
-      ui::PAGE_TRANSITION_TYPED,
-      std::string());
+      embedded_test_server()->GetURL(url), content::Referrer(),
+      ui::PAGE_TRANSITION_TYPED, std::string());
   url_loaded_runner.Run();
 
   std::unique_ptr<SourcePageHandleWebContents> source_page_handle(
@@ -447,14 +442,11 @@ IN_PROC_BROWSER_TEST_F(DistillerPageWebContentsTest,
       current_web_contents);
 
   base::RunLoop url_loaded_runner;
-  WebContentsMainFrameHelper main_frame_loaded(current_web_contents,
-                                               url_loaded_runner.QuitClosure(),
-                                               true);
+  WebContentsMainFrameHelper main_frame_loaded(
+      current_web_contents, url_loaded_runner.QuitClosure(), true);
   current_web_contents->GetController().LoadURL(
-      embedded_test_server()->GetURL(kSimpleArticlePath),
-      content::Referrer(),
-      ui::PAGE_TRANSITION_TYPED,
-      std::string());
+      embedded_test_server()->GetURL(kSimpleArticlePath), content::Referrer(),
+      ui::PAGE_TRANSITION_TYPED, std::string());
   url_loaded_runner.Run();
 
   std::unique_ptr<SourcePageHandleWebContents> source_page_handle(
@@ -567,33 +559,26 @@ IN_PROC_BROWSER_TEST_F(DistillerPageWebContentsTest, MAYBE_TestPinch) {
   dom_distiller::WebContentsMainFrameObserver::CreateForWebContents(
       web_contents);
   base::RunLoop url_loaded_runner;
-  WebContentsMainFrameHelper main_frame_loaded(web_contents,
-                                               url_loaded_runner.QuitClosure(),
-                                               true);
+  WebContentsMainFrameHelper main_frame_loaded(
+      web_contents, url_loaded_runner.QuitClosure(), true);
   web_contents->GetController().LoadURL(
-      embedded_test_server()->GetURL("/pinch_tester.html"),
-      content::Referrer(),
-      ui::PAGE_TRANSITION_TYPED,
-      std::string());
+      embedded_test_server()->GetURL("/pinch_tester.html"), content::Referrer(),
+      ui::PAGE_TRANSITION_TYPED, std::string());
   url_loaded_runner.Run();
 
   // Execute the JS to run the tests, and wait until it has finished.
   base::RunLoop run_loop;
   web_contents->GetMainFrame()->ExecuteJavaScriptForTests(
       base::UTF8ToUTF16("(function() {return pinchtest.run();})();"),
-      base::Bind(&DistillerPageWebContentsTest::OnJsExecutionDone,
-                 base::Unretained(this), run_loop.QuitClosure()));
+      base::BindOnce(&DistillerPageWebContentsTest::OnJsExecutionDone,
+                     base::Unretained(this), run_loop.QuitClosure()));
   run_loop.Run();
 
-  // Convert to dictionary and parse the results.
-  const base::DictionaryValue* dict;
-  ASSERT_TRUE(js_result_);
-  ASSERT_TRUE(js_result_->GetAsDictionary(&dict));
+  ASSERT_TRUE(js_result_.is_dict());
 
-  ASSERT_TRUE(dict->HasKey("success"));
-  bool success;
-  ASSERT_TRUE(dict->GetBoolean("success", &success));
-  EXPECT_TRUE(success);
+  base::Optional<bool> value = js_result_.FindBoolKey("success");
+  ASSERT_TRUE(value.has_value());
+  EXPECT_TRUE(value.value());
 }
 
 }  // namespace dom_distiller
