@@ -10,6 +10,7 @@
 #include <map>
 #include <utility>
 
+#include "base/bind.h"
 #include "base/bind_helpers.h"
 #include "base/logging.h"
 #include "base/metrics/histogram_macros.h"
@@ -170,8 +171,7 @@ BrowsingHistoryService::BrowsingHistoryService(
       driver_(driver),
       local_history_(local_history),
       sync_service_(sync_service),
-      clock_(new base::DefaultClock()),
-      weak_factory_(this) {
+      clock_(new base::DefaultClock()) {
   DCHECK(driver_);
 
   // Get notifications when history is cleared.
@@ -253,8 +253,8 @@ void BrowsingHistoryService::QueryHistoryInternal(
           state->search_text,
           OptionsWithEndTime(state->original_options,
                              state->local_end_time_for_continuation),
-          base::Bind(&BrowsingHistoryService::QueryComplete,
-                     weak_factory_.GetWeakPtr(), state),
+          base::BindOnce(&BrowsingHistoryService::QueryComplete,
+                         weak_factory_.GetWeakPtr(), state),
           &query_task_tracker_);
     }
   } else {
@@ -536,11 +536,11 @@ void BrowsingHistoryService::MergeDuplicateResults(
 
 void BrowsingHistoryService::QueryComplete(
     scoped_refptr<QueryHistoryState> state,
-    QueryResults* results) {
+    QueryResults results) {
   std::vector<HistoryEntry>& output = state->local_results;
-  output.reserve(output.size() + results->size());
+  output.reserve(output.size() + results.size());
 
-  for (const auto& page : *results) {
+  for (const auto& page : results) {
     // TODO(dubroy): Use sane time (crbug.com/146090) here when it's ready.
     output.emplace_back(HistoryEntry(
         HistoryEntry::LOCAL_ENTRY, page.url(), page.title(), page.visit_time(),
@@ -549,7 +549,7 @@ void BrowsingHistoryService::QueryComplete(
   }
 
   state->local_status =
-      results->reached_beginning() ? REACHED_BEGINNING : MORE_RESULTS;
+      results.reached_beginning() ? REACHED_BEGINNING : MORE_RESULTS;
 
   if (!web_history_timer_->IsRunning())
     ReturnResultsToDriver(std::move(state));
