@@ -12,9 +12,9 @@
 
 #include "base/bind.h"
 #include "base/bind_helpers.h"
+#include "base/hash/md5.h"
 #include "base/location.h"
 #include "base/logging.h"
-#include "base/md5.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/string_util.h"
@@ -30,7 +30,6 @@
 #include "components/history/core/browser/page_usage_data.h"
 #include "components/history/core/browser/top_sites_cache.h"
 #include "components/history/core/browser/top_sites_observer.h"
-#include "components/history/core/browser/top_sites_provider.h"
 #include "components/history/core/browser/url_utils.h"
 #include "components/prefs/pref_registry_simple.h"
 #include "components/prefs/pref_service.h"
@@ -89,7 +88,6 @@ bool TopSitesImpl::histogram_recorded_ = false;
 
 TopSitesImpl::TopSitesImpl(PrefService* pref_service,
                            HistoryService* history_service,
-                           std::unique_ptr<TopSitesProvider> provider,
                            const PrepopulatedPageList& prepopulated_pages,
                            const CanAddURLToHistoryFn& can_add_url_to_history)
     : backend_(nullptr),
@@ -98,13 +96,11 @@ TopSitesImpl::TopSitesImpl(PrefService* pref_service,
       prepopulated_pages_(prepopulated_pages),
       pref_service_(pref_service),
       history_service_(history_service),
-      provider_(std::move(provider)),
       can_add_url_to_history_(can_add_url_to_history),
       loaded_(false),
       history_service_observer_(this) {
   DCHECK(pref_service_);
   DCHECK(!can_add_url_to_history_.is_null());
-  DCHECK(provider_);
 }
 
 void TopSitesImpl::Init(const base::FilePath& db_name) {
@@ -245,16 +241,18 @@ void TopSitesImpl::RegisterPrefs(PrefRegistrySimple* registry) {
 TopSitesImpl::~TopSitesImpl() = default;
 
 void TopSitesImpl::StartQueryForMostVisited() {
+  const int kDaysOfHistory = 90;
+
   DCHECK(loaded_);
   timer_.Stop();
 
   if (!history_service_)
     return;
 
-  provider_->ProvideTopSites(
-      num_results_to_request_from_history(),
-      base::Bind(&TopSitesImpl::OnTopSitesAvailableFromHistory,
-                 base::Unretained(this)),
+  history_service_->QueryMostVisitedURLs(
+      num_results_to_request_from_history(), kDaysOfHistory,
+      base::BindOnce(&TopSitesImpl::OnTopSitesAvailableFromHistory,
+                     base::Unretained(this)),
       &cancelable_task_tracker_);
 }
 
@@ -471,10 +469,8 @@ void TopSitesImpl::OnGotMostVisitedURLs(
                &TopSitesImpl::StartQueryForMostVisited);
 }
 
-void TopSitesImpl::OnTopSitesAvailableFromHistory(
-    const MostVisitedURLList* pages) {
-  DCHECK(pages);
-  SetTopSites(*pages, CALL_LOCATION_FROM_OTHER_PLACES);
+void TopSitesImpl::OnTopSitesAvailableFromHistory(MostVisitedURLList pages) {
+  SetTopSites(pages, CALL_LOCATION_FROM_OTHER_PLACES);
 }
 
 void TopSitesImpl::OnURLsDeleted(HistoryService* history_service,
