@@ -20,7 +20,7 @@
 namespace metrics {
 
 DriveMetricsProvider::DriveMetricsProvider(int local_state_path_key)
-    : local_state_path_key_(local_state_path_key), weak_ptr_factory_(this) {}
+    : local_state_path_key_(local_state_path_key) {}
 
 DriveMetricsProvider::~DriveMetricsProvider() {}
 
@@ -50,7 +50,8 @@ DriveMetricsProvider::SeekPenaltyResponse::SeekPenaltyResponse()
 DriveMetricsProvider::DriveMetrics
 DriveMetricsProvider::GetDriveMetricsOnBackgroundThread(
     int local_state_path_key) {
-  base::ScopedBlockingCall scoped_blocking_call(base::BlockingType::WILL_BLOCK);
+  base::ScopedBlockingCall scoped_blocking_call(FROM_HERE,
+                                                base::BlockingType::WILL_BLOCK);
 
   DriveMetricsProvider::DriveMetrics metrics;
   QuerySeekPenalty(base::FILE_EXE, &metrics.app_drive);
