@@ -35,15 +35,27 @@ class NetworkMetricsProviderTest : public testing::Test {
  public:
  protected:
   NetworkMetricsProviderTest()
-      : test_thread_bundle_(TestThreadBundle::IO_MAINLOOP) {
+      : test_thread_bundle_(TestThreadBundle::IO_MAINLOOP) {}
+  ~NetworkMetricsProviderTest() override {}
+
+  void SetUp() override {
 #if defined(OS_CHROMEOS)
     chromeos::DBusThreadManager::Initialize();
     chromeos::NetworkHandler::Initialize();
 #endif  // OS_CHROMEOS
   }
 
+  void TearDown() override {
+#if defined(OS_CHROMEOS)
+    chromeos::NetworkHandler::Shutdown();
+    chromeos::DBusThreadManager::Shutdown();
+#endif  // OS_CHROMEOS
+  }
+
  private:
   TestThreadBundle test_thread_bundle_;
+
+  DISALLOW_COPY_AND_ASSIGN(NetworkMetricsProviderTest);
 };
 
 // Verifies that the effective connection type is correctly set.
