@@ -6,6 +6,7 @@
 
 #include <utility>
 
+#include "base/bind.h"
 #include "base/files/file_path.h"
 #include "base/system/sys_info.h"
 #include "base/task/post_task.h"
@@ -16,13 +17,6 @@ using leveldb_proto::ProtoDatabase;
 using leveldb_proto::ProtoDatabaseProvider;
 
 namespace {
-// Statistics are logged to UMA with this string as part of histogram name. They
-// can all be found under LevelDB.*.NTPSnippets. Changing this needs to
-// synchronize with histograms.xml, AND will also become incompatible with older
-// browsers still reporting the previous values.
-const char kDatabaseUMAClientName[] = "NTPSnippets";
-const char kImageDatabaseUMAClientName[] = "NTPSnippetImages";
-
 const char kSnippetDatabaseFolder[] = "snippets";
 const char kImageDatabaseFolder[] = "images";
 
@@ -32,44 +26,47 @@ const size_t kDatabaseWriteBufferSizeBytes = 128 << 10;
 namespace ntp_snippets {
 
 RemoteSuggestionsDatabase::RemoteSuggestionsDatabase(
+    leveldb_proto::ProtoDatabaseProvider* proto_database_provider,
     const base::FilePath& database_dir)
     : RemoteSuggestionsDatabase(
+          proto_database_provider,
+          database_dir,
           base::CreateSequencedTaskRunnerWithTraits(
               {base::MayBlock(), base::TaskPriority::BEST_EFFORT,
-               base::TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN}),
-          database_dir) {}
+               base::TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN})) {}
 
 RemoteSuggestionsDatabase::RemoteSuggestionsDatabase(
-    scoped_refptr<base::SequencedTaskRunner> task_runner,
-    const base::FilePath& database_dir)
+    leveldb_proto::ProtoDatabaseProvider* proto_database_provider,
+    const base::FilePath& database_dir,
+    scoped_refptr<base::SequencedTaskRunner> task_runner)
     : RemoteSuggestionsDatabase(
-          ProtoDatabaseProvider::CreateUniqueDB<SnippetProto>(task_runner),
-          ProtoDatabaseProvider::CreateUniqueDB<SnippetImageProto>(task_runner),
-          database_dir) {}
+          proto_database_provider->GetDB<SnippetProto>(
+              leveldb_proto::ProtoDbType::REMOTE_SUGGESTIONS_DATABASE,
+              database_dir.AppendASCII(kSnippetDatabaseFolder),
+              task_runner),
+          proto_database_provider->GetDB<SnippetImageProto>(
+              leveldb_proto::ProtoDbType::REMOTE_SUGGESTIONS_IMAGE_DATABASE,
+              database_dir.AppendASCII(kImageDatabaseFolder),
+              task_runner)) {}
 
 RemoteSuggestionsDatabase::RemoteSuggestionsDatabase(
     std::unique_ptr<ProtoDatabase<SnippetProto>> database,
-    std::unique_ptr<ProtoDatabase<SnippetImageProto>> image_database,
-    const base::FilePath& database_dir)
+    std::unique_ptr<ProtoDatabase<SnippetImageProto>> image_database)
     : database_(std::move(database)),
       database_initialized_(false),
       image_database_(std::move(image_database)),
-      image_database_initialized_(false),
-      weak_ptr_factory_(this) {
-  base::FilePath snippet_dir = database_dir.AppendASCII(kSnippetDatabaseFolder);
+      image_database_initialized_(false) {
   leveldb_env::Options options = leveldb_proto::CreateSimpleOptions();
   options.reuse_logs = false;  // Consumes less RAM over time.
   options.write_buffer_size = kDatabaseWriteBufferSizeBytes;
 
-  database_->Init(kDatabaseUMAClientName, snippet_dir, options,
+  database_->Init(options,
                   base::BindOnce(&RemoteSuggestionsDatabase::OnDatabaseInited,
                                  weak_ptr_factory_.GetWeakPtr()));
 
-  base::FilePath image_dir = database_dir.AppendASCII(kImageDatabaseFolder);
   image_database_->Init(
-      kImageDatabaseUMAClientName, image_dir, options,
-      base::BindOnce(&RemoteSuggestionsDatabase::OnImageDatabaseInited,
-                     weak_ptr_factory_.GetWeakPtr()));
+      options, base::BindOnce(&RemoteSuggestionsDatabase::OnImageDatabaseInited,
+                              weak_ptr_factory_.GetWeakPtr()));
 }
 
 RemoteSuggestionsDatabase::~RemoteSuggestionsDatabase() = default;
@@ -179,9 +176,10 @@ void RemoteSuggestionsDatabase::GarbageCollectImages(
       weak_ptr_factory_.GetWeakPtr(), std::move(alive_snippet_ids)));
 }
 
-void RemoteSuggestionsDatabase::OnDatabaseInited(bool success) {
+void RemoteSuggestionsDatabase::OnDatabaseInited(
+    leveldb_proto::Enums::InitStatus status) {
   DCHECK(!database_initialized_);
-  if (!success) {
+  if (status != leveldb_proto::Enums::InitStatus::kOK) {
     DVLOG(1) << "RemoteSuggestionsDatabase init failed.";
     OnDatabaseError();
     return;
@@ -238,9 +236,10 @@ void RemoteSuggestionsDatabase::OnDatabaseSaved(bool success) {
   }
 }
 
-void RemoteSuggestionsDatabase::OnImageDatabaseInited(bool success) {
+void RemoteSuggestionsDatabase::OnImageDatabaseInited(
+    leveldb_proto::Enums::InitStatus status) {
   DCHECK(!image_database_initialized_);
-  if (!success) {
+  if (status != leveldb_proto::Enums::InitStatus::kOK) {
     DVLOG(1) << "RemoteSuggestionsDatabase init failed.";
     OnDatabaseError();
     return;
