@@ -4,22 +4,23 @@
 
 #include "components/password_manager/core/browser/browser_save_password_progress_logger.h"
 
+#include "base/strings/strcat.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
 #include "components/autofill/core/browser/field_types.h"
 #include "components/autofill/core/browser/form_structure.h"
+#include "components/autofill/core/browser/logging/log_manager.h"
 #include "components/autofill/core/browser/proto/server.pb.h"
 #include "components/autofill/core/common/password_form.h"
 #include "components/autofill/core/common/signatures_util.h"
-#include "components/password_manager/core/browser/log_manager.h"
 #include "components/password_manager/core/browser/password_form_manager.h"
+#include "components/password_manager/core/browser/password_form_metrics_recorder.h"
 #include "components/password_manager/core/browser/password_manager.h"
 
 using autofill::AutofillUploadContents;
 using base::NumberToString;
-using base::UintToString;
 
 namespace password_manager {
 
@@ -77,10 +78,17 @@ std::string VoteTypeToString(
   }
 }
 
+std::string FormSignatureToDebugString(autofill::FormSignature form_signature) {
+  return base::StrCat(
+      {NumberToString(form_signature), " - ",
+       NumberToString(
+           PasswordFormMetricsRecorder::HashFormSignature(form_signature))});
+}
+
 }  // namespace
 
 BrowserSavePasswordProgressLogger::BrowserSavePasswordProgressLogger(
-    const LogManager* log_manager)
+    const autofill::LogManager* log_manager)
     : log_manager_(log_manager) {
   DCHECK(log_manager_);
 }
@@ -93,7 +101,7 @@ void BrowserSavePasswordProgressLogger::LogFormSignatures(
   autofill::FormStructure form_structure(form.form_data);
   std::string message = GetStringFromID(label) + ": {\n";
   message += GetStringFromID(STRING_FORM_SIGNATURE) + ": " +
-             ScrubNonDigit(form_structure.FormSignatureAsStr()) + "\n";
+             FormSignatureToDebugString(form_structure.form_signature()) + "\n";
   message += GetStringFromID(STRING_SIGNON_REALM) + ": " +
              ScrubURL(GURL(form.signon_realm)) + "\n";
   message +=
@@ -112,7 +120,7 @@ void BrowserSavePasswordProgressLogger::LogFormStructure(
     const autofill::FormStructure& form_structure) {
   std::string message = GetStringFromID(label) + ": {\n";
   message += GetStringFromID(STRING_FORM_SIGNATURE) + ": " +
-             ScrubNonDigit(form_structure.FormSignatureAsStr()) + "\n";
+             FormSignatureToDebugString(form_structure.form_signature()) + "\n";
   message += GetStringFromID(STRING_ORIGIN) + ": " +
              ScrubURL(form_structure.source_url()) + "\n";
   message += GetStringFromID(STRING_ACTION) + ": " +
@@ -145,7 +153,8 @@ std::string BrowserSavePasswordProgressLogger::FormStructureToFieldsLogString(
         ScrubNonDigit(field->FieldSignatureAsStr()) +
         ", type=" + ScrubElementID(field->form_control_type);
 
-    field_info += ", renderer_id = " + UintToString(field->unique_renderer_id);
+    field_info +=
+        ", renderer_id = " + NumberToString(field->unique_renderer_id);
 
     if (!field->autocomplete_attribute.empty())
       field_info +=
@@ -204,7 +213,7 @@ void BrowserSavePasswordProgressLogger::LogString(StringID label,
 }
 
 void BrowserSavePasswordProgressLogger::LogSuccessfulSubmissionIndicatorEvent(
-    autofill::SubmissionIndicatorEvent event) {
+    autofill::mojom::SubmissionIndicatorEvent event) {
   std::ostringstream submission_event_string_stream;
   submission_event_string_stream << event;
   std::string message =
@@ -217,10 +226,10 @@ void BrowserSavePasswordProgressLogger::LogFormData(
     StringID label,
     const autofill::FormData& form) {
   std::string message = GetStringFromID(label) + ": {\n";
-  message += GetStringFromID(STRING_FORM_SIGNATURE) + ": " +
-             NumberToString(autofill::CalculateFormSignature(form)) + "\n";
   message +=
-      GetStringFromID(STRING_ORIGIN) + ": " + ScrubURL(form.origin) + "\n";
+      GetStringFromID(STRING_FORM_SIGNATURE) + ": " +
+      FormSignatureToDebugString(autofill::CalculateFormSignature(form)) + "\n";
+  message += GetStringFromID(STRING_ORIGIN) + ": " + ScrubURL(form.url) + "\n";
   message +=
       GetStringFromID(STRING_ACTION) + ": " + ScrubURL(form.action) + "\n";
   if (form.main_frame_origin.GetURL().is_valid())
@@ -234,7 +243,7 @@ void BrowserSavePasswordProgressLogger::LogFormData(
 
   if (form.is_form_tag) {
     message +=
-        "Form renderer id: " + UintToString(form.unique_renderer_id) + "\n";
+        "Form renderer id: " + NumberToString(form.unique_renderer_id) + "\n";
   }
 
   // Log fields.
@@ -250,7 +259,7 @@ void BrowserSavePasswordProgressLogger::LogFormData(
     std::string field_info =
         ScrubElementID(field.name) +
         ": type=" + ScrubElementID(field.form_control_type) +
-        ", renderer_id = " + UintToString(field.unique_renderer_id) + ", " +
+        ", renderer_id = " + NumberToString(field.unique_renderer_id) + ", " +
         is_visible + ", " + is_empty + autocomplete + "\n";
     message += field_info;
   }
@@ -259,7 +268,7 @@ void BrowserSavePasswordProgressLogger::LogFormData(
 }
 
 void BrowserSavePasswordProgressLogger::SendLog(const std::string& log) {
-  log_manager_->LogSavePasswordProgress(log);
+  log_manager_->LogTextMessage(log);
 }
 
 }  // namespace password_manager
