@@ -6,13 +6,14 @@
 
 #include <utility>
 
+#include "base/bind.h"
 #include "base/guid.h"
 #include "base/memory/weak_ptr.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/scoped_feature_list.h"
-#include "components/autofill/core/browser/autofill_profile.h"
 #include "components/autofill/core/browser/autofill_test_utils.h"
-#include "components/autofill/core/browser/credit_card.h"
+#include "components/autofill/core/browser/data_model/autofill_profile.h"
+#include "components/autofill/core/browser/data_model/credit_card.h"
 #include "components/autofill/core/browser/test_personal_data_manager.h"
 #include "components/payments/content/payment_request_spec.h"
 #include "components/payments/content/test_content_payment_request_delegate.h"
@@ -56,7 +57,8 @@ class PaymentRequestStateTest : public testing::Test,
   // PaymentRequestState::Delegate:
   void OnPaymentResponseAvailable(mojom::PaymentResponsePtr response) override {
     payment_response_ = std::move(response);
-  };
+  }
+  void OnPaymentResponseError(const std::string& error_message) override {}
   void OnShippingOptionIdSelected(std::string shipping_option_id) override {}
   void OnShippingAddressSelected(mojom::PaymentAddressPtr address) override {
     selected_shipping_address_ = std::move(address);
@@ -67,6 +69,8 @@ class PaymentRequestStateTest : public testing::Test,
       mojom::PaymentOptionsPtr options,
       mojom::PaymentDetailsPtr details,
       std::vector<mojom::PaymentMethodDataPtr> method_data) {
+    if (!details->total)
+      details->total = mojom::PaymentItem::New();
     // The spec will be based on the |options| and |details| passed in.
     spec_ = std::make_unique<PaymentRequestSpec>(
         std::move(options), std::move(details), std::move(method_data),
@@ -103,7 +107,8 @@ class PaymentRequestStateTest : public testing::Test,
   std::vector<mojom::PaymentMethodDataPtr> GetMethodDataForVisa() {
     std::vector<mojom::PaymentMethodDataPtr> method_data;
     mojom::PaymentMethodDataPtr entry = mojom::PaymentMethodData::New();
-    entry->supported_method = "visa";
+    entry->supported_method = "basic-card";
+    entry->supported_networks.push_back(mojom::BasicCardNetwork::VISA);
     method_data.push_back(std::move(entry));
     return method_data;
   }
@@ -139,34 +144,45 @@ class PaymentRequestStateTest : public testing::Test,
   autofill::CreditCard credit_card_visa_;
 };
 
-TEST_F(PaymentRequestStateTest, HasEnrolledInstrument) {
+TEST_F(PaymentRequestStateTest, CanMakePayment) {
   // Default options.
   RecreateStateWithOptions(mojom::PaymentOptions::New());
 
-  // HasEnrolledInstrument returns true because the method data requires Visa,
+  // Legacy CanMakePayment returns true because the method data requires Visa,
   // and the user has a Visa card on file.
+  state()->CanMakePayment(
+      /*legacy_mode=*/true, base::BindOnce([](bool can_make_payment) {
+        EXPECT_TRUE(can_make_payment);
+      }));
   state()->HasEnrolledInstrument(
       base::BindOnce([](bool has_enrolled_instrument) {
         EXPECT_TRUE(has_enrolled_instrument);
       }));
 
   // CanMakePayment returns true because the requested method is supported.
-  state()->CanMakePayment(base::BindOnce(
-      [](bool can_make_payment) { EXPECT_TRUE(can_make_payment); }));
+  state()->CanMakePayment(/*legacy_mode=*/false,
+                          base::BindOnce([](bool can_make_payment) {
+                            EXPECT_TRUE(can_make_payment);
+                          }));
 }
 
 TEST_F(PaymentRequestStateTest, CanMakePayment_NoEnrolledInstrument) {
   // The method data requires MasterCard.
   std::vector<mojom::PaymentMethodDataPtr> method_data;
   mojom::PaymentMethodDataPtr entry = mojom::PaymentMethodData::New();
-  entry->supported_method = "mastercard";
+  entry->supported_method = "basic-card";
+  entry->supported_networks.push_back(mojom::BasicCardNetwork::MASTERCARD);
   method_data.push_back(std::move(entry));
   RecreateStateWithOptionsAndDetails(mojom::PaymentOptions::New(),
                                      mojom::PaymentDetails::New(),
                                      std::move(method_data));
 
-  // HasEnrolledInstrument returns false because the method data requires
+  // Legacy CanMakePayment returns false because the method data requires
   // MasterCard, and the user doesn't have such an instrument.
+  state()->CanMakePayment(
+      /*legacy_mode=*/true, base::BindOnce([](bool can_make_payment) {
+        EXPECT_FALSE(can_make_payment);
+      }));
   state()->HasEnrolledInstrument(
       base::BindOnce([](bool has_enrolled_instrument) {
         EXPECT_FALSE(has_enrolled_instrument);
@@ -174,8 +190,10 @@ TEST_F(PaymentRequestStateTest, CanMakePayment_NoEnrolledInstrument) {
 
   // CanMakePayment returns true because the requested method is supported, even
   // though the payment instrument is not ready to pay.
-  state()->CanMakePayment(base::BindOnce(
-      [](bool can_make_payment) { EXPECT_TRUE(can_make_payment); }));
+  state()->CanMakePayment(/*legacy_mode=*/false,
+                          base::BindOnce([](bool can_make_payment) {
+                            EXPECT_TRUE(can_make_payment);
+                          }));
 }
 
 TEST_F(PaymentRequestStateTest, CanMakePayment_UnsupportedPaymentMethod) {
@@ -187,8 +205,12 @@ TEST_F(PaymentRequestStateTest, CanMakePayment_UnsupportedPaymentMethod) {
                                      mojom::PaymentDetails::New(),
                                      std::move(method_data));
 
-  // HasEnrolledInstrument returns false because the method data requires
+  // Legacy CanMakePayment returns false because the method data requires
   // MasterCard, and the user doesn't have such an instrument.
+  state()->CanMakePayment(
+      /*legacy_mode=*/true, base::BindOnce([](bool can_make_payment) {
+        EXPECT_FALSE(can_make_payment);
+      }));
   state()->HasEnrolledInstrument(
       base::BindOnce([](bool has_enrolled_instrument) {
         EXPECT_FALSE(has_enrolled_instrument);
@@ -196,11 +218,13 @@ TEST_F(PaymentRequestStateTest, CanMakePayment_UnsupportedPaymentMethod) {
 
   // CanMakePayment returns true because the requested method is supported, even
   // though the payment instrument is not ready to pay.
-  state()->CanMakePayment(base::BindOnce(
-      [](bool can_make_payment) { EXPECT_FALSE(can_make_payment); }));
+  state()->CanMakePayment(/*legacy_mode=*/false,
+                          base::BindOnce([](bool can_make_payment) {
+                            EXPECT_FALSE(can_make_payment);
+                          }));
 }
 
-TEST_F(PaymentRequestStateTest, HasEnrolledInstrument_OnlyBasicCard) {
+TEST_F(PaymentRequestStateTest, CanMakePayment_OnlyBasicCard) {
   // The method data supports everything in basic-card.
   mojom::PaymentMethodDataPtr entry = mojom::PaymentMethodData::New();
   entry->supported_method = "basic-card";
@@ -210,20 +234,25 @@ TEST_F(PaymentRequestStateTest, HasEnrolledInstrument_OnlyBasicCard) {
                                      mojom::PaymentDetails::New(),
                                      std::move(method_data));
 
-  // HasEnrolledInstrument returns true because the method data supports
+  // Legacy CanMakePayment returns true because the method data supports
   // everything, and the user has at least one instrument.
+  state()->CanMakePayment(
+      /*legacy_mode=*/true, base::BindOnce([](bool can_make_payment) {
+        EXPECT_TRUE(can_make_payment);
+      }));
   state()->HasEnrolledInstrument(
       base::BindOnce([](bool has_enrolled_instrument) {
         EXPECT_TRUE(has_enrolled_instrument);
       }));
 
   // CanMakePayment returns true because the requested method is supported.
-  state()->CanMakePayment(base::BindOnce(
-      [](bool can_make_payment) { EXPECT_TRUE(can_make_payment); }));
+  state()->CanMakePayment(
+      /*legacy_mode=*/false, base::BindOnce([](bool can_make_payment) {
+        EXPECT_TRUE(can_make_payment);
+      }));
 }
 
-TEST_F(PaymentRequestStateTest,
-       HasEnrolledInstrument_BasicCard_SpecificAvailable) {
+TEST_F(PaymentRequestStateTest, CanMakePayment_BasicCard_SpecificAvailable) {
   // The method data supports visa through basic-card.
   mojom::PaymentMethodDataPtr entry = mojom::PaymentMethodData::New();
   entry->supported_method = "basic-card";
@@ -234,20 +263,26 @@ TEST_F(PaymentRequestStateTest,
                                      mojom::PaymentDetails::New(),
                                      std::move(method_data));
 
-  // HasEnrolledInstrument returns true because the method data supports visa,
+  // Legacy CanMakePayment returns true because the method data supports visa,
   // and the user has a Visa instrument.
+  state()->CanMakePayment(
+      /*legacy_mode=*/true, base::BindOnce([](bool can_make_payment) {
+        EXPECT_TRUE(can_make_payment);
+      }));
   state()->HasEnrolledInstrument(
       base::BindOnce([](bool has_enrolled_instrument) {
         EXPECT_TRUE(has_enrolled_instrument);
       }));
 
   // CanMakePayment returns true because the requested method is supported.
-  state()->CanMakePayment(base::BindOnce(
-      [](bool can_make_payment) { EXPECT_TRUE(can_make_payment); }));
+  state()->CanMakePayment(
+      /*legacy_mode=*/false, base::BindOnce([](bool can_make_payment) {
+        EXPECT_TRUE(can_make_payment);
+      }));
 }
 
 TEST_F(PaymentRequestStateTest,
-       HasEnrolledInstrument_BasicCard_SpecificAvailableButInvalid) {
+       CanMakePayment_BasicCard_SpecificAvailableButInvalid) {
   // The method data supports jcb through basic-card.
   mojom::PaymentMethodDataPtr entry = mojom::PaymentMethodData::New();
   entry->supported_method = "basic-card";
@@ -258,8 +293,12 @@ TEST_F(PaymentRequestStateTest,
                                      mojom::PaymentDetails::New(),
                                      std::move(method_data));
 
-  // HasEnrolledInstrument returns false because the method data supports jcb,
+  // Legacy CanMakePayment returns false because the method data supports jcb,
   // and the user has a JCB instrument, but it's invalid.
+  state()->CanMakePayment(
+      /*legacy_mode=*/true, base::BindOnce([](bool can_make_payment) {
+        EXPECT_FALSE(can_make_payment);
+      }));
   state()->HasEnrolledInstrument(
       base::BindOnce([](bool has_enrolled_instrument) {
         EXPECT_FALSE(has_enrolled_instrument);
@@ -267,12 +306,13 @@ TEST_F(PaymentRequestStateTest,
 
   // CanMakePayment returns true because the requested method is supported, even
   // though there is no enrolled instrument.
-  state()->CanMakePayment(base::BindOnce(
-      [](bool can_make_payment) { EXPECT_TRUE(can_make_payment); }));
+  state()->CanMakePayment(
+      /*legacy_mode=*/false, base::BindOnce([](bool can_make_payment) {
+        EXPECT_TRUE(can_make_payment);
+      }));
 }
 
-TEST_F(PaymentRequestStateTest,
-       HasEnrolledInstrument_BasicCard_SpecificUnavailable) {
+TEST_F(PaymentRequestStateTest, CanMakePayment_BasicCard_SpecificUnavailable) {
   // The method data supports mastercard through basic-card.
   mojom::PaymentMethodDataPtr entry = mojom::PaymentMethodData::New();
   entry->supported_method = "basic-card";
@@ -283,8 +323,12 @@ TEST_F(PaymentRequestStateTest,
                                      mojom::PaymentDetails::New(),
                                      std::move(method_data));
 
-  // HasEnrolledInstrument returns false because the method data supports
+  // Legacy CanMakePayment returns false because the method data supports
   // mastercard, and the user doesn't have such an instrument.
+  state()->CanMakePayment(
+      /*legacy_mode=*/true, base::BindOnce([](bool can_make_payment) {
+        EXPECT_FALSE(can_make_payment);
+      }));
   state()->HasEnrolledInstrument(
       base::BindOnce([](bool has_enrolled_instrument) {
         EXPECT_FALSE(has_enrolled_instrument);
@@ -292,8 +336,10 @@ TEST_F(PaymentRequestStateTest,
 
   // CanMakePayment returns true because the requested method is supported, even
   // though there is no enrolled instrument.
-  state()->CanMakePayment(base::BindOnce(
-      [](bool can_make_payment) { EXPECT_TRUE(can_make_payment); }));
+  state()->CanMakePayment(
+      /*legacy_mode=*/false, base::BindOnce([](bool can_make_payment) {
+        EXPECT_TRUE(can_make_payment);
+      }));
 }
 
 TEST_F(PaymentRequestStateTest, ReadyToPay_DefaultSelections) {
@@ -308,7 +354,8 @@ TEST_F(PaymentRequestStateTest, ReadyToPay_DefaultSelections) {
   // Therefore we are not ready to pay.
   EXPECT_FALSE(state()->is_ready_to_pay());
 
-  state()->SetSelectedShippingProfile(test_address());
+  state()->SetSelectedShippingProfile(
+      test_address(), PaymentRequestState::SectionSelectionStatus::kSelected);
   EXPECT_EQ(0, num_on_selected_information_changed_called());
 
   // Simulate that the merchant has validated the shipping address change.
@@ -353,12 +400,14 @@ TEST_F(PaymentRequestStateTest, ReadyToPay_ContactInfo) {
   EXPECT_TRUE(state()->is_ready_to_pay());
 
   // Unselecting contact profile.
-  state()->SetSelectedContactProfile(nullptr);
+  state()->SetSelectedContactProfile(
+      nullptr, PaymentRequestState::SectionSelectionStatus::kSelected);
   EXPECT_EQ(1, num_on_selected_information_changed_called());
 
   EXPECT_FALSE(state()->is_ready_to_pay());
 
-  state()->SetSelectedContactProfile(test_address());
+  state()->SetSelectedContactProfile(
+      test_address(), PaymentRequestState::SectionSelectionStatus::kSelected);
   EXPECT_EQ(2, num_on_selected_information_changed_called());
 
   // Ready to pay!
@@ -379,7 +428,8 @@ TEST_F(PaymentRequestStateTest, SelectedShippingAddressMessage_Normalized) {
 
   // Select an address, nothing should happen until the normalization is
   // completed and the merchant has validated the address.
-  state()->SetSelectedShippingProfile(test_address());
+  state()->SetSelectedShippingProfile(
+      test_address(), PaymentRequestState::SectionSelectionStatus::kSelected);
   EXPECT_EQ(0, num_on_selected_information_changed_called());
   EXPECT_FALSE(state()->is_ready_to_pay());
 
@@ -405,7 +455,6 @@ TEST_F(PaymentRequestStateTest, SelectedShippingAddressMessage_Normalized) {
   EXPECT_EQ("", selected_shipping_address()->dependent_locality);
   EXPECT_EQ("91111", selected_shipping_address()->postal_code);
   EXPECT_EQ("", selected_shipping_address()->sorting_code);
-  EXPECT_EQ("", selected_shipping_address()->language_code);
   EXPECT_EQ("Underworld", selected_shipping_address()->organization);
   EXPECT_EQ("John H. Doe", selected_shipping_address()->recipient);
   EXPECT_EQ("16502111111", selected_shipping_address()->phone);
@@ -431,9 +480,9 @@ TEST_F(PaymentRequestStateTest, JaLatnShippingAddress) {
                                  "jon.doe@exampl.com", "Example Inc",
                                  "Roppongi", "6 Chrome-10-1", "Tokyo", "",
                                  "106-6126", "JP", "+81363849000");
-  profile.set_language_code("ja-Latn");
 
-  state()->SetSelectedShippingProfile(&profile);
+  state()->SetSelectedShippingProfile(
+      &profile, PaymentRequestState::SectionSelectionStatus::kSelected);
   EXPECT_EQ(0, num_on_selected_information_changed_called());
   EXPECT_FALSE(state()->is_ready_to_pay());
 
@@ -459,8 +508,6 @@ TEST_F(PaymentRequestStateTest, JaLatnShippingAddress) {
   EXPECT_EQ("", selected_shipping_address()->dependent_locality);
   EXPECT_EQ("106-6126", selected_shipping_address()->postal_code);
   EXPECT_EQ("", selected_shipping_address()->sorting_code);
-  EXPECT_EQ("ja", selected_shipping_address()->language_code);
-  EXPECT_EQ("Latn", selected_shipping_address()->script_code);
   EXPECT_EQ("Example Inc", selected_shipping_address()->organization);
   EXPECT_EQ("Jon V. Doe", selected_shipping_address()->recipient);
   EXPECT_EQ("+81363849000", selected_shipping_address()->phone);
@@ -475,7 +522,8 @@ TEST_F(PaymentRequestStateTest, RetryWithShippingAddressErrors) {
   // Therefore we are not ready to pay.
   EXPECT_FALSE(state()->is_ready_to_pay());
 
-  state()->SetSelectedShippingProfile(test_address());
+  state()->SetSelectedShippingProfile(
+      test_address(), PaymentRequestState::SectionSelectionStatus::kSelected);
   EXPECT_EQ(0, num_on_selected_information_changed_called());
 
   // Simulate that the merchant has validated the shipping address change.
@@ -518,7 +566,8 @@ TEST_F(PaymentRequestStateTest, RetryWithPayerErrors) {
   options->request_payer_email = true;
   RecreateStateWithOptions(std::move(options));
 
-  state()->SetSelectedContactProfile(test_address());
+  state()->SetSelectedContactProfile(
+      test_address(), PaymentRequestState::SectionSelectionStatus::kSelected);
   EXPECT_EQ(1, num_on_selected_information_changed_called());
   EXPECT_TRUE(state()->is_ready_to_pay());
 
