@@ -4,16 +4,27 @@
 
 #include "components/safe_browsing/password_protection/password_protection_request.h"
 
+#include <cstddef>
 #include <memory>
 
+#include "base/bind.h"
 #include "base/memory/weak_ptr.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/task/post_task.h"
-#include "components/data_use_measurement/core/data_use_user_data.h"
+#include "base/time/time.h"
+#include "components/password_manager/core/browser/password_manager_metrics_util.h"
 #include "components/password_manager/core/browser/password_reuse_detector.h"
-#include "components/safe_browsing/db/whitelist_checker_client.h"
+#include "components/safe_browsing/common/safe_browsing.mojom.h"
+#include "components/safe_browsing/db/allowlist_checker_client.h"
+#include "components/safe_browsing/features.h"
+#include "components/safe_browsing/password_protection/metrics_util.h"
 #include "components/safe_browsing/password_protection/password_protection_navigation_throttle.h"
+#include "components/safe_browsing/password_protection/visual_utils.h"
+#include "components/safe_browsing/proto/csd.pb.h"
 #include "components/safe_browsing/web_ui/safe_browsing_ui.h"
+#include "components/zoom/zoom_controller.h"
 #include "content/public/browser/browser_task_traits.h"
+#include "content/public/browser/render_widget_host_view.h"
 #include "content/public/browser/web_contents.h"
 #include "net/base/escape.h"
 #include "net/base/load_flags.h"
@@ -21,6 +32,7 @@
 #include "net/http/http_status_code.h"
 #include "net/traffic_annotation/network_traffic_annotation.h"
 #include "services/network/public/cpp/simple_url_loader.h"
+#include "services/service_manager/public/cpp/interface_provider.h"
 #include "url/origin.h"
 
 using content::BrowserThread;
@@ -28,12 +40,29 @@ using content::WebContents;
 
 namespace safe_browsing {
 
+using ReusedPasswordAccountType =
+    LoginReputationClientRequest::PasswordReuseEvent::ReusedPasswordAccountType;
+
 namespace {
 
 // Cap on how many reused domains can be included in a report, to limit
 // the size of the report. UMA suggests 99.9% will have < 200 domains.
 const int kMaxReusedDomains = 200;
 
+// Parameters chosen to ensure privacy is preserved by visual features.
+const int kMinWidthForVisualFeatures = 576;
+const int kMinHeightForVisualFeatures = 576;
+const float kMaxZoomForVisualFeatures = 2.0;
+
+std::unique_ptr<VisualFeatures> ExtractVisualFeatures(
+    const SkBitmap& screenshot) {
+  auto features = std::make_unique<VisualFeatures>();
+  visual_utils::GetHistogramForImage(screenshot,
+                                     features->mutable_color_histogram());
+  visual_utils::GetBlurredImage(screenshot, features->mutable_image());
+  return features;
+}
+
 }  // namespace
 
 PasswordProtectionRequest::PasswordProtectionRequest(
@@ -41,32 +70,33 @@ PasswordProtectionRequest::PasswordProtectionRequest(
     const GURL& main_frame_url,
     const GURL& password_form_action,
     const GURL& password_form_frame_url,
-    ReusedPasswordType reused_password_type,
+    const std::string& username,
+    PasswordType password_type,
     const std::vector<std::string>& matching_domains,
     LoginReputationClientRequest::TriggerType type,
     bool password_field_exists,
     PasswordProtectionService* pps,
     int request_timeout_in_ms)
-    : web_contents_(web_contents),
+    : content::WebContentsObserver(web_contents),
+      web_contents_(web_contents),
       main_frame_url_(main_frame_url),
       password_form_action_(password_form_action),
       password_form_frame_url_(password_form_frame_url),
-      reused_password_type_(reused_password_type),
+      username_(username),
+      password_type_(password_type),
       matching_domains_(matching_domains),
       trigger_type_(type),
       password_field_exists_(password_field_exists),
       password_protection_service_(pps),
       request_timeout_in_ms_(request_timeout_in_ms),
       request_proto_(std::make_unique<LoginReputationClientRequest>()),
-      is_modal_warning_showing_(false),
-      weakptr_factory_(this) {
+      is_modal_warning_showing_(false) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
   DCHECK(trigger_type_ == LoginReputationClientRequest::UNFAMILIAR_LOGIN_PAGE ||
          trigger_type_ == LoginReputationClientRequest::PASSWORD_REUSE_EVENT);
   DCHECK(trigger_type_ != LoginReputationClientRequest::PASSWORD_REUSE_EVENT ||
-         reused_password_type_ !=
-             LoginReputationClientRequest::PasswordReuseEvent::SAVED_PASSWORD ||
+         password_type_ != PasswordType::SAVED_PASSWORD ||
          matching_domains_.size() > 0);
 
   request_proto_->set_trigger_type(trigger_type_);
@@ -99,7 +129,7 @@ void PasswordProtectionRequest::CheckWhitelist() {
   tracker_.PostTask(
       base::CreateSingleThreadTaskRunnerWithTraits({BrowserThread::IO}).get(),
       FROM_HERE,
-      base::BindOnce(&WhitelistCheckerClient::StartCheckCsdWhitelist,
+      base::BindOnce(&AllowlistCheckerClient::StartCheckCsdWhitelist,
                      password_protection_service_->database_manager(),
                      main_frame_url_, result_callback));
 }
@@ -117,10 +147,15 @@ void PasswordProtectionRequest::OnWhitelistCheckDoneOnIO(
 
 void PasswordProtectionRequest::OnWhitelistCheckDone(bool match_whitelist) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  if (match_whitelist)
+  if (match_whitelist) {
     Finish(RequestOutcome::MATCHED_WHITELIST, nullptr);
-  else
+  } else {
+    // In case the request to Safe Browsing takes too long,
+    // we set a timer to cancel that request and return an "unspecified verdict"
+    // so that the navigation isn't blocked indefinitely.
+    StartTimeout();
     CheckCachedVerdicts();
+  }
 }
 
 void PasswordProtectionRequest::CheckCachedVerdicts() {
@@ -133,12 +168,11 @@ void PasswordProtectionRequest::CheckCachedVerdicts() {
   std::unique_ptr<LoginReputationClientResponse> cached_response =
       std::make_unique<LoginReputationClientResponse>();
   auto verdict = password_protection_service_->GetCachedVerdict(
-      main_frame_url_, trigger_type_, reused_password_type_,
-      cached_response.get());
+      main_frame_url_, trigger_type_, password_type_, cached_response.get());
   if (verdict != LoginReputationClientResponse::VERDICT_TYPE_UNSPECIFIED)
     Finish(RequestOutcome::RESPONSE_ALREADY_CACHED, std::move(cached_response));
   else
-    SendRequest();
+    FillRequestProto();
 }
 
 void PasswordProtectionRequest::FillRequestProto() {
@@ -187,12 +221,13 @@ void PasswordProtectionRequest::FillRequestProto() {
       main_frame->set_has_password_field(password_field_exists_);
       LoginReputationClientRequest::PasswordReuseEvent* reuse_event =
           request_proto_->mutable_password_reuse_event();
-      bool matches_sync_password =
-          reused_password_type_ ==
-          LoginReputationClientRequest::PasswordReuseEvent::SIGN_IN_PASSWORD;
-      reuse_event->set_is_chrome_signin_password(matches_sync_password);
-      reuse_event->set_reused_password_type(reused_password_type_);
-      if (matches_sync_password) {
+      bool matches_signin_password =
+          password_type_ == PasswordType::PRIMARY_ACCOUNT_PASSWORD;
+      reuse_event->set_is_chrome_signin_password(matches_signin_password);
+      reuse_event->set_reused_password_type(
+          password_protection_service_->GetPasswordProtectionReusedPasswordType(
+              password_type_));
+      if (matches_signin_password) {
         reuse_event->set_sync_account_type(
             password_protection_service_->GetSyncAccountType());
         LogSyncAccountType(reuse_event->sync_account_type());
@@ -206,16 +241,118 @@ void PasswordProtectionRequest::FillRequestProto() {
             break;
         }
       }
+      if (base::FeatureList::IsEnabled(
+              safe_browsing::kPasswordProtectionForSignedInUsers)) {
+        ReusedPasswordAccountType password_account_type_to_add =
+            password_protection_service_
+                ->GetPasswordProtectionReusedPasswordAccountType(
+                    password_type_);
+        *reuse_event->mutable_reused_password_account_type() =
+            password_account_type_to_add;
+      }
       break;
     }
     default:
       NOTREACHED();
   }
+
+  // Get the page DOM features.
+  content::RenderFrameHost* rfh = web_contents_->GetMainFrame();
+  password_protection_service_->GetPhishingDetector(rfh->GetRemoteInterfaces(),
+                                                    &phishing_detector_);
+  phishing_detector_->StartPhishingDetection(
+      main_frame_url_,
+      base::BindRepeating(&PasswordProtectionRequest::OnGetDomFeatures,
+                          GetWeakPtr()));
+  dom_feature_start_time_ = base::TimeTicks::Now();
+}
+
+void PasswordProtectionRequest::OnGetDomFeatures(const std::string& verdict) {
+  ClientPhishingRequest dom_features_request;
+  bool parsed = dom_features_request.ParseFromString(verdict);
+  UMA_HISTOGRAM_BOOLEAN("PasswordProtection.DomFeatureParsing", parsed);
+  if (parsed) {
+    for (const ClientPhishingRequest::Feature& feature :
+         dom_features_request.feature_map()) {
+      DomFeatures::Feature* new_feature =
+          request_proto_->mutable_dom_features()->add_feature_map();
+      new_feature->set_name(feature.name());
+      new_feature->set_value(feature.value());
+    }
+
+    for (const ClientPhishingRequest::Feature& feature :
+         dom_features_request.non_model_feature_map()) {
+      DomFeatures::Feature* new_feature =
+          request_proto_->mutable_dom_features()->add_feature_map();
+      new_feature->set_name(feature.name());
+      new_feature->set_value(feature.value());
+    }
+
+    request_proto_->mutable_dom_features()->mutable_shingle_hashes()->Swap(
+        dom_features_request.mutable_shingle_hashes());
+    request_proto_->mutable_dom_features()->set_model_version(
+        dom_features_request.model_version());
+  }
+
+  UMA_HISTOGRAM_TIMES("PasswordProtection.DomFeatureExtractionDuration",
+                      base::TimeTicks::Now() - dom_feature_start_time_);
+
+  // Once the DOM features are collected, either collect visual features, or go
+  // straight to sending the ping.
+  if (trigger_type_ == LoginReputationClientRequest::UNFAMILIAR_LOGIN_PAGE &&
+      password_protection_service_->IsExtendedReporting() &&
+      zoom::ZoomController::GetZoomLevelForWebContents(web_contents_) <=
+          kMaxZoomForVisualFeatures &&
+      request_proto_->content_area_width() >= kMinWidthForVisualFeatures &&
+      request_proto_->content_area_height() >= kMinHeightForVisualFeatures) {
+    CollectVisualFeatures();
+  } else {
+    SendRequest();
+  }
+}
+
+void PasswordProtectionRequest::CollectVisualFeatures() {
+  content::RenderWidgetHostView* view =
+      web_contents_ ? web_contents_->GetRenderWidgetHostView() : nullptr;
+
+  if (!view) {
+    SendRequest();
+    return;
+  }
+
+  visual_feature_start_time_ = base::TimeTicks::Now();
+
+  view->CopyFromSurface(
+      gfx::Rect(), gfx::Size(),
+      base::BindOnce(&PasswordProtectionRequest::OnScreenshotTaken,
+                     GetWeakPtr()));
+}
+
+void PasswordProtectionRequest::OnScreenshotTaken(const SkBitmap& screenshot) {
+  // Do the feature extraction on a worker thread, to avoid blocking the UI.
+  base::PostTaskWithTraitsAndReplyWithResult(
+      FROM_HERE,
+      {base::MayBlock(), base::TaskPriority::BEST_EFFORT,
+       base::TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN},
+      base::BindOnce(&ExtractVisualFeatures, screenshot),
+      base::BindOnce(&PasswordProtectionRequest::OnVisualFeatureCollectionDone,
+                     GetWeakPtr()));
+}
+
+void PasswordProtectionRequest::OnVisualFeatureCollectionDone(
+    std::unique_ptr<VisualFeatures> visual_features) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+
+  request_proto_->mutable_visual_features()->Swap(visual_features.get());
+
+  UMA_HISTOGRAM_TIMES("PasswordProtection.VisualFeatureExtractionDuration",
+                      base::TimeTicks::Now() - visual_feature_start_time_);
+
+  SendRequest();
 }
 
 void PasswordProtectionRequest::SendRequest() {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  FillRequestProto();
 
   web_ui_token_ =
       WebUIInfoSingleton::GetInstance()->AddToPGPings(*request_proto_);
@@ -226,8 +363,6 @@ void PasswordProtectionRequest::SendRequest() {
     return;
   }
 
-  // In case the request take too long, we set a timer to cancel this request.
-  StartTimeout();
   net::NetworkTrafficAnnotationTag traffic_annotation =
       net::DefineNetworkTrafficAnnotation("password_protection_request", R"(
         semantics {
@@ -325,29 +460,31 @@ void PasswordProtectionRequest::Finish(
     std::unique_ptr<LoginReputationClientResponse> response) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   tracker_.TryCancelAll();
-  if (trigger_type_ == LoginReputationClientRequest::UNFAMILIAR_LOGIN_PAGE) {
-    LogPasswordOnFocusRequestOutcome(outcome);
-  } else {
-    LogPasswordEntryRequestOutcome(
-        outcome, reused_password_type_,
-        password_protection_service_->GetSyncAccountType());
-    if (reused_password_type_ ==
-        LoginReputationClientRequest::PasswordReuseEvent::SIGN_IN_PASSWORD) {
-      password_protection_service_->MaybeLogPasswordReuseLookupEvent(
-          web_contents_, outcome, response.get());
+
+  // If the request is canceled, the PasswordProtectionService is already
+  // partially destroyed, and we won't be able to log accurate metrics.
+  if (outcome != RequestOutcome::CANCELED) {
+    ReusedPasswordAccountType password_account_type =
+        password_protection_service_
+            ->GetPasswordProtectionReusedPasswordAccountType(password_type_);
+    if (trigger_type_ == LoginReputationClientRequest::UNFAMILIAR_LOGIN_PAGE) {
+      LogPasswordOnFocusRequestOutcome(outcome);
+    } else {
+      LogPasswordEntryRequestOutcome(outcome, password_account_type);
+      // TODO(crbug/914410): Account for non sync users.
+      if (password_type_ == PasswordType::PRIMARY_ACCOUNT_PASSWORD) {
+        password_protection_service_->MaybeLogPasswordReuseLookupEvent(
+            web_contents_, outcome, response.get());
+      }
+    }
+
+    if (outcome == RequestOutcome::SUCCEEDED && response) {
+      LogPasswordProtectionVerdict(trigger_type_, password_account_type,
+                                   response->verdict_type());
     }
   }
-
-  if (outcome == RequestOutcome::SUCCEEDED && response) {
-    LogPasswordProtectionVerdict(
-        trigger_type_, reused_password_type_,
-        password_protection_service_->GetSyncAccountType(),
-        response->verdict_type());
-  }
-
-  password_protection_service_->RequestFinished(
-      this, outcome == RequestOutcome::RESPONSE_ALREADY_CACHED,
-      std::move(response));
+  password_protection_service_->RequestFinished(this, outcome,
+                                                std::move(response));
 }
 
 void PasswordProtectionRequest::Cancel(bool timed_out) {
@@ -355,8 +492,9 @@ void PasswordProtectionRequest::Cancel(bool timed_out) {
   url_loader_.reset();
   // If request is canceled because |password_protection_service_| is shutting
   // down, ignore all these deferred navigations.
-  if (!timed_out)
+  if (!timed_out) {
     throttles_.clear();
+  }
 
   Finish(timed_out ? RequestOutcome::TIMEDOUT : RequestOutcome::CANCELED,
          nullptr);
@@ -372,4 +510,8 @@ void PasswordProtectionRequest::HandleDeferredNavigations() {
   throttles_.clear();
 }
 
+void PasswordProtectionRequest::WebContentsDestroyed() {
+  Cancel(/*timed_out=*/false);
+}
+
 }  // namespace safe_browsing
