@@ -6,20 +6,22 @@
 
 #import <WebKit/WebKit.h>
 
+#include "base/bind.h"
 #include "base/logging.h"
 #import "base/mac/foundation_util.h"
 #include "base/macros.h"
 #include "base/strings/sys_string_conversions.h"
+#include "components/content_settings/core/browser/cookie_settings.h"
 #include "components/google/core/common/google_util.h"
 #include "components/prefs/pref_registry_simple.h"
+#include "components/prefs/pref_service.h"
 #include "components/prefs/scoped_user_pref_update.h"
-#include "components/signin/core/browser/account_consistency_method.h"
 #include "components/signin/core/browser/account_reconcilor.h"
-#include "components/signin/core/browser/signin_client.h"
 #include "components/signin/core/browser/signin_header_helper.h"
+#include "components/signin/public/base/account_consistency_method.h"
+#include "ios/web/common/web_view_creation_util.h"
 #include "ios/web/public/browser_state.h"
-#include "ios/web/public/web_state/web_state_policy_decider.h"
-#include "ios/web/public/web_view_creation_util.h"
+#import "ios/web/public/navigation/web_state_policy_decider.h"
 #include "net/base/mac/url_conversions.h"
 #include "net/base/registry_controlled_domains/registry_controlled_domain.h"
 #include "url/gurl.h"
@@ -42,7 +44,8 @@ NSString* const kChromeConnectedCookieTemplate =
      "document.cookie=\"X-CHROME-CONNECTED=; path=/; domain=\" + domain + \";"
      " expires=Thu, 01-Jan-1970 00:00:01 GMT\";"
      "document.cookie=\"CHROME_CONNECTED=%@; path=/; domain=\" + domain + \";"
-     " expires=\" + new Date(%f).toGMTString() + \"; secure;\"</script></html>";
+     " expires=\" + new Date(%f).toGMTString() + \"; secure;"
+     " samesite=lax;\"</script></html>";
 
 // WebStatePolicyDecider that monitors the HTTP headers on Gaia responses,
 // reacting on the X-Chrome-Manage-Accounts header and notifying its delegate.
@@ -122,7 +125,6 @@ bool AccountConsistencyHandler::ShouldAllowResponse(NSURLResponse* response,
       [delegate_ onAddAccount];
       break;
     case signin::GAIA_SERVICE_TYPE_SIGNOUT:
-    case signin::GAIA_SERVICE_TYPE_REAUTH:
     case signin::GAIA_SERVICE_TYPE_DEFAULT:
       [delegate_ onManageAccounts];
       break;
@@ -217,19 +219,16 @@ AccountConsistencyService::CookieRequest::CookieRequest(
 
 AccountConsistencyService::AccountConsistencyService(
     web::BrowserState* browser_state,
+    PrefService* prefs,
     AccountReconcilor* account_reconcilor,
     scoped_refptr<content_settings::CookieSettings> cookie_settings,
-    GaiaCookieManagerService* gaia_cookie_manager_service,
-    SigninClient* signin_client,
-    identity::IdentityManager* identity_manager)
+    signin::IdentityManager* identity_manager)
     : browser_state_(browser_state),
+      prefs_(prefs),
       account_reconcilor_(account_reconcilor),
       cookie_settings_(cookie_settings),
-      gaia_cookie_manager_service_(gaia_cookie_manager_service),
-      signin_client_(signin_client),
       identity_manager_(identity_manager),
       applying_cookie_requests_(false) {
-  gaia_cookie_manager_service_->AddObserver(this);
   identity_manager_->AddObserver(this);
   ActiveStateManager::FromBrowserState(browser_state_)->AddObserver(this);
   LoadFromPrefs();
@@ -325,14 +324,13 @@ void AccountConsistencyService::RemoveChromeConnectedCookieFromDomain(
 
 void AccountConsistencyService::LoadFromPrefs() {
   const base::DictionaryValue* dict =
-      signin_client_->GetPrefs()->GetDictionary(kDomainsWithCookiePref);
+      prefs_->GetDictionary(kDomainsWithCookiePref);
   for (base::DictionaryValue::Iterator it(*dict); !it.IsAtEnd(); it.Advance()) {
     last_cookie_update_map_[it.key()] = base::Time();
   }
 }
 
 void AccountConsistencyService::Shutdown() {
-  gaia_cookie_manager_service_->RemoveObserver(this);
   identity_manager_->RemoveObserver(this);
   ActiveStateManager::FromBrowserState(browser_state_)->RemoveObserver(this);
   ResetWKWebView();
@@ -396,8 +394,7 @@ void AccountConsistencyService::FinishedApplyingCookieRequest(bool success) {
   CookieRequest& request = cookie_requests_.front();
   if (success) {
     DictionaryPrefUpdate update(
-        signin_client_->GetPrefs(),
-        AccountConsistencyService::kDomainsWithCookiePref);
+        prefs_, AccountConsistencyService::kDomainsWithCookiePref);
     switch (request.request_type) {
       case ADD_CHROME_CONNECTED_COOKIE:
         // Add request.domain to prefs, use |true| as a dummy value (that is
@@ -471,37 +468,32 @@ void AccountConsistencyService::OnBrowsingDataRemoved() {
   cookie_requests_.clear();
   last_cookie_update_map_.clear();
   base::DictionaryValue dict;
-  signin_client_->GetPrefs()->Set(kDomainsWithCookiePref, dict);
+  prefs_->Set(kDomainsWithCookiePref, dict);
 
   // APISID cookie has been removed, notify the GCMS.
-  gaia_cookie_manager_service_->ForceOnCookieChangeProcessing();
-}
-
-void AccountConsistencyService::OnAddAccountToCookieCompleted(
-    const std::string& account_id,
-    const GoogleServiceAuthError& error) {
-  AddChromeConnectedCookies();
-}
-
-void AccountConsistencyService::OnGaiaAccountsInCookieUpdated(
-    const std::vector<gaia::ListedAccount>& accounts,
-    const std::vector<gaia::ListedAccount>& signed_out_accounts,
-    const GoogleServiceAuthError& error) {
-  AddChromeConnectedCookies();
+  // TODO(https://crbug.com/930582) : Remove the need to expose this method
+  // or move it to the network::CookieManager.
+  identity_manager_->ForceTriggerOnCookieChange();
 }
 
 void AccountConsistencyService::OnPrimaryAccountSet(
-    const AccountInfo& account_info) {
+    const CoreAccountInfo& account_info) {
   AddChromeConnectedCookies();
 }
 
 void AccountConsistencyService::OnPrimaryAccountCleared(
-    const AccountInfo& previous_account_info) {
+    const CoreAccountInfo& previous_account_info) {
   // There is not need to remove CHROME_CONNECTED cookies on |GoogleSignedOut|
   // events as these cookies will be removed by the GaiaCookieManagerServer
   // right before fetching the Gaia logout request.
 }
 
+void AccountConsistencyService::OnAccountsInCookieUpdated(
+    const signin::AccountsInCookieJarInfo& accounts_in_cookie_jar_info,
+    const GoogleServiceAuthError& error) {
+  AddChromeConnectedCookies();
+}
+
 void AccountConsistencyService::OnActive() {
   // |browser_state_| is now active. There might be some pending cookie requests
   // to apply.
