@@ -6,7 +6,9 @@
 
 #include "base/bind.h"
 #include "base/metrics/histogram_macros.h"
+#include "build/build_config.h"
 #include "components/spellcheck/common/spellcheck.mojom.h"
+#include "components/spellcheck/common/spellcheck_features.h"
 #include "components/spellcheck/common/spellcheck_result.h"
 #include "components/spellcheck/renderer/spellcheck.h"
 #include "components/spellcheck/renderer/spellcheck_language.h"
@@ -83,8 +85,7 @@ SpellCheckProvider::SpellCheckProvider(
     : content::RenderFrameObserver(render_frame),
       content::RenderFrameObserverTracker<SpellCheckProvider>(render_frame),
       spellcheck_(spellcheck),
-      embedder_provider_(embedder_provider),
-      weak_factory_(this) {
+      embedder_provider_(embedder_provider) {
   DCHECK(spellcheck_);
   DCHECK(embedder_provider);
   if (render_frame)  // NULL in unit tests.
@@ -111,7 +112,7 @@ spellcheck::mojom::SpellCheckHost& SpellCheckProvider::GetSpellCheckHost() {
 
 void SpellCheckProvider::RequestTextChecking(
     const base::string16& text,
-    WebTextCheckingCompletion* completion) {
+    std::unique_ptr<WebTextCheckingCompletion> completion) {
   // Ignore invalid requests.
   if (text.empty() || !HasWordCharacters(text, 0)) {
     completion->DidCancelCheckingText();
@@ -119,40 +120,50 @@ void SpellCheckProvider::RequestTextChecking(
   }
 
   // Try to satisfy check from cache.
-  if (SatisfyRequestFromCache(text, completion))
+  if (SatisfyRequestFromCache(text, completion.get()))
     return;
 
   // Send this text to a browser. A browser checks the user profile and send
   // this text to the Spelling service only if a user enables this feature.
   last_request_.clear();
   last_results_.Assign(blink::WebVector<blink::WebTextCheckingResult>());
-  last_identifier_ = text_check_completions_.Add(completion);
+  last_identifier_ = text_check_completions_.Add(std::move(completion));
 
 #if BUILDFLAG(USE_BROWSER_SPELLCHECKER)
-  // Text check (unified request for grammar and spell check) is only
-  // available for browser process, so we ask the system spellchecker
-  // over mojo or return an empty result if the checker is not available.
-  GetSpellCheckHost().RequestTextCheck(
-      text, routing_id(),
-      base::BindOnce(&SpellCheckProvider::OnRespondTextCheck,
-                     weak_factory_.GetWeakPtr(), last_identifier_, text));
-#else
-  GetSpellCheckHost().CallSpellingService(
-      text, base::BindOnce(&SpellCheckProvider::OnRespondSpellingService,
-                           weak_factory_.GetWeakPtr(), last_identifier_, text));
-#endif  // !USE_BROWSER_SPELLCHECKER
+  if (spellcheck::UseBrowserSpellChecker()) {
+    // Text check (unified request for grammar and spell check) is only
+    // available for browser process, so we ask the system spellchecker
+    // over mojo or return an empty result if the checker is not available.
+    GetSpellCheckHost().RequestTextCheck(
+        text, routing_id(),
+        base::BindOnce(&SpellCheckProvider::OnRespondTextCheck,
+                       weak_factory_.GetWeakPtr(), last_identifier_, text));
+  }
+#endif
+#if BUILDFLAG(USE_RENDERER_SPELLCHECKER)
+  if (!spellcheck::UseBrowserSpellChecker()) {
+    GetSpellCheckHost().CallSpellingService(
+        text,
+        base::BindOnce(&SpellCheckProvider::OnRespondSpellingService,
+                       weak_factory_.GetWeakPtr(), last_identifier_, text));
+  }
+#endif
 }
 
-void SpellCheckProvider::FocusedNodeChanged(const blink::WebNode& unused) {
-#if BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+void SpellCheckProvider::FocusedElementChanged(
+    const blink::WebElement& unused) {
+#if defined(OS_ANDROID)
+  if (!spell_check_host_.is_bound())
+    return;
+
   WebLocalFrame* frame = render_frame()->GetWebFrame();
   WebElement element = frame->GetDocument().IsNull()
                            ? WebElement()
                            : frame->GetDocument().FocusedElement();
   bool enabled = !element.IsNull() && element.IsEditable();
-  bool checked = enabled && IsSpellCheckingEnabled();
-  GetSpellCheckHost().ToggleSpellCheck(enabled, checked);
-#endif  // USE_BROWSER_SPELLCHECKER
+  if (!enabled)
+    GetSpellCheckHost().DisconnectSessionBridge();
+#endif  // defined(OS_ANDROID)
 }
 
 bool SpellCheckProvider::IsSpellCheckingEnabled() const {
@@ -161,8 +172,8 @@ bool SpellCheckProvider::IsSpellCheckingEnabled() const {
 
 void SpellCheckProvider::CheckSpelling(
     const WebString& text,
-    int& offset,
-    int& length,
+    size_t& offset,
+    size_t& length,
     WebVector<WebString>* optional_suggestions) {
   base::string16 word = text.Utf16();
   std::vector<base::string16> suggestions;
@@ -176,9 +187,11 @@ void SpellCheckProvider::CheckSpelling(
         suggestions.begin(), suggestions.end(), web_suggestions.begin(),
         [](const base::string16& s) { return WebString::FromUTF16(s); });
     *optional_suggestions = web_suggestions;
-    UMA_HISTOGRAM_COUNTS_1M("SpellCheck.api.check.suggestions", word.size());
+    UMA_HISTOGRAM_COUNTS_1M("SpellCheck.api.check.suggestions",
+                            base::saturated_cast<int>(word.size()));
   } else {
-    UMA_HISTOGRAM_COUNTS_1M("SpellCheck.api.check", word.size());
+    UMA_HISTOGRAM_COUNTS_1M("SpellCheck.api.check",
+                            base::saturated_cast<int>(word.size()));
     // If optional_suggestions is not requested, the API is called
     // for marking.  So we use this for counting markable words.
     GetSpellCheckHost().NotifyChecked(word, 0 < length);
@@ -187,37 +200,35 @@ void SpellCheckProvider::CheckSpelling(
 
 void SpellCheckProvider::RequestCheckingOfText(
     const WebString& text,
-    WebTextCheckingCompletion* completion) {
-  RequestTextChecking(text.Utf16(), completion);
-  UMA_HISTOGRAM_COUNTS_1M("SpellCheck.api.async", text.length());
+    std::unique_ptr<WebTextCheckingCompletion> completion) {
+  RequestTextChecking(text.Utf16(), std::move(completion));
+  UMA_HISTOGRAM_COUNTS_1M("SpellCheck.api.async",
+                          base::saturated_cast<int>(text.length()));
 }
 
-#if !BUILDFLAG(USE_BROWSER_SPELLCHECKER)
+#if BUILDFLAG(USE_RENDERER_SPELLCHECKER)
 void SpellCheckProvider::OnRespondSpellingService(
     int identifier,
     const base::string16& line,
     bool success,
     const std::vector<SpellCheckResult>& results) {
-  WebTextCheckingCompletion* completion =
-      text_check_completions_.Lookup(identifier);
-  if (!completion)
+  if (!text_check_completions_.Lookup(identifier))
     return;
+  std::unique_ptr<WebTextCheckingCompletion> completion(
+      text_check_completions_.Replace(identifier, nullptr));
   text_check_completions_.Remove(identifier);
 
   // If |success| is false, we use local spellcheck as a fallback.
   if (!success) {
-    spellcheck_->RequestTextChecking(line, completion);
+    spellcheck_->RequestTextChecking(line, std::move(completion));
     return;
   }
 
   // Double-check the returned spellchecking results with our spellchecker to
   // visualize the differences between ours and the on-line spellchecker.
   blink::WebVector<blink::WebTextCheckingResult> textcheck_results;
-  spellcheck_->CreateTextCheckingResults(SpellCheck::USE_NATIVE_CHECKER,
-                                         0,
-                                         line,
-                                         results,
-                                         &textcheck_results);
+  spellcheck_->CreateTextCheckingResults(SpellCheck::USE_NATIVE_CHECKER, 0,
+                                         line, results, &textcheck_results);
   completion->DidFinishCheckingText(textcheck_results);
 
   // Cache the request and the converted results.
@@ -226,11 +237,10 @@ void SpellCheckProvider::OnRespondSpellingService(
 }
 #endif
 
-bool SpellCheckProvider::HasWordCharacters(
-    const base::string16& text,
-    int index) const {
+bool SpellCheckProvider::HasWordCharacters(const base::string16& text,
+                                           size_t index) const {
   const base::char16* data = text.data();
-  int length = text.length();
+  size_t length = text.length();
   while (index < length) {
     uint32_t code = 0;
     U16_NEXT(data, index, length, code);
@@ -248,10 +258,10 @@ void SpellCheckProvider::OnRespondTextCheck(
     const std::vector<SpellCheckResult>& results) {
   // TODO(groby): Unify with SpellCheckProvider::OnRespondSpellingService
   DCHECK(spellcheck_);
-  WebTextCheckingCompletion* completion =
-      text_check_completions_.Lookup(identifier);
-  if (!completion)
+  if (!text_check_completions_.Lookup(identifier))
     return;
+  std::unique_ptr<WebTextCheckingCompletion> completion(
+      text_check_completions_.Replace(identifier, nullptr));
   text_check_completions_.Remove(identifier);
   blink::WebVector<blink::WebTextCheckingResult> textcheck_results;
   spellcheck_->CreateTextCheckingResults(SpellCheck::DO_NOT_MODIFY,
