@@ -8,7 +8,9 @@
 #include <utility>
 #include <vector>
 
+#include "base/bind.h"
 #include "base/metrics/histogram_macros.h"
+#include "base/optional.h"
 #include "base/timer/timer.h"
 #include "base/trace_event/trace_event.h"
 #include "base/trace_event/traced_value.h"
@@ -21,15 +23,43 @@
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/navigation_handle.h"
 #include "content/public/browser/web_contents.h"
-#include "content/public/common/console_message_level.h"
 #include "services/metrics/public/cpp/ukm_builders.h"
 #include "services/metrics/public/cpp/ukm_recorder.h"
 #include "services/metrics/public/cpp/ukm_source.h"
+#include "third_party/blink/public/mojom/devtools/console_message.mojom.h"
 #include "ui/base/page_transition_types.h"
 #include "url/gurl.h"
 
 namespace subresource_filter {
 
+namespace {
+
+using CheckResults =
+    std::vector<SubresourceFilterSafeBrowsingClient::CheckResult>;
+
+base::Optional<RedirectPosition> GetEnforcementRedirectPosition(
+    const CheckResults& results) {
+  // Safe cast since we have strict limits on HTTP redirects.
+  int num_results = static_cast<int>(results.size());
+  for (int i = num_results - 1; i >= 0; --i) {
+    bool warning = false;
+    ActivationList list = GetListForThreatTypeAndMetadata(
+        results[i].threat_type, results[i].threat_metadata, &warning);
+    if (!warning && list != ActivationList::NONE) {
+      if (num_results == 1)
+        return RedirectPosition::kOnly;
+      if (i == 0)
+        return RedirectPosition::kFirst;
+      if (i == num_results - 1)
+        return RedirectPosition::kLast;
+      return RedirectPosition::kMiddle;
+    }
+  }
+  return base::nullopt;
+}
+
+}  // namespace
+
 SubresourceFilterSafeBrowsingActivationThrottle::
     SubresourceFilterSafeBrowsingActivationThrottle(
         content::NavigationHandle* handle,
@@ -84,14 +114,13 @@ void SubresourceFilterSafeBrowsingActivationThrottle::OnCheckUrlResultOnUI(
   DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
   size_t request_id = result.request_id;
   DCHECK_LT(request_id, check_results_.size());
-  DCHECK_LT(request_id, check_start_times_.size());
 
   auto& stored_result = check_results_.at(request_id);
   CHECK(!stored_result.finished);
   stored_result = result;
 
   UMA_HISTOGRAM_TIMES("SubresourceFilter.SafeBrowsing.TotalCheckTime",
-                      base::TimeTicks::Now() - check_start_times_[request_id]);
+                      base::TimeTicks::Now() - result.start_time);
   if (deferring_ && HasFinishedAllSafeBrowsingChecks()) {
     NotifyResult();
 
@@ -121,14 +150,14 @@ SubresourceFilterSafeBrowsingActivationThrottle::ConfigResult::~ConfigResult() =
 
 void SubresourceFilterSafeBrowsingActivationThrottle::CheckCurrentUrl() {
   DCHECK(database_client_);
-  check_start_times_.push_back(base::TimeTicks::Now());
   check_results_.emplace_back();
   size_t id = check_results_.size() - 1;
   io_task_runner_->PostTask(
       FROM_HERE,
       base::BindOnce(&SubresourceFilterSafeBrowsingClient::CheckUrlOnIO,
                      base::Unretained(database_client_.get()),
-                     navigation_handle()->GetURL(), id));
+                     navigation_handle()->GetURL(), id,
+                     base::TimeTicks::Now()));
 }
 
 void SubresourceFilterSafeBrowsingActivationThrottle::NotifyResult() {
@@ -136,9 +165,8 @@ void SubresourceFilterSafeBrowsingActivationThrottle::NotifyResult() {
                "SubresourceFilterSafeBrowsingActivationThrottle::NotifyResult");
   DCHECK(!check_results_.empty());
 
-  // Determine which results to consider for safebrowsing/abusive.
-  std::vector<SubresourceFilterSafeBrowsingClient::CheckResult>
-      check_results_to_consider = {check_results_.back()};
+  // Determine which results to consider for safebrowsing/abusive enforcement.
+  CheckResults check_results_to_consider = {check_results_.back()};
   if (check_results_.size() >= 2 &&
       base::FeatureList::IsEnabled(
           kSafeBrowsingSubresourceFilterConsiderRedirects)) {
@@ -171,7 +199,7 @@ void SubresourceFilterSafeBrowsingActivationThrottle::NotifyResult() {
   if (selection.warning &&
       activation_level == mojom::ActivationLevel::kEnabled) {
     NavigationConsoleLogger::LogMessageOnCommit(
-        navigation_handle(), content::CONSOLE_MESSAGE_LEVEL_WARNING,
+        navigation_handle(), blink::mojom::ConsoleMessageLevel::kWarning,
         kActivationWarningConsoleMessage);
     activation_level = mojom::ActivationLevel::kDisabled;
   }
@@ -210,6 +238,15 @@ void SubresourceFilterSafeBrowsingActivationThrottle::
     DCHECK_EQ(ActivationDecision::ACTIVATED, decision);
     builder.SetDryRun(true);
   }
+
+  if (auto optional_position = GetEnforcementRedirectPosition(check_results_)) {
+    RedirectPosition position = *optional_position;
+    UMA_HISTOGRAM_ENUMERATION(
+        "SubresourceFilter.PageLoad.Activation.RedirectPosition2.Enforcement",
+        position);
+    builder.SetEnforcementRedirectPosition(static_cast<int64_t>(position));
+  }
+
   builder.Record(ukm::UkmRecorder::Get());
 
   UMA_HISTOGRAM_ENUMERATION("SubresourceFilter.PageLoad.ActivationDecision",
@@ -296,26 +333,6 @@ SubresourceFilterSafeBrowsingActivationThrottle::GetActivationDecision(
   // Get the activation level for the matching configuration.
   auto activation_level =
       selected_config->config.activation_options.activation_level;
-
-  // If there is an activation triggered by the activation list (not a dry run),
-  // report where in the redirect chain it was triggered.
-  if (selected_config->config.activation_conditions.activation_scope ==
-          ActivationScope::ACTIVATION_LIST &&
-      activation_level == mojom::ActivationLevel::kEnabled) {
-    ActivationPosition position;
-    if (configs.size() == 1) {
-      position = ActivationPosition::kOnly;
-    } else if (selected_index == 0) {
-      position = ActivationPosition::kFirst;
-    } else if (selected_index == configs.size() - 1) {
-      position = ActivationPosition::kLast;
-    } else {
-      position = ActivationPosition::kMiddle;
-    }
-    UMA_HISTOGRAM_ENUMERATION(
-        "SubresourceFilter.PageLoad.Activation.RedirectPosition", position);
-  }
-
   // Compute and return the activation decision.
   return activation_level == mojom::ActivationLevel::kDisabled
              ? ActivationDecision::ACTIVATION_DISABLED
