@@ -8,16 +8,18 @@
 #include <utility>
 #include <vector>
 
+#include "base/bind.h"
 #include "base/bind_helpers.h"
+#include "base/cancelable_callback.h"
 #include "base/run_loop.h"
 #include "base/test/mock_callback.h"
 #include "base/test/scoped_task_environment.h"
 #include "components/prefs/testing_pref_service.h"
 #include "components/sync/base/hash_util.h"
-#include "components/sync/device_info/device_info.h"
 #include "components/sync/model/model_type_store_test_util.h"
 #include "components/sync/protocol/session_specifics.pb.h"
 #include "components/sync/test/test_matchers.h"
+#include "components/sync_device_info/local_device_info_util.h"
 #include "components/sync_sessions/mock_sync_sessions_client.h"
 #include "components/sync_sessions/session_sync_prefs.h"
 #include "components/sync_sessions/test_matchers.h"
@@ -50,20 +52,19 @@ using testing::UnorderedElementsAre;
 using testing::_;
 
 const char kCacheGuid[] = "SomeCacheGuid";
-const char kClientName[] = "Some Client Name";
 
 // A mock callback that a) can be used as mock to verify call expectations and
 // b) conveniently exposes the last instantiated session store.
-class MockFactoryCompletionCallback {
+class MockOpenCallback {
  public:
   MOCK_METHOD3(Run,
                void(const base::Optional<syncer::ModelError>& error,
                     SessionStore* store,
                     MetadataBatch* metadata_batch));
 
-  SessionStore::FactoryCompletionCallback Get() {
+  SessionStore::OpenCallback Get() {
     return base::BindOnce(
-        [](MockFactoryCompletionCallback* callback,
+        [](MockOpenCallback* callback,
            const base::Optional<syncer::ModelError>& error,
            std::unique_ptr<SessionStore> store,
            std::unique_ptr<MetadataBatch> metadata_batch) {
@@ -152,68 +153,61 @@ std::map<std::string, SessionSpecifics> ReadAllPersistedDataFrom(
   return result;
 }
 
-class SessionStoreFactoryTest : public ::testing::Test {
+class SessionStoreOpenTest : public ::testing::Test {
  protected:
-  SessionStoreFactoryTest()
-      : local_device_info_(kCacheGuid,
-                           kClientName,
-                           "Chromium 10k",
-                           "Chrome 10k",
-                           sync_pb::SyncEnums_DeviceType_TYPE_LINUX,
-                           "device_id"),
-        session_sync_prefs_(&pref_service_),
+  SessionStoreOpenTest()
+      : session_sync_prefs_(&pref_service_),
         underlying_store_(
             syncer::ModelTypeStoreTestUtil::CreateInMemoryStoreForTest(
                 syncer::SESSIONS)) {
     SessionSyncPrefs::RegisterProfilePrefs(pref_service_.registry());
 
-    ON_CALL(mock_sync_sessions_client_, GetSessionSyncPrefs())
+    mock_sync_sessions_client_ =
+        std::make_unique<testing::NiceMock<MockSyncSessionsClient>>();
+
+    ON_CALL(*mock_sync_sessions_client_, GetSessionSyncPrefs())
         .WillByDefault(Return(&session_sync_prefs_));
-    ON_CALL(mock_sync_sessions_client_, GetStoreFactory())
+    ON_CALL(*mock_sync_sessions_client_, GetStoreFactory())
         .WillByDefault(
             Return(syncer::ModelTypeStoreTestUtil::FactoryForForwardingStore(
                 underlying_store_.get())));
-
-    factory_ = SessionStore::CreateFactory(
-        &mock_sync_sessions_client_, mock_restored_foreign_tab_callback_.Get());
   }
 
-  ~SessionStoreFactoryTest() override {}
+  ~SessionStoreOpenTest() override {}
 
   base::test::ScopedTaskEnvironment task_environment_;
-  const syncer::DeviceInfo local_device_info_;
   TestingPrefServiceSimple pref_service_;
   SessionSyncPrefs session_sync_prefs_;
-  testing::NiceMock<MockSyncSessionsClient> mock_sync_sessions_client_;
+  std::unique_ptr<MockSyncSessionsClient> mock_sync_sessions_client_;
   testing::NiceMock<
       base::MockCallback<SessionStore::RestoredForeignTabCallback>>
       mock_restored_foreign_tab_callback_;
-
   std::unique_ptr<ModelTypeStore> underlying_store_;
-  SessionStore::Factory factory_;
 };
 
-TEST_F(SessionStoreFactoryTest, ShouldCreateStore) {
+TEST_F(SessionStoreOpenTest, ShouldCreateStore) {
   ASSERT_THAT(session_sync_prefs_.GetSyncSessionsGUID(), IsEmpty());
 
-  MockFactoryCompletionCallback completion;
+  MockOpenCallback completion;
   EXPECT_CALL(completion, Run(NoModelError(), /*store=*/NotNull(),
                               MetadataBatchContains(_, IsEmpty())));
-  factory_.Run(local_device_info_, completion.Get());
+  SessionStore::Open(kCacheGuid, mock_restored_foreign_tab_callback_.Get(),
+                     mock_sync_sessions_client_.get(), completion.Get());
   completion.Wait();
   ASSERT_THAT(completion.GetResult(), NotNull());
   EXPECT_THAT(completion.GetResult()->local_session_info().client_name,
-              Eq(kClientName));
+              Eq(syncer::GetSessionNameBlocking()));
   EXPECT_THAT(session_sync_prefs_.GetSyncSessionsGUID(),
               Eq(std::string("session_sync") + kCacheGuid));
 }
 
-TEST_F(SessionStoreFactoryTest, ShouldReadSessionsGuidFromPrefs) {
+TEST_F(SessionStoreOpenTest, ShouldReadSessionsGuidFromPrefs) {
   const std::string kCachedGuid = "cachedguid1";
   session_sync_prefs_.SetSyncSessionsGUID(kCachedGuid);
 
-  NiceMock<MockFactoryCompletionCallback> completion;
-  factory_.Run(local_device_info_, completion.Get());
+  NiceMock<MockOpenCallback> completion;
+  SessionStore::Open(kCacheGuid, mock_restored_foreign_tab_callback_.Get(),
+                     mock_sync_sessions_client_.get(), completion.Get());
   completion.Wait();
   ASSERT_THAT(completion.GetResult(), NotNull());
   EXPECT_THAT(completion.GetResult()->local_session_info().session_tag,
@@ -221,8 +215,46 @@ TEST_F(SessionStoreFactoryTest, ShouldReadSessionsGuidFromPrefs) {
   EXPECT_THAT(session_sync_prefs_.GetSyncSessionsGUID(), Eq(kCachedGuid));
 }
 
+TEST_F(SessionStoreOpenTest, ShouldNotUseClientIfCancelled) {
+  // Mimics a caller that uses a weak pointer.
+  class Caller {
+   public:
+    explicit Caller(SessionStore::OpenCallback cb) : cb_(std::move(cb)) {}
+
+    SessionStore::OpenCallback GetCancelableCallback() {
+      return base::BindOnce(&Caller::Completed, weak_ptr_factory_.GetWeakPtr());
+    }
+
+   private:
+    void Completed(const base::Optional<syncer::ModelError>& error,
+                   std::unique_ptr<SessionStore> store,
+                   std::unique_ptr<syncer::MetadataBatch> metadata_batch) {
+      std::move(cb_).Run(error, std::move(store), std::move(metadata_batch));
+    }
+
+    SessionStore::OpenCallback cb_;
+    base::WeakPtrFactory<Caller> weak_ptr_factory_{this};
+  };
+
+  NiceMock<MockOpenCallback> mock_completion;
+  auto caller = std::make_unique<Caller>(mock_completion.Get());
+
+  EXPECT_CALL(mock_completion, Run(_, _, _)).Times(0);
+
+  SessionStore::Open(kCacheGuid, mock_restored_foreign_tab_callback_.Get(),
+                     mock_sync_sessions_client_.get(),
+                     caller->GetCancelableCallback());
+
+  // The client gets destroyed before callback completion.
+  mock_sync_sessions_client_.reset();
+  caller.reset();
+
+  // Run until idle to test for crashes due to use-after-free.
+  base::RunLoop().RunUntilIdle();
+}
+
 // Test fixture that creates an initial session store.
-class SessionStoreTest : public SessionStoreFactoryTest {
+class SessionStoreTest : public SessionStoreOpenTest {
  protected:
   const std::string kLocalSessionTag = "localsessiontag";
 
@@ -232,8 +264,9 @@ class SessionStoreTest : public SessionStoreFactoryTest {
   }
 
   std::unique_ptr<SessionStore> CreateSessionStore() {
-    NiceMock<MockFactoryCompletionCallback> completion;
-    factory_.Run(local_device_info_, completion.Get());
+    NiceMock<MockOpenCallback> completion;
+    SessionStore::Open(kCacheGuid, mock_restored_foreign_tab_callback_.Get(),
+                       mock_sync_sessions_client_.get(), completion.Get());
     completion.Wait();
     EXPECT_THAT(completion.GetResult(), NotNull());
     return completion.StealResult();
@@ -296,12 +329,13 @@ TEST_F(SessionStoreTest, ShouldWriteAndRestoreMetadata) {
                                     ElementsAre(Pair(kStorageKey1, _))));
 
   // Create second session store.
-  NiceMock<MockFactoryCompletionCallback> completion;
+  NiceMock<MockOpenCallback> completion;
   EXPECT_CALL(completion, Run(NoModelError(), /*store=*/NotNull(),
                               MetadataBatchContains(
                                   HasEncryptionKeyName(kEncryptionKeyName1),
                                   ElementsAre(Pair(kStorageKey1, _)))));
-  factory_.Run(local_device_info_, completion.Get());
+  SessionStore::Open(kCacheGuid, mock_restored_foreign_tab_callback_.Get(),
+                     mock_sync_sessions_client_.get(), completion.Get());
   completion.Wait();
   EXPECT_THAT(completion.GetResult(), NotNull());
   EXPECT_NE(session_store(), completion.GetResult());
