@@ -15,6 +15,8 @@ namespace viz {
 SkColorType ResourceFormatToClosestSkColorType(bool gpu_compositing,
                                                ResourceFormat format) {
   if (!gpu_compositing) {
+    // TODO(crbug.com/986405): Remove this assumption and have clients tag
+    // resources with the correct format.
     // In software compositing we lazily use RGBA_8888 throughout the system,
     // but actual pixel encodings are the native skia bit ordering, which can be
     // RGBA or BGRA.
@@ -36,8 +38,8 @@ SkColorType ResourceFormatToClosestSkColorType(bool gpu_compositing,
     case LUMINANCE_8:
       return kGray_8_SkColorType;
     case RGBX_8888:
-      return kRGB_888x_SkColorType;
     case ETC1:
+      return kRGB_888x_SkColorType;
     case RED_8:
     case LUMINANCE_F16:
     case R16_EXT:
@@ -49,6 +51,7 @@ SkColorType ResourceFormatToClosestSkColorType(bool gpu_compositing,
     case YVU_420:
     case YUV_420_BIPLANAR:
     case UYVY_422:
+    case P010:
       return kN32_SkColorType;
     case RGBA_F16:
       return kRGBA_F16_SkColorType;
@@ -67,6 +70,7 @@ int BitsPerPixel(ResourceFormat format) {
     case BGRX_8888:
     case RGBX_1010102:
     case BGRX_1010102:
+    case P010:
       return 32;
     case RGBA_4444:
     case RGB_565:
@@ -113,6 +117,7 @@ unsigned int GLDataType(ResourceFormat format) {
       GL_ZERO,                             // YVU_420
       GL_ZERO,                             // YUV_420_BIPLANAR
       GL_ZERO,                             // UYVY_422
+      GL_ZERO,                             // P010
   };
   static_assert(base::size(format_gl_data_type) == (RESOURCE_FORMAT_MAX + 1),
                 "format_gl_data_type does not handle all cases.");
@@ -143,6 +148,7 @@ unsigned int GLDataFormat(ResourceFormat format) {
       GL_ZERO,       // YVU_420
       GL_ZERO,       // YUV_420_BIPLANAR
       GL_ZERO,       // UYVY_422
+      GL_ZERO,       // P010
   };
   static_assert(base::size(format_gl_data_format) == (RESOURCE_FORMAT_MAX + 1),
                 "format_gl_data_format does not handle all cases.");
@@ -194,6 +200,7 @@ unsigned int GLCopyTextureInternalFormat(ResourceFormat format) {
       GL_ZERO,       // YVU_420
       GL_ZERO,       // YUV_420_BIPLANAR
       GL_ZERO,       // UYVY_422
+      GL_ZERO,       // P010
   };
 
   static_assert(base::size(format_gl_data_format) == (RESOURCE_FORMAT_MAX + 1),
@@ -234,6 +241,8 @@ gfx::BufferFormat BufferFormat(ResourceFormat format) {
       return gfx::BufferFormat::YUV_420_BIPLANAR;
     case UYVY_422:
       return gfx::BufferFormat::UYVY_422;
+    case P010:
+      return gfx::BufferFormat::P010;
     case ETC1:
     case ALPHA_8:
     case LUMINANCE_8:
@@ -275,16 +284,17 @@ unsigned int TextureStorageFormat(ResourceFormat format) {
     case R16_EXT:
       return GL_R16_EXT;
     case RGBX_8888:
+    case ETC1:
       return GL_RGB8_OES;
     case RGBX_1010102:
       return GL_RGB10_A2_EXT;
     case BGR_565:
-    case ETC1:
     case BGRX_8888:
     case BGRX_1010102:
     case YVU_420:
     case YUV_420_BIPLANAR:
     case UYVY_422:
+    case P010:
       break;
   }
   NOTREACHED();
@@ -316,6 +326,7 @@ bool IsGpuMemoryBufferFormatSupported(ResourceFormat format) {
     case YVU_420:
     case YUV_420_BIPLANAR:
     case UYVY_422:
+    case P010:
       return false;
   }
   NOTREACHED();
@@ -345,6 +356,7 @@ bool IsBitmapFormatSupported(ResourceFormat format) {
     case YVU_420:
     case YUV_420_BIPLANAR:
     case UYVY_422:
+    case P010:
       return false;
   }
   NOTREACHED();
@@ -383,10 +395,11 @@ ResourceFormat GetResourceFormat(gfx::BufferFormat format) {
       return YUV_420_BIPLANAR;
     case gfx::BufferFormat::UYVY_422:
       return UYVY_422;
-    default:
-      NOTREACHED();
-      return RGBA_8888;
+    case gfx::BufferFormat::P010:
+      return P010;
   }
+  NOTREACHED();
+  return RGBA_8888;
 }
 
 bool GLSupportsFormat(ResourceFormat format) {
@@ -397,6 +410,7 @@ bool GLSupportsFormat(ResourceFormat format) {
     case YVU_420:
     case YUV_420_BIPLANAR:
     case UYVY_422:
+    case P010:
       return false;
     default:
       return true;
@@ -421,27 +435,30 @@ VkFormat ToVkFormat(ResourceFormat format) {
     case RG_88:
       return VK_FORMAT_R8G8_UNORM;
     case RGBA_F16:
-      return VK_FORMAT_R16_SFLOAT;
+      return VK_FORMAT_R16G16B16A16_SFLOAT;
     case R16_EXT:
       return VK_FORMAT_R16_UNORM;
     case RGBX_8888:
-      return VK_FORMAT_R8G8B8_UNORM;
+      return VK_FORMAT_R8G8B8A8_UNORM;
     case BGRX_8888:
-      return VK_FORMAT_B8G8R8_UNORM;
+      return VK_FORMAT_B8G8R8A8_UNORM;
     case RGBX_1010102:
       return VK_FORMAT_A2R10G10B10_UNORM_PACK32;
     case BGRX_1010102:
       return VK_FORMAT_A2B10G10R10_UNORM_PACK32;
     case ALPHA_8:
+      return VK_FORMAT_R8_UNORM;
     case LUMINANCE_8:
+      return VK_FORMAT_R8_UNORM;
     case LUMINANCE_F16:
     case YVU_420:
     case YUV_420_BIPLANAR:
     case UYVY_422:
     case ETC1:
+    case P010:
       break;
   }
-  NOTREACHED();
+  NOTREACHED() << "Unsupported format " << format;
   return VK_FORMAT_UNDEFINED;
 }
 #endif
