@@ -10,13 +10,14 @@
 
 #include "base/bind.h"
 #include "base/command_line.h"
+#include "base/feature_list.h"
 #include "base/lazy_instance.h"
 #include "base/memory/shared_memory.h"
 #include "base/task/post_task.h"
 #include "base/task_runner_util.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "build/build_config.h"
-#include "components/crash/core/common/crash_key.h"
+#include "components/viz/common/gpu/metal_context_provider.h"
 #include "components/viz/common/gpu/vulkan_context_provider.h"
 #include "components/viz/common/gpu/vulkan_in_process_context_provider.h"
 #include "gpu/command_buffer/client/gpu_memory_buffer_manager.h"
@@ -25,6 +26,7 @@
 #include "gpu/command_buffer/service/shared_context_state.h"
 #include "gpu/command_buffer/service/sync_point_manager.h"
 #include "gpu/config/dx_diag_node.h"
+#include "gpu/config/gpu_finch_features.h"
 #include "gpu/config/gpu_info_collector.h"
 #include "gpu/config/gpu_switches.h"
 #include "gpu/config/gpu_util.h"
@@ -36,16 +38,16 @@
 #include "gpu/ipc/service/gpu_channel_manager.h"
 #include "gpu/ipc/service/gpu_memory_buffer_factory.h"
 #include "gpu/ipc/service/gpu_watchdog_thread.h"
+#include "gpu/ipc/service/image_decode_accelerator_worker.h"
 #include "gpu/vulkan/buildflags.h"
 #include "ipc/ipc_channel_handle.h"
 #include "ipc/ipc_sync_channel.h"
 #include "ipc/ipc_sync_message_filter.h"
+#include "media/gpu/buildflags.h"
 #include "media/gpu/gpu_video_accelerator_util.h"
 #include "media/gpu/gpu_video_encode_accelerator_factory.h"
 #include "media/gpu/ipc/service/gpu_video_decode_accelerator.h"
 #include "media/gpu/ipc/service/media_gpu_channel_manager.h"
-#include "media/mojo/services/mojo_jpeg_decode_accelerator_service.h"
-#include "media/mojo/services/mojo_jpeg_encode_accelerator_service.h"
 #include "media/mojo/services/mojo_video_encode_accelerator_provider.h"
 #include "mojo/public/cpp/bindings/strong_binding.h"
 #include "third_party/skia/include/gpu/GrContext.h"
@@ -60,6 +62,10 @@
 #include "ui/gl/init/gl_factory.h"
 #include "url/gurl.h"
 
+#if BUILDFLAG(USE_VAAPI)
+#include "media/gpu/vaapi/vaapi_jpeg_decode_accelerator_worker.h"
+#endif  // BUILDFLAG(USE_VAAPI)
+
 #if defined(OS_ANDROID)
 #include "components/viz/service/gl/throw_uncaught_exception.h"
 #endif
@@ -70,10 +76,13 @@
 #include "components/arc/video_accelerator/gpu_arc_video_protected_buffer_allocator.h"
 #include "components/arc/video_accelerator/protected_buffer_manager.h"
 #include "components/arc/video_accelerator/protected_buffer_manager_proxy.h"
+#include "components/chromeos_camera/gpu_mjpeg_decode_accelerator_factory.h"
+#include "components/chromeos_camera/mojo_jpeg_encode_accelerator_service.h"
+#include "components/chromeos_camera/mojo_mjpeg_decode_accelerator_service.h"
 #endif  // defined(OS_CHROMEOS)
 
 #if defined(OS_WIN)
-#include "gpu/ipc/service/direct_composition_surface_win.h"
+#include "ui/gl/direct_composition_surface_win.h"
 #endif
 
 #if defined(OS_MACOSX)
@@ -88,6 +97,15 @@ static base::LazyInstance<base::RepeatingCallback<
     void(int severity, size_t message_start, const std::string& message)>>::
     Leaky g_log_callback = LAZY_INSTANCE_INITIALIZER;
 
+bool IsAcceleratedJpegDecodeSupported() {
+#if defined(OS_CHROMEOS)
+  return chromeos_camera::GpuMjpegDecodeAcceleratorFactory::
+      IsAcceleratedJpegDecodeSupported();
+#else
+  return false;
+#endif  // defined(OS_CHROMEOS)
+}
+
 bool GpuLogMessageHandler(int severity,
                           const char* file,
                           int line,
@@ -130,24 +148,23 @@ GpuServiceImpl::GpuServiceImpl(
     const base::Optional<gpu::GPUInfo>& gpu_info_for_hardware_gpu,
     const base::Optional<gpu::GpuFeatureInfo>&
         gpu_feature_info_for_hardware_gpu,
+    const gpu::GpuExtraInfo& gpu_extra_info,
     gpu::VulkanImplementation* vulkan_implementation,
     base::OnceClosure exit_callback)
     : main_runner_(base::ThreadTaskRunnerHandle::Get()),
       io_runner_(std::move(io_runner)),
       watchdog_thread_(std::move(watchdog_thread)),
-      gpu_memory_buffer_factory_(
-          gpu::GpuMemoryBufferFactory::CreateNativeType()),
       gpu_preferences_(gpu_preferences),
       gpu_info_(gpu_info),
       gpu_feature_info_(gpu_feature_info),
       gpu_info_for_hardware_gpu_(gpu_info_for_hardware_gpu),
       gpu_feature_info_for_hardware_gpu_(gpu_feature_info_for_hardware_gpu),
+      gpu_extra_info_(gpu_extra_info),
 #if BUILDFLAG(ENABLE_VULKAN)
       vulkan_implementation_(vulkan_implementation),
 #endif
       exit_callback_(std::move(exit_callback)),
-      bindings_(std::make_unique<mojo::BindingSet<mojom::GpuService>>()),
-      weak_ptr_factory_(this) {
+      bindings_(std::make_unique<mojo::BindingSet<mojom::GpuService>>()) {
   DCHECK(!io_runner_->BelongsToCurrentThread());
   DCHECK(exit_callback_);
 
@@ -159,11 +176,35 @@ GpuServiceImpl::GpuServiceImpl(
   if (vulkan_implementation_) {
     vulkan_context_provider_ =
         VulkanInProcessContextProvider::Create(vulkan_implementation_);
-    if (!vulkan_context_provider_)
+    if (vulkan_context_provider_) {
+      // If Vulkan is supported, then OOP-R is supported.
+      gpu_info_.oop_rasterization_supported = true;
+      gpu_feature_info_.status_values[gpu::GPU_FEATURE_TYPE_OOP_RASTERIZATION] =
+          gpu::kGpuFeatureStatusEnabled;
+    } else {
       DLOG(WARNING) << "Failed to create Vulkan context provider.";
+    }
   }
 #endif
 
+#if BUILDFLAG(USE_VAAPI)
+  if (base::FeatureList::IsEnabled(
+          features::kVaapiJpegImageDecodeAcceleration)) {
+    jpeg_decode_accelerator_worker_ =
+        media::VaapiJpegDecodeAcceleratorWorker::Create();
+  }
+#endif
+
+#if defined(OS_MACOSX)
+  if (gpu_feature_info_.status_values[gpu::GPU_FEATURE_TYPE_METAL] ==
+      gpu::kGpuFeatureStatusEnabled) {
+    metal_context_provider_ = MetalContextProvider::Create();
+  }
+#endif
+
+  gpu_memory_buffer_factory_ =
+      gpu::GpuMemoryBufferFactory::CreateNativeType(vulkan_context_provider());
+
   weak_ptr_ = weak_ptr_factory_.GetWeakPtr();
 }
 
@@ -182,18 +223,18 @@ GpuServiceImpl::~GpuServiceImpl() {
     wait.Wait();
   }
 
-  // The sequence id and scheduler_ could be null for unit tests.
-  if (!skia_output_surface_sequence_id_.is_null()) {
-    DCHECK(scheduler_);
-    scheduler_->DestroySequence(skia_output_surface_sequence_id_);
-  }
-
   media_gpu_channel_manager_.reset();
   gpu_channel_manager_.reset();
 
   // Scheduler must be destroyed before sync point manager is destroyed.
   scheduler_.reset();
   owned_sync_point_manager_.reset();
+  owned_shared_image_manager_.reset();
+
+  // The image decode accelerator worker must outlive the GPU channel manager so
+  // that it doesn't get any decode requests during/after destruction.
+  DCHECK(!gpu_channel_manager_);
+  jpeg_decode_accelerator_worker_.reset();
 
   // Signal this event before destroying the child process. That way all
   // background threads can cleanup. For example, in the renderer the
@@ -215,25 +256,33 @@ void GpuServiceImpl::UpdateGPUInfo() {
       media::GpuVideoAcceleratorUtil::ConvertMediaToGpuEncodeProfiles(
           media::GpuVideoEncodeAcceleratorFactory::GetSupportedProfiles(
               gpu_preferences_));
-  gpu_info_.jpeg_decode_accelerator_supported = media::
-      GpuJpegDecodeAcceleratorFactory::IsAcceleratedJpegDecodeSupported();
+  gpu_info_.jpeg_decode_accelerator_supported =
+      IsAcceleratedJpegDecodeSupported();
+
+  if (jpeg_decode_accelerator_worker_) {
+    gpu_info_.image_decode_accelerator_supported_profiles =
+        jpeg_decode_accelerator_worker_->GetSupportedProfiles();
+  }
+
   // Record initialization only after collecting the GPU info because that can
   // take a significant amount of time.
   gpu_info_.initialization_time = base::Time::Now() - start_time_;
 }
 
 void GpuServiceImpl::InitializeWithHost(
-    mojom::GpuHostPtr gpu_host,
+    mojo::PendingRemote<mojom::GpuHost> pending_gpu_host,
     gpu::GpuProcessActivityFlags activity_flags,
     scoped_refptr<gl::GLSurface> default_offscreen_surface,
     gpu::SyncPointManager* sync_point_manager,
+    gpu::SharedImageManager* shared_image_manager,
     base::WaitableEvent* shutdown_event) {
   DCHECK(main_runner_->BelongsToCurrentThread());
+
+  mojo::Remote<mojom::GpuHost> gpu_host(std::move(pending_gpu_host));
   gpu_host->DidInitialize(gpu_info_, gpu_feature_info_,
                           gpu_info_for_hardware_gpu_,
-                          gpu_feature_info_for_hardware_gpu_);
-  gpu_host_ =
-      mojom::ThreadSafeGpuHostPtr::Create(gpu_host.PassInterface(), io_runner_);
+                          gpu_feature_info_for_hardware_gpu_, gpu_extra_info_);
+  gpu_host_ = mojo::SharedRemote<mojom::GpuHost>(gpu_host.Unbind(), io_runner_);
   if (!in_host_process()) {
     // The global callback is reset from the dtor. So Unretained() here is safe.
     // Note that the callback can be called from any thread. Consequently, the
@@ -243,10 +292,17 @@ void GpuServiceImpl::InitializeWithHost(
     logging::SetLogMessageHandler(GpuLogMessageHandler);
   }
 
-  sync_point_manager_ = sync_point_manager;
-  if (!sync_point_manager_) {
+  if (!sync_point_manager) {
     owned_sync_point_manager_ = std::make_unique<gpu::SyncPointManager>();
-    sync_point_manager_ = owned_sync_point_manager_.get();
+    sync_point_manager = owned_sync_point_manager_.get();
+  }
+
+  if (!shared_image_manager) {
+    // The shared image will be only used on GPU main thread, so it doesn't need
+    // to be thread safe.
+    owned_shared_image_manager_ =
+        std::make_unique<gpu::SharedImageManager>(false /* thread_safe */);
+    shared_image_manager = owned_shared_image_manager_.get();
   }
 
   shutdown_event_ = shutdown_event;
@@ -258,20 +314,18 @@ void GpuServiceImpl::InitializeWithHost(
   }
 
   scheduler_ =
-      std::make_unique<gpu::Scheduler>(main_runner_, sync_point_manager_);
-
-  skia_output_surface_sequence_id_ =
-      scheduler_->CreateSequence(gpu::SchedulingPriority::kHigh);
+      std::make_unique<gpu::Scheduler>(main_runner_, sync_point_manager);
 
   // Defer creation of the render thread. This is to prevent it from handling
   // IPC messages before the sandbox has been enabled and all other necessary
   // initialization has succeeded.
   gpu_channel_manager_ = std::make_unique<gpu::GpuChannelManager>(
       gpu_preferences_, this, watchdog_thread_.get(), main_runner_, io_runner_,
-      scheduler_.get(), sync_point_manager_, gpu_memory_buffer_factory_.get(),
-      gpu_feature_info_, std::move(activity_flags),
-      std::move(default_offscreen_surface),
-      nullptr /* image_decode_accelerator_worker */, vulkan_context_provider());
+      scheduler_.get(), sync_point_manager, shared_image_manager,
+      gpu_memory_buffer_factory_.get(), gpu_feature_info_,
+      std::move(activity_flags), std::move(default_offscreen_surface),
+      jpeg_decode_accelerator_worker_.get(), vulkan_context_provider(),
+      metal_context_provider_.get());
 
   media_gpu_channel_manager_.reset(
       new media::MediaGpuChannelManager(gpu_channel_manager_.get()));
@@ -292,26 +346,11 @@ void GpuServiceImpl::Bind(mojom::GpuServiceRequest request) {
 
 void GpuServiceImpl::DisableGpuCompositing() {
   // Can be called from any thread.
-  (*gpu_host_)->DisableGpuCompositing();
+  gpu_host_->DisableGpuCompositing();
 }
 
-scoped_refptr<gpu::SharedContextState>
-GpuServiceImpl::GetContextStateForGLSurface(gl::GLSurface* surface) {
+scoped_refptr<gpu::SharedContextState> GpuServiceImpl::GetContextState() {
   DCHECK(main_runner_->BelongsToCurrentThread());
-  DCHECK(!is_using_vulkan());
-  gpu::ContextResult result;
-  auto context_state = gpu_channel_manager_->GetSharedContextState(&result);
-  // TODO(penghuang): https://crbug.com/899740 Support GLSurface which is not
-  // compatible.
-  DCHECK_EQ(surface->GetCompatibilityKey(),
-            context_state->surface()->GetCompatibilityKey());
-  return context_state;
-}
-
-scoped_refptr<gpu::SharedContextState>
-GpuServiceImpl::GetContextStateForVulkan() {
-  DCHECK(main_runner_->BelongsToCurrentThread());
-  DCHECK(is_using_vulkan());
   gpu::ContextResult result;
   return gpu_channel_manager_->GetSharedContextState(&result);
 }
@@ -328,7 +367,7 @@ void GpuServiceImpl::RecordLogMessage(int severity,
   // This can be run from any thread.
   std::string header = str.substr(0, message_start);
   std::string message = str.substr(message_start);
-  (*gpu_host_)->RecordLogMessage(severity, header, message);
+  gpu_host_->RecordLogMessage(severity, header, message);
 }
 
 #if defined(OS_CHROMEOS)
@@ -408,19 +447,21 @@ void GpuServiceImpl::CreateArcProtectedBufferManagerOnMainThread(
           protected_buffer_manager_),
       std::move(pbm_request));
 }
-#endif  // defined(OS_CHROMEOS)
 
 void GpuServiceImpl::CreateJpegDecodeAccelerator(
-    media::mojom::JpegDecodeAcceleratorRequest jda_request) {
+    chromeos_camera::mojom::MjpegDecodeAcceleratorRequest jda_request) {
   DCHECK(io_runner_->BelongsToCurrentThread());
-  media::MojoJpegDecodeAcceleratorService::Create(std::move(jda_request));
+  chromeos_camera::MojoMjpegDecodeAcceleratorService::Create(
+      std::move(jda_request));
 }
 
 void GpuServiceImpl::CreateJpegEncodeAccelerator(
-    media::mojom::JpegEncodeAcceleratorRequest jea_request) {
+    chromeos_camera::mojom::JpegEncodeAcceleratorRequest jea_request) {
   DCHECK(io_runner_->BelongsToCurrentThread());
-  media::MojoJpegEncodeAcceleratorService::Create(std::move(jea_request));
+  chromeos_camera::MojoJpegEncodeAcceleratorService::Create(
+      std::move(jea_request));
 }
+#endif  // defined(OS_CHROMEOS)
 
 void GpuServiceImpl::CreateVideoEncodeAcceleratorProvider(
     media::mojom::VideoEncodeAcceleratorProviderRequest vea_provider_request) {
@@ -441,8 +482,8 @@ void GpuServiceImpl::CreateGpuMemoryBuffer(
     CreateGpuMemoryBufferCallback callback) {
   DCHECK(io_runner_->BelongsToCurrentThread());
   // This needs to happen in the IO thread.
-  std::move(callback).Run(gpu_memory_buffer_factory_->CreateGpuMemoryBuffer(
-      id, size, format, usage, client_id, surface_handle));
+  gpu_memory_buffer_factory_->CreateGpuMemoryBufferAsync(
+      id, size, format, usage, client_id, surface_handle, std::move(callback));
 }
 
 void GpuServiceImpl::DestroyGpuMemoryBuffer(gfx::GpuMemoryBufferId id,
@@ -533,7 +574,7 @@ void GpuServiceImpl::RequestHDRStatusOnMainThread(
   DCHECK(main_runner_->BelongsToCurrentThread());
   bool hdr_enabled = false;
 #if defined(OS_WIN)
-  hdr_enabled = gpu::DirectCompositionSurfaceWin::IsHDRSupported();
+  hdr_enabled = gl::DirectCompositionSurfaceWin::IsHDRSupported();
 #endif
   io_runner_->PostTask(FROM_HERE,
                        base::BindOnce(std::move(callback), hdr_enabled));
@@ -575,39 +616,62 @@ void GpuServiceImpl::UpdateGpuInfoPlatform(
 }
 #endif
 
+void GpuServiceImpl::RegisterDisplayContext(
+    gpu::DisplayContext* display_context) {
+  DCHECK(main_runner_->BelongsToCurrentThread());
+  display_contexts_.AddObserver(display_context);
+}
+
+void GpuServiceImpl::UnregisterDisplayContext(
+    gpu::DisplayContext* display_context) {
+  DCHECK(main_runner_->BelongsToCurrentThread());
+  display_contexts_.RemoveObserver(display_context);
+}
+
+void GpuServiceImpl::LoseAllContexts() {
+  DCHECK(main_runner_->BelongsToCurrentThread());
+
+  if (IsExiting())
+    return;
+
+  for (auto& display_context : display_contexts_)
+    display_context.MarkContextLost();
+  gpu_channel_manager_->LoseAllContexts();
+}
+
 void GpuServiceImpl::DidCreateContextSuccessfully() {
   DCHECK(main_runner_->BelongsToCurrentThread());
-  (*gpu_host_)->DidCreateContextSuccessfully();
+  gpu_host_->DidCreateContextSuccessfully();
 }
 
 void GpuServiceImpl::DidCreateOffscreenContext(const GURL& active_url) {
   DCHECK(main_runner_->BelongsToCurrentThread());
-  (*gpu_host_)->DidCreateOffscreenContext(active_url);
+  gpu_host_->DidCreateOffscreenContext(active_url);
 }
 
 void GpuServiceImpl::DidDestroyChannel(int client_id) {
   DCHECK(main_runner_->BelongsToCurrentThread());
   media_gpu_channel_manager_->RemoveChannel(client_id);
-  (*gpu_host_)->DidDestroyChannel(client_id);
+  gpu_host_->DidDestroyChannel(client_id);
 }
 
 void GpuServiceImpl::DidDestroyOffscreenContext(const GURL& active_url) {
   DCHECK(main_runner_->BelongsToCurrentThread());
-  (*gpu_host_)->DidDestroyOffscreenContext(active_url);
+  gpu_host_->DidDestroyOffscreenContext(active_url);
 }
 
 void GpuServiceImpl::DidLoseContext(bool offscreen,
                                     gpu::error::ContextLostReason reason,
                                     const GURL& active_url) {
   DCHECK(main_runner_->BelongsToCurrentThread());
-  (*gpu_host_)->DidLoseContext(offscreen, reason, active_url);
+  gpu_host_->DidLoseContext(offscreen, reason, active_url);
 }
 
 void GpuServiceImpl::StoreShaderToDisk(int client_id,
                                        const std::string& key,
                                        const std::string& shader) {
   DCHECK(main_runner_->BelongsToCurrentThread());
-  (*gpu_host_)->StoreShaderToDisk(client_id, key, shader);
+  gpu_host_->StoreShaderToDisk(client_id, key, shader);
 }
 
 void GpuServiceImpl::MaybeExitOnContextLost() {
@@ -622,22 +686,18 @@ bool GpuServiceImpl::IsExiting() const {
 void GpuServiceImpl::SendCreatedChildWindow(gpu::SurfaceHandle parent_window,
                                             gpu::SurfaceHandle child_window) {
   // This can be called from main or display compositor thread.
-  (*gpu_host_)->SetChildSurface(parent_window, child_window);
+  gpu_host_->SetChildSurface(parent_window, child_window);
 }
 #endif
 
-void GpuServiceImpl::SetActiveURL(const GURL& url) {
-  DCHECK(main_runner_->BelongsToCurrentThread());
-  static crash_reporter::CrashKeyString<1024> crash_key("url-chunk");
-  crash_key.Set(url.possibly_invalid_spec());
-}
-
 void GpuServiceImpl::EstablishGpuChannel(int32_t client_id,
                                          uint64_t client_tracing_id,
                                          bool is_gpu_host,
                                          bool cache_shaders_on_disk,
                                          EstablishGpuChannelCallback callback) {
   if (gpu::IsReservedClientId(client_id)) {
+    // This returns a null handle, which is treated by the client as a failure
+    // case.
     std::move(callback).Run(mojo::ScopedMessagePipeHandle());
     return;
   }
@@ -661,6 +721,12 @@ void GpuServiceImpl::EstablishGpuChannel(int32_t client_id,
   gpu::GpuChannel* gpu_channel = gpu_channel_manager_->EstablishChannel(
       client_id, client_tracing_id, is_gpu_host, cache_shaders_on_disk);
 
+  if (!gpu_channel) {
+    // This returns a null handle, which is treated by the client as a failure
+    // case.
+    std::move(callback).Run(mojo::ScopedMessagePipeHandle());
+    return;
+  }
   mojo::MessagePipe pipe;
   gpu_channel->Init(pipe.handle0.release(), shutdown_event_);
 
