@@ -17,16 +17,13 @@ namespace content {
 // AppCacheRequest wrapper for the URLRequest class.
 class CONTENT_EXPORT AppCacheURLRequest : public AppCacheRequest {
  public:
-  // Factory function to create an instance of the AppCacheURLRequest class.
-  static std::unique_ptr<AppCacheURLRequest> Create(
-      net::URLRequest* url_request);
-
+  explicit AppCacheURLRequest(net::URLRequest* url_request);
   ~AppCacheURLRequest() override;
 
   // AppCacheRequest overrides.
   const GURL& GetURL() const override;
   const std::string& GetMethod() const override;
-  const GURL& GetFirstPartyForCookies() const override;
+  const GURL& GetSiteForCookies() const override;
   const GURL GetReferrer() const override;
   bool IsSuccess() const override;
   bool IsCancelled() const override;
@@ -36,9 +33,6 @@ class CONTENT_EXPORT AppCacheURLRequest : public AppCacheRequest {
   net::URLRequest* GetURLRequest() override;
   AppCacheURLRequest* AsURLRequest() override;
 
- protected:
-  explicit AppCacheURLRequest(net::URLRequest* url_request);
-
  private:
   net::URLRequest* url_request_;
 
