@@ -9,18 +9,21 @@
 #include <memory>
 #include <utility>
 
+#include "base/containers/circular_deque.h"
 #include "base/location.h"
 #include "base/logging.h"
-#include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
 #include "base/single_thread_task_runner.h"
+#include "base/stl_util.h"
+#include "base/test/scoped_feature_list.h"
 #include "base/test/scoped_task_environment.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "content/browser/renderer_host/input/timeout_monitor.h"
 #include "content/common/input/synthetic_web_input_event_builders.h"
 #include "content/common/input/web_touch_event_traits.h"
+#include "content/public/common/content_features.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/WebKit/public/platform/WebInputEvent.h"
+#include "third_party/blink/public/platform/web_input_event.h"
 #include "ui/events/base_event_utils.h"
 
 using blink::WebGestureEvent;
@@ -41,7 +44,7 @@ base::TimeDelta DefaultTouchTimeoutDelay() {
 }  // namespace
 
 class PassthroughTouchEventQueueTest : public testing::Test,
-                                       public TouchEventQueueClient {
+                                       public PassthroughTouchEventQueueClient {
  public:
   PassthroughTouchEventQueueTest()
       : scoped_task_environment_(
@@ -54,13 +57,13 @@ class PassthroughTouchEventQueueTest : public testing::Test,
 
   // testing::Test
   void SetUp() override {
-    ResetQueueWithConfig(TouchEventQueue::Config());
+    ResetQueueWithConfig(PassthroughTouchEventQueue::Config());
     sent_events_ids_.clear();
   }
 
   void TearDown() override { queue_.reset(); }
 
-  // TouchEventQueueClient
+  // PassthroughTouchEventQueueClient
   void SendTouchEventImmediately(
       const TouchEventWithLatencyInfo& event) override {
     sent_events_.push_back(event.event);
@@ -73,6 +76,7 @@ class PassthroughTouchEventQueueTest : public testing::Test,
   }
 
   void OnTouchEventAck(const TouchEventWithLatencyInfo& event,
+                       InputEventAckSource ack_source,
                        InputEventAckState ack_result) override {
     ++acked_event_count_;
     if (followup_touch_event_) {
@@ -93,6 +97,8 @@ class PassthroughTouchEventQueueTest : public testing::Test,
   void OnFilteringTouchEvent(const blink::WebTouchEvent& touch_event) override {
   }
 
+  void FlushDeferredGestureQueue() override {}
+
  protected:
   void SetUpForTouchMoveSlopTesting(double slop_length_dips) {
     slop_length_dips_ = slop_length_dips;
@@ -100,7 +106,7 @@ class PassthroughTouchEventQueueTest : public testing::Test,
 
   void SetUpForTimeoutTesting(base::TimeDelta desktop_timeout_delay,
                               base::TimeDelta mobile_timeout_delay) {
-    TouchEventQueue::Config config;
+    PassthroughTouchEventQueue::Config config;
     config.desktop_touch_ack_timeout_delay = desktop_timeout_delay;
     config.mobile_touch_ack_timeout_delay = mobile_timeout_delay;
     config.touch_ack_timeout_supported = true;
@@ -112,6 +118,15 @@ class PassthroughTouchEventQueueTest : public testing::Test,
                            DefaultTouchTimeoutDelay());
   }
 
+  void SetUpForSkipFilterTesting(const std::string& events_to_always_forward) {
+    base::test::ScopedFeatureList feature_list;
+    feature_list.InitAndEnableFeatureWithParameters(
+        features::kSkipTouchEventFilter,
+        {{features::kSkipTouchEventFilterTypeParamName,
+          events_to_always_forward}});
+    ResetQueueWithConfig(PassthroughTouchEventQueue::Config());
+  }
+
   void SendTouchEvent(WebTouchEvent event) {
     if (slop_length_dips_) {
       event.moved_beyond_slop_region = false;
@@ -131,38 +146,36 @@ class PassthroughTouchEventQueueTest : public testing::Test,
 
   void SendGestureEvent(WebInputEvent::Type type) {
     WebGestureEvent event(type, WebInputEvent::kNoModifiers,
-                          ui::EventTimeStampToSeconds(ui::EventTimeForNow()));
+                          ui::EventTimeForNow());
     queue_->OnGestureScrollEvent(
         GestureEventWithLatencyInfo(event, ui::LatencyInfo()));
   }
 
   void SendTouchEventAck(InputEventAckState ack_result) {
     DCHECK(!sent_events_ids_.empty());
-    queue_->ProcessTouchAck(ack_result, ui::LatencyInfo(),
-                            sent_events_ids_.front());
+    queue_->ProcessTouchAck(InputEventAckSource::COMPOSITOR_THREAD, ack_result,
+                            ui::LatencyInfo(), sent_events_ids_.front(), true);
     sent_events_ids_.pop_front();
   }
 
   void SendTouchEventAckLast(InputEventAckState ack_result) {
     DCHECK(!sent_events_ids_.empty());
-    queue_->ProcessTouchAck(ack_result, ui::LatencyInfo(),
-                            sent_events_ids_.back());
+    queue_->ProcessTouchAck(InputEventAckSource::COMPOSITOR_THREAD, ack_result,
+                            ui::LatencyInfo(), sent_events_ids_.back(), true);
     sent_events_ids_.pop_back();
   }
 
   void SendTouchEventAckWithID(InputEventAckState ack_result,
                                int unique_event_id) {
-    queue_->ProcessTouchAck(ack_result, ui::LatencyInfo(), unique_event_id);
-    sent_events_ids_.erase(std::remove(sent_events_ids_.begin(),
-                                       sent_events_ids_.end(), unique_event_id),
-                           sent_events_ids_.end());
+    queue_->ProcessTouchAck(InputEventAckSource::COMPOSITOR_THREAD, ack_result,
+                            ui::LatencyInfo(), unique_event_id, true);
+    base::Erase(sent_events_ids_, unique_event_id);
   }
 
   void SendGestureEventAck(WebInputEvent::Type type,
                            InputEventAckState ack_result) {
-    GestureEventWithLatencyInfo event(
-        type, blink::WebInputEvent::kNoModifiers,
-        ui::EventTimeStampToSeconds(ui::EventTimeForNow()), ui::LatencyInfo());
+    GestureEventWithLatencyInfo event(type, blink::WebInputEvent::kNoModifiers,
+                                      ui::EventTimeForNow(), ui::LatencyInfo());
     queue_->OnGestureEventAck(event, ack_result);
   }
 
@@ -179,12 +192,12 @@ class PassthroughTouchEventQueueTest : public testing::Test,
   }
 
   void PressTouchPoint(float x, float y) {
-    touch_event_.PressPoint(x, y);
+    touch_event_.PressPoint(x, y, radius_x_, radius_y_);
     SendTouchEvent();
   }
 
   void MoveTouchPoint(int index, float x, float y) {
-    touch_event_.MovePoint(index, x, y);
+    touch_event_.MovePoint(index, x, y, radius_x_, radius_y_);
     SendTouchEvent();
   }
 
@@ -208,8 +221,7 @@ class PassthroughTouchEventQueueTest : public testing::Test,
     touch_event_.touches[index].state = WebTouchPoint::kStateMoved;
     touch_event_.moved_beyond_slop_region = true;
     WebTouchEventTraits::ResetType(WebInputEvent::kTouchMove,
-                                   touch_event_.TimeStampSeconds(),
-                                   &touch_event_);
+                                   touch_event_.TimeStamp(), &touch_event_);
     SendTouchEvent();
   }
 
@@ -221,8 +233,7 @@ class PassthroughTouchEventQueueTest : public testing::Test,
     touch_event_.touches[index].state = WebTouchPoint::kStateMoved;
     touch_event_.moved_beyond_slop_region = true;
     WebTouchEventTraits::ResetType(WebInputEvent::kTouchMove,
-                                   touch_event_.TimeStampSeconds(),
-                                   &touch_event_);
+                                   touch_event_.TimeStamp(), &touch_event_);
     SendTouchEvent();
   }
 
@@ -234,8 +245,7 @@ class PassthroughTouchEventQueueTest : public testing::Test,
     touch_event_.touches[index].state = WebTouchPoint::kStateMoved;
     touch_event_.moved_beyond_slop_region = true;
     WebTouchEventTraits::ResetType(WebInputEvent::kTouchMove,
-                                   touch_event_.TimeStampSeconds(),
-                                   &touch_event_);
+                                   touch_event_.TimeStamp(), &touch_event_);
     SendTouchEvent();
   }
 
@@ -254,11 +264,10 @@ class PassthroughTouchEventQueueTest : public testing::Test,
   }
 
   void AdvanceTouchTime(double seconds) {
-    touch_event_.SetTimeStampSeconds(touch_event_.TimeStampSeconds() + seconds);
+    touch_event_.SetTimeStamp(touch_event_.TimeStamp() +
+                              base::TimeDelta::FromSecondsD(seconds));
   }
 
-  void ResetTouchEvent() { touch_event_ = SyntheticWebTouchEvent(); }
-
   size_t GetAndResetAckedEventCount() {
     size_t count = acked_event_count_;
     acked_event_count_ = 0;
@@ -289,10 +298,6 @@ class PassthroughTouchEventQueueTest : public testing::Test,
 
   size_t queued_event_count() const { return queue_->SizeForTesting(); }
 
-  const WebTouchEvent& latest_event() const {
-    return queue_->GetLatestEventForTesting().event;
-  }
-
   const WebTouchEvent& acked_event() const { return last_acked_event_; }
 
   const WebTouchEvent& sent_event() const {
@@ -309,20 +314,29 @@ class PassthroughTouchEventQueueTest : public testing::Test,
   }
 
   static void RunTasksAndWait(base::TimeDelta delay) {
+    base::RunLoop run_loop;
     base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-        FROM_HERE, base::MessageLoop::QuitWhenIdleClosure(), delay);
-    base::RunLoop().Run();
+        FROM_HERE, run_loop.QuitClosure(), delay);
+    run_loop.Run();
+  }
+
+  PassthroughTouchEventQueue::PreFilterResult FilterBeforeForwarding(
+      const blink::WebTouchEvent& event) {
+    return queue_->FilterBeforeForwarding(event);
   }
 
   int GetUniqueTouchEventID() { return sent_events_ids_.back(); }
 
+  const float radius_x_ = 20.0f;
+  const float radius_y_ = 20.0f;
+
  private:
   void SendTouchEvent() {
     SendTouchEvent(touch_event_);
     touch_event_.ResetPoints();
   }
 
-  void ResetQueueWithConfig(const TouchEventQueue::Config& config) {
+  void ResetQueueWithConfig(const PassthroughTouchEventQueue::Config& config) {
     queue_.reset(new PassthroughTouchEventQueue(this, config));
     queue_->OnHasTouchEventHandlers(true);
   }
@@ -339,7 +353,7 @@ class PassthroughTouchEventQueueTest : public testing::Test,
   std::unique_ptr<InputEventAckState> sync_ack_result_;
   double slop_length_dips_;
   gfx::PointF anchor_;
-  std::deque<int> sent_events_ids_;
+  base::circular_deque<int> sent_events_ids_;
 };
 
 // Tests that touch-events are queued properly.
@@ -481,36 +495,36 @@ TEST_F(PassthroughTouchEventQueueTest,
   EXPECT_EQ(INPUT_EVENT_ACK_STATE_CONSUMED, acked_event_state());
 }
 
-// Tests that addition of a touch handler during a touch sequence will not cause
-// the remaining sequence to be forwarded.
+// Tests that addition of a touch handler during a touch sequence will continue
+// forwarding events.
 TEST_F(PassthroughTouchEventQueueTest,
-       ActiveSequenceNotForwardedWhenHandlersAdded) {
+       ActiveSequenceStillForwardedWhenHandlersAdded) {
   OnHasTouchEventHandlers(false);
 
   // Send a touch-press event while there is no handler.
   PressTouchPoint(1, 1);
-  EXPECT_EQ(1U, GetAndResetAckedEventCount());
-  EXPECT_EQ(0U, GetAndResetSentEventCount());
-  EXPECT_EQ(0U, queued_event_count());
+  EXPECT_EQ(0U, GetAndResetAckedEventCount());
+  EXPECT_EQ(1U, GetAndResetSentEventCount());
+  EXPECT_EQ(1U, queued_event_count());
 
   OnHasTouchEventHandlers(true);
 
-  // The remaining touch sequence should not be forwarded.
+  // The remaining touch sequence should be forwarded.
   MoveTouchPoint(0, 5, 5);
   ReleaseTouchPoint(0);
-  EXPECT_EQ(2U, GetAndResetAckedEventCount());
-  EXPECT_EQ(0U, GetAndResetSentEventCount());
-  EXPECT_EQ(0U, queued_event_count());
+  EXPECT_EQ(0U, GetAndResetAckedEventCount());
+  EXPECT_EQ(1U, GetAndResetSentEventCount());
+  EXPECT_EQ(3U, queued_event_count());
 
-  // A new touch sequence should resume forwarding.
+  // A new touch sequence should continue forwarding.
   PressTouchPoint(1, 1);
-  EXPECT_EQ(1U, queued_event_count());
+  EXPECT_EQ(4U, queued_event_count());
   EXPECT_EQ(1U, GetAndResetSentEventCount());
 }
 
-// Tests that removal of a touch handler during a touch sequence will prevent
-// the remaining sequence from being forwarded, even if another touch handler is
-// registered during the same touch sequence.
+// Tests that removal of a touch handler during a touch sequence will not
+// prevent the remaining sequence from being forwarded, even if another touch
+// handler is registered during the same touch sequence.
 TEST_F(PassthroughTouchEventQueueTest,
        ActiveSequenceDroppedWhenHandlersRemoved) {
   // Send a touch-press event.
@@ -529,8 +543,7 @@ TEST_F(PassthroughTouchEventQueueTest,
   EXPECT_EQ(0U, GetAndResetAckedEventCount());
   EXPECT_EQ(2U, queued_event_count());
 
-  // Repeated registration/unregstration of handlers should have no effect as
-  // we're still awaiting the ack arrival.
+  // Repeated registration/unregstration of handlers should have no effect.
   OnHasTouchEventHandlers(true);
   EXPECT_EQ(0U, GetAndResetAckedEventCount());
   EXPECT_EQ(2U, queued_event_count());
@@ -538,13 +551,13 @@ TEST_F(PassthroughTouchEventQueueTest,
   EXPECT_EQ(0U, GetAndResetAckedEventCount());
   EXPECT_EQ(2U, queued_event_count());
 
-  // clear the queue .
+  // Clear the queue.
   SendTouchEventAck(INPUT_EVENT_ACK_STATE_NO_CONSUMER_EXISTS);
   SendTouchEventAck(INPUT_EVENT_ACK_STATE_NO_CONSUMER_EXISTS);
   EXPECT_EQ(2U, GetAndResetAckedEventCount());
   EXPECT_EQ(0U, queued_event_count());
 
-  // Events should be dropped while there is no touch handler.
+  // TouchMove events should be dropped while there is no touch handler.
   MoveTouchPoint(0, 10, 10);
   EXPECT_EQ(0U, queued_event_count());
   EXPECT_EQ(1U, GetAndResetAckedEventCount());
@@ -553,15 +566,15 @@ TEST_F(PassthroughTouchEventQueueTest,
   // Simulate touch handler registration in the middle of a touch sequence.
   OnHasTouchEventHandlers(true);
 
-  // The touch end for the interrupted sequence should be dropped.
+  // The touch end for the interrupted sequence should be sent.
   ReleaseTouchPoint(0);
-  EXPECT_EQ(0U, queued_event_count());
-  EXPECT_EQ(1U, GetAndResetAckedEventCount());
-  EXPECT_EQ(0U, GetAndResetSentEventCount());
+  EXPECT_EQ(1U, queued_event_count());
+  EXPECT_EQ(0U, GetAndResetAckedEventCount());
+  EXPECT_EQ(1U, GetAndResetSentEventCount());
 
   // A new touch sequence should be forwarded properly.
   PressTouchPoint(1, 1);
-  EXPECT_EQ(1U, queued_event_count());
+  EXPECT_EQ(2U, queued_event_count());
   EXPECT_EQ(1U, GetAndResetSentEventCount());
 }
 
@@ -612,16 +625,14 @@ TEST_F(PassthroughTouchEventQueueTest, SpuriousAcksIgnored) {
   EXPECT_EQ(0U, GetAndResetAckedEventCount());
 }
 
-// Tests that touch-move events are not sent to the renderer if the preceding
-// touch-press event did not have a consumer (and consequently, did not hit the
-// main thread in the renderer).
+// Tests that touch-move events are not sent to the renderer even if the
+// preceding touch-press event did not have a consumer.
 TEST_F(PassthroughTouchEventQueueTest, NoConsumer) {
   // The first touch-press should reach the renderer.
   PressTouchPoint(1, 1);
   EXPECT_EQ(1U, GetAndResetSentEventCount());
 
-  // The second touch should be sent since we don't know if there is
-  // a consumer or not.
+  // The second touch should be sent too.
   MoveTouchPoint(0, 5, 5);
   EXPECT_EQ(1U, GetAndResetSentEventCount());
   EXPECT_EQ(2U, queued_event_count());
@@ -633,14 +644,13 @@ TEST_F(PassthroughTouchEventQueueTest, NoConsumer) {
   EXPECT_EQ(2U, GetAndResetAckedEventCount());
   EXPECT_EQ(0U, GetAndResetSentEventCount());
 
-  // Send a release event. This should not reach the renderer.
+  // Send a release event. This should reach the renderer.
   ReleaseTouchPoint(0);
-  EXPECT_EQ(0U, GetAndResetSentEventCount());
-  EXPECT_EQ(WebInputEvent::kTouchEnd, acked_event().GetType());
-  EXPECT_EQ(1U, GetAndResetAckedEventCount());
+  EXPECT_EQ(1U, GetAndResetSentEventCount());
+  EXPECT_EQ(WebInputEvent::kTouchMove, acked_event().GetType());
+  EXPECT_EQ(0U, GetAndResetAckedEventCount());
 
-  // Send a press-event, followed by move a following move should not
-  // be sent but held in the queue.
+  // Send a press-event, followed by a move should be sent.
   PressTouchPoint(10, 10);
   MoveTouchPoint(0, 5, 5);
 
@@ -651,9 +661,9 @@ TEST_F(PassthroughTouchEventQueueTest, NoConsumer) {
   MoveTouchPoint(0, 6, 5);
   EXPECT_EQ(0U, GetAndResetSentEventCount());
   EXPECT_EQ(0U, GetAndResetAckedEventCount());
-  EXPECT_EQ(2U, queued_event_count());
+  EXPECT_EQ(3U, queued_event_count());
   SendTouchEventAck(INPUT_EVENT_ACK_STATE_CONSUMED);
-  EXPECT_EQ(2U, GetAndResetAckedEventCount());
+  EXPECT_EQ(1U, GetAndResetAckedEventCount());
 }
 
 TEST_F(PassthroughTouchEventQueueTest, AckTouchEventInReverse) {
@@ -694,9 +704,9 @@ TEST_F(PassthroughTouchEventQueueTest, AckWithFollowupEvents) {
 
   // Create a touch event that will be queued synchronously by a touch ack.
   // Note, this will be triggered by all subsequent touch acks.
-  WebTouchEvent followup_event(
-      WebInputEvent::kTouchMove, WebInputEvent::kNoModifiers,
-      ui::EventTimeStampToSeconds(ui::EventTimeForNow()));
+  WebTouchEvent followup_event(WebInputEvent::kTouchMove,
+                               WebInputEvent::kNoModifiers,
+                               ui::EventTimeForNow());
   followup_event.touches_length = 1;
   followup_event.touches[0].id = 0;
   followup_event.touches[0].state = WebTouchPoint::kStateMoved;
@@ -781,9 +791,9 @@ TEST_F(PassthroughTouchEventQueueTest, SynchronousAcksInOrder) {
   EXPECT_EQ(0U, GetAndResetAckedEventCount());
 
   // Create a touch event that will be queued synchronously by a touch ack.
-  WebTouchEvent followup_event(
-      WebInputEvent::kTouchMove, WebInputEvent::kNoModifiers,
-      ui::EventTimeStampToSeconds(ui::EventTimeForNow()));
+  WebTouchEvent followup_event(WebInputEvent::kTouchMove,
+                               WebInputEvent::kNoModifiers,
+                               ui::EventTimeForNow());
   followup_event.touches_length = 1;
   followup_event.touches[0].id = 0;
   followup_event.unique_touch_event_id = 100;
@@ -804,9 +814,9 @@ TEST_F(PassthroughTouchEventQueueTest, SynchronousAcksInOrder) {
 // TouchEventQueue::QueueEvent() are properly handled.
 TEST_F(PassthroughTouchEventQueueTest, ImmediateAckWithFollowupEvents) {
   // Create a touch event that will be queued synchronously by a touch ack.
-  WebTouchEvent followup_event(
-      WebInputEvent::kTouchStart, WebInputEvent::kNoModifiers,
-      ui::EventTimeStampToSeconds(ui::EventTimeForNow()));
+  WebTouchEvent followup_event(WebInputEvent::kTouchStart,
+                               WebInputEvent::kNoModifiers,
+                               ui::EventTimeForNow());
   followup_event.touches_length = 1;
   followup_event.touches[0].id = 1;
   followup_event.touches[0].state = WebTouchPoint::kStatePressed;
@@ -815,9 +825,9 @@ TEST_F(PassthroughTouchEventQueueTest, ImmediateAckWithFollowupEvents) {
   // Now, enqueue a stationary touch that will not be forwarded.  This should be
   // immediately ack'ed with "NO_CONSUMER_EXISTS".  The followup event should
   // then be enqueued and immediately sent to the renderer.
-  WebTouchEvent stationary_event(
-      WebInputEvent::kTouchMove, WebInputEvent::kNoModifiers,
-      ui::EventTimeStampToSeconds(ui::EventTimeForNow()));
+  WebTouchEvent stationary_event(WebInputEvent::kTouchMove,
+                                 WebInputEvent::kNoModifiers,
+                                 ui::EventTimeForNow());
   stationary_event.touches_length = 1;
   stationary_event.touches[0].id = 1;
   stationary_event.touches[0].state = WebTouchPoint::kStateStationary;
@@ -830,31 +840,32 @@ TEST_F(PassthroughTouchEventQueueTest, ImmediateAckWithFollowupEvents) {
   EXPECT_EQ(WebInputEvent::kTouchMove, acked_event().GetType());
 }
 
-// Tests basic TouchEvent forwarding suppression.
+// Tests that basic TouchEvent forwarding suppression has been disabled.
 TEST_F(PassthroughTouchEventQueueTest, NoTouchBasic) {
-  // Disable TouchEvent forwarding.
+  // The old behaviour was to suppress events when there were no handlers.
+  // Signal the no-handler case and test that events still get forwarded.
   OnHasTouchEventHandlers(false);
   PressTouchPoint(30, 5);
-  EXPECT_EQ(0U, GetAndResetSentEventCount());
-  EXPECT_EQ(1U, GetAndResetAckedEventCount());
+  EXPECT_EQ(1U, GetAndResetSentEventCount());
+  EXPECT_EQ(0U, GetAndResetAckedEventCount());
 
   // TouchMove should not be sent to renderer.
   MoveTouchPoint(0, 65, 10);
   EXPECT_EQ(0U, GetAndResetSentEventCount());
-  EXPECT_EQ(1U, GetAndResetAckedEventCount());
+  EXPECT_EQ(0U, GetAndResetAckedEventCount());
 
-  // TouchEnd should not be sent to renderer.
+  // TouchEnd should be sent to renderer.
   ReleaseTouchPoint(0);
-  EXPECT_EQ(0U, GetAndResetSentEventCount());
-  EXPECT_EQ(1U, GetAndResetAckedEventCount());
+  EXPECT_EQ(1U, GetAndResetSentEventCount());
+  EXPECT_EQ(0U, GetAndResetAckedEventCount());
 
-  // Enable TouchEvent forwarding.
+  // Signal handlers-present and make sure events are still getting forwarded.
   OnHasTouchEventHandlers(true);
 
   PressTouchPoint(80, 10);
   EXPECT_EQ(1U, GetAndResetSentEventCount());
   SendTouchEventAck(INPUT_EVENT_ACK_STATE_NOT_CONSUMED);
-  EXPECT_EQ(1U, GetAndResetAckedEventCount());
+  EXPECT_EQ(2U, GetAndResetAckedEventCount());
 
   MoveTouchPoint(0, 80, 20);
   EXPECT_EQ(1U, GetAndResetSentEventCount());
@@ -1138,9 +1149,9 @@ TEST_F(PassthroughTouchEventQueueTest, TouchTimeoutWithFollowupGesture) {
   EXPECT_EQ(1U, GetAndResetSentEventCount());
 
   // The cancelled sequence may turn into a scroll gesture.
-  WebGestureEvent followup_scroll(
-      WebInputEvent::kGestureScrollBegin, WebInputEvent::kNoModifiers,
-      ui::EventTimeStampToSeconds(ui::EventTimeForNow()));
+  WebGestureEvent followup_scroll(WebInputEvent::kGestureScrollBegin,
+                                  WebInputEvent::kNoModifiers,
+                                  ui::EventTimeForNow());
   SetFollowupEvent(followup_scroll);
 
   // Delay the ack.
@@ -1158,23 +1169,23 @@ TEST_F(PassthroughTouchEventQueueTest, TouchTimeoutWithFollowupGesture) {
   EXPECT_EQ(1U, GetAndResetSentEventCount());
   EXPECT_EQ(0U, GetAndResetAckedEventCount());
 
-  // Ack the cancel event.  Normally, this would resume touch forwarding,
+  // Ack the cancel event. Normally, this would resume touch forwarding,
   // but we're still within a scroll gesture so it remains disabled.
   SendTouchEventAck(INPUT_EVENT_ACK_STATE_CONSUMED);
   EXPECT_FALSE(IsTimeoutRunning());
   EXPECT_EQ(0U, GetAndResetSentEventCount());
   EXPECT_EQ(0U, GetAndResetAckedEventCount());
 
-  // Try to forward touch events for the current sequence.
+  // Forward touch events for the current sequence.
   GetAndResetSentEventCount();
   GetAndResetAckedEventCount();
   MoveTouchPoint(0, 1, 1);
   ReleaseTouchPoint(0);
   EXPECT_FALSE(IsTimeoutRunning());
-  EXPECT_EQ(0U, GetAndResetSentEventCount());
-  EXPECT_EQ(2U, GetAndResetAckedEventCount());
+  EXPECT_EQ(1U, GetAndResetSentEventCount());
+  EXPECT_EQ(1U, GetAndResetAckedEventCount());
 
-  // Now end the scroll sequence, resuming touch handling.
+  // Now end the scroll sequence.
   SendGestureEvent(blink::WebInputEvent::kGestureScrollEnd);
   PressTouchPoint(0, 1);
   EXPECT_TRUE(IsTimeoutRunning());
@@ -1195,9 +1206,9 @@ TEST_F(PassthroughTouchEventQueueTest,
   EXPECT_EQ(1U, GetAndResetSentEventCount());
 
   // The cancelled sequence may turn into a scroll gesture.
-  WebGestureEvent followup_scroll(
-      WebInputEvent::kGestureScrollBegin, WebInputEvent::kNoModifiers,
-      ui::EventTimeStampToSeconds(ui::EventTimeForNow()));
+  WebGestureEvent followup_scroll(WebInputEvent::kGestureScrollBegin,
+                                  WebInputEvent::kNoModifiers,
+                                  ui::EventTimeForNow());
   SetFollowupEvent(followup_scroll);
 
   // Delay the ack.
@@ -1371,7 +1382,7 @@ TEST_F(PassthroughTouchEventQueueTest,
   ASSERT_EQ(1U, GetAndResetSentEventCount());
   ASSERT_EQ(1U, GetAndResetAckedEventCount());
 
-  // Events should not be forwarded, as the point had no consumer.
+  // TouchMove events should not be forwarded, as the point had no consumer.
   MoveTouchPoint(0, 0, 15);
   EXPECT_EQ(0U, queued_event_count());
   EXPECT_EQ(0U, GetAndResetSentEventCount());
@@ -1403,7 +1414,7 @@ TEST_F(PassthroughTouchEventQueueTest,
 
   WebGestureEvent followup_scroll(WebInputEvent::kGestureScrollBegin,
                                   WebInputEvent::kNoModifiers,
-                                  WebInputEvent::kTimeStampForTesting);
+                                  WebInputEvent::GetStaticTimeStampForTests());
   SetFollowupEvent(followup_scroll);
   MoveTouchPoint(0, 20, 5);
   EXPECT_EQ(0U, GetAndResetSentEventCount());
@@ -1448,7 +1459,7 @@ TEST_F(PassthroughTouchEventQueueTest, TouchAbsorptionWithConsumedFirstMove) {
   MoveTouchPoint(0, 20, 5);
   WebGestureEvent followup_scroll(WebInputEvent::kGestureScrollUpdate,
                                   WebInputEvent::kNoModifiers,
-                                  WebInputEvent::kTimeStampForTesting);
+                                  WebInputEvent::GetStaticTimeStampForTests());
   SetFollowupEvent(followup_scroll);
   SendTouchEventAck(INPUT_EVENT_ACK_STATE_NOT_CONSUMED);
   SendGestureEventAck(WebInputEvent::kGestureScrollUpdate,
@@ -1552,11 +1563,11 @@ TEST_F(PassthroughTouchEventQueueTest, UnseenTouchPointerIdsNotForwarded) {
   EXPECT_EQ(0U, GetAndResetSentEventCount());
   EXPECT_EQ(1U, GetAndResetAckedEventCount());
 
-  // Give the touchmove a valid id after release it shouldn't be sent.
+  // Give the touchmove a valid id after release; it should be sent.
   event.touches[0].id = press_id;
   SendTouchEvent(event);
-  EXPECT_EQ(0U, GetAndResetSentEventCount());
-  EXPECT_EQ(1U, GetAndResetAckedEventCount());
+  EXPECT_EQ(1U, GetAndResetSentEventCount());
+  EXPECT_EQ(0U, GetAndResetAckedEventCount());
 }
 
 // Tests that touch points states are correct in TouchMove events.
@@ -1622,8 +1633,8 @@ TEST_F(PassthroughTouchEventQueueTest,
   PressTouchPoint(1, 1);
   SendTouchEventAck(INPUT_EVENT_ACK_STATE_CONSUMED);
 
-  // Default initial radiusX/Y is (1.f, 1.f).
-  // Default initial rotationAngle is 1.f.
+  // Default initial radiusX/Y is (20.f, 20.f).
+  // Default initial rotationAngle is 0.f.
   // Default initial force is 1.f.
 
   // Change touch point radius only.
@@ -1682,7 +1693,7 @@ TEST_F(PassthroughTouchEventQueueTest, FilterTouchMovesWhenNoPointerChanged) {
 
   // Do not really move any touch points, but use previous values.
   MoveTouchPoint(0, 10, 10);
-  ChangeTouchPointRadius(1, 1, 1);
+  ChangeTouchPointRadius(1, radius_x_, radius_y_);
   MoveTouchPoint(1, 2, 2);
   EXPECT_EQ(4U, queued_event_count());
   EXPECT_EQ(0U, GetAndResetSentEventCount());
@@ -1774,4 +1785,202 @@ TEST_F(PassthroughTouchEventQueueTest, TouchStartOrFirstTouchMove) {
   EXPECT_FALSE(sent_event().touch_start_or_first_touch_move);
 }
 
+TEST_F(PassthroughTouchEventQueueTest, TouchScrollStartedUnfiltered) {
+  SyntheticWebTouchEvent event;
+  event.SetType(WebInputEvent::kTouchScrollStarted);
+  EXPECT_EQ(PassthroughTouchEventQueue::PreFilterResult::kUnfiltered,
+            FilterBeforeForwarding(event));
+}
+
+TEST_F(PassthroughTouchEventQueueTest,
+       TouchStartWithoutPageHandlersUnfiltered) {
+  OnHasTouchEventHandlers(false);
+  SyntheticWebTouchEvent event;
+  event.PressPoint(1, 1);
+
+  EXPECT_EQ(PassthroughTouchEventQueue::PreFilterResult::kUnfiltered,
+            FilterBeforeForwarding(event));
+}
+
+TEST_F(PassthroughTouchEventQueueTest, TouchStartWithPageHandlersUnfiltered) {
+  OnHasTouchEventHandlers(true);
+  SyntheticWebTouchEvent event;
+  event.PressPoint(1, 1);
+
+  EXPECT_EQ(PassthroughTouchEventQueue::PreFilterResult::kUnfiltered,
+            FilterBeforeForwarding(event));
+}
+
+TEST_F(PassthroughTouchEventQueueTest, TouchMoveFilteredAfterTimeout) {
+  SetUpForTimeoutTesting();
+  OnHasTouchEventHandlers(true);
+  PressTouchPoint(1, 1);
+
+  // Allow the initial touch start event to time out.
+  RunTasksAndWait(DefaultTouchTimeoutDelay() * 2);
+
+  // Any subsequent touch move events are filtered.
+  SyntheticWebTouchEvent event;
+  int id = event.PressPoint(1, 1);
+  event.MovePoint(id, 2, 2);
+
+  EXPECT_EQ(PassthroughTouchEventQueue::PreFilterResult::kFilteredTimeout,
+            FilterBeforeForwarding(event));
+}
+
+TEST_F(PassthroughTouchEventQueueTest, TouchMoveWithoutPageHandlersUnfiltered) {
+  OnHasTouchEventHandlers(false);
+  // Start the touch sequence.
+  PressTouchPoint(1, 1);
+
+  SyntheticWebTouchEvent event;
+  int id = event.PressPoint(1, 1);
+  event.MovePoint(id, 2, 2);
+
+  EXPECT_EQ(
+      PassthroughTouchEventQueue::PreFilterResult::kFilteredNoPageHandlers,
+      FilterBeforeForwarding(event));
+}
+
+TEST_F(PassthroughTouchEventQueueTest, StationaryTouchMoveFiltered) {
+  OnHasTouchEventHandlers(true);
+  // Start the touch sequence.
+  PressTouchPoint(1, 1);
+
+  // Touch move events with stationary pointers are filtered.
+  SyntheticWebTouchEvent event;
+  int id = event.PressPoint(1, 1);
+  event.MovePoint(id, 1, 1);
+
+  EXPECT_EQ(PassthroughTouchEventQueue::PreFilterResult::
+                kFilteredNoNonstationaryPointers,
+            FilterBeforeForwarding(event));
+}
+
+TEST_F(PassthroughTouchEventQueueTest,
+       StationaryTouchMoveWithActualTouchMoveUnfiltered) {
+  OnHasTouchEventHandlers(true);
+  // Start the touch sequence.
+  PressTouchPoint(1, 1);
+  PressTouchPoint(2, 2);
+
+  // Touch move events with non stationary pointers are unfiltered, even if
+  // there's another touch with a stationary pointer.
+  SyntheticWebTouchEvent event;
+  int id1 = event.PressPoint(1, 1);
+  event.MovePoint(id1, 1, 1);
+  int id2 = event.PressPoint(2, 2);
+  event.MovePoint(id2, 3, 3);
+
+  EXPECT_EQ(PassthroughTouchEventQueue::PreFilterResult::kUnfiltered,
+            FilterBeforeForwarding(event));
+}
+
+TEST_F(PassthroughTouchEventQueueTest, NonTouchMoveUnfiltered) {
+  OnHasTouchEventHandlers(true);
+  // Start the touch sequence.
+  PressTouchPoint(1, 1);
+
+  // Non-touchmove events are never filtered.
+  SyntheticWebTouchEvent event;
+  int id = event.PressPoint(1, 1);
+  event.ReleasePoint(id);
+
+  EXPECT_EQ(PassthroughTouchEventQueue::PreFilterResult::kUnfiltered,
+            FilterBeforeForwarding(event));
+}
+
+TEST_F(PassthroughTouchEventQueueTest, TouchMoveWithNonTouchMoveUnfiltered) {
+  OnHasTouchEventHandlers(true);
+  // Start the touch sequence.
+  PressTouchPoint(1, 1);
+  PressTouchPoint(1, 1);
+
+  // Non-touchmove events are never filtered, even if they're paired with a
+  // touchmove event that would otherwise be filtered.
+  SyntheticWebTouchEvent event;
+  int id1 = event.PressPoint(1, 1);
+  event.MovePoint(id1, 1, 1);
+  int id2 = event.PressPoint(2, 2);
+  event.ReleasePoint(id2);
+
+  EXPECT_EQ(PassthroughTouchEventQueue::PreFilterResult::kUnfiltered,
+            FilterBeforeForwarding(event));
+}
+
+TEST_F(PassthroughTouchEventQueueTest,
+       TouchMoveWithoutSequenceHandlerUnfiltered) {
+  OnHasTouchEventHandlers(true);
+  // Start the touch sequence.
+  PressTouchPoint(1, 1);
+
+  // Send an ack indicating that there's no handler for the current sequence.
+  SendTouchEventAck(INPUT_EVENT_ACK_STATE_NO_CONSUMER_EXISTS);
+
+  // Any subsequent touches in the sequence should be unfiltered.
+  SyntheticWebTouchEvent event;
+  int id = event.PressPoint(1, 1);
+  event.MovePoint(id, 3, 3);
+
+  EXPECT_EQ(PassthroughTouchEventQueue::PreFilterResult::
+                kFilteredNoHandlerForSequence,
+            FilterBeforeForwarding(event));
+}
+
+TEST_F(PassthroughTouchEventQueueTest,
+       TouchStartUnfilteredWithForwardDiscrete) {
+  SetUpForSkipFilterTesting(
+      features::kSkipTouchEventFilterTypeParamValueDiscrete);
+
+  OnHasTouchEventHandlers(false);
+  SyntheticWebTouchEvent event;
+  event.PressPoint(1, 1);
+
+  EXPECT_EQ(PassthroughTouchEventQueue::PreFilterResult::kUnfiltered,
+            FilterBeforeForwarding(event));
+}
+
+TEST_F(PassthroughTouchEventQueueTest, TouchMoveFilteredWithForwardDiscrete) {
+  SetUpForSkipFilterTesting(
+      features::kSkipTouchEventFilterTypeParamValueDiscrete);
+
+  OnHasTouchEventHandlers(false);
+  // Start the touch sequence.
+  PressTouchPoint(1, 1);
+
+  SyntheticWebTouchEvent event;
+  int id = event.PressPoint(1, 1);
+  event.MovePoint(id, 2, 2);
+
+  EXPECT_EQ(
+      PassthroughTouchEventQueue::PreFilterResult::kFilteredNoPageHandlers,
+      FilterBeforeForwarding(event));
+}
+
+TEST_F(PassthroughTouchEventQueueTest, TouchStartUnfilteredWithForwardAll) {
+  SetUpForSkipFilterTesting(features::kSkipTouchEventFilterTypeParamValueAll);
+
+  OnHasTouchEventHandlers(false);
+  SyntheticWebTouchEvent event;
+  event.PressPoint(1, 1);
+
+  EXPECT_EQ(PassthroughTouchEventQueue::PreFilterResult::kUnfiltered,
+            FilterBeforeForwarding(event));
+}
+
+TEST_F(PassthroughTouchEventQueueTest, TouchMoveUnfilteredWithForwardAll) {
+  SetUpForSkipFilterTesting(features::kSkipTouchEventFilterTypeParamValueAll);
+
+  OnHasTouchEventHandlers(false);
+  // Start the touch sequence.
+  PressTouchPoint(1, 1);
+
+  SyntheticWebTouchEvent event;
+  int id = event.PressPoint(1, 1);
+  event.MovePoint(id, 2, 2);
+
+  EXPECT_EQ(PassthroughTouchEventQueue::PreFilterResult::kUnfiltered,
+            FilterBeforeForwarding(event));
+}
+
 }  // namespace content
