@@ -4,16 +4,19 @@
 
 #include "content/browser/renderer_host/input/touch_emulator.h"
 
+#include "base/containers/queue.h"
+#include "base/time/time.h"
 #include "build/build_config.h"
 #include "content/browser/renderer_host/input/motion_event_web.h"
+#include "content/browser/renderer_host/render_widget_host_view_base.h"
 #include "content/browser/renderer_host/ui_events_helper.h"
 #include "content/common/input/web_touch_event_traits.h"
 #include "content/grit/content_resources.h"
 #include "content/public/common/content_client.h"
 #include "content/public/common/content_switches.h"
-#include "third_party/WebKit/public/platform/WebCursorInfo.h"
-#include "third_party/WebKit/public/platform/WebKeyboardEvent.h"
-#include "third_party/WebKit/public/platform/WebMouseEvent.h"
+#include "third_party/blink/public/platform/web_cursor_info.h"
+#include "third_party/blink/public/platform/web_keyboard_event.h"
+#include "third_party/blink/public/platform/web_mouse_event.h"
 #include "ui/base/ui_base_types.h"
 #include "ui/events/base_event_utils.h"
 #include "ui/events/blink/blink_event_util.h"
@@ -33,12 +36,17 @@ namespace content {
 namespace {
 
 ui::GestureProvider::Config GetEmulatorGestureProviderConfig(
-    ui::GestureProviderConfigType config_type) {
+    ui::GestureProviderConfigType config_type,
+    TouchEmulator::Mode mode) {
   ui::GestureProvider::Config config =
       ui::GetGestureProviderConfig(config_type);
   config.gesture_begin_end_types_enabled = false;
   config.gesture_detector_config.swipe_enabled = false;
   config.gesture_detector_config.two_finger_tap_enabled = false;
+  if (mode == TouchEmulator::Mode::kInjectingTouchEvents) {
+    config.gesture_detector_config.longpress_timeout = base::TimeDelta::Max();
+    config.gesture_detector_config.showpress_timeout = base::TimeDelta::Max();
+  }
   return config;
 }
 
@@ -51,7 +59,8 @@ int ModifiersWithoutMouseButtons(const WebInputEvent& event) {
 
 // Time between two consecutive mouse moves, during which second mouse move
 // is not converted to touch.
-const double kMouseMoveDropIntervalSeconds = 5.f / 1000;
+constexpr base::TimeDelta kMouseMoveDropInterval =
+    base::TimeDelta::FromMilliseconds(5);
 
 } // namespace
 
@@ -62,8 +71,11 @@ TouchEmulator::TouchEmulator(TouchEmulatorClient* client,
           ui::GestureProviderConfigType::CURRENT_PLATFORM),
       double_tap_enabled_(true),
       use_2x_cursors_(false),
+      pinch_gesture_mode_for_testing_(false),
       emulated_stream_active_sequence_count_(0),
-      native_stream_active_sequence_count_(0) {
+      native_stream_active_sequence_count_(0),
+      last_emulated_start_target_(nullptr),
+      pending_taps_count_(0) {
   DCHECK(client_);
   ResetState();
   InitCursors(device_scale_factor, true);
@@ -77,7 +89,7 @@ TouchEmulator::~TouchEmulator() {
 
 void TouchEmulator::ResetState() {
   last_mouse_event_was_move_ = false;
-  last_mouse_move_timestamp_ = 0;
+  last_mouse_move_timestamp_ = base::TimeTicks();
   mouse_pressed_ = false;
   shift_pressed_ = false;
   suppress_next_fling_cancel_ = false;
@@ -85,15 +97,23 @@ void TouchEmulator::ResetState() {
   pinch_gesture_active_ = false;
 }
 
-void TouchEmulator::Enable(ui::GestureProviderConfigType config_type) {
-  if (!gesture_provider_ || gesture_provider_config_type_ != config_type) {
+void TouchEmulator::Enable(Mode mode,
+                           ui::GestureProviderConfigType config_type) {
+  if (gesture_provider_ && mode_ != mode)
+    client_->SetCursor(pointer_cursor_);
+
+  if (!gesture_provider_ || gesture_provider_config_type_ != config_type ||
+      mode_ != mode) {
+    mode_ = mode;
     gesture_provider_config_type_ = config_type;
     gesture_provider_.reset(new ui::FilteredGestureProvider(
-        GetEmulatorGestureProviderConfig(config_type), this));
-    // TODO(dgozman): Use synthetic secondary touch to support multi-touch.
-    gesture_provider_->SetMultiTouchZoomSupportEnabled(false);
+        GetEmulatorGestureProviderConfig(config_type, mode), this));
     gesture_provider_->SetDoubleTapSupportForPageEnabled(double_tap_enabled_);
+    // TODO(dgozman): Use synthetic secondary touch to support multi-touch.
+    gesture_provider_->SetMultiTouchZoomSupportEnabled(
+        mode != Mode::kEmulatingTouchFromMouse);
   }
+
   UpdateCursor();
 }
 
@@ -101,9 +121,12 @@ void TouchEmulator::Disable() {
   if (!enabled())
     return;
 
+  mode_ = Mode::kEmulatingTouchFromMouse;
   CancelTouch();
   gesture_provider_.reset();
-  UpdateCursor();
+  base::queue<base::OnceClosure> empty;
+  injected_touch_completion_callbacks_.swap(empty);
+  client_->SetCursor(pointer_cursor_);
   ResetState();
 }
 
@@ -135,9 +158,7 @@ bool TouchEmulator::InitCursors(float device_scale_factor, bool force) {
       use_2x ? IDR_DEVTOOLS_PINCH_CURSOR_ICON_2X :
           IDR_DEVTOOLS_PINCH_CURSOR_ICON);
 
-  CursorInfo cursor_info;
-  cursor_info.type = blink::WebCursorInfo::kTypePointer;
-  pointer_cursor_.InitFromCursorInfo(cursor_info);
+  pointer_cursor_ = WebCursor(CursorInfo(ui::CursorType::kPointer));
   return true;
 }
 
@@ -146,18 +167,19 @@ gfx::SizeF TouchEmulator::InitCursorFromResource(
   gfx::Image& cursor_image =
       content::GetContentClient()->GetNativeImageNamed(resource_id);
   CursorInfo cursor_info;
-  cursor_info.type = blink::WebCursorInfo::kTypeCustom;
+  cursor_info.type = ui::CursorType::kCustom;
   cursor_info.image_scale_factor = scale;
   cursor_info.custom_image = cursor_image.AsBitmap();
   cursor_info.hotspot =
       gfx::Point(cursor_image.Width() / 2, cursor_image.Height() / 2);
 
-  cursor->InitFromCursorInfo(cursor_info);
+  *cursor = WebCursor(cursor_info);
   return gfx::ScaleSize(gfx::SizeF(cursor_image.Size()), 1.f / scale);
 }
 
-bool TouchEmulator::HandleMouseEvent(const WebMouseEvent& mouse_event) {
-  if (!enabled())
+bool TouchEmulator::HandleMouseEvent(const WebMouseEvent& mouse_event,
+                                     RenderWidgetHostViewBase* target_view) {
+  if (!enabled() || mode_ != Mode::kEmulatingTouchFromMouse)
     return false;
 
   if (mouse_event.button == WebMouseEvent::Button::kRight &&
@@ -173,12 +195,12 @@ bool TouchEmulator::HandleMouseEvent(const WebMouseEvent& mouse_event) {
 
   if (mouse_event.GetType() == WebInputEvent::kMouseMove) {
     if (last_mouse_event_was_move_ &&
-        mouse_event.TimeStampSeconds() <
-            last_mouse_move_timestamp_ + kMouseMoveDropIntervalSeconds)
+        mouse_event.TimeStamp() <
+            last_mouse_move_timestamp_ + kMouseMoveDropInterval)
       return true;
 
     last_mouse_event_was_move_ = true;
-    last_mouse_move_timestamp_ = mouse_event.TimeStampSeconds();
+    last_mouse_move_timestamp_ = mouse_event.TimeStamp();
   } else {
     last_mouse_event_was_move_ = false;
   }
@@ -197,15 +219,18 @@ bool TouchEmulator::HandleMouseEvent(const WebMouseEvent& mouse_event) {
     return true;
   }
 
-  FillTouchEventAndPoint(mouse_event);
-  HandleEmulatedTouchEvent(touch_event_);
+  gfx::PointF pos_in_root = mouse_event.PositionInWidget();
+  if (target_view)
+    pos_in_root = target_view->TransformPointToRootCoordSpaceF(pos_in_root);
+  FillTouchEventAndPoint(mouse_event, pos_in_root);
+  HandleEmulatedTouchEvent(touch_event_, target_view);
 
   // Do not pass mouse events to the renderer.
   return true;
 }
 
 bool TouchEmulator::HandleMouseWheelEvent(const WebMouseWheelEvent& event) {
-  if (!enabled())
+  if (!enabled() || mode_ != Mode::kEmulatingTouchFromMouse)
     return false;
 
   // Send mouse wheel for easy scrolling when there is no active touch.
@@ -213,7 +238,7 @@ bool TouchEmulator::HandleMouseWheelEvent(const WebMouseWheelEvent& event) {
 }
 
 bool TouchEmulator::HandleKeyboardEvent(const WebKeyboardEvent& event) {
-  if (!enabled())
+  if (!enabled() || mode_ != Mode::kEmulatingTouchFromMouse)
     return false;
 
   if (!UpdateShiftPressed((event.GetModifiers() & WebInputEvent::kShiftKey) !=
@@ -251,12 +276,14 @@ bool TouchEmulator::HandleTouchEvent(const blink::WebTouchEvent& event) {
   return false;
 }
 
-void TouchEmulator::HandleEmulatedTouchEvent(blink::WebTouchEvent event) {
+bool TouchEmulator::HandleEmulatedTouchEvent(
+    blink::WebTouchEvent event,
+    RenderWidgetHostViewBase* target_view) {
   DCHECK(gesture_provider_);
   event.unique_touch_event_id = ui::GetNextTouchEventId();
   auto result = gesture_provider_->OnTouchEvent(MotionEventWeb(event));
   if (!result.succeeded)
-    return;
+    return true;
 
   const bool event_consumed = true;
   const bool is_source_touch_event_set_non_blocking = false;
@@ -265,7 +292,7 @@ void TouchEmulator::HandleEmulatedTouchEvent(blink::WebTouchEvent event) {
     gesture_provider_->OnTouchEventAck(event.unique_touch_event_id,
                                        event_consumed,
                                        is_source_touch_event_set_non_blocking);
-    return;
+    return true;
   }
 
   bool is_sequence_start = WebTouchEventTraits::IsTouchSequenceStart(event);
@@ -274,14 +301,17 @@ void TouchEmulator::HandleEmulatedTouchEvent(blink::WebTouchEvent event) {
     gesture_provider_->OnTouchEventAck(event.unique_touch_event_id,
                                        event_consumed,
                                        is_source_touch_event_set_non_blocking);
-    return;
+    return true;
   }
 
-  if (is_sequence_start)
+  if (is_sequence_start) {
     emulated_stream_active_sequence_count_++;
+    last_emulated_start_target_ = target_view;
+  }
 
   event.moved_beyond_slop_region = result.moved_beyond_slop_region;
-  client_->ForwardEmulatedTouchEvent(event);
+  client_->ForwardEmulatedTouchEvent(event, target_view);
+  return false;
 }
 
 bool TouchEmulator::HandleTouchEventAck(
@@ -291,11 +321,15 @@ bool TouchEmulator::HandleTouchEventAck(
     if (is_sequence_end)
       emulated_stream_active_sequence_count_--;
 
+    int taps_count_before = pending_taps_count_;
     const bool event_consumed = ack_result == INPUT_EVENT_ACK_STATE_CONSUMED;
-    if (gesture_provider_)
+    if (gesture_provider_) {
       gesture_provider_->OnTouchEventAck(
           event.unique_touch_event_id, event_consumed,
           InputEventAckStateIsSetNonBlocking(ack_result));
+    }
+    if (pending_taps_count_ == taps_count_before)
+      OnInjectedTouchCompleted();
     return true;
   }
 
@@ -306,10 +340,30 @@ bool TouchEmulator::HandleTouchEventAck(
   return false;
 }
 
+void TouchEmulator::OnGestureEventAck(const WebGestureEvent& event,
+                                      RenderWidgetHostViewBase*) {
+  if (event.GetType() != WebInputEvent::kGestureTap)
+    return;
+  if (pending_taps_count_) {
+    pending_taps_count_--;
+    OnInjectedTouchCompleted();
+  }
+}
+
+void TouchEmulator::OnViewDestroyed(RenderWidgetHostViewBase* destroyed_view) {
+  if (destroyed_view != last_emulated_start_target_)
+    return;
+
+  last_emulated_start_target_ = nullptr;
+  emulated_stream_active_sequence_count_ = 0;
+}
+
 void TouchEmulator::OnGestureEvent(const ui::GestureEventData& gesture) {
   WebGestureEvent gesture_event =
       ui::CreateWebGestureEventFromGestureEventData(gesture);
 
+  DCHECK(gesture_event.unique_touch_event_id);
+
   switch (gesture_event.GetType()) {
     case WebInputEvent::kUndefined:
       NOTREACHED() << "Undefined WebInputEvent type";
@@ -366,32 +420,60 @@ void TouchEmulator::OnGestureEvent(const ui::GestureEventData& gesture) {
       suppress_next_fling_cancel_ = false;
       break;
 
+    case WebInputEvent::kGestureTap:
+      pending_taps_count_++;
+      client_->ForwardEmulatedGestureEvent(gesture_event);
+      break;
+
     default:
       // Everything else goes through.
       client_->ForwardEmulatedGestureEvent(gesture_event);
   }
 }
 
-void TouchEmulator::CancelTouch() {
-  if (!emulated_stream_active_sequence_count_ || !enabled())
+bool TouchEmulator::RequiresDoubleTapGestureEvents() const {
+  return true;
+}
+
+void TouchEmulator::InjectTouchEvent(const blink::WebTouchEvent& event,
+                                     RenderWidgetHostViewBase* target_view,
+                                     base::OnceClosure callback) {
+  DCHECK(enabled() && mode_ == Mode::kInjectingTouchEvents);
+  touch_event_ = event;
+  injected_touch_completion_callbacks_.push(std::move(callback));
+  if (HandleEmulatedTouchEvent(touch_event_, target_view))
+    OnInjectedTouchCompleted();
+}
+
+void TouchEmulator::OnInjectedTouchCompleted() {
+  if (injected_touch_completion_callbacks_.empty())
     return;
+  if (!injected_touch_completion_callbacks_.front().is_null())
+    std::move(injected_touch_completion_callbacks_.front()).Run();
+  injected_touch_completion_callbacks_.pop();
+}
+
+void TouchEmulator::CancelTouch() {
+  if (!emulated_stream_active_sequence_count_ || !enabled() ||
+      mode_ != Mode::kEmulatingTouchFromMouse) {
+    return;
+  }
 
   WebTouchEventTraits::ResetTypeAndTouchStates(
-      WebInputEvent::kTouchCancel,
-      ui::EventTimeStampToSeconds(ui::EventTimeForNow()), &touch_event_);
+      WebInputEvent::kTouchCancel, ui::EventTimeForNow(), &touch_event_);
   DCHECK(gesture_provider_);
   if (gesture_provider_->GetCurrentDownEvent())
-    HandleEmulatedTouchEvent(touch_event_);
+    HandleEmulatedTouchEvent(touch_event_, last_emulated_start_target_);
 }
 
 void TouchEmulator::UpdateCursor() {
-  if (!enabled())
-    client_->SetCursor(pointer_cursor_);
-  else
+  DCHECK(enabled());
+  if (mode_ == Mode::kEmulatingTouchFromMouse)
     client_->SetCursor(InPinchGestureMode() ? pinch_cursor_ : touch_cursor_);
 }
 
 bool TouchEmulator::UpdateShiftPressed(bool shift_pressed) {
+  DCHECK(enabled() && mode_ == Mode::kEmulatingTouchFromMouse);
   if (shift_pressed_ == shift_pressed)
     return false;
   shift_pressed_ = shift_pressed;
@@ -403,7 +485,7 @@ void TouchEmulator::PinchBegin(const WebGestureEvent& event) {
   DCHECK(InPinchGestureMode());
   DCHECK(!pinch_gesture_active_);
   pinch_gesture_active_ = true;
-  pinch_anchor_ = gfx::Point(event.x, event.y);
+  pinch_anchor_ = event.PositionInWidget();
   pinch_scale_ = 1.f;
   WebGestureEvent pinch_event =
       GetPinchGestureEvent(WebInputEvent::kGesturePinchBegin, event);
@@ -412,7 +494,7 @@ void TouchEmulator::PinchBegin(const WebGestureEvent& event) {
 
 void TouchEmulator::PinchUpdate(const WebGestureEvent& event) {
   DCHECK(pinch_gesture_active_);
-  int dy = pinch_anchor_.y() - event.y;
+  float dy = pinch_anchor_.y() - event.PositionInWidget().y;
   float scale = exp(dy * 0.002f);
   WebGestureEvent pinch_event =
       GetPinchGestureEvent(WebInputEvent::kGesturePinchUpdate, event);
@@ -430,25 +512,26 @@ void TouchEmulator::PinchEnd(const WebGestureEvent& event) {
 }
 
 void TouchEmulator::ScrollEnd(const WebGestureEvent& event) {
-  WebGestureEvent scroll_event(WebInputEvent::kGestureScrollEnd,
-                               ModifiersWithoutMouseButtons(event),
-                               event.TimeStampSeconds());
-  scroll_event.source_device = blink::kWebGestureDeviceTouchscreen;
+  WebGestureEvent scroll_event(
+      WebInputEvent::kGestureScrollEnd, ModifiersWithoutMouseButtons(event),
+      event.TimeStamp(), blink::WebGestureDevice::kTouchscreen);
+  scroll_event.unique_touch_event_id = event.unique_touch_event_id;
   client_->ForwardEmulatedGestureEvent(scroll_event);
 }
 
 WebGestureEvent TouchEmulator::GetPinchGestureEvent(
     WebInputEvent::Type type,
-    const WebInputEvent& original_event) {
+    const WebGestureEvent& original_event) {
   WebGestureEvent event(type, ModifiersWithoutMouseButtons(original_event),
-                        original_event.TimeStampSeconds());
-  event.source_device = blink::kWebGestureDeviceTouchscreen;
-  event.x = pinch_anchor_.x();
-  event.y = pinch_anchor_.y();
+                        original_event.TimeStamp(),
+                        blink::WebGestureDevice::kTouchscreen);
+  event.SetPositionInWidget(pinch_anchor_);
+  event.unique_touch_event_id = original_event.unique_touch_event_id;
   return event;
 }
 
-void TouchEmulator::FillTouchEventAndPoint(const WebMouseEvent& mouse_event) {
+void TouchEmulator::FillTouchEventAndPoint(const WebMouseEvent& mouse_event,
+                                           const gfx::PointF& pos_in_root) {
   WebInputEvent::Type eventType;
   switch (mouse_event.GetType()) {
     case WebInputEvent::kMouseDown:
@@ -468,15 +551,18 @@ void TouchEmulator::FillTouchEventAndPoint(const WebMouseEvent& mouse_event) {
   touch_event_.touches_length = 1;
   touch_event_.SetModifiers(ModifiersWithoutMouseButtons(mouse_event));
   WebTouchEventTraits::ResetTypeAndTouchStates(
-      eventType, mouse_event.TimeStampSeconds(), &touch_event_);
+      eventType, mouse_event.TimeStamp(), &touch_event_);
   WebTouchPoint& point = touch_event_.touches[0];
   point.id = 0;
   point.radius_x = 0.5f * cursor_size_.width();
   point.radius_y = 0.5f * cursor_size_.height();
   point.force = eventType == WebInputEvent::kTouchEnd ? 0.f : 1.f;
   point.rotation_angle = 0.f;
-  point.SetPositionInWidget(mouse_event.PositionInWidget().x,
-                            mouse_event.PositionInWidget().y);
+  // We need to convert this to the root-view's coord space, otherwise the
+  // GestureRecognizer will potentially receive events for a moving widget,
+  // for example when scroll bubbling is taking place. The GestureRecognizer
+  // isn't designed to handle that.
+  point.SetPositionInWidget(pos_in_root);
   point.SetPositionInScreen(mouse_event.PositionInScreen().x,
                             mouse_event.PositionInScreen().y);
   point.tilt_x = 0;
@@ -485,7 +571,11 @@ void TouchEmulator::FillTouchEventAndPoint(const WebMouseEvent& mouse_event) {
 }
 
 bool TouchEmulator::InPinchGestureMode() const {
-  return shift_pressed_;
+  return shift_pressed_ || pinch_gesture_mode_for_testing_;
+}
+
+void TouchEmulator::SetPinchGestureModeForTesting(bool pinch_gesture_mode) {
+  pinch_gesture_mode_for_testing_ = pinch_gesture_mode;
 }
 
 }  // namespace content
