@@ -12,8 +12,8 @@
 #include <vector>
 
 #include "base/bind.h"
-#include "base/memory/ptr_util.h"
 #include "base/strings/string_number_conversions.h"
+#include "base/task/post_task.h"
 #include "base/values.h"
 #include "content/browser/devtools/devtools_agent_host_impl.h"
 #include "content/browser/devtools/service_worker_devtools_agent_host.h"
@@ -25,7 +25,9 @@
 #include "content/browser/service_worker/service_worker_version.h"
 #include "content/grit/content_resources.h"
 #include "content/public/browser/browser_context.h"
+#include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
+#include "content/public/browser/console_message.h"
 #include "content/public/browser/render_process_host.h"
 #include "content/public/browser/storage_partition.h"
 #include "content/public/browser/web_contents.h"
@@ -33,6 +35,7 @@
 #include "content/public/browser/web_ui_data_source.h"
 #include "content/public/common/child_process_host.h"
 #include "content/public/common/url_constants.h"
+#include "third_party/blink/public/mojom/service_worker/service_worker_object.mojom.h"
 
 using base::DictionaryValue;
 using base::ListValue;
@@ -44,18 +47,17 @@ namespace content {
 namespace {
 
 using GetRegistrationsCallback =
-    base::Callback<void(const std::vector<ServiceWorkerRegistrationInfo>&,
-                        const std::vector<ServiceWorkerVersionInfo>&,
-                        const std::vector<ServiceWorkerRegistrationInfo>&)>;
+    base::OnceCallback<void(const std::vector<ServiceWorkerRegistrationInfo>&,
+                            const std::vector<ServiceWorkerVersionInfo>&,
+                            const std::vector<ServiceWorkerRegistrationInfo>&)>;
 
 void OperationCompleteCallback(WeakPtr<ServiceWorkerInternalsUI> internals,
                                int callback_id,
-                               ServiceWorkerStatusCode status) {
+                               blink::ServiceWorkerStatusCode status) {
   if (!BrowserThread::CurrentlyOn(BrowserThread::UI)) {
-    BrowserThread::PostTask(
-        BrowserThread::UI,
-        FROM_HERE,
-        base::Bind(OperationCompleteCallback, internals, callback_id, status));
+    base::PostTaskWithTraits(FROM_HERE, {BrowserThread::UI},
+                             base::BindOnce(OperationCompleteCallback,
+                                            internals, callback_id, status));
     return;
   }
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
@@ -66,32 +68,6 @@ void OperationCompleteCallback(WeakPtr<ServiceWorkerInternalsUI> internals,
   }
 }
 
-void CallServiceWorkerVersionMethodWithVersionID(
-    ServiceWorkerInternalsUI::ServiceWorkerVersionMethod method,
-    scoped_refptr<ServiceWorkerContextWrapper> context,
-    int64_t version_id,
-    const ServiceWorkerInternalsUI::StatusCallback& callback) {
-  if (!BrowserThread::CurrentlyOn(BrowserThread::IO)) {
-    BrowserThread::PostTask(
-        BrowserThread::IO,
-        FROM_HERE,
-        base::Bind(CallServiceWorkerVersionMethodWithVersionID,
-                   method,
-                   context,
-                   version_id,
-                   callback));
-    return;
-  }
-
-  scoped_refptr<ServiceWorkerVersion> version =
-      context->GetLiveVersion(version_id);
-  if (!version.get()) {
-    callback.Run(SERVICE_WORKER_ERROR_NOT_FOUND);
-    return;
-  }
-  (*version.get().*method)(callback);
-}
-
 std::vector<const Value*> ConvertToRawPtrVector(
     const std::vector<std::unique_ptr<const Value>>& args) {
   std::vector<const Value*> args_rawptrs(args.size());
@@ -109,7 +85,7 @@ base::ProcessId GetRealProcessId(int process_host_id) {
   if (!rph)
     return base::kNullProcessId;
 
-  base::ProcessHandle handle = rph->GetHandle();
+  base::ProcessHandle handle = rph->GetProcess().Handle();
   if (handle == base::kNullProcessHandle)
     return base::kNullProcessId;
   // TODO(nhiroki): On Windows, |rph->GetHandle()| does not duplicate ownership
@@ -171,7 +147,7 @@ void UpdateVersionInfo(const ServiceWorkerVersionInfo& version,
   }
 
   info->SetString("script_url", version.script_url.spec());
-  info->SetString("version_id", base::Int64ToString(version.version_id));
+  info->SetString("version_id", base::NumberToString(version.version_id));
   info->SetInteger("process_id",
                    static_cast<int>(GetRealProcessId(version.process_id)));
   info->SetInteger("process_host_id", version.process_id);
@@ -181,27 +157,29 @@ void UpdateVersionInfo(const ServiceWorkerVersionInfo& version,
 
 std::unique_ptr<ListValue> GetRegistrationListValue(
     const std::vector<ServiceWorkerRegistrationInfo>& registrations) {
-  auto result = base::MakeUnique<ListValue>();
-  for (std::vector<ServiceWorkerRegistrationInfo>::const_iterator it =
-           registrations.begin();
-       it != registrations.end();
-       ++it) {
+  auto result = std::make_unique<ListValue>();
+  for (auto it = registrations.begin(); it != registrations.end(); ++it) {
     const ServiceWorkerRegistrationInfo& registration = *it;
-    auto registration_info = base::MakeUnique<DictionaryValue>();
-    registration_info->SetString("scope", registration.pattern.spec());
+    auto registration_info = std::make_unique<DictionaryValue>();
+    registration_info->SetString("scope", registration.scope.spec());
     registration_info->SetString(
-        "registration_id", base::Int64ToString(registration.registration_id));
+        "registration_id", base::NumberToString(registration.registration_id));
+    registration_info->SetBoolean("navigation_preload_enabled",
+                                  registration.navigation_preload_enabled);
+    registration_info->SetInteger(
+        "navigation_preload_header_length",
+        registration.navigation_preload_header_length);
 
     if (registration.active_version.version_id !=
-        kInvalidServiceWorkerVersionId) {
-      auto active_info = base::MakeUnique<DictionaryValue>();
+        blink::mojom::kInvalidServiceWorkerVersionId) {
+      auto active_info = std::make_unique<DictionaryValue>();
       UpdateVersionInfo(registration.active_version, active_info.get());
       registration_info->Set("active", std::move(active_info));
     }
 
     if (registration.waiting_version.version_id !=
-        kInvalidServiceWorkerVersionId) {
-      auto waiting_info = base::MakeUnique<DictionaryValue>();
+        blink::mojom::kInvalidServiceWorkerVersionId) {
+      auto waiting_info = std::make_unique<DictionaryValue>();
       UpdateVersionInfo(registration.waiting_version, waiting_info.get());
       registration_info->Set("waiting", std::move(waiting_info));
     }
@@ -213,12 +191,9 @@ std::unique_ptr<ListValue> GetRegistrationListValue(
 
 std::unique_ptr<ListValue> GetVersionListValue(
     const std::vector<ServiceWorkerVersionInfo>& versions) {
-  auto result = base::MakeUnique<ListValue>();
-  for (std::vector<ServiceWorkerVersionInfo>::const_iterator it =
-           versions.begin();
-       it != versions.end();
-       ++it) {
-    auto info = base::MakeUnique<DictionaryValue>();
+  auto result = std::make_unique<ListValue>();
+  for (auto it = versions.begin(); it != versions.end(); ++it) {
+    auto info = std::make_unique<DictionaryValue>();
     UpdateVersionInfo(*it, info.get());
     result->Append(std::move(info));
   }
@@ -227,22 +202,22 @@ std::unique_ptr<ListValue> GetVersionListValue(
 
 void DidGetStoredRegistrationsOnIOThread(
     scoped_refptr<ServiceWorkerContextWrapper> context,
-    const GetRegistrationsCallback& callback,
-    ServiceWorkerStatusCode status,
+    GetRegistrationsCallback callback,
+    blink::ServiceWorkerStatusCode status,
     const std::vector<ServiceWorkerRegistrationInfo>& stored_registrations) {
   DCHECK_CURRENTLY_ON(BrowserThread::IO);
-  BrowserThread::PostTask(
-      BrowserThread::UI, FROM_HERE,
-      base::Bind(callback, context->GetAllLiveRegistrationInfo(),
-                 context->GetAllLiveVersionInfo(), stored_registrations));
+  base::PostTaskWithTraits(
+      FROM_HERE, {BrowserThread::UI},
+      base::BindOnce(std::move(callback), context->GetAllLiveRegistrationInfo(),
+                     context->GetAllLiveVersionInfo(), stored_registrations));
 }
 
 void GetRegistrationsOnIOThread(
     scoped_refptr<ServiceWorkerContextWrapper> context,
-    const GetRegistrationsCallback& callback) {
+    GetRegistrationsCallback callback) {
   DCHECK_CURRENTLY_ON(BrowserThread::IO);
-  context->GetAllRegistrations(
-      base::Bind(DidGetStoredRegistrationsOnIOThread, context, callback));
+  context->GetAllRegistrations(base::BindOnce(
+      DidGetStoredRegistrationsOnIOThread, context, std::move(callback)));
 }
 
 void DidGetRegistrations(
@@ -260,8 +235,8 @@ void DidGetRegistrations(
   args.push_back(GetRegistrationListValue(live_registrations));
   args.push_back(GetVersionListValue(live_versions));
   args.push_back(GetRegistrationListValue(stored_registrations));
-  args.push_back(base::MakeUnique<Value>(partition_id));
-  args.push_back(base::MakeUnique<Value>(context_path.value()));
+  args.push_back(std::make_unique<Value>(partition_id));
+  args.push_back(std::make_unique<Value>(context_path.value()));
   internals->web_ui()->CallJavascriptFunctionUnsafe(
       "serviceworker.onPartitionData", ConvertToRawPtrVector(args));
 }
@@ -280,26 +255,23 @@ class ServiceWorkerInternalsUI::PartitionObserver
     DCHECK_CURRENTLY_ON(BrowserThread::UI);
     web_ui_->CallJavascriptFunctionUnsafe(
         "serviceworker.onRunningStateChanged", Value(partition_id_),
-        Value(base::Int64ToString(version_id)));
+        Value(base::NumberToString(version_id)));
   }
   void OnVersionStateChanged(int64_t version_id,
+                             const GURL& scope,
                              ServiceWorkerVersion::Status) override {
     DCHECK_CURRENTLY_ON(BrowserThread::UI);
     web_ui_->CallJavascriptFunctionUnsafe(
         "serviceworker.onVersionStateChanged", Value(partition_id_),
-        Value(base::Int64ToString(version_id)));
+        Value(base::NumberToString(version_id)));
   }
   void OnErrorReported(int64_t version_id,
-                       int process_id,
-                       int thread_id,
                        const ErrorInfo& info) override {
     DCHECK_CURRENTLY_ON(BrowserThread::UI);
     std::vector<std::unique_ptr<const Value>> args;
-    args.push_back(base::MakeUnique<Value>(partition_id_));
-    args.push_back(base::MakeUnique<Value>(base::Int64ToString(version_id)));
-    args.push_back(base::MakeUnique<Value>(process_id));
-    args.push_back(base::MakeUnique<Value>(thread_id));
-    auto value = base::MakeUnique<DictionaryValue>();
+    args.push_back(std::make_unique<Value>(partition_id_));
+    args.push_back(std::make_unique<Value>(base::NumberToString(version_id)));
+    auto value = std::make_unique<DictionaryValue>();
     value->SetString("message", info.error_message);
     value->SetInteger("lineNumber", info.line_number);
     value->SetInteger("columnNumber", info.column_number);
@@ -309,18 +281,14 @@ class ServiceWorkerInternalsUI::PartitionObserver
                                           ConvertToRawPtrVector(args));
   }
   void OnReportConsoleMessage(int64_t version_id,
-                              int process_id,
-                              int thread_id,
                               const ConsoleMessage& message) override {
     DCHECK_CURRENTLY_ON(BrowserThread::UI);
     std::vector<std::unique_ptr<const Value>> args;
-    args.push_back(base::MakeUnique<Value>(partition_id_));
-    args.push_back(base::MakeUnique<Value>(base::Int64ToString(version_id)));
-    args.push_back(base::MakeUnique<Value>(process_id));
-    args.push_back(base::MakeUnique<Value>(thread_id));
-    auto value = base::MakeUnique<DictionaryValue>();
-    value->SetInteger("sourceIdentifier", message.source_identifier);
-    value->SetInteger("message_level", message.message_level);
+    args.push_back(std::make_unique<Value>(partition_id_));
+    args.push_back(std::make_unique<Value>(base::NumberToString(version_id)));
+    auto value = std::make_unique<DictionaryValue>();
+    value->SetInteger("sourceIdentifier", static_cast<int>(message.source));
+    value->SetInteger("message_level", static_cast<int>(message.message_level));
     value->SetString("message", message.message);
     value->SetInteger("lineNumber", message.line_number);
     value->SetString("sourceURL", message.source_url.spec());
@@ -328,16 +296,16 @@ class ServiceWorkerInternalsUI::PartitionObserver
     web_ui_->CallJavascriptFunctionUnsafe(
         "serviceworker.onConsoleMessageReported", ConvertToRawPtrVector(args));
   }
-  void OnRegistrationStored(int64_t registration_id,
-                            const GURL& pattern) override {
+  void OnRegistrationCompleted(int64_t registration_id,
+                               const GURL& scope) override {
     DCHECK_CURRENTLY_ON(BrowserThread::UI);
-    web_ui_->CallJavascriptFunctionUnsafe("serviceworker.onRegistrationStored",
-                                          Value(pattern.spec()));
+    web_ui_->CallJavascriptFunctionUnsafe(
+        "serviceworker.onRegistrationCompleted", Value(scope.spec()));
   }
   void OnRegistrationDeleted(int64_t registration_id,
-                             const GURL& pattern) override {
+                             const GURL& scope) override {
     web_ui_->CallJavascriptFunctionUnsafe("serviceworker.onRegistrationDeleted",
-                                          Value(pattern.spec()));
+                                          Value(scope.spec()));
   }
   int partition_id() const { return partition_id_; }
 
@@ -350,6 +318,8 @@ ServiceWorkerInternalsUI::ServiceWorkerInternalsUI(WebUI* web_ui)
     : WebUIController(web_ui), next_partition_id_(0) {
   WebUIDataSource* source =
       WebUIDataSource::Create(kChromeUIServiceWorkerInternalsHost);
+  source->OverrideContentSecurityPolicyScriptSrc(
+      "script-src chrome://resources 'self' 'unsafe-eval';");
   source->SetJsonPath("strings.js");
   source->AddResourcePath("serviceworker_internals.js",
                           IDR_SERVICE_WORKER_INTERNALS_JS);
@@ -357,40 +327,33 @@ ServiceWorkerInternalsUI::ServiceWorkerInternalsUI(WebUI* web_ui)
                           IDR_SERVICE_WORKER_INTERNALS_CSS);
   source->SetDefaultResource(IDR_SERVICE_WORKER_INTERNALS_HTML);
   source->DisableDenyXFrameOptions();
-  source->UseGzip(std::unordered_set<std::string>());
 
   BrowserContext* browser_context =
       web_ui->GetWebContents()->GetBrowserContext();
   WebUIDataSource::Add(browser_context, source);
 
   web_ui->RegisterMessageCallback(
-      "GetOptions",
-      base::Bind(&ServiceWorkerInternalsUI::GetOptions,
-                 base::Unretained(this)));
+      "GetOptions", base::BindRepeating(&ServiceWorkerInternalsUI::GetOptions,
+                                        base::Unretained(this)));
   web_ui->RegisterMessageCallback(
-      "SetOption",
-      base::Bind(&ServiceWorkerInternalsUI::SetOption, base::Unretained(this)));
+      "SetOption", base::BindRepeating(&ServiceWorkerInternalsUI::SetOption,
+                                       base::Unretained(this)));
   web_ui->RegisterMessageCallback(
       "getAllRegistrations",
-      base::Bind(&ServiceWorkerInternalsUI::GetAllRegistrations,
-                 base::Unretained(this)));
+      base::BindRepeating(&ServiceWorkerInternalsUI::GetAllRegistrations,
+                          base::Unretained(this)));
   web_ui->RegisterMessageCallback(
-      "stop",
-      base::Bind(&ServiceWorkerInternalsUI::CallServiceWorkerVersionMethod,
-                 base::Unretained(this),
-                 &ServiceWorkerVersion::StopWorker));
+      "stop", base::BindRepeating(&ServiceWorkerInternalsUI::StopWorker,
+                                  base::Unretained(this)));
   web_ui->RegisterMessageCallback(
-      "inspect",
-      base::Bind(&ServiceWorkerInternalsUI::InspectWorker,
-                 base::Unretained(this)));
+      "inspect", base::BindRepeating(&ServiceWorkerInternalsUI::InspectWorker,
+                                     base::Unretained(this)));
   web_ui->RegisterMessageCallback(
-      "unregister",
-      base::Bind(&ServiceWorkerInternalsUI::Unregister,
-                 base::Unretained(this)));
+      "unregister", base::BindRepeating(&ServiceWorkerInternalsUI::Unregister,
+                                        base::Unretained(this)));
   web_ui->RegisterMessageCallback(
-      "start",
-      base::Bind(&ServiceWorkerInternalsUI::StartWorker,
-                 base::Unretained(this)));
+      "start", base::BindRepeating(&ServiceWorkerInternalsUI::StartWorker,
+                                   base::Unretained(this)));
 }
 
 ServiceWorkerInternalsUI::~ServiceWorkerInternalsUI() {
@@ -399,9 +362,11 @@ ServiceWorkerInternalsUI::~ServiceWorkerInternalsUI() {
   // Safe to use base::Unretained(this) because
   // ForEachStoragePartition is synchronous.
   BrowserContext::StoragePartitionCallback remove_observer_cb =
-      base::Bind(&ServiceWorkerInternalsUI::RemoveObserverFromStoragePartition,
-                 base::Unretained(this));
-  BrowserContext::ForEachStoragePartition(browser_context, remove_observer_cb);
+      base::BindRepeating(
+          &ServiceWorkerInternalsUI::RemoveObserverFromStoragePartition,
+          base::Unretained(this));
+  BrowserContext::ForEachStoragePartition(browser_context,
+                                          std::move(remove_observer_cb));
 }
 
 void ServiceWorkerInternalsUI::GetOptions(const ListValue* args) {
@@ -429,10 +394,11 @@ void ServiceWorkerInternalsUI::GetAllRegistrations(const ListValue* args) {
       web_ui()->GetWebContents()->GetBrowserContext();
   // Safe to use base::Unretained(this) because
   // ForEachStoragePartition is synchronous.
-  BrowserContext::StoragePartitionCallback add_context_cb =
-      base::Bind(&ServiceWorkerInternalsUI::AddContextFromStoragePartition,
-                 base::Unretained(this));
-  BrowserContext::ForEachStoragePartition(browser_context, add_context_cb);
+  BrowserContext::StoragePartitionCallback add_context_cb = base::BindRepeating(
+      &ServiceWorkerInternalsUI::AddContextFromStoragePartition,
+      base::Unretained(this));
+  BrowserContext::ForEachStoragePartition(browser_context,
+                                          std::move(add_context_cb));
 }
 
 void ServiceWorkerInternalsUI::AddContextFromStoragePartition(
@@ -447,18 +413,19 @@ void ServiceWorkerInternalsUI::AddContextFromStoragePartition(
   } else {
     partition_id = next_partition_id_++;
     auto new_observer =
-        base::MakeUnique<PartitionObserver>(partition_id, web_ui());
+        std::make_unique<PartitionObserver>(partition_id, web_ui());
     context->AddObserver(new_observer.get());
     observers_[reinterpret_cast<uintptr_t>(partition)] =
         std::move(new_observer);
   }
 
-  BrowserThread::PostTask(
-      BrowserThread::IO, FROM_HERE,
-      base::Bind(GetRegistrationsOnIOThread, context,
-                 base::Bind(DidGetRegistrations, AsWeakPtr(), partition_id,
-                            context->is_incognito() ? base::FilePath()
-                                                    : partition->GetPath())));
+  base::PostTaskWithTraits(
+      FROM_HERE, {BrowserThread::IO},
+      base::BindOnce(
+          GetRegistrationsOnIOThread, context,
+          base::BindOnce(DidGetRegistrations, AsWeakPtr(), partition_id,
+                         context->is_incognito() ? base::FilePath()
+                                                 : partition->GetPath())));
 }
 
 void ServiceWorkerInternalsUI::RemoveObserverFromStoragePartition(
@@ -489,13 +456,13 @@ bool ServiceWorkerInternalsUI::GetServiceWorkerContext(
     scoped_refptr<ServiceWorkerContextWrapper>* context) const {
   BrowserContext* browser_context =
       web_ui()->GetWebContents()->GetBrowserContext();
-  StoragePartition* result_partition(NULL);
+  StoragePartition* result_partition(nullptr);
   BrowserContext::StoragePartitionCallback find_context_cb =
-      base::Bind(&ServiceWorkerInternalsUI::FindContext,
-                 base::Unretained(this),
-                 partition_id,
-                 &result_partition);
-  BrowserContext::ForEachStoragePartition(browser_context, find_context_cb);
+      base::BindRepeating(&ServiceWorkerInternalsUI::FindContext,
+                          base::Unretained(this), partition_id,
+                          &result_partition);
+  BrowserContext::ForEachStoragePartition(browser_context,
+                                          std::move(find_context_cb));
   if (!result_partition)
     return false;
   *context = static_cast<ServiceWorkerContextWrapper*>(
@@ -503,12 +470,10 @@ bool ServiceWorkerInternalsUI::GetServiceWorkerContext(
   return true;
 }
 
-void ServiceWorkerInternalsUI::CallServiceWorkerVersionMethod(
-    ServiceWorkerVersionMethod method,
-    const ListValue* args) {
+void ServiceWorkerInternalsUI::StopWorker(const ListValue* args) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   int callback_id;
-  const DictionaryValue* cmd_args = NULL;
+  const DictionaryValue* cmd_args = nullptr;
   int partition_id;
   scoped_refptr<ServiceWorkerContextWrapper> context;
   std::string version_id_string;
@@ -522,16 +487,15 @@ void ServiceWorkerInternalsUI::CallServiceWorkerVersionMethod(
     return;
   }
 
-  base::Callback<void(ServiceWorkerStatusCode)> callback =
-      base::Bind(OperationCompleteCallback, AsWeakPtr(), callback_id);
-  CallServiceWorkerVersionMethodWithVersionID(
-      method, context, version_id, callback);
+  base::OnceCallback<void(blink::ServiceWorkerStatusCode)> callback =
+      base::BindOnce(OperationCompleteCallback, AsWeakPtr(), callback_id);
+  StopWorkerWithId(context, version_id, std::move(callback));
 }
 
 void ServiceWorkerInternalsUI::InspectWorker(const ListValue* args) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   int callback_id;
-  const DictionaryValue* cmd_args = NULL;
+  const DictionaryValue* cmd_args = nullptr;
   int process_host_id = 0;
   int devtools_agent_route_id = 0;
   if (!args->GetInteger(0, &callback_id) ||
@@ -541,18 +505,18 @@ void ServiceWorkerInternalsUI::InspectWorker(const ListValue* args) {
                             &devtools_agent_route_id)) {
     return;
   }
-  base::Callback<void(ServiceWorkerStatusCode)> callback =
-      base::Bind(OperationCompleteCallback, AsWeakPtr(), callback_id);
+  base::OnceCallback<void(blink::ServiceWorkerStatusCode)> callback =
+      base::BindOnce(OperationCompleteCallback, AsWeakPtr(), callback_id);
   scoped_refptr<ServiceWorkerDevToolsAgentHost> agent_host(
       ServiceWorkerDevToolsManager::GetInstance()
           ->GetDevToolsAgentHostForWorker(process_host_id,
                                           devtools_agent_route_id));
   if (!agent_host.get()) {
-    callback.Run(SERVICE_WORKER_ERROR_NOT_FOUND);
+    std::move(callback).Run(blink::ServiceWorkerStatusCode::kErrorNotFound);
     return;
   }
   agent_host->Inspect();
-  callback.Run(SERVICE_WORKER_OK);
+  std::move(callback).Run(blink::ServiceWorkerStatusCode::kOk);
 }
 
 void ServiceWorkerInternalsUI::Unregister(const ListValue* args) {
@@ -560,7 +524,7 @@ void ServiceWorkerInternalsUI::Unregister(const ListValue* args) {
   int callback_id;
   int partition_id;
   std::string scope_string;
-  const DictionaryValue* cmd_args = NULL;
+  const DictionaryValue* cmd_args = nullptr;
   scoped_refptr<ServiceWorkerContextWrapper> context;
   if (!args->GetInteger(0, &callback_id) ||
       !args->GetDictionary(1, &cmd_args) ||
@@ -570,9 +534,9 @@ void ServiceWorkerInternalsUI::Unregister(const ListValue* args) {
     return;
   }
 
-  base::Callback<void(ServiceWorkerStatusCode)> callback =
-      base::Bind(OperationCompleteCallback, AsWeakPtr(), callback_id);
-  UnregisterWithScope(context, GURL(scope_string), callback);
+  base::OnceCallback<void(blink::ServiceWorkerStatusCode)> callback =
+      base::BindOnce(OperationCompleteCallback, AsWeakPtr(), callback_id);
+  UnregisterWithScope(context, GURL(scope_string), std::move(callback));
 }
 
 void ServiceWorkerInternalsUI::StartWorker(const ListValue* args) {
@@ -580,7 +544,7 @@ void ServiceWorkerInternalsUI::StartWorker(const ListValue* args) {
   int callback_id;
   int partition_id;
   std::string scope_string;
-  const DictionaryValue* cmd_args = NULL;
+  const DictionaryValue* cmd_args = nullptr;
   scoped_refptr<ServiceWorkerContextWrapper> context;
   if (!args->GetInteger(0, &callback_id) ||
       !args->GetDictionary(1, &cmd_args) ||
@@ -589,31 +553,59 @@ void ServiceWorkerInternalsUI::StartWorker(const ListValue* args) {
       !cmd_args->GetString("scope", &scope_string)) {
     return;
   }
-  base::Callback<void(ServiceWorkerStatusCode)> callback =
-      base::Bind(OperationCompleteCallback, AsWeakPtr(), callback_id);
-  context->StartServiceWorker(GURL(scope_string), callback);
+  base::OnceCallback<void(blink::ServiceWorkerStatusCode)> callback =
+      base::BindOnce(OperationCompleteCallback, AsWeakPtr(), callback_id);
+  context->StartServiceWorker(GURL(scope_string), std::move(callback));
+}
+
+void ServiceWorkerInternalsUI::StopWorkerWithId(
+    scoped_refptr<ServiceWorkerContextWrapper> context,
+    int64_t version_id,
+    StatusCallback callback) {
+  if (!BrowserThread::CurrentlyOn(BrowserThread::IO)) {
+    base::PostTaskWithTraits(
+        FROM_HERE, {BrowserThread::IO},
+        base::BindOnce(&ServiceWorkerInternalsUI::StopWorkerWithId,
+                       base::Unretained(this), context, version_id,
+                       std::move(callback)));
+    return;
+  }
+
+  scoped_refptr<ServiceWorkerVersion> version =
+      context->GetLiveVersion(version_id);
+  if (!version.get()) {
+    std::move(callback).Run(blink::ServiceWorkerStatusCode::kErrorNotFound);
+    return;
+  }
+
+  // ServiceWorkerVersion::StopWorker() takes a base::OnceClosure for argument,
+  // so bind blink::ServiceWorkerStatusCode::kOk to callback here.
+  version->StopWorker(
+      base::BindOnce(std::move(callback), blink::ServiceWorkerStatusCode::kOk));
 }
 
 void ServiceWorkerInternalsUI::UnregisterWithScope(
     scoped_refptr<ServiceWorkerContextWrapper> context,
     const GURL& scope,
-    const ServiceWorkerInternalsUI::StatusCallback& callback) const {
+    ServiceWorkerInternalsUI::StatusCallback callback) const {
   if (!BrowserThread::CurrentlyOn(BrowserThread::IO)) {
-    BrowserThread::PostTask(
-        BrowserThread::IO, FROM_HERE,
-        base::Bind(&ServiceWorkerInternalsUI::UnregisterWithScope,
-                   base::Unretained(this), context, scope, callback));
+    base::PostTaskWithTraits(
+        FROM_HERE, {BrowserThread::IO},
+        base::BindOnce(&ServiceWorkerInternalsUI::UnregisterWithScope,
+                       base::Unretained(this), context, scope,
+                       std::move(callback)));
     return;
   }
 
   if (!context->context()) {
-    callback.Run(SERVICE_WORKER_ERROR_ABORT);
+    std::move(callback).Run(blink::ServiceWorkerStatusCode::kErrorAbort);
     return;
   }
 
   // ServiceWorkerContextWrapper::UnregisterServiceWorker doesn't work here
   // because that reduces a status code to boolean.
-  context->context()->UnregisterServiceWorker(scope, callback);
+  context->context()->UnregisterServiceWorker(
+      scope, base::AdaptCallbackForRepeating(std::move(callback)));
 }
 
 }  // namespace content
