@@ -264,6 +264,9 @@ class CONTENT_EXPORT StoragePartition {
   // Wait until all deletions tasks are finished. For test use only.
   virtual void WaitForDeletionTasksForTesting() = 0;
 
+  // Wait until code cache's shutdown is complete. For test use only.
+  virtual void WaitForCodeCacheShutdownForTesting() = 0;
+
  protected:
   virtual ~StoragePartition() {}
 };
