@@ -4,15 +4,19 @@
 
 #include "extensions/browser/api/networking_private/networking_private_service_client.h"
 
+#include <memory>
 #include <utility>
 
 #include "base/base64.h"
 #include "base/bind.h"
 #include "base/memory/ptr_util.h"
 #include "base/sequenced_task_runner.h"
+#include "base/task/lazy_task_runner.h"
+#include "base/task/post_task.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "components/onc/onc_constants.h"
 #include "content/public/browser/browser_thread.h"
+#include "content/public/browser/network_service_instance.h"
 #include "extensions/browser/api/networking_private/networking_private_api.h"
 #include "extensions/browser/api/networking_private/networking_private_delegate_observer.h"
 
@@ -23,14 +27,19 @@ namespace extensions {
 
 namespace {
 
-const char kNetworkingPrivateSequenceTokenName[] = "NetworkingPrivate";
-
 // Deletes WiFiService object on the worker thread.
 void ShutdownWifiServiceOnWorkerThread(
     std::unique_ptr<WiFiService> wifi_service) {
   DCHECK(wifi_service.get());
 }
 
+// Ensure that all calls to WiFiService are called from the same task runner
+// since the implementations do not provide any thread safety gaurantees.
+base::LazySequencedTaskRunner g_sequenced_task_runner =
+    LAZY_SEQUENCED_TASK_RUNNER_INITIALIZER(base::TaskTraits(
+        {base::ThreadPool(), base::MayBlock(), base::TaskPriority::USER_VISIBLE,
+         base::TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN}));
+
 }  // namespace
 
 NetworkingPrivateServiceClient::ServiceCallbacks::ServiceCallbacks() {
@@ -41,20 +50,16 @@ NetworkingPrivateServiceClient::ServiceCallbacks::~ServiceCallbacks() {
 
 NetworkingPrivateServiceClient::NetworkingPrivateServiceClient(
     std::unique_ptr<WiFiService> wifi_service)
-    : wifi_service_(std::move(wifi_service)), weak_factory_(this) {
-  sequence_token_ = BrowserThread::GetBlockingPool()->GetNamedSequenceToken(
-      kNetworkingPrivateSequenceTokenName);
-  task_runner_ =
-      BrowserThread::GetBlockingPool()
-          ->GetSequencedTaskRunnerWithShutdownBehavior(
-              sequence_token_, base::SequencedWorkerPool::CONTINUE_ON_SHUTDOWN);
+    : wifi_service_(std::move(wifi_service)),
+      task_runner_(g_sequenced_task_runner.Get()),
+      weak_factory_(this) {
   task_runner_->PostTask(
       FROM_HERE,
-      base::Bind(&WiFiService::Initialize,
-                 base::Unretained(wifi_service_.get()), task_runner_));
+      base::BindOnce(&WiFiService::Initialize,
+                     base::Unretained(wifi_service_.get()), task_runner_));
   task_runner_->PostTask(
       FROM_HERE,
-      base::Bind(
+      base::BindOnce(
           &WiFiService::SetEventObservers,
           base::Unretained(wifi_service_.get()),
           base::ThreadTaskRunnerHandle::Get(),
@@ -64,7 +69,7 @@ NetworkingPrivateServiceClient::NetworkingPrivateServiceClient(
           base::Bind(&NetworkingPrivateServiceClient::
                          OnNetworkListChangedEventOnUIThread,
                      weak_factory_.GetWeakPtr())));
-  net::NetworkChangeNotifier::AddNetworkChangeObserver(this);
+  content::GetNetworkConnectionTracker()->AddNetworkConnectionObserver(this);
 }
 
 NetworkingPrivateServiceClient::~NetworkingPrivateServiceClient() {
@@ -75,14 +80,14 @@ NetworkingPrivateServiceClient::~NetworkingPrivateServiceClient() {
 
 void NetworkingPrivateServiceClient::Shutdown() {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  net::NetworkChangeNotifier::RemoveNetworkChangeObserver(this);
+  content::GetNetworkConnectionTracker()->RemoveNetworkConnectionObserver(this);
   // Clear callbacks map to release callbacks from UI thread.
   callbacks_map_.Clear();
   // Post ShutdownWifiServiceOnWorkerThread task to delete services when all
   // posted tasks are done.
   task_runner_->PostTask(FROM_HERE,
-                         base::Bind(&ShutdownWifiServiceOnWorkerThread,
-                                    base::Passed(&wifi_service_)));
+                         base::BindOnce(&ShutdownWifiServiceOnWorkerThread,
+                                        std::move(wifi_service_)));
 }
 
 void NetworkingPrivateServiceClient::AddObserver(
@@ -95,11 +100,11 @@ void NetworkingPrivateServiceClient::RemoveObserver(
   network_events_observers_.RemoveObserver(observer);
 }
 
-void NetworkingPrivateServiceClient::OnNetworkChanged(
-    net::NetworkChangeNotifier::ConnectionType type) {
-  task_runner_->PostTask(FROM_HERE,
-                         base::Bind(&WiFiService::RequestConnectedNetworkUpdate,
-                                    base::Unretained(wifi_service_.get())));
+void NetworkingPrivateServiceClient::OnConnectionChanged(
+    network::mojom::ConnectionType type) {
+  task_runner_->PostTask(
+      FROM_HERE, base::BindOnce(&WiFiService::RequestConnectedNetworkUpdate,
+                                base::Unretained(wifi_service_.get())));
 }
 
 NetworkingPrivateServiceClient::ServiceCallbacks*
@@ -335,9 +340,17 @@ void NetworkingPrivateServiceClient::SetCellularSimState(
   failure_callback.Run(networking_private::kErrorNotSupported);
 }
 
+void NetworkingPrivateServiceClient::SelectCellularMobileNetwork(
+    const std::string& guid,
+    const std::string& network_id,
+    const VoidCallback& success_callback,
+    const FailureCallback& failure_callback) {
+  failure_callback.Run(networking_private::kErrorNotSupported);
+}
+
 std::unique_ptr<base::ListValue>
 NetworkingPrivateServiceClient::GetEnabledNetworkTypes() {
-  std::unique_ptr<base::ListValue> network_list;
+  auto network_list = std::make_unique<base::ListValue>();
   network_list->AppendString(::onc::network_type::kWiFi);
   return network_list;
 }
@@ -355,12 +368,12 @@ NetworkingPrivateServiceClient::GetDeviceStateList() {
 
 std::unique_ptr<base::DictionaryValue>
 NetworkingPrivateServiceClient::GetGlobalPolicy() {
-  return base::MakeUnique<base::DictionaryValue>();
+  return std::make_unique<base::DictionaryValue>();
 }
 
 std::unique_ptr<base::DictionaryValue>
 NetworkingPrivateServiceClient::GetCertificateLists() {
-  return base::MakeUnique<base::DictionaryValue>();
+  return std::make_unique<base::DictionaryValue>();
 }
 
 bool NetworkingPrivateServiceClient::EnableNetworkType(
@@ -373,10 +386,11 @@ bool NetworkingPrivateServiceClient::DisableNetworkType(
   return false;
 }
 
-bool NetworkingPrivateServiceClient::RequestScan() {
+bool NetworkingPrivateServiceClient::RequestScan(
+    const std::string& /* type */) {
   task_runner_->PostTask(FROM_HERE,
-                         base::Bind(&WiFiService::RequestNetworkScan,
-                                    base::Unretained(wifi_service_.get())));
+                         base::BindOnce(&WiFiService::RequestNetworkScan,
+                                        base::Unretained(wifi_service_.get())));
   return true;
 }
 
